@@ -17,22 +17,22 @@
 -define(NEED_OPENGL, 1).
 -define(NEED_ESDL, 1).
 -define(ERROR, error_msg(?LINE)).
--include("wings.hrl").
--include("e3d_image.hrl").
+-include_lib("wings/src/wings.hrl").
+-include_lib("wings/e3d/e3d_image.hrl").
+-include_lib("wings/e3d/e3d.hrl").
 -include("auv.hrl").
--include("e3d.hrl").
 
 -import(lists, [foreach/2,reverse/1,sort/1,foldl/3,member/2]).
 -import(auv_segment, [map_vertex/2]).
 
--define(OPT_BG, [{type_sel,color},{undefined,ignore},{1.0,1.0,1.0,0.0}]).
--define(OPT_EDGES, [all_edges,{0.0,0.0,0.0},1.0,false]).
+-define(OPT_BG, [auv_background, {type_sel,color},{undefined,ignore},{1.0,1.0,1.0,0.0}]).
+-define(OPT_EDGES, [auv_edges, all_edges,{0.0,0.0,0.0},1.0,false]).
 -define(OPT_FACES, [texture]).
 
 -record(opt, {texsz = {512,512},   %% Texture size
 	      no_renderers = 4,
 	      renderers = [{auv_background, ?OPT_BG},
-			   {auv_edges, [all_edges]}]
+			   {auv_edges, [?OPT_EDGES]}]
 	     }).
 
 -record(sh, {id=ignore,
@@ -60,7 +60,8 @@
  	 n,         % Normal            (binary) Optional
 	 bi,        % BiNormal          (binary) Optional
 	 bb,        % BoundingBox 3D pos
-	 vc        % Vertex colors     (binary)
+         vc,        % Vertex colors     (binary)
+         vbo        % Vbo               (binary)
 	}).
 
 -record(chart, 
@@ -80,53 +81,44 @@
 
 draw_options() ->
     [MaxTxs0|_] = gl:getIntegerv(?GL_MAX_TEXTURE_SIZE),
-    MaxTxs = max(min(4096, MaxTxs0), 256),
+    MaxTxs = max(min(8192, MaxTxs0), 256),
     Prefs = get_pref(tx_prefs, pref_to_list(#opt{})),
     TexSz = proplists:get_value(texsz, Prefs, 512),
     Shaders = shaders(),
-    Qs = [{hframe,[{menu,gen_tx_sizes(MaxTxs, []),TexSz,
-		    [{key,texsz}]}],
-	   [{title,?__(1,"Size")}]},
-	  {vframe,render_passes(Prefs, Shaders), [{title,?__(2,"Render")}]},
-	  {hframe,[{button,?__(4,"New Pass"),done,[{key,add_shader}]},
-		   {button,?__(5,"Delete Unused Pass"),done,[{key,del_shader}]}]}
+    Qs = [{hframe,[{menu, gen_tx_sizes(MaxTxs, []),TexSz,
+		    [{key,texsz}]}],[{title,?__(1,"Size")}]},
+	  {vframe, render_passes(Prefs, Shaders), [{title,?__(2,"Render")}]}
 	 ],
-    
-    wings_ask:dialog(?__(3,"Draw Options"), Qs,
-		     fun(Options) ->
-			     {{changed,New},Opt} = list_to_prefs(Options),
-			     set_pref([{tx_prefs,pref_to_list(Opt)}]),
-			     if New -> 
-				     {auv,{draw_options,restart}};
-				true  ->
-				     {auv,{draw_options,{Opt,Shaders}}}
-			     end
-		     end).
+
+    wings_dialog:dialog(?__(3,"Draw Options"), Qs,
+			fun(Options) ->
+				Opt = list_to_prefs(Options),
+				set_pref([{tx_prefs,pref_to_list(Opt)}]),
+				{auv,{draw_options,{Opt,Shaders}}}
+			end).
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %% Menu handling
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 render_passes(Prefs, Shaders) ->
-    %% Ugly count of shaders
-    NoOfPasses = max(length([ok || {{auv_pass,_},_} <- Prefs])-1, 1),
+    NoOfPasses = 7,
     Menu = renderers(Shaders),
-    Background = 
-	{hframe, 
-	 [{label, integer_to_list(0) ++ ": "},
-	  {menu,[{?__(1,"Background"), auv_background}],auv_background,
-	   [{key,{auv_pass,0}},layout]}, 
-	  {value,get_def(Prefs,auv_opt,0), store_opt(0)},
-	  {button,?__(2,"Options"),keep,[option_hook(0,background(),[]),
-					 drop_flags(0)]}],[]},
-    Other = [{hframe, 
-	      [{label, integer_to_list(Id) ++ ": "},
-	       {menu,Menu,default_menu(Id, Prefs),
-		[{key,{auv_pass,Id}},layout,pass_hook(Id)]}, 
-	       {value,get_def(Prefs, auv_opt, Id),store_opt(Id)},
+    Background =
+	{hframe,
+	 [{menu,[{?__(1,"Background"), auv_background}],auv_background,
+	   [{key,{auv_pass,0}}, enable_opt()]},
+	  {value, get_def(Prefs,auv_opt,0), [{key,{auv_opt,0}}]},
+	  {button,?__(2,"Options"),keep,
+	   [{key, {opt_butt, 0}}, option_hook(0,background(),[])]}],
+	 []},
+    Other = [{hframe,
+	      [{menu,Menu,default_menu(Id, Prefs),
+		[{key,{auv_pass,Id}}, enable_opt()]},
+	       {value,get_def(Prefs, auv_opt, Id),[{key,{auv_opt,Id}}]},
 	       {button,?__(3,"Options"),keep,
-		[option_hook(Id, Menu, Shaders),
-		 drop_flags(Id)]}],
+		[{key, {opt_butt, Id}}, option_hook(Id, Menu, Shaders)]}
+	      ],
 	      []} || Id <- lists:seq(1, NoOfPasses)],
     [Background|Other].
 
@@ -155,27 +147,55 @@ renderers(Shaders) ->
      {?__(2,"Draw Edges"),auv_edges},
      {?__(3,"Draw Faces"),auv_faces}|Menu0].
 
-options(auv_background, [{type_sel,Type},{Image,_},Color],_) ->
+option_dialog(Id, Fields, Renderers, Shaders) ->
+    try
+	Name = wings_dialog:get_value({auv_pass, Id}, Fields),
+	Opts = wings_dialog:get_value({auv_opt, Id}, Fields),
+	{StrName, Name} = renderer(Name,Renderers),
+	SetValue = fun(Res) ->
+			   %% Change the value in the parent dialog
+			   wings_dialog:set_value({auv_opt, Id}, [Name|Res], Fields),
+			   ignore
+		   end,
+	wings_dialog:dialog(StrName,options(Name,Opts,Shaders),SetValue)
+    catch _:Crash ->
+	    io:format("EXIT: ~p ~p~n",[Crash, erlang:get_stacktrace()])
+    end.
+
+options(auv_background, [auv_background, {type_sel,Type},{Image,_},Color],_) ->
+    Enable = fun(_, What, Fields) ->
+		     IsImage = image == What,
+		     wings_dialog:enable(image_sel, IsImage, Fields),
+		     wings_dialog:enable(col_sel,  not IsImage, Fields)
+	     end,
     [{hradio,[{?__(1,"Image"),image},{?__(2,"Color"),color}],
-      Type,[{key,type_sel},layout]},
+      Type,[{key,type_sel},{hook, Enable}]},
      {hframe,[{label,?__(1,"Image")},image_selector(Image)],
-      [is_enabled(image)]},
+      [{key, image_sel}]},
      {hframe,[{label,?__(2,"Color")},{color,fix(Color,wings_gl:have_fbo())}],
-      [is_enabled(color)]}];
-options(auv_background, _Bad,Sh) ->  
+      [{key, col_sel}]}];
+options(auv_background, _Bad,Sh) ->
     options(auv_background, ?OPT_BG,Sh);
-options(auv_edges,[Type,Color,Size,UseVtxColors],_) ->
+
+options(auv_edges,[auv_edges, Type,Color,Size,UseVtxColors],_) ->
     [{vradio,[{?__(3,"Draw All Edges"),all_edges},
-	      {?__(4,"Draw Border Edges"), border_edges}], 
+	      {?__(4,"Draw Border Edges"), border_edges}],
       Type, []},
      {hframe,[{label,?__(5,"Edge Color:")},{color,Color}]},
      {hframe,[{label,?__(6,"Edge Width:")},{text,Size,[{range,{0.0,100.0}}]}]},
      {?__(8,"Use vertex colors (on border edges)"),UseVtxColors}
     ];
 options(auv_edges,_,Sh) -> options(auv_edges,?OPT_EDGES,Sh);
-options({shader,Id},Vals,Sh) ->
+
+options({shader,Id}, Vals0, Sh) ->
     {value,Shader} = lists:keysearch(Id,#sh.id,Sh),
-    shader_options(Shader,Vals);
+    case Vals0 of
+	[{shader,Id}|Vals] ->
+	    shader_options(Shader,Vals);
+	_ ->
+	    shader_options(Shader,[])
+    end;
+
 options(Command,Vals,_) ->
     io:format("~p: ~p~n",[Command, Vals]),
     exit(unknown_default).
@@ -184,7 +204,7 @@ shader_options(#sh{args=Args,def=Defs,file=File}, Vals) ->
     case shader_menu(Args,reverse(Vals),[]) of
 	{failed,_} ->
 	    case shader_menu(Args,Defs,[]) of
-		{failed,What} -> 
+		{failed,What} ->
 		    io:format("AUV: Bad default value ~p in ~p~n",
 			      [What, File]),
 		    [];
@@ -246,56 +266,34 @@ image_selector(Default) ->
     end,
     {menu,Menu,Def,[layout]}.
 
-is_enabled(Type) ->
-    {hook, fun(is_minimized, {_Var,_I,Store}) -> 
-		   case gb_trees:get(type_sel, Store) of
-		       Type ->   false;
-		       _Other -> true
-		   end;
-	      (_,_) -> void
-	   end}.
+enable_opt() ->
+    {hook,
+     fun({_W, Id}, Pass, Fields) ->
+	     Disable = Pass =:= ignore orelse Pass =:= auv_faces,
+	     wings_dialog:enable({opt_butt, Id},  not Disable, Fields),
+	     case wings_dialog:get_value({auv_opt, Id}, Fields) of
+		 [Pass|_] ->
+		     ok;
+		 _ -> %% Changed type reset options
+		     wings_dialog:set_value({auv_opt, Id}, [], Fields)
+	     end
+     end}.
 
 option_hook(Id,Renderers,Shaders) ->
-    {hook, fun(is_disabled,{_Var,_I,Sto}) ->
-		   Pass = gb_trees:get({auv_pass,Id}, Sto),
-		   Pass =:= ignore orelse Pass =:= auv_faces;
-	      (is_minimized, _) ->
-		   false;
-	      (update,{_Var,_I,_B,Sto}) ->
-		   Name = gb_trees:get({auv_pass,Id},Sto),
-		   render_option_dialog(Id,renderer(Name,Renderers),
-					Shaders,Sto);
-	      (_,_) -> void
-	   end}.
-
-pass_hook(Id) ->
-    {hook, fun(update,{Var,_I,B,Sto0}) ->
-		   VarE = {auv_opt,Id},
-		   Sto1 = gb_trees:enter(VarE,[],Sto0),
-		   {store, gb_trees:enter(Var,B,Sto1)};
-	      (_,_) -> void
-	   end}.
+    {hook,
+     fun(_Key, button_pressed, Fields) ->
+	     Env = wx:get_env(),
+	     spawn(fun() ->
+			   %% Neeed open dialog in dialog from another process
+			   wx:set_env(Env),
+			   option_dialog(Id, Fields, Renderers, Shaders)
+		   end)
+     end
+    }.
 
 renderer(Id,[Renderer={_,Id}|_R]) ->  Renderer;
 renderer(Id,[_|R]) ->  renderer(Id,R).
 
-render_option_dialog(Id,{StrName,Name},Shaders,Sto) ->
-    Fun = render_option_fun(wings_wm:this()),
-    Opt = gb_trees:get({auv_opt,Id},Sto),
-    wings_ask:dialog(StrName,options(Name,Opt,Shaders),Fun).
-
-render_option_fun(Parent) ->
-    fun(What) -> wpa:drop(Parent, {render_opt, What}) end.
-
-store_opt(Id) ->
-    [{key,{auv_opt,Id}},
-     {hook, fun(update,{Var,_I,_B={render_opt,Opt},Sto}) -> 
-		    {store, gb_trees:enter(Var,Opt,Sto)};
-	       (_,_) -> void
-	    end}].
-drop_flags(Id) ->
-    {drop_flags, [{index,-1}|store_opt(Id)]}.
-
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %% Texture creation
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -313,76 +311,83 @@ get_texture(St = #st{bb=#uvstate{}}, {Options,Shaders}) ->
 %% Texture Rendering
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-render_image(#ts{uv=UVpos,pos=Pos,n=Ns,bi=BiNs,vc=Vc}=Geom,
-	     Passes, #opt{texsz={TexW,TexH}}, Reqs) ->
+render_image(Geom0, Passes, #opt{texsz={TexW,TexH}}, Reqs) ->
     gl:pushAttrib(?GL_ALL_ATTRIB_BITS),
-    
+
     Current = wings_wm:viewport(),
     UsingFbo = setup_fbo(TexW,TexH),
-    {W0,H0} = if not UsingFbo -> wings_wm:top_size();
-		 true -> {TexW,TexH}
+    {W0,H0} = if not UsingFbo ->
+                      wings_wm:top_size();
+		 true ->
+                      gl:blendEquationSeparate(?GL_FUNC_ADD, ?GL_MAX),
+                      {TexW,TexH}
 	      end,
     {W,Wd} = calc_texsize(W0, TexW),
     {H,Hd} = calc_texsize(H0, TexH),
 %%    io:format("Get texture sz ~p ~p ~n", [{W,Wd},{H,Hd}]),
     set_viewport({0,0,W,H}),
-    %% Load Pointers
-    gl:vertexPointer(2, ?GL_FLOAT, 0, UVpos),
-    case lists:member(normal, Reqs) of
-	true -> gl:normalPointer(?GL_FLOAT, 0, Ns);
-	false -> ignore
-    end,
-
-    gl:colorPointer(3, ?GL_FLOAT, 0, Vc),
-    case have_shaders() of
-	false -> ignore;
-	true  -> 
- 	    gl:clientActiveTexture(?GL_TEXTURE1),
- 	    gl:texCoordPointer(3,?GL_FLOAT,0,Pos),
- 	    gl:clientActiveTexture(?GL_TEXTURE0)
-    end,
-    case lists:member(binormal, Reqs) of
-	true -> 	    
-	    gl:clientActiveTexture(?GL_TEXTURE2),
-	    gl:texCoordPointer(3,?GL_FLOAT,0,BiNs),
-	    gl:clientActiveTexture(?GL_TEXTURE0);
-	false ->
-	    ignore
-    end,
-
-    try 
-        Dl = fun() ->
-		     foreach(fun(Pass) -> 
-				     if not UsingFbo -> 
-					     Pass(Geom,undefined);
-					true ->
-					     fill_bg_tex(UsingFbo),
-					     Pass(Geom,UsingFbo)
-				     end
-			     end,
-			     Passes) 
-	     end,
+    Geom = make_vbo(Geom0, Reqs),
+    try
+        Do = fun(Pass) ->
+                     if not UsingFbo ->
+                             Pass(Geom,undefined);
+                        true ->
+                             fill_bg_tex(UsingFbo),
+                             Pass(Geom,UsingFbo)
+                     end
+             end,
+        Dl = fun() -> foreach(Do, Passes) end,
 	ImageBins = get_texture(0, Wd, 0, Hd, {W,H}, Dl, UsingFbo,[]),
 	ImageBin = merge_texture(ImageBins, Wd, Hd, W*3, H, []),
-	if not UsingFbo -> 
+	if not UsingFbo ->
 		#e3d_image{image=ImageBin,width=TexW,height=TexH};
-	   true -> 
+	   true ->
 		#e3d_image{image=ImageBin,width=TexW,height=TexH,
 			   type=r8g8b8a8,bytes_pp=4}
 	end
     catch _:What ->
 	    Where = erlang:get_stacktrace(),
 	    exit({What,Where})
-    after 
-	case UsingFbo of 
-	  false -> ignore;
-	  #sh_conf{fbo_d=DeleteMe} -> DeleteMe()
-      end,
-      set_viewport(Current),
-      gl:readBuffer(?GL_BACK),
-      gl:popAttrib(),
-      gl:clear(?GL_COLOR_BUFFER_BIT bor ?GL_DEPTH_BUFFER_BIT),
-      ?ERROR    
+    after
+	case UsingFbo of
+            false -> ignore;
+            #sh_conf{fbo_d=DeleteMe} -> DeleteMe()
+        end,
+        wings_vbo:delete(Geom#ts.vbo),
+        set_viewport(Current),
+        gl:readBuffer(?GL_BACK),
+        gl:popAttrib(),
+        gl:blendEquationSeparate(?GL_FUNC_ADD, ?GL_FUNC_ADD),
+        gl:clear(?GL_COLOR_BUFFER_BIT bor ?GL_DEPTH_BUFFER_BIT),
+        ?ERROR
+    end.
+
+make_vbo(#ts{uv=UV}=Ts, Reqs) ->
+    {Layout, Bin} = make_bin_normal(Ts, Reqs, [{vertex2d, 0, 0}], UV),
+    Ts#ts{vbo=wings_vbo:new(dynamic, Bin, {predefined, Layout})}.
+
+make_bin_normal(#ts{n=Ns}=Ts, Reqs, Layout, Bin) ->
+    case lists:member(normal, Reqs) of
+	true  -> make_bin_color(Ts, Reqs, [{normal, 0, byte_size(Bin)}|Layout],
+                                <<Bin/binary, Ns/binary>>);
+	false -> make_bin_color(Ts, Reqs, Layout, Bin)
+    end.
+
+make_bin_color(#ts{vc=Vc}=Ts, Reqs, Layout, Bin) ->
+    make_bin_pos3d(Ts, Reqs, [{color, 0, byte_size(Bin)}|Layout], <<Bin/binary, Vc/binary>>).
+
+make_bin_pos3d(#ts{pos=Pos}=Ts, Reqs, Layout, Bin) ->
+    case have_shaders() of
+        true  -> make_bin_binormal(Ts, Reqs, [{{tex,1,3}, 0, byte_size(Bin)}|Layout],
+                                   <<Bin/binary, Pos/binary>>);
+	false -> make_bin_binormal(Ts, Reqs, Layout, Bin)
+    end.
+
+make_bin_binormal(#ts{bi=BiNs}, Reqs, Layout, Bin) ->
+    case lists:member(binormal, Reqs) of
+	true -> {lists:reverse([{{tex,2,4}, 0, byte_size(Bin)}|Layout]),
+                 <<Bin/binary, BiNs/binary>>};
+	false -> {lists:reverse(Layout), Bin}
     end.
 
 %%%%%%%%% FBO stuff
@@ -403,17 +408,16 @@ error_msg(Line) ->
     end.
 
 draw_texture_square() ->
-    gl:'begin'(?GL_QUADS),
-    gl:texCoord2f(0,0), gl:vertex2f(0,0),
-    gl:texCoord2f(1,0), gl:vertex2f(1,0),
-    gl:texCoord2f(1,1), gl:vertex2f(1,1),
-    gl:texCoord2f(0,1), gl:vertex2f(0,1),
-    gl:'end'().
-
+    VertexUvQ = << 0.0:?F32,0.0:?F32, 0.0:?F32,0.0:?F32,
+                   1.0:?F32,0.0:?F32, 1.0:?F32,0.0:?F32,
+                   1.0:?F32,1.0:?F32, 1.0:?F32,1.0:?F32,
+                   0.0:?F32,1.0:?F32, 0.0:?F32,1.0:?F32>>,
+    wings_vbo:draw(fun() -> gl:drawArrays(?GL_QUADS, 0, 4) end, VertexUvQ, [vertex2d, uv]).
 
 fill_bg_tex(#sh_conf{fbo_w=Prev}) ->
     gl:drawBuffer(?GL_COLOR_ATTACHMENT1_EXT),
     gl:bindTexture(?GL_TEXTURE_2D, Prev),
+    gl:texEnvi(?GL_TEXTURE_ENV, ?GL_TEXTURE_ENV_MODE, ?GL_REPLACE),
     gl:enable(?GL_TEXTURE_2D),
     gl:disable(?GL_BLEND),
     draw_texture_square(),
@@ -424,7 +428,7 @@ fill_bg_tex(#sh_conf{fbo_w=Prev}) ->
 get_texture(Wc, Wd, Hc, Hd, {W,H}=Info, DL, UsingFbo, ImageAcc)
   when Wc < Wd, Hc < Hd ->
     gl:pixelStorei(?GL_UNPACK_ALIGNMENT, 1),
-    gl:clearColor(1, 1, 1, 1),
+    gl:clearColor(1.0, 1.0, 1.0, 1.0),
     gl:shadeModel(?GL_SMOOTH),
     gl:disable(?GL_CULL_FACE),
     gl:disable(?GL_LIGHTING),
@@ -496,33 +500,20 @@ pref_to_list(#opt{texsz={TexSz,_TexSz}, no_renderers=NoR,
      r2list(Rs,1,NoR)].
 
 r2list([{Type, Opts}|Rest], Id, Max) when Id < Max ->
-    [{{auv_pass,Id}, Type},{{auv_opt,Id},Opts}|r2list(Rest,Id+1,Max)];
+    [{{auv_pass,Id}, Type},{{auv_opt,Id}, Opts}|r2list(Rest,Id+1,Max)];
 r2list([], Id, Max) when Id < Max ->
     [{{auv_pass,Id}, ignore},{{auv_opt,Id},[]}|r2list([],Id+1,Max)];
 r2list([], _Id, _Max)  -> [].
 
-list_to_prefs([{texsz, TexSz}|Rest]) ->    
-    case reverse(Rest) of
-	[{del_shader, true},{add_shader,_}|ROpts] -> 
-	    LR = listOfRenders(reverse(ROpts),delete,[]),
-	    Changed = true;
-	[{del_shader, false},{add_shader, true}|ROpts] -> 
-	    LR = listOfRenders(reverse(ROpts),keep,[]) ++ [{ignore, none}],
-	    Changed = true;
-	[{del_shader, false},{add_shader, false}|ROpts] ->
-	    LR = listOfRenders(reverse(ROpts),keep,[]),
-	    Changed = false
-    end,
-    {{changed, Changed},
-     #opt{texsz={TexSz,TexSz},no_renderers=length(LR),renderers=LR}}.
+list_to_prefs([{texsz, TexSz}|Rest]) ->
+    LR = listOfRenders(Rest,[]),
+    #opt{texsz={TexSz,TexSz},no_renderers=length(LR),renderers=LR}.
 
-listOfRenders([{{auv_pass,_},ignore},_|Rest],delete,Acc) ->
-    listOfRenders(Rest,delete,Acc);
-listOfRenders([{{auv_pass,_},ignore},_|Rest],keep,Acc) ->
-    listOfRenders(Rest,keep,[{ignore,[]}|Acc]);
-listOfRenders([{{auv_pass,_},Type},{{auv_opt,_},Opts}|Rest],Op,Acc) ->
-    listOfRenders(Rest,Op,[{Type,Opts}|Acc]);
-listOfRenders([],_,Acc) -> reverse(Acc).
+listOfRenders([{{auv_pass,_},ignore},_|Rest],Acc) ->
+    listOfRenders(Rest,[{ignore,[]}|Acc]);
+listOfRenders([{{auv_pass,_},Type},{{auv_opt,_},Opts}|Rest],Acc) ->
+    listOfRenders(Rest,[{Type,Opts}|Acc]);
+listOfRenders([],Acc) -> reverse(Acc).
     
 gen_tx_sizes(Sz, Acc) when Sz < 128 -> Acc;
 gen_tx_sizes(Sz, Acc) ->
@@ -551,19 +542,21 @@ setup(#st{bb=#uvstate{id=RId,st=#st{shapes=Sh0}}}=St, Reqs) ->
     We = gb_trees:get(RId,Sh0),
     {Charts,{_Cnt,UVpos,Vpos,Ns,Ts,BB,Vc}} = setup_charts(St, We, Reqs),
     #ts{charts=Charts,
-	uv =to_bin(UVpos,uv),
-	pos=to_bin(Vpos,pos),
-	n  =to_bin(Ns,pos),
-	bi =to_bin(Ts,pos),
+	uv = to_bin(UVpos,uv),
+	pos= to_bin(Vpos,pos),
+	n  = to_bin(Ns,pos),
+	bi = build_tangents(Ts),
 	bb = BB,
-	vc=to_bin(Vc, vertex)}.
+	vc = to_bin(Vc, vertex)}.
 
 setup_charts(#st{shapes=Cs0,selmode=Mode,sel=Sel}, We, Reqs) ->
     Ns = case member(normal, Reqs) orelse member(binormal, Reqs) of
 	     true -> setup_normals(We);
 	     false -> []
 	 end,
-    Start = {0,[],[],[],[],[],[]}, %% {UvPos,3dPos,Normal,Tangent,Vc}
+    Z = e3d_vec:zero(),
+    TSA = array:new([{default, {Z,Z}}]),
+    Start = {0,[],[],[],{TSA,[]},[],[]}, %% {UvPos,3dPos,Normal,Tangent,Vc}
     Shapes = if Sel =:= [] -> 
 		     gb_trees:values(Cs0);
 		Mode =:= body -> 
@@ -579,14 +572,13 @@ setup_charts(#st{shapes=Cs0,selmode=Mode,sel=Sel}, We, Reqs) ->
 setup_chart(#we{id=Id}=Ch, Ns, Reqs, WWe, State0) ->
     OEs0 = outer_verts(Ch),
     BB = [],
-    {Fs,{OEs,UvBB,State}}  = 
-	create_faces(Ch, WWe, Ns, Reqs, {OEs0,BB,State0}),
+    {Fs,{OEs,UvBB,State}} = create_faces(Ch, WWe, Ns, Reqs, {OEs0,BB,State0}),
     {#chart{id=Id,fs=Fs,oes=OEs,bb_uv=UvBB},State}.
 
 create_faces(#we{vp=Vtab,name=#ch{vmap=Vmap}}=We,
 	     #we{vp=Vt3d}=RealWe, NTab, Reqs, State) ->
     Fs = wings_we:visible(We),
-    C=fun(Face,{OEs,UvBB,{Cnt,UVpos,Vpos,Ns,Ts,PosBB,Vc}}) ->
+    C=fun(Face,{OEs,UvBB,{Cnt,UVpos,Vpos,Ns,Ts0,PosBB,Vc}}) ->
 	      Vs0 = wings_face:vertices_ccw(Face,We),
 	      UVcoords = [array:get(V, Vtab) || V <- Vs0],
 	      Coords   = [array:get(map_vertex(V,Vmap),Vt3d) 
@@ -608,7 +600,7 @@ create_faces(#we{vp=Vtab,name=#ch{vmap=Vmap}}=We,
 		       Len -> triangulate(FaceVs,UVcoords)
 		   end,
 %	      io:format("Face ~p Normals ~p~n", [Face,Normals]),
-	      Tangents = fix_tang_vecs(Vs,Coords,UVcoords,Normals,Reqs), 
+	      Ts = calc_tang_vecs(Vs,Vs0,Coords,UVcoords,Normals,Ts0,Reqs),
 	      Indx = fun(I) -> [V+Cnt || V <- I] end,
 	      {#fs{vs=Indx(Vs),vse=Indx(FaceVs),id=Face},
 	       {map_oes(OEs, Vs0, Cnt+Len-1, Face),
@@ -617,7 +609,7 @@ create_faces(#we{vp=Vtab,name=#ch{vmap=Vmap}}=We,
 		 UVcoords ++ UVpos,
 		 Coords   ++ Vpos,
 		 Normals  ++ Ns,
-		 Tangents ++ Ts,
+		 Ts,
 		 e3d_vec:bounding_box(PosBB ++ Coords),
 		 OldVc   ++ Vc}}}
       end,
@@ -645,28 +637,117 @@ fix_normals([V|R],Ns,Vmap) ->
     [find(map_vertex(V,Vmap),Ns)|fix_normals(R,Ns,Vmap)];
 fix_normals([],_,_) -> [].
 
-fix_tang_vecs(_,_,_,[],_) -> [];
-fix_tang_vecs(_Vs,Coords,_UVCoords,Normals,Reqs) ->
+%%%% Tangent calc
+
+calc_tang_vecs(_,_,_,_,[],Ts,_) -> Ts;
+calc_tang_vecs(Vs,VsIds,Coords,UVCoords,Normals,Ts0,Reqs) ->
+    %% io:format("Vs ~p ~n",[Vs]),
+    %% io:format("Coords ~p ~n",[Coords]),
+    %% io:format("UV ~p ~n",[UVCoords]),
+    %% io:format("Normals ~p ~n",[Normals]),
     case lists:member(binormal,Reqs) of
 	true ->
-	    %% Not tested enough, but I think it's better
-	    binormals(Normals,e3d_vec:normal(Coords));
+	    N = e3d_vec:norm(e3d_vec:average(Normals)),
+	    calc_tang_vecs_1([V+1||V <- Vs], VsIds, Coords, UVCoords, N, Ts0);
 	false ->
-	    []
+	    Ts0
     end.
 
-binormals(Normals,FaceNormal) ->
-    Q = e3d_q:rotate_s_to_t({0.0,0.0,1.0},FaceNormal),
-    R = fun(N) -> 
-		D = e3d_q:rotate_s_to_t(FaceNormal,N),
-%		io:format("Z-N ~p~nN-n ~p~n",[Q,D]),
-		R = e3d_q:mul(D,Q),
-		Bi = e3d_q:vec_rotate({0.0,1.0,0.0},R),
-%		io:format("~p ~p~n",[R,Bi]),
-		e3d_vec:norm(e3d_vec:cross(Bi,N))
+calc_tang_vecs_1([Vi1,Vi2,Vi3|Vis], VsIds, Coords, UVCoords, N, {Ts, F2V}) ->
+    {X1,Y1,Z1} = e3d_vec:sub(lists:nth(Vi2,Coords), lists:nth(Vi1,Coords)),
+    {X2,Y2,Z2} = e3d_vec:sub(lists:nth(Vi3,Coords), lists:nth(Vi1,Coords)),
+    {U1,V1,_} = lists:nth(Vi1, UVCoords),
+    {U2,V2,_} = lists:nth(Vi2, UVCoords),
+    {U3,V3,_} = lists:nth(Vi3, UVCoords),
+    S1 = U2-U1,
+    S2 = U3-U1,
+    T1 = V2-V1,
+    T2 = V3-V1,
+    Vs = [lists:nth(Vi1, VsIds),lists:nth(Vi2,VsIds),lists:nth(Vi3,VsIds)],
+    try
+	F = 1.0 / (S1*T2 - S2*T1),
+	Tangent = {F*(T2*X1-T1*X2), F*(T2*Y1-T1*Y2), F*(T2*Z1-T1*Z2)},
+	BiTangent = {F*(S1*X2-S2*X1), F*(S1*Y2-S2*Y1), F*(S1*Z2-S2*Z1)},
+	H = case e3d_vec:dot(e3d_vec:cross(N, Tangent), BiTangent) < 0.0 of
+		true  -> 1;
+		false -> -1
+	    end,
+	Tss0 = {add_tangent(Vs, Tangent, BiTangent, Ts), [{N,H,Vs}|F2V]},
+	calc_tang_vecs_1(Vis, VsIds, Coords, UVCoords, N, Tss0)
+    catch _:badarith ->
+	    Tss1 = {Ts, [{N,0,Vs}|F2V]},
+	    calc_tang_vecs_1(Vis, VsIds, Coords, UVCoords, N, Tss1)
+    end;
+calc_tang_vecs_1([], _, _, _, _, Tss) ->
+    Tss.
+
+add_tangent([V|Vs], Tangent, BiTangent, Ts) ->
+    {T0, B0} = array:get(V,Ts),
+    T1 = e3d_vec:add(T0, Tangent),
+    T2 = e3d_vec:add(B0, BiTangent),
+    add_tangent(Vs, Tangent, BiTangent, array:set(V, {T1, T2}, Ts));
+add_tangent([], _, _, Ts) -> Ts.
+
+build_tangents({VsTs0, RevF2V}) ->
+    VsTs = array:map(fun(_V, {T, BT}) -> {e3d_vec:norm(T), e3d_vec:norm(BT)} end, VsTs0),
+    build_tangents(lists:reverse(RevF2V), VsTs, <<>>).
+
+build_tangents([{N, H, Face}|Fs], Ts, Bin0) ->
+    Bin = add_tangents1(Face, Ts, H, N, undefined, Bin0),
+    build_tangents(Fs, Ts, Bin);
+build_tangents([], _, Bin) -> Bin.
+
+add_tangents1([V|Vs], Ts, H0, N, Prev, Bin0) ->
+    case array:get(V, Ts) of
+	{{0.0, 0.0, 0.0}, BiT} ->
+	    {Tan = {X,Y,Z}, H} = get_tangent(Prev, BiT, H0, N),
+	    Bin = <<Bin0/binary, X:?F32,Y:?F32,Z:?F32, H:?F32>>,
+	    add_tangents1(Vs, Ts, H, N, Tan, Bin);
+	{Tan = {X,Y,Z}, _} when H0 /= 0 ->
+	    Bin = <<Bin0/binary, X:?F32,Y:?F32,Z:?F32, H0:?F32>>,
+	    add_tangents1(Vs, Ts, H0, N, Tan, Bin);
+	{Tan = {X,Y,Z}, BiT} ->
+	    H = case e3d_vec:dot(e3d_vec:cross(N, Tan), BiT) < 0.0 of
+		    true  -> 1;
+		    false -> -1
+		end,
+	    Bin = <<Bin0/binary, X:?F32,Y:?F32,Z:?F32, H:?F32>>,
+	    add_tangents1(Vs, Ts, H, N, Tan, Bin)
+    end;
+add_tangents1([], _, _, _, _, Bin) -> Bin.
+
+get_tangent(undefined, {0.0,0.0,0.0}, H0, N) ->
+    H = if H0 =:= 0 -> -1; true -> H0 end,
+    {cross_axis(N), H};
+get_tangent(undefined, BiT, 0, N) ->
+    T = e3d_vec:cross(BiT, N),
+    H = case e3d_vec:dot(e3d_vec:cross(N, T), BiT) < 0.0 of
+	    true  -> 1;
+	    false -> -1
 	end,
-    [R(N) || N <- Normals].
-   
+    {T, H};
+get_tangent(undefined, BiT, H, N) ->
+    {e3d_vec:cross(BiT, N), H};
+get_tangent(Prev, _, H, _) ->
+    {Prev, H}.
+
+cross_axis(N = {NX,NY,NZ}) ->
+    try
+	V2 = case abs(NX) > abs(NY) of
+		 true ->
+		     ILen = 1.0 / math:sqrt(NX*NX+NZ*NZ),
+		     {-NZ*ILen, 0.0, NX * ILen};
+		 false ->
+		     ILen = 1.0 / math:sqrt(NY*NY+NZ*NZ),
+		     {0.0, NZ*ILen, -NY * ILen}
+	     end,
+	e3d_vec:cross(N, V2)
+    catch _:badarith ->
+	    {1.0, 0.0,0.0}
+    end.
+
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
 fix_vc(Vs, Face, We, Vmap) ->
     try 
 	Uvc = wings_va:face_attr([vertex|color], Face, We),
@@ -736,7 +817,7 @@ vs_lines([B],Last) ->
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 get_passes(Passes,Shaders) ->
-    lists:foldr(fun(Pass,Acc) -> 
+    lists:foldr(fun(Pass,Acc) ->
 			case pass(Pass,Shaders) of
 			    Fun when is_function(Fun) ->
 				[Fun|Acc];
@@ -746,13 +827,13 @@ get_passes(Passes,Shaders) ->
 		end, [], Passes).
 
 pass({ignore,_},_) ->  ignore;
-pass({auv_background,[{type_sel,color},_,Color]},_) ->  
+pass({auv_background,[auv_background, {type_sel,color},_,Color]},_) ->
     fun(_Geom,_) ->
 	    {R,G,B,A} = fix(Color,true),
 	    gl:clearColor(R,G,B,A),
 	    gl:clear(?GL_COLOR_BUFFER_BIT bor ?GL_DEPTH_BUFFER_BIT)
     end;
-pass({auv_background,[{type_sel,image},{_Name,Id},Color]},_) ->
+pass({auv_background,[auv_background, {type_sel,image},{_Name,Id},Color]},_) ->
     fun(_Geom,_) ->
 	    {R,G,B,A} = fix(Color,true),
 	    gl:clearColor(R,G,B,A),
@@ -768,31 +849,36 @@ pass({auv_background,[{type_sel,image},{_Name,Id},Color]},_) ->
     end;
 pass({auv_background, _},Sh) ->
     pass({auv_background, ?OPT_BG},Sh);
-pass({auv_edges, [all_edges,Color,Width,_UseMat]},_) ->
+
+pass({auv_edges, [auv_edges,all_edges,Color,Width,_UseMat]},_) ->
     R=fun(#chart{fs=Fs}) ->
 	      Draw = fun(#fs{vse=Vs}) ->
 			     Patched = vs_lines(Vs,hd(Vs)),
 			     wings_gl:drawElements(?GL_LINES,length(Patched),
-					     ?GL_UNSIGNED_INT,Patched)
-			     end,
+                                                   ?GL_UNSIGNED_INT,Patched)
+                     end,
 	      foreach(Draw,Fs)
       end,
-    fun(#ts{charts=Charts},_) ->  
-	    gl:disable(?GL_DEPTH_TEST),	    
+    fun(#ts{vbo={call,_,{vbo,Vbo}}, charts=Charts},_) ->
+	    gl:disable(?GL_DEPTH_TEST),
 	    gl:color3fv(Color),
 	    gl:lineWidth(Width),
+            gl:bindBuffer(?GL_ARRAY_BUFFER, Vbo),
+            gl:vertexPointer(2, ?GL_FLOAT, 0, 0),
 	    gl:enableClientState(?GL_VERTEX_ARRAY),
 	    foreach(R, Charts),
 	    gl:disableClientState(?GL_VERTEX_ARRAY)
     end;
-pass({auv_edges, [border_edges,Color,Width,UseMat]},_) -> 
+pass({auv_edges, [auv_edges,border_edges,Color,Width,UseMat]},_) ->
     R= fun(#chart{oes=Es0}) ->
 	       Es = foldl(fun([A,B,_],Acc) -> [A,B|Acc] end, [], Es0),
 	       wings_gl:drawElements(?GL_LINES,length(Es),?GL_UNSIGNED_INT,Es)
        end,
-    fun(#ts{charts=Charts},_) ->  
+    fun(#ts{vbo={call,_,{vbo,Vbo}}, charts=Charts},_) ->
 	    gl:color3fv(Color),
 	    gl:lineWidth(Width),
+            gl:bindBuffer(?GL_ARRAY_BUFFER, Vbo),
+            gl:vertexPointer(2, ?GL_FLOAT, 0, 0),
 	    gl:enableClientState(?GL_VERTEX_ARRAY),
 	    gl:disable(?GL_DEPTH_TEST),
 	    case UseMat of
@@ -809,52 +895,45 @@ pass({auv_edges, _},Sh) ->
     pass({auv_edges, ?OPT_EDGES},Sh);
 
 pass({auv_faces,[_]},_) ->
-    fun(#ts{charts=Charts}, _) ->
+    fun(#ts{vbo={call,EnableVbo,{vbo,_Vbo}}, charts=Charts}, _) ->
 	    gl:disable(?GL_DEPTH_TEST),
 	    gl:disable(?GL_ALPHA_TEST),
-	    gl:enable(?GL_BLEND),	   
+	    gl:enable(?GL_BLEND),
 	    gl:blendFunc(?GL_SRC_ALPHA, ?GL_ONE_MINUS_SRC_ALPHA),
-	    gl:enableClientState(?GL_VERTEX_ARRAY),
-	    gl:clientActiveTexture(?GL_TEXTURE0),
-	    gl:enableClientState(?GL_COLOR_ARRAY),
 	    R = fun(#fs{vs=Vs}) ->
-			wings_gl:drawElements(?GL_TRIANGLES,length(Vs),
-					?GL_UNSIGNED_INT,Vs)
+			wings_gl:drawElements(?GL_TRIANGLES,length(Vs),?GL_UNSIGNED_INT,Vs)
 		end,
-	    gl:disable(?GL_TEXTURE_2D),
-	    foreach(fun(#chart{fs=Fs}) -> foreach(R, Fs) end, Charts),
-	    gl:disable(?GL_TEXTURE_2D),
-	    gl:disableClientState(?GL_VERTEX_ARRAY),
-	    gl:disableClientState(?GL_COLOR_ARRAY),
-	    gl:disableClientState(?GL_TEXTURE_COORD_ARRAY)
+	    All = fun() -> foreach(fun(#chart{fs=Fs}) -> foreach(R, Fs) end, Charts) end,
+            EnableVbo(All)
     end;
 pass({auv_faces, _},Sh) ->
     pass({auv_faces,?OPT_FACES},Sh);
+
 pass({{shader,Id}, Opts},{Sh,Compiled}) ->
     shader_pass(lists:keysearch(Id,#sh.id,Sh),
 		lists:keysearch(Id,1,Compiled),Opts);
-pass({_R, _O},_) ->    
+pass({_R, _O},_) ->
     io:format("AUV: ~p:~p: Unknown Render Pass (~p) or options (~p) ~n",
 	      [?MODULE,?LINE,_R,_O]),
     ignore.
 
-shader_pass(Shader={value,#sh{def=Def}},Prog,[]) when Def /= [] ->    
-    shader_pass(Shader, Prog, reverse(Def));
-shader_pass(_,false,_) ->    
+shader_pass(Shader={value,#sh{id=Id, def=Def}},Prog,[]) when Def /= [] ->
+    shader_pass(Shader, Prog, [{shader,Id}|reverse(Def)]);
+shader_pass(_,false,_) ->
     io:format("AUV: No shader program found skipped ~p~n", [?LINE]),
     ignore;
-shader_pass(false,_,_) ->    
+shader_pass(false,_,_) ->
     io:format("AUV: Not shader found skipped ~p~n", [?LINE]),
     ignore;
-shader_pass({value,#sh{name=_Name, args=Args,tex_units=TexUnits,reqs=Reqs}},
-	    {value,{_,Prog}},Opts) ->
-    fun(Ts = #ts{charts=Charts},Config) ->
+shader_pass({value,#sh{id=Id, args=Args, tex_units=TexUnits}},
+	    {value,{_,Prog}}, [{shader,Id}|Opts]) ->
+    fun(Ts = #ts{vbo={call,EnableVbo,_}, charts=Charts}, Config) ->
 	    gl:disable(?GL_DEPTH_TEST),
 	    gl:disable(?GL_ALPHA_TEST),
 	    gl:enable(?GL_BLEND),
 	    gl:blendFunc(?GL_SRC_ALPHA, ?GL_ONE_MINUS_SRC_ALPHA),
 	    gl:useProgram(Prog),
-	    try 
+	    try
 		Conf = Config#sh_conf{prog=Prog,ts=Ts},
 		PerChartUniF = shader_uniforms(reverse(Args),Opts,Conf),
 		case send_texture(reverse(Args),Opts) of
@@ -862,49 +941,35 @@ shader_pass({value,#sh{name=_Name, args=Args,tex_units=TexUnits,reqs=Reqs}},
 			gl:enable(?GL_TEXTURE_2D),
 			gl:disable(?GL_BLEND),
 			draw_texture_square();
-		    _ -> 
-			gl:enableClientState(?GL_VERTEX_ARRAY),
-			gl:clientActiveTexture(?GL_TEXTURE1),
-			gl:enableClientState(?GL_TEXTURE_COORD_ARRAY),
-			case lists:member(binormal,Reqs) of
-			    true -> 
-				gl:clientActiveTexture(?GL_TEXTURE2),
-				gl:enableClientState(?GL_TEXTURE_COORD_ARRAY);
-			    false -> ignore
-			end,
-			case lists:member(normal,Reqs) of
-			    true ->
-				gl:enableClientState(?GL_NORMAL_ARRAY);
-			    false -> ignore
-			end,
-			foreach(fun(Chart = #chart{fs=Fas}) -> 
-					PerFaceUniF = sh_uniforms(PerChartUniF,Chart,Conf),
-					R = fun(Face = #fs{vs=Vss}) ->			    
-						    sh_uniforms(PerFaceUniF,Face,Conf),
-						    wings_gl:drawElements(?GL_TRIANGLES,length(Vss),
-								    ?GL_UNSIGNED_INT,Vss)
-					    end,
-					foreach(R,Fas) 
-				end,
-				Charts)
-		end
-	    catch throw:What ->
-		    io:format("AUV: ERROR ~s ~n",[What]);
+		    _ ->
+                        DC = fun(Chart = #chart{fs=Fas}) ->
+                                     PerFaceUniF = sh_uniforms(PerChartUniF,Chart,Conf),
+                                     R = fun(Face = #fs{vs=Vss}) ->
+                                                 sh_uniforms(PerFaceUniF,Face,Conf),
+                                                 wings_gl:drawElements(?GL_TRIANGLES,length(Vss),
+                                                                       ?GL_UNSIGNED_INT,Vss)
+                                         end,
+                                     foreach(R,Fas)
+                             end,
+                        EnableVbo(fun() -> foreach(DC, Charts) end)
+                end
+            catch throw:What ->
+                    io:format("AUV: ERROR ~s ~n",[What]);
 		_:What ->
 		    Stack = erlang:get_stacktrace(),
 		    io:format("AUV: Internal ERROR ~p:~n~p ~n",[What,Stack])
 	    after
-	      gl:useProgram(0),
-	      gl:disable(?GL_TEXTURE_2D),
-	      gl:disableClientState(?GL_VERTEX_ARRAY),
-	      gl:disableClientState(?GL_NORMAL_ARRAY),
-	      gl:clientActiveTexture(?GL_TEXTURE2),
-	      gl:disableClientState(?GL_TEXTURE_COORD_ARRAY),
-	      gl:clientActiveTexture(?GL_TEXTURE1),
-	      gl:disableClientState(?GL_TEXTURE_COORD_ARRAY),
-	      gl:clientActiveTexture(?GL_TEXTURE0),
-	      disable_tex_units(TexUnits),
-	      gl:activeTexture(?GL_TEXTURE0)
+                gl:useProgram(0),
+                gl:disable(?GL_TEXTURE_2D),
+                gl:disableClientState(?GL_VERTEX_ARRAY),
+                gl:disableClientState(?GL_NORMAL_ARRAY),
+                gl:clientActiveTexture(?GL_TEXTURE2),
+                gl:disableClientState(?GL_TEXTURE_COORD_ARRAY),
+                gl:clientActiveTexture(?GL_TEXTURE1),
+                gl:disableClientState(?GL_TEXTURE_COORD_ARRAY),
+                gl:clientActiveTexture(?GL_TEXTURE0),
+                disable_tex_units(TexUnits),
+                gl:activeTexture(?GL_TEXTURE0)
 	    end
     end.
 
@@ -930,7 +995,16 @@ shader_uniforms([{uniform,float,Name,_,_}|As],[Val|Opts],Conf) ->
 shader_uniforms([{uniform,menu,Name,_,_}|As],[Vals|Opts],Conf) 
   when is_list(Vals) ->
     Loc = wings_gl:uloc(Conf#sh_conf.prog,Name),
-    foldl(fun(Val,Cnt) -> gl:uniform1f(Loc+Cnt,Val),Cnt+1 end,0,Vals),
+    foldl(fun(Val,Cnt) when is_integer(Val) ->
+                gl:uniform1i(Loc+Cnt,Val),Cnt+1;
+             (Val,Cnt) ->
+                gl:uniform1f(Loc+Cnt,Val),Cnt+1
+          end,0,Vals),
+    shader_uniforms(As,Opts,Conf);
+shader_uniforms([{uniform,menu,Name,_,_}|As],[Vals|Opts],Conf)
+    when is_integer(Vals) ->
+    Loc = wings_gl:uloc(Conf#sh_conf.prog,Name),
+    gl:uniform1i(Loc,Vals),
     shader_uniforms(As,Opts,Conf);
 shader_uniforms([{uniform,bool,Name,_,_}|As],[Val|Opts],Conf) ->
     Loc = wings_gl:uloc(Conf#sh_conf.prog,Name),
@@ -939,7 +1013,11 @@ shader_uniforms([{uniform,bool,Name,_,_}|As],[Val|Opts],Conf) ->
     shader_uniforms(As,Opts,Conf);
 shader_uniforms([{uniform,{slider,_,_},Name,_,_}|As],[Val|Opts],Conf) ->
     Loc = wings_gl:uloc(Conf#sh_conf.prog,Name),
-    gl:uniform1f(Loc,Val),
+	if is_integer(Val) ->
+			gl:uniform1i(Loc,Val);
+		true ->
+			gl:uniform1f(Loc,Val)
+	end,
     shader_uniforms(As,Opts,Conf);
 shader_uniforms([{uniform,{image,Unit},Name,_,_}|As],[{_,Id}|Opts],Conf) ->
     Loc = wings_gl:uloc(Conf#sh_conf.prog,Name),
@@ -948,16 +1026,6 @@ shader_uniforms([{uniform,{image,Unit},Name,_,_}|As],[{_,Id}|Opts],Conf) ->
     gl:bindTexture(?GL_TEXTURE_2D, TxId),
     gl:uniform1i(Loc, Unit),
     shader_uniforms(As,Opts,Conf);
-shader_uniforms([{auv,{auv_noise,Unit}}|Rest],Opts,Conf) ->
-    case wings_image:pnoiseid() of
-	0 -> throw("No noise texture available");
-	TxId -> 
-	    Loc = wings_gl:uloc(Conf#sh_conf.prog, "auv_noise"),
-	    gl:activeTexture(?GL_TEXTURE0 + Unit),
-	    gl:bindTexture(?GL_TEXTURE_3D, TxId),
-	    gl:uniform1i(Loc, Unit),
-            shader_uniforms(Rest,Opts,Conf)
-    end;
 shader_uniforms([{auv,{auv_bg,Unit}}|Rest],Opts,Conf) ->
     Loc = wings_gl:uloc(Conf#sh_conf.prog, "auv_bg"),
     gl:activeTexture(?GL_TEXTURE0),
@@ -1050,9 +1118,6 @@ parse_sh_info([{fragment_shader,Name}|Opts],Sh,NI,Acc) ->
     parse_sh_info(Opts, Sh#sh{fs=Name},NI, Acc);
 parse_sh_info([{requires,List}|Opts],Sh,NI,Acc) when is_list(List) ->
     parse_sh_info(Opts, Sh#sh{reqs=List},NI, Acc);
-parse_sh_info([{auv,auv_noise}|Opts],Sh,NI,Acc) ->
-    What = {auv,{auv_noise,1}},
-    parse_sh_info(Opts, Sh#sh{args=[What|Sh#sh.args]},NI+1,Acc);
 parse_sh_info([{auv,auv_bg}|Opts],Sh,NI,Acc) ->
     What = {auv,{auv_bg,0}},
     parse_sh_info(Opts, Sh#sh{args=[What|Sh#sh.args]},NI,Acc);
@@ -1118,14 +1183,14 @@ compile_shaders(Passes, Available) ->
 	  end, [], Passes).
 		   
 compile_shader(Id, {value,#sh{name=Name,vs=VsF,fs=FsF}}, Acc) ->
-    try 
+    try
 	Vs = wings_gl:compile(vertex, read_file(VsF)),
         Fs = wings_gl:compile(fragment, read_file(FsF)),
         Prog = wings_gl:link_prog([Vs,Fs]),
-	io:format("AUV: ~s ok~n", [Name]),
+	io:format("AUV: Shader ´~s´ ok~n", [Name]),
 	[{Id,Prog}|Acc]
     catch throw:What ->
-	    io:format("AUV: Error ~s ~n",[What]),
+	    io:format("AUV: Error ~p ~s ~n",[Name, What]),
 	    Acc;
 	_:Err ->
 	    Stack = erlang:get_stacktrace(),
