@@ -16,8 +16,8 @@
 -define(NEED_OPENGL, 1).
 -define(NEED_ESDL, 1).
 
--include("wings.hrl").
--include("e3d_image.hrl").
+-include_lib("wings/src/wings.hrl").
+-include_lib("wings/e3d/e3d_image.hrl").
 -include("auv.hrl").
  
 -export([init/0,menu/2,command/2,redraw/1]).
@@ -126,13 +126,13 @@ create_window(Action, Name, Id, #st{shapes=Shs}=St) ->
     Op = {replace,fun(Ev) -> auv_event(Ev, St) end},
     Segment = if element(1,Action) == edit -> ""; true -> ?__(1,"Segmenting") end,
     Title = "AutoUV "++ Segment ++": " ++ ObjName,
-    {X0,Y,W,H} = init_drawarea(),
-    Props = [{display_lists,Name}|wings_view:initial_properties()],
-    CreateToolbar = fun(N, P, Wi) -> wings_toolbar:create(N, P, Wi) end,
-    X = if element(1,Action) == edit -> X0; true -> 10 end,
-    wings_wm:toplevel(Name, Title, {X,Y,highest}, {W,H},
-		      [resizable,closable,menubar,{properties,Props},
-		       {toolbar,CreateToolbar}], Op),
+
+    {Pos,Size} = init_drawarea(),
+    {Frame,Ps} = wings_frame:make_win(Title, [{size, Size}, {pos, Pos}]),
+    Context = wxGLCanvas:getContext(?GET(gl_canvas)),
+    Canvas = wings_gl:window(Frame, Context, true, true),
+    Props = [{display_data,Name}|wings_view:initial_properties()++Ps],
+    wings_wm:toplevel(Name, Canvas, Props, Op),
     wings_wm:send(Name, {init,{Action,We}}).
 
 auv_event({init,Op}, St) ->
@@ -186,7 +186,7 @@ init_show_maps(Charts0, Fs, #we{name=WeName,id=Id}, GeomSt0) ->
     GeomSt = insert_initial_uvcoords(Charts, Id, MatName, GeomSt1),
     EditWin = {autouv,Id},
     case wings_wm:is_window(EditWin) of
-	true -> 
+	true ->
 	    wings_wm:send(EditWin, {add_faces,Fs,GeomSt}),
 	    wings_wm:send(geom, {new_state,GeomSt});
 	false ->
@@ -247,68 +247,8 @@ create_uv_state(Charts, MatName, Fs, We, #st{shapes=Shs0}=GeomSt) ->
     end,
     wings:register_postdraw_hook(Win, ?MODULE,
 				 fun draw_background/1),
-    wings_wm:menubar(Win, menubar()),
-    wings_wm:send({menubar,Win}, {current_state,St}),
     St.
 
-menubar() ->
-    [{?__(1,"Edit"),edit,
-      fun(_) ->
-	      [{?__(2,"Undo/Redo"),undo_toggle,?__(3,"Undo or redo the last command")},
-	       {?__(4,"Redo"),redo,?__(5,"Redo the last command that was undone")},
-	       {?__(6,"Undo"),undo,?__(7,"Undo the last command")},
-	       {?__(71,"Repeat"),repeat,?__(72,"Repeat the last command")},
-	       {?__(74,"Repeat Args"),repeat_args,
-		?__(75,"Repeat the last command with same arguments")},
-	       {?__(76,"Repeat Drag"),repeat_drag,
-		?__(77,"Repeat the last command with same arguments the same distance")}
-	      ]
-      end},
-     {?__(8,"View"),view,
-      fun(St) ->
-	      Menu0 = wings_view:menu(St),
-	      ShwBgImg = {?__(9,"Show/Hide Background Image"),toggle_background,
-	                  ?__(10,"Toggle display of the background texture image")},
-	      Menu = [I || I <- Menu0,
-			   keep_view_item(I)],
-	      [ShwBgImg|redundant_separators(Menu)]
-      end},
-     {?__(11,"Select"),select,
-      fun(St) ->
-	      Menu0 = wings_sel_cmd:menu(St),
-	      Menu = [I || I <- Menu0,
-			   keep_sel_item(I)],
-	      redundant_separators(Menu)
-      end}].
-
-
-keep_view_item(separator) -> true;
-keep_view_item({_,aim,_}) -> true;
-keep_view_item({_,highlight_aim,_}) -> true;
-keep_view_item({_,frame,_}) -> true;
-keep_view_item(_) -> false.
-
-keep_sel_item(separator) -> true;
-keep_sel_item({_,more,_}) -> true;
-keep_sel_item({_,less,_}) -> true;
-keep_sel_item({_,similar,_}) -> true;
-keep_sel_item({_,inverse,_}) -> true;
-keep_sel_item({_,all,_}) -> true;
-keep_sel_item({_,deselect,_}) -> true;
-keep_sel_item({_,{edge_loop,_}}) -> true;
-keep_sel_item({_,{adjacent,_}}) -> true;
-keep_sel_item({_,hide_selected,_}) -> true;
-keep_sel_item({_,hide_unselected,_}) -> true;
-keep_sel_item({_,show_all,_}) -> true;
-keep_sel_item(_) -> false.
-
-redundant_separators([]) -> [];
-redundant_separators([separator]) -> [];
-redundant_separators([separator|[separator|_]=T]) ->
-    redundant_separators(T);
-redundant_separators([H|T]) ->
-    [H|redundant_separators(T)].
-
 insert_initial_uvcoords(Charts, Id, MatName, #st{shapes=Shs0}=St) ->
     We0 = gb_trees:get(Id, Shs0),
     We1 = update_uvs(gb_trees:values(Charts), We0),
@@ -706,10 +646,10 @@ handle_event_3({action,{auv,quit}}, _St) ->
     cleanup_before_exit(),
     delete;
 handle_event_3({action,{{auv,_},Cmd}}, St) ->
-%%    io:format("Cmd ~p ~n", [Cmd]),
+    %%    io:format("Cmd ~p ~n", [Cmd]),
     handle_command(Cmd, St);
 handle_event_3({action,{auv,Cmd}}, St) ->
-%%    io:format("Cmd ~p ~n", [Cmd]),
+    %%    io:format("Cmd ~p ~n", [Cmd]),
     handle_command(Cmd, St);
 handle_event_3({action,{select,show_all}}, #st{bb=#uvstate{st=GeomSt,id=Id}}) ->
     wings_wm:send({autouv,Id}, {add_faces,object,GeomSt}),
@@ -720,12 +660,6 @@ handle_event_3({action,{select,Command}}, St0) ->
 	#st{}=St -> ok
     end,
     new_state(St);
-handle_event_3({action,{edit,undo_toggle}}=Act, _) ->
-    wings_wm:send(geom, Act);
-handle_event_3({action,{edit,undo}}=Act, _) ->
-    wings_wm:send(geom, Act);
-handle_event_3({action,{edit,redo}}=Act, _) ->
-    wings_wm:send(geom, Act);
 handle_event_3({action,{edit,repeat}}, St) ->
     repeat(command, St);
 handle_event_3({action,{edit,repeat_args}}, St) ->
@@ -737,7 +671,7 @@ handle_event_3({action,{view,toggle_background}}, _) ->
     put({?MODULE,show_background},not Old),
     wings_wm:dirty();
 
-handle_event_3({action,Ev}, St) ->
+handle_event_3({action,Ev}=Act, #st{selmode=AUVSel, bb=#uvstate{st=#st{selmode=GSel}}}=St) ->
     case Ev of  %% Keyboard shortcuts end up here (I believe)
 	{_, {move,_}} ->
 	    handle_command(move,St);
@@ -756,17 +690,17 @@ handle_event_3({action,Ev}, St) ->
 	    wings_view:command(aim, St1),
 	    get_event(St);
 	{view,highlight_aim} ->
-        #st{sel=Sel} = St,
-        case  Sel =:= [] of
-          true ->
+            #st{sel=Sel} = St,
+            case  Sel =:= [] of
+                true ->
 		    St1 = fake_selection(St),
-            wings_view:command(aim, St1),
-            get_event(St);
-          false ->
-            {{_,Cmd},St1} = wings:highlight_aim_setup(St),
-            wings_view:command(Cmd,St1),
-            get_event(St)
-        end;
+                    wings_view:command(aim, St1),
+                    get_event(St);
+                false ->
+                    {{_,Cmd},St1} = wings:highlight_aim_setup(St),
+                    wings_view:command(Cmd,St1),
+                    get_event(St)
+            end;
 	{view,Cmd} when Cmd == frame ->
 	    wings_view:command(Cmd,St),
 	    get_event(St);
@@ -776,9 +710,16 @@ handle_event_3({action,Ev}, St) ->
 	    repeat(args, St);
 	{edit,repeat_drag} ->
 	    repeat(drag, St);
-	_ ->
-%%	    io:format("Miss Action ~p~n", [Ev]),
-	    keep
+        _ when AUVSel =:= GSel ->
+            wings_wm:send_after_redraw(geom, Act),
+            keep;
+        {body, _} -> keep;
+        {face, _} -> keep;
+        {edge, _} -> keep;
+        {vertex,_} -> keep;
+        _ ->
+            wings_wm:send_after_redraw(geom, Act),
+            keep
     end;
 handle_event_3(got_focus, _) ->
     Msg1 = wings_msg:button_format(?__(1,"Select")),
@@ -788,7 +729,7 @@ handle_event_3(got_focus, _) ->
     wings_wm:message(Message, ""),
     wings_wm:dirty();
 handle_event_3(_Event, _) ->
-%%    io:format("MissEvent ~p~n", [_Event]),
+    %% io:format("MissEvent ~P~n", [_Event, 20]),
     keep.
 
 clear_temp_sel(#st{temp_sel=none}=St) -> St;
@@ -1744,6 +1685,7 @@ align_chart(Dir, V1={X1,Y1,_},V2={X2,Y2,_}, We) ->
     Center = e3d_vec:average(V1,V2),
     rotate_chart(-Deg,Center,We).
 
+-spec align_error() -> no_return().
 align_error() ->
     wings_u:error_msg(?__(1,"Select two vertices or one edge")).
 
@@ -1923,39 +1865,37 @@ draw_background(#st{bb=#uvstate{matname=MatName,st=St,bg_img=Image}}) ->
     %% Draw border around the UV space.
     gl:enable(?GL_DEPTH_TEST),
     gl:polygonMode(?GL_FRONT_AND_BACK, ?GL_LINE),
-    gl:lineWidth(1),
-    gl:color3f(0, 0, 0.7),
-    gl:translatef(0, 0, -0.5),
-    gl:'begin'(?GL_LINES),
-    G = fun(V) ->
-		gl:vertex2f(V,20),  gl:vertex2f(V,-20),
-		gl:vertex2f(20,V),  gl:vertex2f(-20,V)
-	end,
-    lists:foreach(G, lists:seq(-20,20)),
-    gl:'end'(),
-    gl:lineWidth(3),
-    gl:color3f(0, 0, 1.0),
+    gl:lineWidth(1.0),
+    gl:color3f(0.0, 0.0, 0.7),
+    gl:translatef(0.0, 0.0, -0.5),
+    Bin = << <<V:?F32,20.0:?F32, V:?F32,-20:?F32, 20.0:?F32,V:?F32, -20.0:?F32,V:?F32>>
+             || V <- lists:seq(-20,20) >>,
+    wings_vbo:draw(fun() -> gl:drawArrays(?GL_LINES, 0, 4*(20+20+1)) end, Bin, [vertex2d]),
+    gl:lineWidth(3.0),
+    gl:color3f(0.0, 0.0, 1.0),
     gl:recti(0, 0, 1, 1),
 
     %% Draw the background texture.
     gl:polygonMode(?GL_FRONT_AND_BACK, ?GL_FILL),
-    gl:color3f(1, 1, 1),			%Clear
+    gl:color3f(1.0, 1.0, 1.0),			%Clear
     case get({?MODULE,show_background}) of
 	false -> ok;
-	_ -> 
+	_ ->
 	    Tx = case get_texture(MatName,St) of
 		     false -> wings_image:txid(Image);
 		     DiffId -> wings_image:txid(DiffId)
 		 end,
-	    gl:enable(?GL_TEXTURE_2D),
-	    gl:bindTexture(?GL_TEXTURE_2D, Tx)
+            case Tx of
+                none -> ignore; %% Avoid crash if TexImage is deleted
+                Tx ->
+                    gl:enable(?GL_TEXTURE_2D),
+                    gl:bindTexture(?GL_TEXTURE_2D, Tx)
+            end
     end,
-    gl:'begin'(?GL_QUADS),
-    gl:texCoord2f(0, 0),    gl:vertex3f(0, 0, -0.99999),
-    gl:texCoord2f(1, 0),    gl:vertex3f(1, 0, -0.99999),
-    gl:texCoord2f(1, 1),    gl:vertex3f(1, 1, -0.99999),
-    gl:texCoord2f(0, 1),    gl:vertex3f(0, 1, -0.99999),
-    gl:'end'(), 
+    Q = [{0.0, 0.0},{0.0, 0.0, -0.99999}, {1.0, 0.0},{1.0, 0.0, -0.99999},
+         {1.0, 1.0},{1.0, 1.0, -0.99999}, {0.0, 1.0},{0.0, 1.0, -0.99999}],
+    wings_vbo:draw(fun() -> gl:drawArrays(?GL_QUADS, 0, 4) end, Q, [uv, vertex]),
+
     gl:disable(?GL_TEXTURE_2D),
 
     gl:popAttrib().
@@ -1965,10 +1905,10 @@ redraw(St) ->
     wings:redraw(St).
 
 init_drawarea() ->
-    {{X,TopY},{W0,TopH}} = wings_wm:win_rect(desktop),
+    {W0,H0} = wings_wm:top_size(),
     W = W0 div 2,
-    {X+W,TopY+75,W,TopH-100}.
-    
+    {{W,75},{W,H0-100}}.
+
 cleanup_before_exit() ->
     wings:unregister_postdraw_hook(wings_wm:this(), ?MODULE),
     wings_dl:delete_dlists().
