@@ -18,7 +18,7 @@
 -define(NEED_ESDL, 1).
 -define(NEED_OPENGL, 1).
 
--include_lib("wings.hrl").
+-include_lib("wings/src/wings.hrl").
 
 %% State info
 -record(cs, {v=[],  %% Connected vertices
@@ -49,8 +49,6 @@ menu(_, Menu) -> Menu.
 
 command({tools,connect}, St0) ->
     wings:mode_restriction([vertex,edge]), %% ,face
-    Active = wings_wm:this(),
-    wings_wm:callback(fun() -> wings_u:menu_restriction(Active, [view]) end),
     St = wings_undo:init(St0#st{selmode=edge,sel=[],sh=true}),
     wings_draw:refresh_dlists(St),
     C = #cs{ost=St0, st=St},
@@ -455,7 +453,7 @@ select_way(Cuts,We,NMM = {Mode,_}) ->
 check_normal(Face,{Way,MM},We = #we{id=Id}) ->
     {MVM,_PM,_} = wings_u:get_matrices(Id, MM),
     Normal0 = wings_face:normal(Face,We),
-    {_,_,Z} = e3d_mat:mul_vector(list_to_tuple(MVM),Normal0),
+    {_,_,Z} = e3d_mat:mul_vector(MVM, Normal0),
     if 
 	Way == normal, Z > 0.1 -> true;
 	Way == inverted, Z < -0.1 -> true;
@@ -598,16 +596,15 @@ gldraw_connect(Pos0, Pos1) ->
     gl:disable(?GL_LIGHTING),
     gl:disable(?GL_DEPTH_TEST),
     gl:disable(?GL_ALPHA_TEST),
-    gl:color3f(0, 0, 0),
+    gl:color3f(0.0, 0.0, 0.0),
     gl:matrixMode(?GL_PROJECTION),
     gl:loadIdentity(),
-    glu:ortho2D(0, W, 0, H),
+    glu:ortho2D(0.0, W, 0.0, H),
     gl:matrixMode(?GL_MODELVIEW),
     gl:loadIdentity(),
-    gl:'begin'(?GL_LINES),
-    gl:vertex3fv(Pos0),
-    gl:vertex3fv(Pos1),
-    gl:'end'(),
+    wings_vbo:draw(fun() ->
+			   gl:drawArrays(?GL_LINES, 0, 2)
+		   end, [Pos0,Pos1]),
     gl:popAttrib().
 
 slide(C=#cs{st=St=#st{shapes=Sh},we=Shape,v=[#vi{id=Id1,mm=MM}|_]},S,E) ->
