@@ -13,9 +13,9 @@
 
 -module(wpc_constraints).
 -export([init/0,menu/2,command/2]).
--include("wings.hrl").
+-define(NEED_ESDL, 1).
+-include_lib("wings/src/wings.hrl").
 -include("e3d.hrl").
--include_lib("esdl/include/sdl_keyboard.hrl").
 -define(NONZERO, 1.0e-6).
 
 init() ->
@@ -736,6 +736,7 @@ check_element(#st{sel=[{_,Sel}]}) ->
 check_element(_St) ->
     element_error().
 
+-spec element_error() -> no_return().
 element_error() ->
     Str = ?__(1,"Exactly one element must be selected"),
     wings_u:error_msg(Str).
@@ -753,6 +754,7 @@ check_angle_sel(#st{sel=[{_,Sel1},{_,Sel2}]}) ->
 check_angle_sel(_St) ->
     angle_error().
 
+-spec angle_error() -> no_return().
 angle_error() ->
     wings_u:error_msg(?__(1,"Exactly two elements must be selected")).
 
@@ -1048,7 +1050,9 @@ set_constraint({Shift,Ctrl,Alt}, Key, Val) ->
     end,
     ComboStr = mod(ModKeyCombo),
     Tag = tag(Key),
-    io:format("The ~s constraint bound to ~s is now set to ~p\n\n",[Tag,ComboStr,Val]),
+    Msg = io_lib:format(?__(1,"The ~ts constraint bound to ~ts is now set to ~p"),[Tag,ComboStr,Val]),
+    io:format("~ts\n",[Msg]),
+    wings_u:message(Msg),
     wings_pref:set_value(list_to_atom(Key++ModKeyCombo), Val).
 
 tag(Key) ->
