@@ -132,11 +132,11 @@ axis_menu(_, face, Axis) ->
     {AxisStr,{plane_cut,Fun}}.
 
 slices_dialog(Mode,Cmd) ->
-    wings_ask:dialog(?__(1,"Slice into Equal Parts"),
-      [{vframe,[{hframe,[{slider,{text,2,[{range,{2,100}}]}}]}]}],
-      fun([Parts]) ->
-        {Mode,{slice,{Parts,Cmd}}}
-      end).
+    wings_dialog:dialog(?__(1,"Slice into Equal Parts"),
+			[{vframe,[{hframe,[{slider,{text,2,[{range,{2,100}}]}}]}]}],
+			fun([Parts]) ->
+				{Mode,{slice,{Parts,Cmd}}}
+			end).
 
 %%%
 %%% Commands
@@ -339,6 +339,7 @@ plane_cut(Cut, {Axis0,Point}, St0) ->
         {save_state,wings_sel:valid_sel(St)}
     end.
 
+-spec planecut_error() -> no_return().
 planecut_error() ->
     wings_u:error_msg(?__(1,"This is taking too long.\nTry again with a smaller selection.")).
 
