@@ -65,54 +65,56 @@ prefs() ->
     WhRotate = wings_pref:get_value(wh_rot_spd, 7.5),
     ZoomAim = wings_pref:get_value(highlight_zoom_aim),
 
-    Hook = fun (is_disabled, {_Var,_I,Sto}) ->
-            not gb_trees:get(wheel_zooms, Sto);
-             (_, _) -> void
-           end,
-    WHook = fun (is_disabled, {_Var,_I,Sto}) ->
-             not ((gb_trees:get(wheel_adds, Sto)) andalso (gb_trees:get(wheel_zooms, Sto)));
-             (_, _) -> void
-            end,
+    AddHook = fun(wheel_adds, Enable0, Sto) ->
+		      Keys = [wh_scroll_info, wh_pan_spd, wh_rot_spd],
+		      Enable = Enable0 andalso wings_dialog:get_value(wheel_zooms, Sto),
+		      wings_dialog:enable(Keys, Enable, Sto)
+	      end,
+    WheelHook = fun(wheel_zooms, Enable, Sto) ->
+			Keys = [wheel_zoom_factor,wheel_zoom_factor_alt,wheel_adds],
+			wings_dialog:enable(Keys, Enable, Sto),
+			Keys2 = [wh_scroll_info, wh_pan_spd, wh_rot_spd],
+			Enable2 = Enable andalso wings_dialog:get_value(wheel_adds, Sto),
+			wings_dialog:enable(Keys2, Enable2, Sto)
+		end,
+
     {hframe,
-    [{vframe,
-     [{vframe,[mouse_buttons()],[{title,?__(1,"Mouse Buttons")}]},
-      {vframe,[camera_modes()],[{title,?__(2,"Camera Mode")}]},
+     [{vframe,
+       [{vframe,[mouse_buttons()],[{title,?__(1,"Mouse Buttons")}]},
+	{vframe,[camera_modes()],[{title,?__(2,"Camera Mode")}]},
+	{vframe,
+	 [{hframe,[{slider,{text,CamRotSpeed,[{key,cam_rotation_speed},{range,{1,100}}]}}]}],
+	 [{title,?__(19,"Rotation Speed")}]},
+	{vframe,
+	 [{hframe,[{slider,{text,PanSpeed0,[{key,pan_speed},{range,{1,100}}]}}]}],
+	 [{title,?__(3,"Pan Speed")}]},
+	{vframe,
+	 [{hframe,[{slider,{text,ArrowPanSpeed,[{key,pan_speed_arrow_keys},{range,{1,100}}]}}]}],
+	 [{title,?__(16,"Arrow Key Pan Speed")}]}]},
       {vframe,
-       [{hframe,[{slider,{text,CamRotSpeed,[{key,cam_rotation_speed},{range,{1,100}}]}}]}],
-       [{title,?__(19,"Rotation Speed")}]},
-      {vframe,
-       [{hframe,[{slider,{text,PanSpeed0,[{key,pan_speed},{range,{1,100}}]}}]}],
-       [{title,?__(3,"Pan Speed")}]},
-      {vframe,
-       [{hframe,[{slider,{text,ArrowPanSpeed,[{key,pan_speed_arrow_keys},{range,{1,100}}]}}]}],
-       [{title,?__(16,"Arrow Key Pan Speed")}]}]},
-    {vframe,
-      [{vframe,
-       [{hframe,[{?__(4,"Wheel Zooms"),ZoomFlag0,[{key,wheel_zooms}]},
-                 {?__(20,"Zooming in aims Camera"),ZoomAim,[{key,highlight_zoom_aim}]}]},
-    {hframe,
-     [{label,?__(7,"Zoom Factor"),[{hook,Hook}]},
-      {text,ZoomFactor0,
-       [{key,wheel_zoom_factor},
-     {range,{1,50}},{hook,Hook}]},
-      {label,"%",[{hook,Hook}]}]},
-     {hframe,
-      [{label,?__(10,"Alternate Zoom Factor(Alt+Scroll)"),[{hook,Hook}]},
-       {text,ZoomFactorAlt,
-        [{key,wheel_zoom_factor_alt},
-      {range,{1,50}},{hook,Hook}]},
-     {label,"%",[{hook,Hook}]}]} ],[{title,?__(9,"Scroll Wheel")}]},
-    {vframe,
-      [{?__(11,"Wheel Pans & Rotates"),WheelAdds,[{key,wheel_adds},{hook,Hook}]},
-       {?__(17,"Show Info Line Help String"),WhScrollInfo,[{key,wh_scroll_info},{hook,WHook}]}, 
-      {vframe,
-       [{hframe,
-        [{slider,{text,WhPanSpd,[{key, wh_pan_spd},{range,{1,100}}]}}],
-      [{title,?__(12,"Pan Speed")},{hook,WHook}]},
-      {hframe,
-       [{slider,{text,WhRotate,[{key, wh_rot_spd},{range,{?NEARZERO,180.0}}]}}],
-       [{title,?__(13,"Rotation Step in Degrees")},{hook,WHook}]}]}],
-    [{title,?__(15,"Unidirectional Camera")}]}]}]}.
+       [{vframe,
+	 [{hframe,[{?__(4,"Wheel Zooms"),ZoomFlag0,[{key,wheel_zooms}, {hook, WheelHook}]},
+		   {?__(20,"Zooming in aims Camera"),ZoomAim,[{key,highlight_zoom_aim}]}]},
+	  {hframe,
+	   [{label,?__(7,"Zoom Factor")},
+	    {text,ZoomFactor0, [{key,wheel_zoom_factor},{range,{1,50}}]},
+	    {label,"%"}]},
+	  {hframe,
+	   [{label,?__(10,"Alternate Zoom Factor(Alt+Scroll)")},
+	    {text,ZoomFactorAlt, [{key,wheel_zoom_factor_alt},{range,{1,50}}]},
+	    {label,"%"}]}
+	 ],[{title,?__(9,"Scroll Wheel")}]},
+	{vframe,
+	 [{?__(11,"Wheel Pans & Rotates"),WheelAdds,[{key,wheel_adds}, {hook, AddHook}]},
+	  {?__(17,"Show Info Line Help String"),WhScrollInfo,[{key,wh_scroll_info}]},
+	  {vframe,
+	   [{hframe,
+	     [{slider,{text,WhPanSpd,[{key, wh_pan_spd},{range,{1,100}}]}}],
+	     [{title,?__(12,"Pan Speed")}]},
+	    {hframe,
+	     [{slider,{text,WhRotate,[{key, wh_rot_spd},{range,{?NEARZERO,180.0}}]}}],
+	     [{title,?__(13,"Rotation Step in Degrees")}]}]}],
+	 [{title,?__(15,"Unidirectional Camera")}]}]}]}.
 
 mouse_buttons() ->
     {menu,[{desc(1),1,[{info,info(1)}]},
@@ -120,32 +122,29 @@ mouse_buttons() ->
            {desc(3),3,[{info,info(3)}]}],
      wings_pref:get_value(num_buttons),
      [{key,num_buttons},
-      {hook,fun (update, {Var,_I,Val,Sto0}) ->
-		    Sto = gb_trees:update(Var, Val, Sto0),
-		    Mode0 = gb_trees:get(camera_mode, Sto),
-		    Mode = case {Val,Mode0} of
-			       {1,_} -> nendo;
-			       {2,blender} -> blender;
-			       {2,_} -> nendo;
-			       {3,_} -> Mode0
-			   end,
-		    {store,gb_trees:update(camera_mode, Mode, Sto)};
-		(_, _) -> void
+      {hook,fun (_,Val,Sto) ->
+		    Mode0 = wings_dialog:get_value(camera_mode, Sto),
+		    Mode  = case {Val,Mode0} of
+				{1,_} -> nendo;
+				{2,blender} -> blender;
+				{2,_} -> nendo;
+				{3,_} -> Mode0
+			    end,
+		    wings_dialog:set_value(camera_mode, {Mode, camera_modes(Val)}, Sto)
 	    end}]}.
 
 camera_modes() ->
-    Modes = [wings_cam,mirai,nendo,maya,tds,blender,mb,sketchup],
-    {menu,[{desc(Mode),Mode,[{info,info(Mode)}]} || Mode <- Modes],
+    {menu,camera_modes(wings_pref:get_value(num_buttons)),
      wings_pref:get_value(camera_mode),
-     [{key,camera_mode},
-      {hook,fun (menu_disabled, {_Var,_I,Sto}) ->
-		    case gb_trees:get(num_buttons, Sto) of
-			1 -> [mirai,maya,tds,blender,mb,sketchup,wings_cam];
-			2 -> [mirai,maya,tds,mb,sketchup,wings_cam];
-			3 -> []
-		    end;
-		(_, _) -> void
-	    end}]}.
+     [{key,camera_mode}]}.
+
+camera_modes(NumButtons) ->
+    All = [wings_cam,mirai,nendo,maya,tds,blender,mb,sketchup],
+    [{desc(Mode),Mode,[{info,info(Mode)}]} || Mode <- All -- disabled(NumButtons)].
+
+disabled(1) -> [mirai,maya,tds,blender,mb,sketchup,wings_cam];
+disabled(2) -> [mirai,maya,tds,mb,sketchup,wings_cam];
+disabled(3) -> [].
 
 desc(1) -> ?__(1,"One");
 desc(2) -> ?__(2,"Two");
@@ -249,20 +248,21 @@ tweak_cam_event(Ev, Sym, Camera, St) ->
 
 quit_tweak_cam() ->
     case wings_io:get_mouse_state() of
-      {0,X,Y} ->
-        wings_wm:later(#mousebutton{button=1,x=X,y=Y,mod=0,state=?SDL_RELEASED}),
-        pop;
-      _ ->
-        pop
+	{0,X0,Y0} ->
+	    {X,Y} = wings_wm:screen2local({X0,Y0}),
+	    wings_wm:later(#mousebutton{which=wings_wm:this_win(),
+					button=1,x=X,y=Y,mod=0,state=?SDL_RELEASED}),
+	    pop;
+	_ ->
+	    pop
     end.
 
 %%%
 %%% Blender style camera.
 %%%
 
-blender(#mousebutton{button=2,state=?SDL_PRESSED,x=X0,y=Y0,mod=Mod}, Redraw)
+blender(#mousebutton{button=2,state=?SDL_PRESSED,x=X,y=Y,mod=Mod}, Redraw)
   when Mod band ?ALT_BITS =:= 0 ->
-    {X,Y} = wings_wm:local2global(X0, Y0),
     Camera = #camera{x=X,y=Y,ox=X,oy=Y},
     grab(),
     message(blender_help()),
@@ -303,9 +303,8 @@ blender_help() ->
 %%% Nendo style camera.
 %%%
 
-nendo(#mousebutton{button=2,x=X0,y=Y0,mod=Mod,state=?SDL_RELEASED}, Redraw)
+nendo(#mousebutton{button=2,x=X,y=Y,mod=Mod,state=?SDL_RELEASED}, Redraw)
   when Mod band ?CTRL_BITS =:= 0 ->
-    {X,Y} = wings_wm:local2global(X0, Y0),
     Camera = #camera{x=X,y=Y,ox=X,oy=Y},
     grab(),
     MoveTumbles = allow_rotation(),
@@ -373,9 +372,8 @@ nendo_message(false) ->
 %%% Mirai style camera.
 %%%
 
-mirai(#mousebutton{button=2,x=X0,y=Y0,mod=Mod,state=?SDL_RELEASED}, Redraw)
+mirai(#mousebutton{button=2,x=X,y=Y,mod=Mod,state=?SDL_RELEASED}, Redraw)
   when Mod band ?CTRL_BITS =:= 0 ->
-    {X,Y} = wings_wm:local2global(X0, Y0),
     Camera = #camera{x=X,y=Y,ox=X,oy=Y},
     grab(),
     MoveTumbles = allow_rotation(),
@@ -448,8 +446,7 @@ mirai_message(false) ->
 %%% 3ds max style camera.
 %%%
 
-tds(#mousebutton{button=2,x=X0,y=Y0,state=?SDL_PRESSED}, Redraw) ->
-    {X,Y} = wings_wm:local2global(X0, Y0),
+tds(#mousebutton{button=2,x=X,y=Y,state=?SDL_PRESSED}, Redraw) ->
     Camera = #camera{x=X,y=Y,ox=X,oy=Y},
     grab(),
     message(wings_msg:join(tds_help(), 
@@ -498,10 +495,9 @@ tds_help() ->
 %%% Maya style camera.
 %%%
 
-maya(#mousebutton{x=X0,y=Y0,mod=Mod,state=?SDL_PRESSED}, Redraw)
+maya(#mousebutton{x=X,y=Y,mod=Mod,state=?SDL_PRESSED}, Redraw)
   when Mod band ?ALT_BITS =/= 0, Mod band ?CTRL_BITS =:= 0, Mod band ?SHIFT_BITS =:= 0 ->
-    {X,Y} = wings_wm:local2global(X0, Y0),
-    wings_io:change_event_handler(?SDL_KEYUP, ?SDL_ENABLE),
+    wings_io:change_event_handler(?SDL_KEYUP, true),
     Camera = #camera{x=X,y=Y,ox=X,oy=Y},
     grab(),
     message(maya_help()),
@@ -545,7 +541,7 @@ get_maya_event(Camera, Redraw) ->
     {replace,fun(Ev) -> maya_event(Ev, Camera, Redraw) end}.
 
 maya_stop_camera(Camera) ->
-    wings_io:change_event_handler(?SDL_KEYUP, ?SDL_IGNORE),
+    wings_io:change_event_handler(?SDL_KEYUP, false),
     stop_camera(Camera).
 
 maya_help() ->
@@ -560,9 +556,8 @@ maya_help() ->
 %%% Motionbuilder style camera.
 %%%
 
-mb(#mousebutton{button=1,mod=Mod,x=X0,y=Y0,state=?SDL_PRESSED}, Redraw)
+mb(#mousebutton{button=1,mod=Mod,x=X,y=Y,state=?SDL_PRESSED}, Redraw)
   when Mod band (?SHIFT_BITS bor ?CTRL_BITS) =/= 0 ->
-    {X,Y} = wings_wm:local2global(X0, Y0),
     Camera = #camera{x=X,y=Y,ox=X,oy=Y},
     grab(),
     message(mb_help()),
@@ -607,9 +602,8 @@ mb_help() ->
 %%% Sketchup style camera.
 %%%
 
-sketchup(#mousebutton{button=2,state=?SDL_PRESSED,x=X0,y=Y0,mod=Mod}, Redraw)
+sketchup(#mousebutton{button=2,state=?SDL_PRESSED,x=X,y=Y,mod=Mod}, Redraw)
   when (Mod band (?ALT_BITS  bor ?CTRL_BITS)) =:= 0  ->
-    {X,Y} = wings_wm:local2global(X0, Y0),
     Camera = #camera{x=X,y=Y,ox=X,oy=Y},
     grab(),
     message(sketchup_help()),
@@ -648,9 +642,8 @@ sketchup_help() ->
 %%% Wings 3D camera suggested by oort
 %%%
 
-wings_cam(#mousebutton{button=2,x=X0,y=Y0,mod=Mod,state=?SDL_RELEASED}, Redraw)
+wings_cam(#mousebutton{button=2,x=X,y=Y,mod=Mod,state=?SDL_RELEASED}, Redraw)
   when Mod band (?CTRL_BITS bor ?SHIFT_BITS bor ?ALT_BITS) =:= 0 ->
-    {X,Y} = wings_wm:local2global(X0, Y0),
     Camera = #camera{x=X,y=Y,ox=X,oy=Y},
     grab(),
     wings_cam_message(),
@@ -767,7 +760,7 @@ aim_zoom(Dir, St0) ->
             true ->
               Client = wings_wm:this(),
               {X0,Y0} = wings_wm:win_size(Client),
-              {X,Y} = wings_wm:local2global(X0 div 2, Y0 div 2),
+              {X,Y} = {X0 div 2, Y0 div 2},
               wings_io:warp(X,Y),
               zoom_step(Dir)
         end;
@@ -888,19 +881,18 @@ dist_factor(Dist) ->
     max(abs(Dist), 0.2).
 
 stop_camera(#camera{ox=Ox,oy=Oy}) ->
+    wings_wm:release_focus(),
     case wings_io:ungrab(Ox, Oy) of
 	still_grabbed ->
 	    wings_wm:later(view_changed);
 	no_grab ->
-	    wings_wm:release_focus(),
 	    wings_wm:dirty()
     end,
     update_sel(fun show_sel_fun/2),
     pop.
 
-camera_mouse_range(X0, Y0, #camera{x=OX,y=OY, xt=Xt0, yt=Yt0}=Camera) ->
+camera_mouse_range(X1, Y1, #camera{x=OX,y=OY, xt=Xt0, yt=Yt0}=Camera) ->
 %%    io:format("Camera Mouse Range ~p ~p~n", [{X0,Y0}, {OX,OY,Xt0,Yt0}]),
-    {X1,Y1} = wings_wm:local2global(X0, Y0),
     XD0 = (X1 - OX),
     YD0 = (Y1 - OY),
     {XD,YD} = wings_pref:lowpass(XD0 + Xt0, YD0 + Yt0),
