@@ -33,8 +33,6 @@
 -include("wings.hrl").
 -include_lib("cl/include/cl.hrl").
 
--compile(export_all).
-
 -define(PL_UNITS, 4).
 
 -record(v, {pos,                     % position
@@ -70,119 +68,110 @@
 -define(FACE_SZ, (4*4)).
 -define(VERTEX_SZ, ((3*4)+4)).
 -define(LOCK_SZ, 32).
-%%-define(DEFAULT, erlang).
--define(DEFAULT, opencl).
 
 %%%% API %%%%%%%%%
 
 %% Returns opaque data
 init(Plan, Level, We) when is_integer(Level) ->
-    build_data(Plan,Level,We,?DEFAULT);
+    build_data(Plan,Level,We);
 
 init(Plan, #base{level=Level}, We)  ->
-    build_data(Plan,Level,We,?DEFAULT).
+    build_data(Plan,Level,We).
 
 %% Update state with new vertex positions
 update(ChangedVs, Data) ->
-    case ?DEFAULT of 
-	erlang ->  wings_cc_ref:update(ChangedVs, Data);
-	_ ->       update_1(ChangedVs, Data)
-    end.
-    
+    update_1(ChangedVs, Data).
+
 %% Generates a subdivided #vab{}
 gen_vab(Base) ->
     try
-	case subdiv(Base, ?DEFAULT) of
-	    skip -> create_vab(<<>>, []);
-	    Data ->
-		gen_vab_1(Data, Base)
-	end
+	Data = subdiv(Base),
+	gen_vab_1(Data, Base)
     catch
 	exit:{out_of_resources, Wanted, CardMax} ->
-	    io:format(?__(1,"OpenCL subd failed: wanted ~pMB only ~pMB available~n"), 
+	    io:format(?__(1,"OpenCL subd failed: wanted ~pMB only ~pMB available~n"),
 		      [Wanted, CardMax]),
 	    DecBase = decrease_level(Base),
-	    gen_vab(DecBase)	   
+	    gen_vab(DecBase)
     end.
 %% Generates a subdivided #vab{} from Material Plan
 gen_vab(Plan, Base) ->
-    try 
-	case subdiv(Base, ?DEFAULT) of
-	    skip -> create_vab(<<>>, []);
-	    Data ->
-		gen_vab_1(Plan, Data, Base)
-	end
+    try
+	Data = subdiv(Base),
+	gen_vab_1(Plan, Data, Base)
     catch
 	exit:{out_of_resources, Wanted, CardMax} ->
-	    io:format(?__(1,"OpenCL subd failed: wanted ~pMB only ~pMB available~n"), 
+	    io:format(?__(1,"OpenCL subd failed: wanted ~pMB only ~pMB available~n"),
 		      [Wanted, CardMax]),
 	    DecBase = decrease_level(Base),
 	    gen_vab(Plan, DecBase)
     end.
 
 %% Subdivide mesh
-subdiv(Base = #base{level=N, n=Total, type=Type}, Impl) when Total > 0 ->
-    case Impl of
-	opencl ->
-	    {In,Out,CL} = cl_allocate(Base, cl_setup()),
-	    Wait = cl_write_input(Base, In, Out, CL),
-	    subdiv_1(N, In, Out, Type, CL, Wait);
-	erlang ->		
-	    wings_cc_ref:subdiv(Base)
-    end;
-subdiv(_, _) ->
-    skip.
+subdiv(Base = #base{level=N, type=Type}) ->
+    {In,Out,CL} = cl_allocate(Base, cl_setup()),
+    Wait = cl_write_input(Base, In, Out, CL),
+    subdiv_1(N, In, Out, Type, CL, Wait).
 
 %% Generates a vab (and updates Data)
 %% Returns Vab
 gen_vab_1(Data0, Base) ->
-    case ?DEFAULT of
-	erlang -> create_vab(wings_cc_ref:gen_vab(Data0, Base), Base);
-	opencl -> create_vab(gen_vab_2(Data0, Base), Base)
-    end.
+    create_vab(gen_vab_2(Data0, Base), Base).
 
 gen_vab_1(Plan, Data, Base) ->
-    case ?DEFAULT of
-	erlang -> create_vab(wings_cc_ref:gen_vab(Plan, Data, Base), Base);
-	opencl -> create_vab(gen_vab_2(Plan, Data, Base), Base)
-    end.
+    create_vab(gen_vab_2(Plan, Data, Base), Base).
 
-create_vab({Vs, SNs0, {Attrs0, Tangents0}, Edges, MatInfo}, #base{type=Type}) ->
+create_vab({Vs,SNs,{Attrs,Tangents},Edges,MatInfo}, #base{type=Type}) ->
+    L = [{vs,Vs},
+	 {sns,SNs},
+	 {Type,Attrs},
+	 {ts,Tangents}],
+    Data0 = [D || D <- [Vs,SNs,Attrs,Tangents], is_binary(D)],
+    Data = iolist_to_binary(Data0),
+    [Vbo] = gl:genBuffers(1),
+    gl:bindBuffer(?GL_ARRAY_BUFFER, Vbo),
+    gl:bufferData(?GL_ARRAY_BUFFER, byte_size(Data), Data, ?GL_STATIC_DRAW),
+    gl:bindBuffer(?GL_ARRAY_BUFFER, 0),
+    Vab = #vab{id=Vbo,face_es=Edges,mat_map=MatInfo},
+    create_vab_1(L, 0, Vab).
+
+create_vab_1([{_,Data}=H|T], Offset0, Vab0) ->
+    Vab = create_vab_2(H, Offset0, Vab0),
+    Offset = case is_binary(Data) of
+		 false -> Offset0;
+		 true -> Offset0 + byte_size(Data)
+	     end,
+    create_vab_1(T, Offset, Vab);
+create_vab_1([], _, Vab) -> Vab.
+
+create_vab_2({vs,Vs}, Offset, Vab) ->
     Ns = case Vs of
-	     <<>> -> Vs;
-	     <<_:3/unit:32,NsP/bytes>> ->
-		 NsP
+	     <<>> -> Offset;
+	     _ -> Offset + 3 * 4
 	 end,
     S = 3*4+3*4,
+    Vab#vab{face_vs={S,Offset},face_fn={S,Ns}};
+create_vab_2({sns,SNs0}, Offset, Vab=#vab{face_fn=FaceNs}) ->
     SNs = case SNs0 of
-	      <<>> -> {S, Ns};
-	      _ -> {4*4, SNs0}  %% Special case easier cl code
+	      <<>> -> FaceNs;
+	      _ -> {4*4, Offset}
 	  end,
-    Colors = case Attrs0 of 
-		 <<>> -> none;
-		 _ when Type =:= color ->
-		     {3*4, Attrs0};
-		 _ when Type =:= color_uv; Type =:= color_uv_tangent ->
-		     {5*4, Attrs0};
-		 _ -> none		     
-	     end,
-    UVs = case Attrs0 of
-	      <<>> -> none;
-	      _ when Type =:= uv; Type =:= uv_tangent ->
-		  {2*4, Attrs0};
-	      <<_:3/unit:32,UVBin/bytes>> 
-		when Type =:= color_uv; Type =:= color_uv_tangent ->
-		  {5*4, UVBin};
-	      _ -> none
-	  end,
-    Tangents = case Tangents0 of
-		   none -> none;
-		   Ts when is_binary(Ts) ->
-		       {4*4, Tangents0}
-	       end,
-    #vab{face_vs={S,Vs},face_fn={S,Ns}, face_es=Edges,
-	 face_sn=SNs, face_vc=Colors, face_uv=UVs, face_ts=Tangents,
-	 mat_map=MatInfo}.
+    Vab#vab{face_sn=SNs};
+create_vab_2({VCs,Data}, Offset, Vab)
+  when (VCs =:= color orelse VCs =:= color_tangent), Data =/= <<>> ->
+    Vab#vab{face_vc={3*4,Offset}};
+create_vab_2({UV,Data}, Offset, Vab)
+  when (UV =:= uv orelse UV =:= uv_tangent), Data =/= <<>> ->
+    Vab#vab{face_uv={2*4,Offset}};
+create_vab_2({CUV,Data}, Offset, Vab)
+  when (CUV =:= color_uv orelse CUV =:= color_uv_tangent), Data =/= <<>> ->
+    Vab#vab{face_vc={5*4,Offset}, face_uv={5*4,Offset+3*4}};
+create_vab_2({ts,Data}, Offset, Vab)
+  when Data =/= <<>> ->
+    Vab#vab{face_ts={4*4,Offset}};
+create_vab_2({_Type, _Data}, _, Vab) ->
+    %% io:format("Ignore ~p ~p~n",[_Type, size(_Data)]),
+    Vab.
 
 update_1(ChangedVs0, #base{vmap=VMap, v=VsBin0}=Base)
   when is_list(ChangedVs0) ->
@@ -213,7 +202,7 @@ update_vs([], _, Bin) -> [Bin].
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-build_data({Type, MatFs}, N, We0, Impl) ->
+build_data({Type, MatFs}, N, We0) ->
     {FMap0, We} = gen_fmap(MatFs, We0),
     FMap  = gb_trees:from_orddict(lists:sort(FMap0)),
     Empty = array:new(),
@@ -228,7 +217,7 @@ build_data({Type, MatFs}, N, We0, Impl) ->
 		 build_data_attrs(FMap0, [], [], StartAcc, Get, Attrs, We)
 	 end,
     B1 = calc_matmap(MatFs, B0#base{type=Type, level=N}),
-    pack_data(B1, Impl).
+    pack_data(B1).
 
 build_data_plain([{F,_}|Fs], Ftab0, Acc0, Get, We) ->
     {Vs, Acc} = wings_face:fold(Get, {[], Acc0}, F, We), %% CCW
@@ -380,11 +369,7 @@ update_vmap(Orig, VM={N,VMap}) ->
 	    {V, VM}
     end.
 
-pack_data(B=#base{e=Etab0, f=Ftab0}, erlang) ->
-    Etab = array:from_list(array:sparse_to_list(Etab0)),
-    Ftab = array:from_list(Ftab0),
-    B#base{e=Etab, f=Ftab};
-pack_data(B0=#base{v=Vtab0, e=Etab0, f=Ftab0}, opencl) ->
+pack_data(B0=#base{v=Vtab0, e=Etab0, f=Ftab0}) ->
     GetFs = fun(Vs, {No, FI, Fs}) ->
 		    Len = length(Vs),
 		    {No+Len, <<FI/binary, No:?I32, Len:?I32>>,
@@ -707,6 +692,7 @@ create_buffers(Ctxt, [Binary|Szs], Acc) ->
 create_buffers(_, [], Buffers) ->
     lists:reverse(Buffers).
 
+-spec release_buffers(term(), boolean()) -> no_return().
 release_buffers(Buffers, true) ->
     case get({?MODULE, cl}) of
 	undefined ->
