@@ -18,8 +18,8 @@
 	 changed_we/2, update_normals/1,
 	 split/3,original_we/1,update_dynamic/2,join/1,abort_split/1]).
 
-% export for plugins (and wings_proxy) that need to draw stuff
--export([drawVertices/2, draw_flat_faces/2, draw_smooth_faces/2]).
+%% Export for plugins (and wings_proxy) that need to draw stuff
+-export([draw_flat_faces/2,draw_smooth_faces/2]).
 
 -define(NEED_OPENGL, 1).
 -define(NEED_ESDL, 1).
@@ -285,7 +285,7 @@ more_need([], _, Acc) ->
     ordsets:from_list(Acc).
 
 wins_of_same_class() ->
-    case wings_wm:get_prop(display_lists) of
+    case wings_wm:get_dd() of
 	geom_display_lists -> wings_u:geom_windows();
 	_ -> [wings_wm:this()]
     end.
@@ -328,23 +328,14 @@ update_fun_2(smooth, #dlo{smooth=none,proxy=false}=D0, St) ->
 update_fun_2(smooth, #dlo{proxy=true}=D0, St) ->
     wings_proxy:smooth(D0,St);
 update_fun_2({vertex,_PtSize}, #dlo{vs=none,src_we=We}=D, _) ->
-    UnselDlist = gl:genLists(1),
-    gl:newList(UnselDlist, ?GL_COMPILE),
-    gl:enableClientState(?GL_VERTEX_ARRAY),
-    drawVertices(?GL_POINTS, vertices_f32(visible_vertices(We))),
-    gl:disableClientState(?GL_VERTEX_ARRAY),
-    gl:endList(),
-    D#dlo{vs=UnselDlist};
+    Data = vertices_f32(visible_vertices(We)),
+    Unsel = vbo_draw_arrays(?GL_POINTS, Data),
+    D#dlo{vs=Unsel};
 update_fun_2(hard_edges, #dlo{hard=none,src_we=#we{he=Htab}=We}=D, _) ->
-    List = gl:genLists(1),
-    gl:newList(List, ?GL_COMPILE),
-    gl:enableClientState(?GL_VERTEX_ARRAY),
-    VisibleHard = wings_we:visible_edges(Htab, We),
-    Edges= gb_sets:to_list(VisibleHard),
-    drawVertices(?GL_LINES, edges_f32(Edges, We)),
-    gl:disableClientState(?GL_VERTEX_ARRAY),
-    gl:endList(),
-    D#dlo{hard=List};
+    Edges = gb_sets:to_list(wings_we:visible_edges(Htab, We)),
+    Data = edges_f32(Edges, We),
+    Hard = vbo_draw_arrays(?GL_LINES, Data),
+    D#dlo{hard=Hard};
 update_fun_2(edges, #dlo{edges=none,ns=Ns}=D, _) ->
     EdgeDl = make_edge_dl(array:sparse_to_list(Ns)),
     D#dlo{edges=EdgeDl};
@@ -356,26 +347,38 @@ update_fun_2(proxy, D, St) ->
 %%%% Check if plugins using the Pst need a draw list updated.
 update_fun_2({plugin,{Plugin,{_,_}=Data}},#dlo{plugins=Pdl}=D,St) ->
     case lists:keytake(Plugin,1,Pdl) of
-	    false ->
-		  Plugin:update_dlist(Data,D,St);
-	    {_,{Plugin,{_,none}},Pdl0} ->
-	      Plugin:update_dlist(Data,D#dlo{plugins=Pdl0},St);
-		_ -> D
-	end;
+        false ->
+            Plugin:update_dlist(Data,D,St);
+        {_,{Plugin,{_,none}},Pdl0} ->
+            Plugin:update_dlist(Data,D#dlo{plugins=Pdl0},St);
+        _ -> D
+    end;
 
 update_fun_2(_, D, _) -> D.
 
 make_edge_dl(Ns) ->
-    Dl = gl:genLists(1),
-    gl:newList(Dl, ?GL_COMPILE),
     {Tris,Quads,Polys,PsLens} = make_edge_dl_bin(Ns, <<>>, <<>>, <<>>, []),
-    gl:enableClientState(?GL_VERTEX_ARRAY),
-    drawVertices(?GL_TRIANGLES, Tris),    
-    drawVertices(?GL_QUADS, Quads),
-    drawPolygons(Polys, PsLens),
-    gl:disableClientState(?GL_VERTEX_ARRAY),
-    gl:endList(),
-    Dl.
+    T = vbo_draw_arrays(?GL_TRIANGLES, Tris),
+    Q = vbo_draw_arrays(?GL_QUADS, Quads),
+    DP = case wings_util:min_wx({1,8}) of
+             true ->
+                 {_,Ss,Ls} = foldl(fun(N, {Start, Ss, Ls}) ->
+                                           gl:drawArrays(?GL_POLYGON, Start, N),
+                                           {N+Start, <<Ss/binary, Start:?UI32>>, <<Ls/binary, N:?UI32>>}
+                                   end, {0, <<>>, <<>>}, PsLens),
+                 fun() ->
+                         gl:multiDrawArrays(?GL_POLYGON, Ss, Ls)
+                 end;
+             false ->
+                 fun() ->
+                         foldl(fun(Length, Start) ->
+                                       gl:drawArrays(?GL_POLYGON, Start, Length),
+                                       Length+Start
+                               end, 0, PsLens)
+                 end
+         end,
+    P = wings_vbo:new(DP, Polys),
+    [T,Q,P].
 
 make_edge_dl_bin([[_|[{X1,Y1,Z1},{X2,Y2,Z2},{X3,Y3,Z3}]]|Ns],
 		 Tris0, Quads, Polys, PsLens) ->
@@ -411,22 +414,12 @@ vertices_f32([{_,_,_}|_]=List) ->
     << <<X:?F32,Y:?F32,Z:?F32>> || {X,Y,Z} <- List >>;
 vertices_f32([]) -> <<>>.
 
-drawVertices(_Type, <<>>) -> 
-    ok;
-drawVertices(Type, Bin) -> 
-    gl:vertexPointer(3, ?GL_FLOAT, 0, Bin),
-    gl:drawArrays(Type, 0, byte_size(Bin) div 12),
-    wings:keep(Bin).
-
-drawPolygons(<<>>, _PsLens) -> 
-    ok;
-drawPolygons(Polys, PsLens) ->
-    gl:vertexPointer(3, ?GL_FLOAT, 0, Polys), 
-    foldl(fun(Length,Start) ->
-		  gl:drawArrays(?GL_POLYGON, Start, Length),
-		  Length+Start
-	  end, 0, PsLens),
-    wings:keep(Polys).
+vbo_draw_arrays(Type, Data) ->
+    N = byte_size(Data) div 12,
+    D = fun() ->
+		gl:drawArrays(Type, 0, N)
+	end,
+    wings_vbo:new(D, Data).
 
 visible_vertices(#we{vp=Vtab0}=We) ->
     case wings_we:is_open(We) of
@@ -465,78 +458,68 @@ update_sel(#dlo{sel=none,src_sel={face,Faces}}=D) ->
     update_face_sel(gb_sets:to_list(Faces), D);
 update_sel(#dlo{sel=none,src_sel={edge,Edges}}=D) ->
     #dlo{src_we=We} = D,
-    List = gl:genLists(1),
-    gl:newList(List, ?GL_COMPILE),
-    gl:enableClientState(?GL_VERTEX_ARRAY),
-    drawVertices(?GL_LINES, edges_f32(gb_sets:to_list(Edges), We)),
-    gl:disableClientState(?GL_VERTEX_ARRAY),
-    gl:endList(),
-    D#dlo{sel=List};
+    Data = edges_f32(gb_sets:to_list(Edges), We),
+    Sel = vbo_draw_arrays(?GL_LINES, Data),
+    D#dlo{sel=Sel};
 update_sel(#dlo{sel=none,src_sel={vertex,Vs}}=D) ->
     #dlo{src_we=#we{vp=Vtab0}} = D,
-    SelDlist = gl:genLists(1),
     Vtab1 = array:sparse_to_orddict(Vtab0),
-    case length(Vtab1) =:= gb_sets:size(Vs) of
-	true ->
-	    gl:newList(SelDlist, ?GL_COMPILE),
-	    gl:enableClientState(?GL_VERTEX_ARRAY),
-	    drawVertices(?GL_POINTS, vertices_f32(Vtab1)),
-	    gl:disableClientState(?GL_VERTEX_ARRAY),
-	    gl:endList(),
-	    D#dlo{sel=SelDlist};
-	false ->
-	    Vtab = sofs:from_external(Vtab1, [{vertex,data}]),
-	    R = sofs:from_external(gb_sets:to_list(Vs), [vertex]),
-	    Sel = sofs:to_external(sofs:image(Vtab, R)),
-
-	    gl:newList(SelDlist, ?GL_COMPILE),
-	    gl:enableClientState(?GL_VERTEX_ARRAY),
-	    drawVertices(?GL_POINTS, vertices_f32(Sel)),
-	    gl:disableClientState(?GL_VERTEX_ARRAY),
-	    gl:endList(),
-
-	    D#dlo{sel=SelDlist}
-    end;
+    Sel = case length(Vtab1) =:= gb_sets:size(Vs) of
+	      true ->
+		  Vtab1;
+	      false ->
+		  Vtab = sofs:from_external(Vtab1, [{vertex,data}]),
+		  R = sofs:from_external(gb_sets:to_list(Vs), [vertex]),
+		  sofs:to_external(sofs:image(Vtab, R))
+	  end,
+    Data = vertices_f32(Sel),
+    Points = vbo_draw_arrays(?GL_POINTS, Data),
+    D#dlo{sel=Points};
 update_sel(#dlo{}=D) -> D.
 
 %% Select all faces.
-update_sel_all(#dlo{vab=#vab{face_vs=Vs}}=D) when Vs =/= none ->
-    List = gl:genLists(1),
-    gl:newList(List, ?GL_COMPILE),
-    wings_draw_setup:enableVertexPointer(Vs),
+update_sel_all(#dlo{vab=#vab{face_vs=Vs}=Vab}=D) when Vs =/= none ->
     Count = wings_draw_setup:face_vertex_count(D),
-    gl:drawArrays(?GL_TRIANGLES, 0, Count),
-    wings_draw_setup:disableVertexPointer(Vs),
-    gl:endList(),
-    D#dlo{sel=List};
+    F = fun() ->
+		wings_draw_setup:enable_pointers(Vab, []),
+		gl:drawArrays(?GL_TRIANGLES, 0, Count),
+		wings_draw_setup:disable_pointers(Vab, [])
+	end,
+    D#dlo{sel={call,F,Vab}};
 update_sel_all(#dlo{src_we=#we{fs=Ftab}}=D) ->
     %% No vertex arrays to re-use. Rebuild from scratch.
     update_face_sel(gb_trees:keys(Ftab), D).
 
-update_face_sel(Fs0, #dlo{src_we=We,vab=#vab{face_vs=Vs,face_map=Map}}=D)
+update_face_sel(Fs0, #dlo{src_we=We,vab=#vab{face_vs=Vs,face_map=Map}=Vab}=D)
   when Vs =/= none ->
     Fs = wings_we:visible(Fs0, We),
-    List = gl:genLists(1),
-    gl:newList(List, ?GL_COMPILE),
-    wings_draw_setup:enableVertexPointer(Vs),
-    Draw = fun(Face) ->
-		   {Start,NoElements} = array:get(Face, Map),
-		   gl:drawArrays(?GL_TRIANGLES, Start, NoElements)
-	   end,
-    [Draw(Face) || Face <- Fs],
-    wings_draw_setup:disableVertexPointer(Vs),
-    gl:endList(),
-    D#dlo{sel=List};
+    F = case wings_util:min_wx({1,8}) of
+            true ->
+                Collect = fun(Face, {Ss,Es}) ->
+                                  {Start,NoElements} = array:get(Face, Map),
+                                  {<<Ss/binary, Start:?UI32>>, <<Es/binary, NoElements:?UI32>>}
+                          end,
+                {Start,NoElements} = lists:foldl(Collect, {<<>>,<<>>}, lists:reverse(Fs)),
+                fun() ->
+                        wings_draw_setup:enable_pointers(Vab, []),
+                        gl:multiDrawArrays(?GL_TRIANGLES, Start, NoElements),
+                        wings_draw_setup:disable_pointers(Vab, [])
+                end;
+            false ->
+                SN = [array:get(Face, Map) || Face <- Fs],
+                fun() ->
+                        wings_draw_setup:enable_pointers(Vab, []),
+                        [gl:drawArrays(?GL_TRIANGLES, S, N) || {S,N} <- SN],
+                        wings_draw_setup:disable_pointers(Vab, [])
+                end
+        end,
+    Sel = {call,F,Vab},
+    D#dlo{sel=Sel};
 update_face_sel(Fs0, #dlo{src_we=We}=D) ->
     Fs = wings_we:visible(Fs0, We),
-    List = gl:genLists(1),
-    gl:newList(List, ?GL_COMPILE),     
-    BinFs = update_face_sel_1(Fs, D, <<>>),
-    gl:enableClientState(?GL_VERTEX_ARRAY),
-    drawVertices(?GL_TRIANGLES, BinFs),
-    gl:disableClientState(?GL_VERTEX_ARRAY),
-    gl:endList(),
-    D#dlo{sel=List}.
+    Data = update_face_sel_1(Fs, D, <<>>),
+    Sel = vbo_draw_arrays(?GL_TRIANGLES, Data),
+    D#dlo{sel=Sel}.
 
 update_face_sel_1(Fs, #dlo{ns=none,src_we=We}, Bin) ->
     update_face_sel_2(Fs, We, Bin);
@@ -599,8 +582,7 @@ unlit_plain_face_1([], _, Bin) -> Bin.
 %%% Splitting of objects into two display lists.
 %%%
 
-split(#dlo{ns={Ns0},src_we=#we{fs=Ftab}}=D, Vs, St) ->
-    Ns = remove_stale_ns(Ns0, Ftab),
+split(#dlo{ns={Ns}}=D, Vs, St) ->
     split_1(D#dlo{ns=Ns}, Vs, St);
 split(D, Vs, St) -> split_1(D, Vs, St).
 
@@ -617,13 +599,14 @@ split_2(#dlo{mirror=M,src_sel=Sel,src_we=#we{fs=Ftab}=We,
     StaticVs = static_vs(Faces, Vs, We),
 
     VisFtab = wings_we:visible(gb_trees:to_list(Ftab), We),
-    {Work,#dlo{ns=Ns},FtabDyn} = split_faces(D, VisFtab, Faces, St),
+    {Work,#dlo{ns=Ns},FtabStatic,FtabDyn} =
+	split_faces(D, VisFtab, Faces, St),
 
     %% To support commands that create new objects with static
     %% geometry (such as Shell Extrude), we must be sure to pass
     %% on the new normals table or the static edges will not be
     %% visible.
-    StaticEdgeDl = make_static_edges(Faces, D#dlo{ns=Ns}),
+    StaticEdgeDl = make_static_edges(FtabStatic, D#dlo{ns=Ns}),
     {DynVs,VsDlist} = split_vs_dlist(Vs, StaticVs, Sel, We),
 
     WeDyn = wings_facemat:gc(We#we{fs=gb_trees:from_orddict(FtabDyn)}),
@@ -640,16 +623,6 @@ split_2(#dlo{mirror=M,src_sel=Sel,src_we=#we{fs=Ftab}=We,
 	 proxy=UsesProxy, proxy_data=Pd,
 	 needed=Needed,open=Open}.
 
-remove_stale_ns(none, _) -> none;
-remove_stale_ns(Ns, Ftab) ->
-    Deleted = array:default(Ns),
-    array:sparse_map(fun(Face, Term) ->
-			     case gb_trees:is_defined(Face, Ftab) of
-				 true  -> Term;
-				 false -> Deleted
-			     end
-		     end, Ns).
-
 static_vs(Fs, Vs, We) ->
     VsSet = gb_sets:from_ordset(Vs),
     Fun = fun(V, _, _, A) ->
@@ -667,18 +640,17 @@ static_vs_1([], _, _, Acc) -> ordsets:from_list(Acc).
 split_faces(#dlo{needed=Need}=D0, Ftab0, Fs0, St) ->
     Ftab = sofs:from_external(Ftab0, [{face,data}]),
     Fs = sofs:from_external(Fs0, [face]),
+    {DynFtab0,StaticFtab0} = sofs:partition(1, Ftab, Fs),
+    DynFtab = sofs:to_external(DynFtab0),
+    StaticFtab = sofs:to_external(StaticFtab0),
     case member(work, Need) orelse member(smooth, Need) of
 	false ->
 	    %% This is wireframe mode. We don't need any
 	    %% 'work' display list for faces.
-	    FtabDyn = sofs:to_external(sofs:restriction(Ftab, Fs)),
-	    {none,D0,FtabDyn};
+	    {none,D0,StaticFtab,DynFtab};
 	true ->
 	    %% Faces needed. (Either workmode or smooth mode.)
-	    {FtabDyn0,StaticFtab0} = sofs:partition(1, Ftab, Fs),
-	    FtabDyn = sofs:to_external(FtabDyn0),
-	    StaticFtab = sofs:to_external(StaticFtab0),
-
+	    %%
 	    %% Make sure that every static face has an entry
 	    %% in the normals array. Most of the time, this is
 	    %% not needed because newly created faces are
@@ -690,7 +662,7 @@ split_faces(#dlo{needed=Need}=D0, Ftab0, Fs0, St) ->
 	    StaticPlan = wings_draw_setup:prepare(StaticFtab, D1, St),
 	    D = wings_draw_setup:flat_faces(StaticPlan, D1),
 	    Dl = draw_flat_faces(D, St),
-	    {[Dl],D1,FtabDyn}
+	    {[Dl],D1,StaticFtab,DynFtab}
     end.
 
 split_new_normals(Ftab, #dlo{ns=Ns0,src_we=We}=D) ->
@@ -714,22 +686,10 @@ split_new_normals([{Face,Edge}|T], We, Ns0) ->
     end;
 split_new_normals([], _, Ns) -> Ns.
 
-make_static_edges(DynFaces, #dlo{ns=none}) ->
-    make_static_edges_1(DynFaces, [], []);
-make_static_edges(DynFaces, #dlo{ns=Ns}) ->
-    make_static_edges_1(DynFaces, array:sparse_to_orddict(Ns), []).
-
-make_static_edges_1([F|Fs], [{F,_}|Ns], Acc) ->
-    make_static_edges_1(Fs, Ns, Acc);
-make_static_edges_1([_|_]=Fs, [{_,N}|Ns], Acc) ->
-    make_static_edges_1(Fs, Ns, [N|Acc]);
-make_static_edges_1(_, Ns, Acc) ->
-    make_static_edges_2(Ns, Acc).
-
-make_static_edges_2([{_,N}|Ns], Acc) ->
-    make_static_edges_2(Ns, [N|Acc]);
-make_static_edges_2([], Acc) ->
-    make_edge_dl(Acc).
+make_static_edges(_Ftab, #dlo{ns=none}) ->
+    make_edge_dl([]);
+make_static_edges(Ftab, #dlo{ns=Ns}) ->
+    make_edge_dl([array:get(F, Ns) || {F,_} <- Ftab]).
 
 insert_vtx_data([V|Vs], Vtab, Acc) ->
     insert_vtx_data(Vs, Vtab, [{V,array:get(V, Vtab)}|Acc]);
@@ -746,19 +706,14 @@ split_vs_dlist(Vs, StaticVs, {vertex,SelVs0}, #we{vp=Vtab}=We) ->
 			    true -> DynVs
 			end,
 	    UnselDyn = sofs:to_external(UnselDyn0),
-	    UnselDlist = gl:genLists(1),
-	    gl:newList(UnselDlist, ?GL_COMPILE),
 	    List0 = sofs:from_external(array:sparse_to_orddict(Vtab),
 				       [{vertex,info}]),
 	    List1 = sofs:drestriction(List0, DynVs),
 	    List2 = sofs:to_external(List1),
 	    List = wings_we:visible_vs(List2, We),
-	    VisibleVs = << <<X:?F32,Y:?F32,Z:?F32>> || {_,{X,Y,Z}} <- List >>,
-	    gl:enableClientState(?GL_VERTEX_ARRAY),
-	    drawVertices(?GL_POINTS, VisibleVs),
-	    gl:disableClientState(?GL_VERTEX_ARRAY),
-	    gl:endList(),
-	    {UnselDyn,[UnselDlist]}
+	    Data = << <<X:?F32,Y:?F32,Z:?F32>> || {_,{X,Y,Z}} <- List >>,
+	    Unsel = vbo_draw_arrays(?GL_POINTS, Data),
+	    {UnselDyn,[Unsel]}
     end;
 split_vs_dlist(_, _, _, _) -> {none,none}.
     
@@ -808,17 +763,12 @@ dynamic_edges(#dlo{edges=[StaticEdge|_],ns=Ns}=D) ->
 dynamic_vs(#dlo{split=#split{dyn_vs=none}}=D) -> D;
 dynamic_vs(#dlo{src_we=#we{vp=Vtab},vs=[Static|_],
 		split=#split{dyn_vs=DynVs}}=D) ->
-    UnselDlist = gl:genLists(1),
-    gl:newList(UnselDlist, ?GL_COMPILE),
-    DynVsBin = foldl(fun(V, Bin) ->
-			     {X1,Y1,Z1} = array:get(V, Vtab),
-			     <<Bin/binary, X1:?F32,Y1:?F32,Z1:?F32>>
-		      end, <<>>, DynVs),
-    gl:enableClientState(?GL_VERTEX_ARRAY),
-    drawVertices(?GL_POINTS, DynVsBin),
-    gl:disableClientState(?GL_VERTEX_ARRAY),
-    gl:endList(),
-    D#dlo{vs=[Static,UnselDlist]}.
+    Data = foldl(fun(V, Bin) ->
+			 {X1,Y1,Z1} = array:get(V, Vtab),
+			 <<Bin/binary, X1:?F32,Y1:?F32,Z1:?F32>>
+		 end, <<>>, DynVs),
+    Unsel = vbo_draw_arrays(?GL_POINTS, Data),
+    D#dlo{vs=[Static,Unsel]}.
 
 % added by Micheus
 dynamic_influence(#dlo{src_we=#we{pst=Pst},split=#split{orig_st=St}}=D) ->
@@ -910,25 +860,9 @@ tricky_share({X,Y,Z}, {_,_,Z}=Old) ->
 
 draw_flat_faces(#dlo{vab=Vab}, St) ->
     draw_flat_faces(Vab, St);
-draw_flat_faces(#vab{face_vs=BinVs,face_fn=Ns,face_uv=UV,face_ts=TS,
-		     face_vc=Col,mat_map=MatMap}=D,
-	       #st{mat=Mtab}) ->
-    wings_draw_setup:enableVertexPointer(BinVs),
-    wings_draw_setup:enableNormalPointer(Ns),
-    ActiveColor = wings_draw_setup:enableColorPointer(Col),
-    wings_draw_setup:enableTexCoordPointer(UV),
-    wings_draw_setup:enableTangentCoordPointer(TS),
-    Dl = gl:genLists(1),
-    gl:newList(Dl, ?GL_COMPILE),
-    draw_mat_faces(MatMap, Mtab, ActiveColor),
-    gl:endList(),
-    wings_draw_setup:disableVertexPointer(BinVs),
-    wings_draw_setup:disableNormalPointer(Ns),
-    wings_draw_setup:disableColorPointer(Col),
-    wings_draw_setup:disableTexCoordPointer(UV),
-    wings_draw_setup:disableTangentCoordPointer(TS),
-    wings:keep(D),
-    Dl.
+draw_flat_faces(#vab{mat_map=MatMap}=Vab, #st{mat=Mtab}) ->
+    Extra = [colors,face_normals,uvs,tangents],
+    draw_mat_faces(Vab, Extra, MatMap, Mtab).
 
 %%%
 %%% Smooth drawing.
@@ -937,14 +871,8 @@ draw_flat_faces(#vab{face_vs=BinVs,face_fn=Ns,face_uv=UV,face_ts=TS,
 draw_smooth_faces(#dlo{vab=Vab},St) ->
     draw_smooth_faces(Vab,St);
 
-draw_smooth_faces(#vab{face_vs=BinVs,face_sn=Ns,face_uv=UV,face_ts=TS,
-		       face_vc=Col,mat_map=MatMap}=D,
-		  #st{mat=Mtab}) ->
-    wings_draw_setup:enableVertexPointer(BinVs),
-    wings_draw_setup:enableNormalPointer(Ns),
-    ActiveColor = wings_draw_setup:enableColorPointer(Col),
-    wings_draw_setup:enableTexCoordPointer(UV),
-    wings_draw_setup:enableTangentCoordPointer(TS),
+draw_smooth_faces(#vab{mat_map=MatMap}=Vab, #st{mat=Mtab}) ->
+    Extra = [colors,vertex_normals,uvs,tangents],
 
     %% Partition into transparent and solid material face groups.
     {Transparent,Solid} =
@@ -953,91 +881,71 @@ draw_smooth_faces(#vab{face_vs=BinVs,face_sn=Ns,face_uv=UV,face_ts=TS,
 			end, MatMap),
 
     %% Create display list for solid faces.
-    ListOp = gl:genLists(1),
-    gl:newList(ListOp, ?GL_COMPILE),
-    draw_mat_faces(Solid, Mtab, ActiveColor),
-    gl:endList(),
+    DrawSolid = draw_mat_faces(Vab, Extra, Solid, Mtab),
 
     %% Create display list for transparent faces if there are
     %% any transparent faces.
-    Res = case Transparent of
-	      [] ->
-		  %% All faces are solid.
-		  {[ListOp,none],false};
-	      _ ->
-		  %% Create the display list for the transparent faces.
-		  ListTr = gl:genLists(1),
-		  gl:newList(ListTr, ?GL_COMPILE),
-		  draw_mat_faces(Transparent, Mtab, ActiveColor),
-		  gl:endList(),
-		  {[ListOp,ListTr],true}
-	  end,
-
-    wings_draw_setup:disableVertexPointer(BinVs),
-    wings_draw_setup:disableNormalPointer(Ns),
-    wings_draw_setup:disableColorPointer(Col),
-    wings_draw_setup:disableTexCoordPointer(UV),
-    wings_draw_setup:disableTangentCoordPointer(TS),
-    wings:keep(D),
-    Res.
-
-draw_mat_faces(MatGroups, Mtab, ActiveColor) ->
-    case wings_pref:get_value(show_materials) of
-	false ->
-	    %% Showing of materials has been turned off. Use
-	    %% the 'default' material for all faces.
-	    gl:pushAttrib(?GL_TEXTURE_BIT),
-	    DeApply = wings_material:apply_material(default, Mtab,
-						    ActiveColor),
-	    foreach(
-	      fun({_,Type,Start,NumElements}) ->
-		      gl:drawArrays(Type, Start, NumElements)
-	      end, MatGroups),
-	    DeApply(),
-	    gl:popAttrib();
-	true ->
-	    %% Setup shader for materials 
-	    Progs = get(light_shaders),
-	    UseSceneLights = wings_pref:get_value(scene_lights) andalso
-		wings_light:any_enabled_lights(),
-	    UseShaders = Progs =/= undefined andalso not UseSceneLights 
-		andalso wings_pref:get_value(number_of_lights) =:= 2,
-	    case UseShaders of
-	    	false -> ignore;
-	    	_ -> 
-	    	    NumShaders = wings_pref:get_value(active_shader),
-	    	    {Prog,_Name} = element(NumShaders, Progs),
-	    	    put(active_shader, Prog),
-		    ok
-	    end,
-	    %% Show materials.
-	    foreach(
-	      fun({Mat,Type,Start,NumElements}) ->
-		      gl:pushAttrib(?GL_TEXTURE_BIT),
-		      DeApply = wings_material:apply_material(Mat, Mtab,
-							      ActiveColor),
-		      gl:drawArrays(Type, Start, NumElements),
-		      DeApply(),
-		      gl:popAttrib()
-	      end, MatGroups),
-	    put(active_shader, 0),
-	    ok
+    case Transparent of
+	[] ->
+	    %% All faces are solid.
+	    {[DrawSolid,none],false};
+	_ ->
+	    %% Create the display list for the transparent faces.
+	    DrawTr = draw_mat_faces(Vab, Extra, Transparent, Mtab),
+	    {[DrawSolid,DrawTr],true}
     end.
 
+draw_mat_faces(Vab, Extra, MatGroups, Mtab) ->
+    ActiveColor = wings_draw_setup:has_active_color(Vab),
+    D = fun() ->
+		wings_draw_setup:enable_pointers(Vab, Extra),
+		do_draw_mat_faces(MatGroups, Mtab, ActiveColor),
+		wings_draw_setup:disable_pointers(Vab, Extra)
+	end,
+    {call,D,Vab}.
+
+do_draw_mat_faces(MatGroups, Mtab, ActiveColor) ->
+    %% Setup shader for materials
+    Progs = get(light_shaders),
+    UseSceneLights = wings_pref:get_value(scene_lights) andalso
+	wings_light:any_enabled_lights(),
+    UseShaders = Progs =/= undefined andalso not UseSceneLights
+	andalso wings_pref:get_value(number_of_lights) =:= 2,
+    case UseShaders of
+	false -> ignore;
+	_ ->
+	    NumShaders = wings_pref:get_value(active_shader),
+	    {Prog,_Name} = element(NumShaders, Progs),
+	    put(active_shader, Prog),
+	    ok
+    end,
+    %% Show materials.
+    foreach(
+      fun({Mat,Type,Start,NumElements}) ->
+	      gl:pushAttrib(?GL_TEXTURE_BIT),
+	      DeApply = wings_material:apply_material(Mat, Mtab,
+						      ActiveColor),
+	      gl:drawArrays(Type, Start, NumElements),
+	      DeApply(),
+	      gl:popAttrib()
+      end, MatGroups),
+    put(active_shader, 0),
+    ok.
+
 %%
 %% Draw normals for the selected elements.
 %%
 
 make_normals_dlist(#dlo{normals=none,src_we=We,src_sel={Mode,Elems}}=D) ->
-    List = gl:genLists(1),
-    gl:newList(List, ?GL_COMPILE),
-    gl:color3fv(wings_pref:get_value(normal_vector_color)),
     Normals = make_normals_dlist_1(Mode, Elems, We),
-    gl:enableClientState(?GL_VERTEX_ARRAY),
-    drawVertices(?GL_LINES, Normals),
-    gl:disableClientState(?GL_VERTEX_ARRAY),
-    gl:endList(),
-    D#dlo{normals=List};
+    VectorColor = wings_pref:get_value(normal_vector_color),
+    N = byte_size(Normals) div 12,
+    Draw0 = fun() ->
+		    gl:color3fv(VectorColor),
+		    gl:drawArrays(?GL_LINES, 0, N)
+	    end,
+    Draw = wings_vbo:new(Draw0, Normals),
+    D#dlo{normals=Draw};
 make_normals_dlist(#dlo{src_sel=none}=D) -> D#dlo{normals=none};
 make_normals_dlist(D) -> D.
 
@@ -1076,4 +984,3 @@ make_normals_dlist_1(face, Faces, We) ->
 	    end, <<>>, wings_we:visible(gb_sets:to_list(Faces), We));
 make_normals_dlist_1(body, _, #we{fs=Ftab}=We) ->
     make_normals_dlist_1(face, gb_sets:from_list(gb_trees:keys(Ftab)), We).
-
