@@ -39,21 +39,24 @@ menu(X, Y, St) ->
 	    wings_menu_util:flatten(),
 	    separator,
 	    cut_line(St),
-	    {?__(6,"Connect"),connect,
-	     ?__(7,"Create a new edge by connecting midpoints of selected edges")},
+	    {?__(6,"Connect"),connect(),
+	     	{?__(7,"Create a new edge by connecting midpoints of selected edges"), [],
+		 ?__(25,"Create a new edge and slides it along neighbor edges")},[]},
 	    {?__(8,"Bevel"),bevel,
 	     ?__(9,"Round off selected edges")},
 	    separator,
 	    {?__(10,"Dissolve"), dslv(),
-		 {?__(11,"Eliminate selected edges"), [],
-		  ?__(50,"Eliminate selected edges and remove remaining isolated verts")},[]},
-		{?__(12,"Collapse"),collapse_fun(),
-	     {?__(13,"Delete edges, replacing them with vertices"), [],
-		  ?__(21,"Delete edges, replacing them with vertices and remove any newly created isolated vertices")},[]},
+		{?__(11,"Eliminate selected edges"), [],
+		 ?__(50,"Eliminate selected edges and remove remaining isolated verts")},[]},
+	    {?__(12,"Collapse"),collapse_fun(),
+	     	{?__(13,"Delete edges, replacing them with vertices"), [],
+		 ?__(21,"Delete edges, replacing them with vertices and remove any newly created isolated vertices")},[]},
 	    separator,
 	    {?__(14,"Hardness"),
 	     {hardness,[{?__(15,"Soft"),soft},
-			{?__(16,"Hard"),hard}]}},
+			        {?__(16,"Hard"),hard},
+			        {?__(24,"Invert"),invert}
+			        ]}},
 	    separator,
 	    {?__(17,"Loop Cut"),loop_cut,
 	     ?__(18,"Cut into two objects along edge loop")},
@@ -62,6 +65,12 @@ menu(X, Y, St) ->
 	     ?__(20,"Apply vertex colors to selected edges")}],
     wings_menu:popup_menu(X, Y, edge, Menu).
 
+connect() ->
+    fun
+	(1, _Ns) -> {edge,connect};
+	(3, _Ns) -> {edge,connect_slide};
+	(_, _) -> ignore
+    end.
 dslv() ->
     fun
 	(1, _Ns) -> {edge,dissolve};
@@ -127,13 +136,15 @@ command(slide, St) ->
 command(cut_pick, St) ->
     cut_pick(St);
 command({cut,ask}, St) ->
-    wings_ask:ask(cut_command(),
-		  [{?__(1,"Segments"), 2}],
-		  fun([Ret]) -> cut(Ret, St) end);
+    wings_dialog:ask(cut_command(),
+		     [{?__(1,"Segments"), 2, []}],
+		     fun([Ret]) -> cut(Ret, St) end);
 command({cut,Num}, St) ->
     {save_state,cut(Num, St)};
 command(connect, St) ->
     {save_state,connect(St)};
+command(connect_slide, St) ->
+    connect_slide(St);
 command(clean_dissolve, St) ->
     {save_state,wings_shape:update_folders(clean_dissolve(St))};
 command(dissolve, St) ->
@@ -178,6 +189,11 @@ connect(Es0, #we{id=Id}=We0, Acc) ->
     We = wings_edge:dissolve_isolated_vs(Vs, We2),
     {We,[{Id,Sel}|Acc]}.
 
+connect_slide(St1) ->
+    {St0,Sel} = wings_sel:mapfold(fun connect/3, [], St1),
+    St=wings_sel:set(Sel, St0),
+    slide(St).
+
 cut_edges(Es, We) ->
     mapfoldl(fun(Edge, W0) ->
 		     {W,V} = wings_edge:cut(Edge, 2, W0),
@@ -249,6 +265,7 @@ cut_pick(St) ->
     Flags = [{initial,[0]}],
     wings_drag:setup(Tvs, Units, Flags, wings_sel:set(vertex, Sel, St)).
 
+-spec cut_pick_error() -> no_return().
 cut_pick_error() ->
     wings_u:error_msg(?__(1,"Only one edge can be cut at an arbitrary position.")).
 
@@ -257,7 +274,7 @@ cut_pick_make_tvs(Edge, #we{id=Id,es=Etab,vp=Vtab,next_id=NewV}=We) ->
     Start = array:get(Va, Vtab),
     End = array:get(Vb, Vtab),
     Dir = e3d_vec:sub(End, Start),
-    Char = {7,7,3,3,7,0,
+    Char = {7,7,3.0,3.0,7.0,0.0,
 	    <<2#01111100,
 	     2#10000010,
 	     2#10000010,
@@ -273,29 +290,28 @@ cut_pick_marker([I], D, Edge, We0, Start, Dir, Char) ->
     {X,Y,Z} = Pos = e3d_vec:add_prod(Start, Dir, I),
     {MM,PM,ViewPort} = wings_u:get_matrices(0, original),
     {Sx,Sy,_} = wings_gl:project(X, Y, Z, MM, PM, ViewPort),
-    Dl = gl:genLists(1),
-    gl:newList(Dl, ?GL_COMPILE),
-    gl:pushAttrib(?GL_ALL_ATTRIB_BITS),
-    gl:color3f(1, 0, 0),
-    gl:shadeModel(?GL_FLAT),
-    gl:disable(?GL_DEPTH_TEST),
-    gl:matrixMode(?GL_PROJECTION),
-    gl:pushMatrix(),
-    gl:loadIdentity(),
-    {W,H} = wings_wm:win_size(),
-    glu:ortho2D(0, W, 0, H),
-    gl:matrixMode(?GL_MODELVIEW),
-    gl:pushMatrix(),
-    gl:loadIdentity(),
-    gl:rasterPos2f(Sx, Sy),
-    wings_io:draw_char(Char),
-    gl:popMatrix(),
-    gl:matrixMode(?GL_PROJECTION),
-    gl:popMatrix(),
-    gl:popAttrib(),
-    gl:endList(),
+    Draw = fun() ->
+		   gl:pushAttrib(?GL_ALL_ATTRIB_BITS),
+		   gl:color3f(1.0, 0.0, 0.0),
+		   gl:shadeModel(?GL_FLAT),
+		   gl:disable(?GL_DEPTH_TEST),
+		   gl:matrixMode(?GL_PROJECTION),
+		   gl:pushMatrix(),
+		   gl:loadIdentity(),
+		   {W,H} = wings_wm:win_size(),
+		   glu:ortho2D(0.0, W, 0.0, H),
+		   gl:matrixMode(?GL_MODELVIEW),
+		   gl:pushMatrix(),
+		   gl:loadIdentity(),
+		   gl:rasterPos2f(Sx, Sy),
+		   wings_io:draw_bitmap(Char),
+		   gl:popMatrix(),
+		   gl:matrixMode(?GL_PROJECTION),
+		   gl:popMatrix(),
+		   gl:popAttrib()
+	   end,
     {We,_} = wings_edge:fast_cut(Edge, Pos, We0),
-    D#dlo{hilite={call_in_this_win,wings_wm:this(),Dl},src_we=We};
+    D#dlo{hilite={edge, {call_in_this_win,wings_wm:this(),Draw}},src_we=We};
 cut_pick_marker({finish,[I]}, D0, Edge, We, Start, Dir, Char) ->
     D = cut_pick_marker([I], D0, Edge, We, Start, Dir, Char),
     D#dlo{vs=none,hilite=none}.
@@ -342,6 +358,13 @@ dissolve(St0) ->
 %%% The Hardness command.
 %%%
 
+hardness(invert, St) ->
+    wings_sel:map(fun(Edges, #we{he=Htab0}=We) ->
+			  WereSoft = gb_sets:subtract(Edges,Htab0),
+			  Outside = gb_sets:subtract(Htab0,Edges),
+			  NowHard = gb_sets:union(WereSoft,Outside),
+			  We#we{he=NowHard}
+		  end, St);
 hardness(soft, St) ->
     wings_sel:map(fun(Edges, #we{he=Htab0}=We) ->
 			  Htab = gb_sets:difference(Htab0, Edges),
