@@ -18,9 +18,9 @@
 
 %% Useful functions.
 -export([mirror_faces/2,set_color/2,force_bridge/5,
-	 extrude_faces/1,extrude_region/1,extract_region/1]).
+	 extrude_faces/1,extrude_region/1,extract_region/1,shell_extrude/1]).
 	 % not currently used outside this module (but could be)
-	 % extract_faces/1, shell_extrude/1
+	 % extract_faces/1
 
 -include("wings.hrl").
 -import(lists, [foldl/3,reverse/1,sort/1,member/2]).
@@ -41,9 +41,10 @@ menu(X, Y, St) ->
 	     ?__(9,"Carve out interior of object, making selected faces holes")},
 	    {?__(10,"Bevel"),bevel,
 	     ?__(11,"Round off edges of selected faces")},
-	    {?__(12,"Bridge"),bridge,
-	     ?__(13,"Create a bridge or tunnel between two faces") ++
-	     ?__(40," or face regions")},
+	    {?__(12,"Bridge"),bridge_fun(),
+	     {?__(13,"Create a bridge or tunnel between two faces") ++
+	      ?__(40," or face regions"),[],
+	      ?__(43,"Create a bridge or tunnel from reference vertexes")},[]},
 	    separator,
 	    {?__(14,"Bump"),bump,
 	     ?__(15,"Create bump of selected faces")},
@@ -57,11 +58,12 @@ menu(X, Y, St) ->
 	      ?__(22,"Mirror and create separate objects")},[]},
 	    {?__(23,"Dissolve"),dissolve_fun(),
 	     {?__(24,"Eliminate all edges between selected faces"),[],
-		  ?__(34,"Eliminate selected faces and remove remaining isolated verts")},[]},
+	      ?__(34,"Eliminate selected faces and remove remaining isolated verts")},[]},
 	    {?__(25,"Collapse"),collapse,
 	     ?__(26,"Delete faces, replacing them with vertices")},
-	    {?__(27,"Smooth"),smooth,
-	     ?__(28,"Subdivide selected faces to smooth them (Catmull-Clark)")},
+	    {?__(27,"Subdivide"),subdiv_fun(),
+	     {?__(28,"Subdivide selected faces to smooth them (Catmull-Clark)"),[],
+	      ?__(42,"Subdivide selected faces")},[]},
 	    {?__(29,"Tesselate"),{tesselate,wings_tesselation:submenu()}},
 	    separator,
 	    {?__(32,"Hide"),hide_fun(),
@@ -83,6 +85,20 @@ dissolve_fun() ->
 	(_, _) -> ignore
     end.
 
+subdiv_fun() ->
+    fun
+	(1, _Ns) -> {face,smooth};
+	(3, _Ns) -> {face,subdiv};
+	(_, _) -> ignore
+end.
+
+bridge_fun() ->
+    fun
+	(1, _Ns) -> {face,bridge};
+	(3, _Ns) -> {face,{bridge,reference}};
+	(_, _) -> ignore
+    end.
+
 lift_fun(St) ->
     fun(help, _Ns) ->
 	    {?__(1,"Lift, rotating face around edge or vertex"),[],
@@ -146,9 +162,13 @@ command(dissolve, St) ->
 command(clean_dissolve, St) ->
     {save_state,wings_shape:update_folders(clean_dissolve(St))};
 command(bridge, St) ->
-    {save_state,wings_shape:recreate_folder_system(bridge(St))};
+    {save_state,wings_shape:recreate_folder_system(bridge([],St))};
+command({bridge,reference}, St) ->
+    bridge_ref(St);
 command(smooth, St) ->
     ?SLOW({save_state,smooth(St)});
+command(subdiv, St) ->
+    ?SLOW({save_state,subdiv(St)});
 command(auto_smooth, St) ->
     wings_body:auto_smooth(St);
 command({lift,Lift}, St) ->
@@ -613,6 +633,33 @@ do_flatten_normal(Faces, Center, We) ->
     Vs = wings_face:to_vertices(Faces, We),
     wings_vertex:flatten(Vs, N, Center, We).
 
+%%%
+%%% The Subdivide command.
+%%%
+
+subdiv(St0) ->
+    {St,Sel} = wings_sel:mapfold(fun subdiv/3, [], St0),
+    wings_sel:set(Sel, St).
+
+subdiv(Faces0, #we{id=Id}=We0, Acc) ->
+    Rs = wings_sel:face_regions(Faces0, We0),
+    wings_pb:start(?__(1,"subdividing")),
+    We1 = wings_pb:done(subdiv_regions(Rs, 1, length(Rs), We0)),
+    NewSelFaces = wings_we:new_items_as_ordset(face, We0, We1),
+    We = wings_we:mirror_flatten(We0, We1),
+    {We,[{Id,gb_sets:from_ordset(NewSelFaces)}|Acc]}.
+
+subdiv_regions([Faces0|Rs], I, N, #we{he=Htab}=We0) ->
+    wings_pb:update(I/N, io_lib:format("~p/~p\n", [I,N])),
+    HardEdges0 = wings_face:outer_edges(Faces0, We0),
+    HardEdges = gb_sets:union(gb_sets:from_list(HardEdges0), Htab),
+    Faces = gb_sets:to_list(Faces0),
+    {Vs,Es} = all_edges(Faces0, We0),
+    We = wings_subdiv:subdiv(Faces, Vs, Es, HardEdges, We0),
+    subdiv_regions(Rs, I+1, N, We);
+subdiv_regions([], _, _, We) -> We.
+
+
 %%%
 %%% The Smooth command.
 %%%
@@ -704,7 +751,7 @@ smooth_connect_3(Va, Vb, Face, Hide, We0) ->
 %%% The Bridge command.
 %%%
 
-bridge(#st{shapes=Shapes0,sel=[{IdA,FacesA},{IdB,FacesB}]}=St0) ->
+bridge(Reference, #st{shapes=Shapes0,sel=[{IdA,FacesA},{IdB,FacesB}]}=St0) ->
     case {gb_sets:to_list(FacesA),gb_sets:to_list(FacesB)} of
 	{[FA],[FB0]} ->
 	    #we{next_id=Id}=WeA = wings_we:break_mirror(gb_trees:get(IdA, Shapes0)),
@@ -716,7 +763,15 @@ bridge(#st{shapes=Shapes0,sel=[{IdA,FacesA},{IdB,FacesB}]}=St0) ->
 	    Sel = [{IdA,gb_sets:from_list([FA,FB])}],
 	    St1 = wings_sel:set(Sel, St0),
 	    St = St1#st{shapes=Shapes},
-	    bridge(St);
+	    Reference0 =
+		case Reference of
+		    [{IdA,VrA},{IdB,VrB}] ->
+			[gb_sets:smallest(VrA),new_vertex_id(gb_sets:smallest(VrB),wings_face:vertices_ccw(FB0,WeB0),wings_face:vertices_ccw(FB,We))];
+		    [{IdB,VrB},{IdA,VrA}] ->
+			[gb_sets:smallest(VrB),new_vertex_id(gb_sets:smallest(VrA),wings_face:vertices_ccw(FA,We),wings_face:vertices_ccw(FB,We))];
+		    _ -> []
+		end,
+	    bridge(Reference0, St);
 	{[_],FB0} ->
 	    WeB0 = gb_trees:get(IdB, Shapes0),
 	    case wings_sel:face_regions(FB0, WeB0) of
@@ -725,7 +780,7 @@ bridge(#st{shapes=Shapes0,sel=[{IdA,FacesA},{IdB,FacesB}]}=St0) ->
 	        Face = wings_we:new_items_as_gbset(face, WeB0, WeB),
 	        Shapes = gb_trees:update(IdB, WeB, Shapes0),
 	        St = St0#st{shapes=Shapes,sel=[{IdA,FacesA},{IdB,Face}]},
-	        bridge(St);
+	        bridge(Reference, St);
 	      _ -> bridge_error()
 	    end;
 	{FA0,[_]} ->
@@ -736,7 +791,7 @@ bridge(#st{shapes=Shapes0,sel=[{IdA,FacesA},{IdB,FacesB}]}=St0) ->
 	        Face = wings_we:new_items_as_gbset(face, WeA0, WeA),
 	        Shapes = gb_trees:update(IdA, WeA, Shapes0),
 	        St = St0#st{shapes=Shapes,sel=[{IdA,Face},{IdB,FacesB}]},
-	        bridge(St);
+	        bridge(Reference, St);
 	      _ -> bridge_error()
 	    end;
 	{FA0,FB0} ->
@@ -753,17 +808,17 @@ bridge(#st{shapes=Shapes0,sel=[{IdA,FacesA},{IdB,FacesB}]}=St0) ->
 	            FaceA = wings_we:new_items_as_gbset(face, WeA0, WeA),
 	            Shapes = gb_trees:update(IdA, WeA, Shapes1),
 	            St = St0#st{shapes=Shapes,sel=[{IdA,FaceA},{IdB,FaceB}]},
-	            bridge(St);
+	            bridge(Reference, St);
 	          _ -> bridge_error()
 	        end;
 	      _ -> bridge_error()
 	    end
     end;
-bridge(#st{shapes=Shapes0,sel=[{Id,Faces0}]}=St0) ->
+bridge(Reference, #st{shapes=Shapes0,sel=[{Id,Faces0}]}=St0) ->
     case gb_sets:to_list(Faces0) of
 	[FA,FB] ->
 	    We0 = gb_trees:get(Id, Shapes0),
-	    We = bridge_0(FA, FB, We0),
+	    We = bridge_0(Reference, FA, FB, We0),
 	    Shapes = gb_trees:update(Id, We, Shapes0),
 	    St0#st{shapes=Shapes,sel=[]};
 	FaceSel ->
@@ -774,21 +829,125 @@ bridge(#st{shapes=Shapes0,sel=[{Id,Faces0}]}=St0) ->
 	        Faces = wings_we:new_items_as_gbset(face, We0, We),
 	        Shapes = gb_trees:update(Id, We, Shapes0),
 	        St = St0#st{shapes=Shapes,sel=[{Id,Faces}]},
-	        bridge(St);
+	        bridge(Reference, St);
 	      _ -> bridge_error()
 	    end
     end;
-bridge(_St) ->
+bridge(_, _St) ->
     bridge_error().
 
-bridge_0(FaceA, FaceB, We0) ->
+bridge_ref(#st{shapes=Shps, sel=[{IdA,FsA},{IdB,FsB}]}=St) ->
+    WeA = gb_trees:get(IdA,Shps),
+    WeB = gb_trees:get(IdB,Shps),
+    VsA = wings_face:to_vertices(FsA, WeA),
+    VsB = wings_face:to_vertices(FsB, WeB),
+    wings:ask(bridge_selection({IdA,VsA}, {IdB,VsB}), St, fun bridge_ref/2);
+bridge_ref(#st{shapes=Shps,sel=[{Id,Faces0}]}=St) ->
+    case gb_sets:to_list(Faces0) of
+	[FsA,FsB] ->
+	    We = gb_trees:get(Id, Shps),
+	    VsA = wings_face:to_vertices([FsA], We),
+	    VsB = wings_face:to_vertices([FsB], We),
+	    wings:ask(bridge_selection({Id,VsA}, {Id,VsB}), St, fun bridge_ref/2);
+	FaceSel ->
+	    We0 = gb_trees:get(Id, Shps),
+	    case wings_sel:face_regions(FaceSel, We0) of
+		[_,_] ->
+		    We = wings_dissolve:faces(FaceSel, We0),
+		    Faces = wings_we:new_items_as_gbset(face, We0, We),
+		    Shapes = gb_trees:update(Id, We, Shps),
+		    bridge_ref(St#st{shapes=Shapes,sel=[{Id,Faces}]});
+		_ -> bridge_error()
+	    end
+    end;
+bridge_ref(_St) ->
+    bridge_error().
+
+bridge_ref(Reference, St) ->
+    {save_state,wings_shape:recreate_folder_system(bridge(Reference, St))}.
+
+bridge_selection({IdA,VsA}, {IdB,VsB}) ->
+    Desc  = ?__(1,"Select a single vertex as reference from each selected face or region border."),
+    Desc1 = ?__(2,"Nothing selected."),
+    Desc2 = ?__(3,"You must select only one vertex on each face/region."),
+    Desc3 = ?__(4,"Invalid selection."),
+    Fun = fun
+	      (check, #st{sel=[]}) ->
+		  {none, Desc1};
+	      (check, #st{sel=[{Id,Vs}]}=_St) when Id =:= IdA; Id =:= IdB ->
+		  case gb_sets:to_list(Vs) of
+		      [Va,Vb] ->
+			  VsA0 = gb_sets:insert(Va ,gb_sets:empty()),
+			  VsB0 = gb_sets:insert(Vb ,gb_sets:empty()),
+			  case sel_validate(VsA, VsB, VsA0, VsB0) of
+			      fail -> {none, Desc2};
+			      _ -> {none, []}
+			  end;
+		      _ -> {none, Desc2}
+		  end;
+	      (check, #st{sel=[{IdA0,VsA0},{IdB0,VsB0}]}=_St)
+		  when IdA0 =:= IdA; IdB0 =:= IdB; IdA0 =:= IdB; IdB0 =:= IdA ->
+
+		  case sel_validate(VsA, VsB, VsA0, VsB0) of
+		      fail -> {none, Desc2};
+		      _ -> {none, []}
+		  end;
+	      (check, _St) ->
+		  {none, Desc3};
+	      (exit, {_,_,#st{sel=[{Id,Vs0}]}}) when Id =:= IdA; Id =:= IdB ->
+		  case gb_sets:to_list(Vs0) of
+		      [Va,Vb]=Vs ->
+			  VsA0 = gb_sets:insert(Va ,gb_sets:empty()),
+			  VsB0 = gb_sets:insert(Vb ,gb_sets:empty()),
+			  case sel_validate(VsA, VsB, VsA0, VsB0) of
+			      normal -> {result,Vs};
+			      invert -> {result,[Vb,Va]};
+			      _ -> error
+			  end;
+		      _ -> error
+		  end;
+	      (exit, {_,_,#st{sel=[{IdA0,VsA0},{IdB0,VsB0}]=Sel}}) ->
+		  case sel_validate(VsA, VsB, VsA0, VsB0) of
+		      normal -> {result,Sel};
+		      invert -> {result,[{IdA0,VsB0},{IdB0,VsA0}]};
+		      _ -> error
+		  end;
+	      (exit,_) -> error
+	  end,
+    {[{Fun,Desc}],[],[],[vertex]}.
+
+sel_validate(VsA, VsB, VsA0, VsB0) ->
+    Res =
+	{bridge_sel_validate(VsA0,VsA),
+	 bridge_sel_validate(VsB0,VsB),
+	 bridge_sel_validate(VsA0,VsB),
+	 bridge_sel_validate(VsB0,VsA)},
+    case Res of
+	{Va,Vb,false,false} when is_integer(Va), is_integer(Vb) -> normal;
+	{false,false,Va,Vb} when is_integer(Va), is_integer(Vb) -> invert;
+	{Va,Vb,Va0,Vb0} when is_integer(Va), is_integer(Vb), is_integer(Va0), is_integer(Vb0) -> normal;
+	{Va0,Vb0,Va,Vb} when is_integer(Va), is_integer(Vb), is_integer(Va0), is_integer(Vb0) -> invert;
+	_ -> fail
+    end.
+
+bridge_sel_validate(Vs, VsList) ->
+    case gb_sets:to_list(Vs) of
+	[V] ->
+	    case lists:member(V, VsList) of
+		false -> false;
+		_ -> V
+	    end;
+	_ -> false
+    end.
+
+bridge_0(Reference, FaceA, FaceB, We0) ->
     VsA0 = wings_face:vertices_ccw(FaceA, We0),
     VsB0 = wings_face:vertices_ccw(FaceB, We0),
     SizeA = length(VsA0),
     SizeB = length(VsB0),
     case SizeA =:= SizeB of
       true ->
-        bridge_1(FaceA, VsA0, FaceB, VsB0, We0);
+        bridge_1(Reference, FaceA, VsA0, FaceB, VsB0, We0);
       false ->
         if
           SizeA > SizeB ->
@@ -796,13 +955,13 @@ bridge_0(FaceA, FaceB, We0) ->
             [Va|Vs] = VsB0,
             We = cut_in_extra_edges_0(Vs, Va, SizeB, Diff, FaceB, We0),
             VsB = wings_face:vertices_ccw(FaceB, We),
-            bridge_1(FaceA, VsA0, FaceB, VsB, We);
+            bridge_1(Reference, FaceA, VsA0, FaceB, VsB, We);
           true ->
             Diff = SizeB - SizeA,
             [Va|Vs] = VsA0,
             We = cut_in_extra_edges_0(Vs, Va, SizeA, Diff, FaceA, We0),
             VsA = wings_face:vertices_ccw(FaceA, We),
-            bridge_1(FaceA, VsA, FaceB, VsB0, We)
+            bridge_1(Reference, FaceA, VsA, FaceB, VsB0, We)
         end
     end.
 
@@ -833,7 +992,7 @@ cut_in_extra_edges([Va|Vs], Vb, V, Parts, Face, 1, N, Diff0, We0) ->
 cut_in_extra_edges([Va|Vs], _, V,Parts, Face, Count, N, Diff, We) ->
     cut_in_extra_edges(Vs, Va, V, Parts, Face, Count-1, N, Diff, We).
 
-bridge_1(FaceA, VsA, FaceB, VsB, #we{vp=Vtab}=We) ->
+bridge_1(Reference, FaceA, VsA, FaceB, VsB, #we{vp=Vtab}=We) ->
     An = wings_face:face_normal_cw(VsA, Vtab),
     Bn = wings_face:face_normal_cw(VsB, Vtab),
     case e3d_vec:dot(An, Bn) of
@@ -844,7 +1003,12 @@ bridge_1(FaceA, VsA, FaceB, VsB, #we{vp=Vtab}=We) ->
 		true ->
 		    bridge_error(?__(3,"Faces must not be neighbors."));
 		false ->
-		    bridge(FaceA, VsA, FaceB, VsB, We)
+		    case Reference of
+			[VsAr,VsBr] ->
+			    bridge_ref(FaceA, VsAr, FaceB, VsBr, We);
+			_ ->
+			    bridge(FaceA, VsA, FaceB, VsB, We)
+		    end
 	    end
     end.
 
@@ -872,6 +1036,29 @@ try_bridge(N, Len, Va0, FaceA, IterA0, Vb, FaceB, IterB, Ids, We0,
     try_bridge(N-1, Len, Va, FaceA, IterA,
 	       Vb, FaceB, IterB, Ids, We0, Best).
 
+bridge_ref(FaceA, Va, FaceB, Vb, We0) ->
+    Len = wings_face:vertices(FaceA, We0),
+    {Ids,We} = wings_we:new_wrap_range(Len, 2, We0),
+    IterA = wings_face:skip_to_cw(Va, wings_face:iterator(FaceA, We)),
+    IterB = wings_face:skip_to_ccw(Vb, wings_face:iterator(FaceB, We)),
+    try_bridge_ref(Len, Len, Va, FaceA, IterA,
+		   Vb, FaceB, IterB, Ids, We, {9.9E307,We}).
+
+try_bridge_ref(0, _Len, _Va, _FaceA, _IterA, _Vb, _FaceB, _IterB, _, _, {_,We}) ->
+    We;
+try_bridge_ref(N, Len, Va0, FaceA, IterA0, Vb0, FaceB, IterB0, Ids, We0,
+	       {EdgeSum0,_}=Best0) ->
+    We = do_bridge(Len, Va0, FaceA, IterA0, Vb0, FaceB, IterB0, Ids, We0, We0),
+    Best = case sum_edge_lens(Len, Ids, We, 0) of
+	       Min when Min < EdgeSum0 -> {Min,We};
+	       _ -> Best0
+	   end,
+    {_,_,_,IterA} = wings_face:next_cw(IterA0),
+    {Va,_,_,_} = wings_face:next_cw(IterA),
+    {Vb,_,_,IterB} = wings_face:next_ccw(IterB0),
+    try_bridge_ref(N-1, Len, Va, FaceA, IterA,
+		   Vb, FaceB, IterB, Ids, We0, Best).
+
 sum_edge_lens(0, _Ids, _We, Sum) -> Sum;
 sum_edge_lens(N, Ids0, #we{es=Etab,vp=Vtab}=We, Sum) ->
     Edge = wings_we:id(0, Ids0),
@@ -956,10 +1143,18 @@ get_edge(Edge, Etab) ->
 	Erec -> Erec
     end.
 
+new_vertex_id(OldV, []=_OldVs, _NewVs) -> OldV;
+new_vertex_id(OldV, _OldVs, []=_NewVs) -> OldV;
+new_vertex_id(OldV, [OldV|_], [NewV|_]) -> NewV;
+new_vertex_id(OldV, [_|OldVs], [_|NewVs]) ->
+    new_vertex_id(OldV, OldVs, NewVs).
+
+-spec bridge_error() -> no_return().
 bridge_error() ->
     %?__(1,"Exactly two faces must be selected.")
     bridge_error(?__(2,"Exactly two face regions must be selected.")).
 
+-spec bridge_error(any()) -> no_return().
 bridge_error(Error) ->
     wings_u:error_msg(Error).
 
@@ -1040,6 +1235,7 @@ lift_from_edge(Dir, EdgeSel, St0) ->
 	{_,_} -> lift_sel_mismatch()
     end.
 
+-spec lift_sel_mismatch() -> no_return().
 lift_sel_mismatch() ->
     wings_u:error_msg(?__(1,"Face and edge selections don't match.")).
 	
@@ -1130,6 +1326,7 @@ lift_from_vertex(Dir, VsSel, St0) ->
 	{_,_} -> lift_vtx_sel_mismatch()
     end.
 
+-spec lift_vtx_sel_mismatch() -> no_return().
 lift_vtx_sel_mismatch() ->
     wings_u:error_msg(?__(1,"Face and vertex selections don't match.")).
 
