@@ -12,9 +12,10 @@
 %%
 
 -module(wings_file).
--export([init/0,init_autosave/0,menu/1,command/2]).
+-export([init/0,init_autosave/0,menu/0,command/2]).
 -export([import_filename/2,export_filename/2,export_filename/3]).
 -export([unsaved_filename/0,autosave_filename/1]).
+-export([file_filters/1]).
 
 -include("wings.hrl").
 -include("e3d.hrl").
@@ -30,13 +31,16 @@
 %% export_filename([Prop], St, Continuation).
 %%   The St will only be used to setup the default filename.
 %%   The Continuation fun will be called like this: Continuation(Filename).
-export_filename(Prop, #st{file=undefined}, Cont) ->
-    export_filename(Prop, Cont);
 export_filename(Prop0, #st{file=File}, Cont) ->
     Prop = case proplists:get_value(ext, Prop0) of
 	       undefined -> Prop0;
 	       Ext ->
-		   Def = filename:rootname(filename:basename(File), ?WINGS) ++ Ext,
+		   BaseName =
+		       case File of
+                           undefined -> "untitled";
+                           _ -> filename:basename(File)
+		       end,
+		   Def = filename:rootname(BaseName, ?WINGS) ++ Ext,
 		   [{default_filename,Def}|Prop0]
 	   end,
     export_filename(Prop, Cont).
@@ -54,16 +58,17 @@ import_filename_1(Ps0, Cont) ->
     This = wings_wm:this(),
     Dir = wings_pref:get_value(current_directory),
     String = case os:type() of
-        {win32,_} -> "Import";
-        _Other    -> ?__(1,"Import")
-    end,
+		 {win32,_} -> "Import";
+		 _Other    -> ?__(1,"Import")
+	     end,
     Ps = Ps0 ++ [{title,String},{directory,Dir}],
     Fun = fun(Name0) ->
-          Name=test_unc_path(Name0),
+		  Name=test_unc_path(Name0),
 		  case catch Cont(Name) of
 		      {command_error,Error} ->
 			  wings_u:message(Error);
 		      #st{}=St ->
+			  set_cwd(dirname(Name)),
 			  wings_wm:send(This, {new_state,St});
 		      Tuple when is_tuple(Tuple) ->
 			  wings_wm:send(This, {action,Tuple});
@@ -88,11 +93,12 @@ export_filename_1(Prop0, Cont) ->
     Dir = wings_pref:get_value(current_directory),
     Prop = Prop0 ++ [{directory,Dir}],
     Fun = fun(Name0) ->
-          Name=test_unc_path(Name0),
+		  Name=test_unc_path(Name0),
 		  case catch Cont(Name) of
 		      {command_error,Error} ->
 			  wings_u:message(Error);
 		      #st{}=St ->
+			  set_cwd(dirname(Name)),
 			  wings_wm:send(This, {new_state,St});
 		      Tuple when is_tuple(Tuple) ->
 			  wings_wm:send(This, {action,Tuple});
@@ -124,7 +130,7 @@ init() ->
 	    end
     end.
 
-menu(_) ->
+menu() ->
     ImpFormats = [{"Nendo (.ndo)...",ndo}],
     ExpFormats = [{"Nendo (.ndo)...",ndo}],
     Tail = [{?__(25,"Exit"),quit,?__(28,"Exit Wings 3D")}],
@@ -172,10 +178,7 @@ menu(_) ->
      separator|recent_files(Tail)].
 
 save_unused_mats() ->
-    case wings_pref:get_value(save_unused_materials) of
-	  true -> [crossmark];
-	  false -> []
-    end.
+    wings_menu_util:crossmark(save_unused_materials).
 
 command(new, St) ->
     new(St);
@@ -270,15 +273,21 @@ command(quit, _) ->
 command(confirmed_quit, _) ->
     del_unsaved_file(),
     quit;
-command(Key, St) when is_integer(Key), 1 =< Key ->
+command({recent_file,Key}, St) when is_integer(Key), 1 =< Key ->
     Recent0 = wings_pref:get_value(recent_files, []),
     {_,File} = lists:nth(Key, Recent0),
     case filelib:is_file(File) of
 	true ->
 	    named_open(File, St);
 	false ->
+	    Last = length(Recent0),
 	    Recent = delete_nth(Recent0, Key),
 	    wings_pref:set_value(recent_files, Recent),
+	    wings_menu:update_menu(file, {recent_file, Last}, delete, []),
+	    lists:foreach(fun({Str, RKey, Help}) ->
+				  wings_menu:update_menu(file, RKey, Str, Help);
+			     (separator) -> ok
+			  end, recent_files(Recent, [])),
 	    wings_u:error_msg(?__(5,"This file has been moved or deleted."))
     end.
 
@@ -378,12 +387,13 @@ merge(Name, St0) ->
 		  %%   Name: Original name of file to be opened.
 		  %%   File: Either original file or the autosave file
 		  St1 = St0#st{saved=wings_image:next_id()},
-		  case ?SLOW(wings_ff_wings:import(File, St0)) of
+		  case wings_ff_wings:merge(File, St0) of
 		      {error,Reason} ->
 			  clean_new_images(St1),
 			  wings_u:error_msg(?__(2,"Read failed: ") ++ Reason);
 		      #st{}=St ->
 			  set_cwd(dirname(Name)),
+			  wings_u:caption(St#st{saved=false}),
 			  wings_shape:recreate_folder_system(St#st{saved=false})
 		  end
 	  end,
@@ -555,7 +565,7 @@ init_autosave() ->
 	    Op = {seq,push,get_autosave_event(make_ref(), #st{saved=auto})},
 	    wings_wm:new(Name, {0,0,1}, {0,0}, Op),
 	    wings_wm:hide(Name),
-	    wings_wm:set_prop(Name, display_lists, geom_display_lists)
+	    wings_wm:set_dd(Name, geom_display_lists)
     end,
     wings_wm:send(Name, start_timer).
 
@@ -637,21 +647,29 @@ update_recent(Old, New) ->
     Recent = add_recent(NewFile, Recent1),
     wings_pref:set_value(recent_files, Recent).
 
-add_recent(File, [A,B,C,D,E|_]) -> [File,A,B,C,D,E];
-add_recent(File, Recent) -> [File|Recent].
+add_recent(NewFile, Present) ->
+    Recent = add_recent_1(NewFile, Present),
+    lists:foreach(fun({Str, Key, Help}) ->
+			  wings_menu:update_menu(file, Key, Str, Help);
+		     (separator) -> ok
+		  end, recent_files(Recent, [])),
+    Recent.
+
+add_recent_1(File, [A,B,C,D,E|_]) -> [File,A,B,C,D,E];
+add_recent_1(File, Recent) -> [File|Recent].
 
 recent_files(Tail) ->
-    case wings_pref:get_value(recent_files, []) of
-	[] -> Tail;
-	Files ->
-	    Help = ?__(1,"Open this recently used file"),
-	    recent_files_1(Files, 1, Help, [separator|Tail])
-    end.
+    recent_files(wings_pref:get_value(recent_files, []), Tail).
+
+recent_files([], Tail) -> Tail;
+recent_files(Files, Tail) ->
+    Help = ?__(1,"Open this recently used file"),
+    recent_files_1(Files, 1, Help, [separator|Tail]).
 
 recent_files_1([{Base0,Base1}|T], I, Help0, Tail) ->
     Base = wings_u:pretty_filename(Base0),
     Help = lists:flatten([Help0," -- "|wings_u:pretty_filename(Base1)]),
-    [{Base,I,Help}|recent_files_1(T, I+1, Help0, Tail)];
+    [{Base,{recent_file,I},Help}|recent_files_1(T, I+1, Help0, Tail)];
 recent_files_1([], _, _, Tail) -> Tail.
 
 %%
@@ -688,7 +706,7 @@ import_ndo(Name, St0) ->
     end.
 
 import_image() ->
-    Ps = [{extensions,wings_image:image_formats()}],
+    Ps = [{extensions,wings_image:image_formats()},{multiple,true}],
     Cont = fun(Name) -> {file,{import_image,Name}} end,
     import_filename(Ps, Cont).
 
@@ -728,25 +746,59 @@ do_export_ndo(Name, St) ->
 
 install_plugin() ->
     Props = case os:type() of
-        {win32,_} ->
+		{win32,_} ->
 		    [{title,"Install Plug-In"},
-             {extensions,
-             [{".gz", "GZip Compressed File"},
-              {".tar", "Tar File"},
-              {".tgz", "Compressed Tar File"},
-              {".beam", "Beam File"}]}];
-        _Other    ->
-            [{title,?__(1,"Install Plug-In")},
-             {extensions,
-             [{".gz",?__(2,"GZip Compressed File")},
-              {".tar",?__(3,"Tar File")},
-              {".tgz",?__(4,"Compressed Tar File")},
-              {".beam",?__(5,"Beam File")}]}]
-    end,
+		     {extensions,
+		      [{".gz", "GZip Compressed File"},
+		       {".tar", "Tar File"},
+		       {".tgz", "Compressed Tar File"},
+		       {".beam", "Beam File"}]}];
+		_Other    ->
+		    [{title,?__(1,"Install Plug-In")},
+		     {extensions,
+		      [{".gz",?__(2,"GZip Compressed File")},
+		       {".tar",?__(3,"Tar File")},
+		       {".tgz",?__(4,"Compressed Tar File")},
+		       {".beam",?__(5,"Beam File")}]}]
+	    end,
     Cont = fun(Name) -> {file,{install_plugin,Name}} end,
     import_filename(Props, Cont).
 
-%%%
+file_filters(Prop) ->
+    Exts = case proplists:get_value(extensions, Prop, none) of
+	       none ->
+		   Ext = proplists:get_value(ext, Prop, ".wings"),
+		   ExtDesc = proplists:get_value(ext_desc, Prop,
+						 ?__(1,"Wings File")),
+		   [{Ext,ExtDesc}];
+	       Other -> Other
+	   end,
+    lists:flatten([file_add_all(Exts),
+		   file_filters_0(Exts++[{".*", ?__(2,"All Files")}])]).
+
+file_filters_0(Exts) ->
+    file_filters_1(lists:reverse(Exts),[]).
+
+file_filters_1([{Ext,Desc}|T], Acc) ->
+    Wildcard = "*" ++ Ext,
+    ExtString = [Desc," (",Wildcard,")","|",Wildcard|Acc],
+    case T of
+	[] -> ExtString;
+	_  ->
+	    file_filters_1(T, ["|"|ExtString])
+    end.
+
+file_add_all([_]) -> [];
+file_add_all(Exts) ->
+    All0 = ["*"++E || {E,_} <- Exts],
+    All = file_add_semicolons(All0),
+    [?__(1,"All Formats")++" (",All,")", "|", All, "|"].
+
+file_add_semicolons([E1|[_|_]=T]) ->
+    [E1,";"|file_add_semicolons(T)];
+file_add_semicolons(Other) -> Other.
+
+
 %%% Utilities.
 %%%
 
