@@ -12,8 +12,9 @@
 %%
 
 -module(wings_gl).
--export([init_extensions/0,is_ext/1,is_ext/2,
-	 init_restrictions/0,is_restriction/1,
+-export([init/1, window/4, attributes/0,
+	 is_ext/1,is_ext/2,
+	 is_restriction/1,
 	 error_string/1]).
 
 %% GLSL exports
@@ -25,9 +26,9 @@
 -export([have_fbo/0, setup_fbo/2, delete_fbo/1]).
 
 %% GL wrappers
--export([callLists/1, project/6, unProject/6, 
+-export([project/6, unProject/6,
 	 triangulate/2, deleteTextures/1,
-	 bindFramebuffer/2, 
+	 bindFramebuffer/2,
 	 drawElements/4
 	]).
 
@@ -37,28 +38,124 @@
 -define(NEED_OPENGL, 1).
 -include("wings.hrl").
 
--ifdef(USE_WX_OPENGL).
--define(genFramebuffers,genFramebuffers).
--define(bindFramebuffer, bindFramebuffer).
--define(framebufferTexture2D,framebufferTexture2D).
--define(genRenderbuffers, genRenderbuffers).
--define(bindRenderbuffer,bindRenderbuffer).
--define(renderbufferStorage,renderbufferStorage).
--define(framebufferRenderbuffer,framebufferRenderbuffer).
--define(checkFramebufferStatus, checkFramebufferStatus).
--define(generateMipmap, generateMipmap).
--else.
--define(genFramebuffers,genFramebuffersEXT).
--define(bindFramebuffer, bindFramebufferEXT).
--define(framebufferTexture2D,framebufferTexture2DEXT).
--define(genRenderbuffers, genRenderbuffersEXT).
--define(bindRenderbuffer,bindRenderbufferEXT).
--define(renderbufferStorage,renderbufferStorageEXT).
--define(framebufferRenderbuffer,framebufferRenderbufferEXT).
--define(checkFramebufferStatus, checkFramebufferStatusEXT).
--define(generateMipmap, generateMipmapExt).
+-ifndef(WX_GL_SAMPLE_BUFFERS).     %% New in wxWidgets-3.0
+-define(WX_GL_SAMPLE_BUFFERS,17).  %% 1 for multisampling support (antialiasing)
+-define(WX_GL_SAMPLES,18).         %% 4 for 2x2 antialiasing supersampling on most graphics cards
 -endif.
 
+init(Parent) ->
+    GL = window(Parent, undefined, true, true),
+    wx_object:cast(Parent, {init_menus, Parent}),
+    init_extensions(),
+    init_restrictions(),
+    GL.
+
+attributes() ->
+    {attribList,
+     [?WX_GL_RGBA,
+      ?WX_GL_MIN_RED,8,?WX_GL_MIN_GREEN,8,?WX_GL_MIN_BLUE,8,
+      ?WX_GL_DEPTH_SIZE, 24, ?WX_GL_STENCIL_SIZE, 8,
+      ?WX_GL_DOUBLEBUFFER,
+      ?WX_GL_SAMPLE_BUFFERS,1,
+      ?WX_GL_SAMPLES, 4,
+      0]}.
+
+window(Parent, Context, Connect, Show) ->
+    Style = ?wxFULL_REPAINT_ON_RESIZE bor ?wxWANTS_CHARS,
+    Flags = [attributes(), {style, Style}],
+    GL = case Context of
+	     undefined ->
+		 Win = wxGLCanvas:new(Parent, Flags),
+		 ?SET(gl_canvas, Win),
+		 Win;
+	     _ ->
+		 wxGLCanvas:new(Parent, Context, Flags)
+	 end,
+    Connect andalso connect_events(GL),
+    case Show of
+	true ->
+	    wxWindow:connect(Parent, show),
+	    wxFrame:show(Parent),
+	    receive #wx{event=#wxShow{}} -> ok end;
+	false ->
+	    ok
+    end,
+    wxGLCanvas:setCurrent(GL),
+    wxWindow:disconnect(Parent, show),
+    GL.
+
+%% Event handling for OpenGL windows
+
+connect_events(Canvas) ->
+    %% Re-attaches the OpenGL Context to Window when is [re] created/showed
+    wxWindow:connect(Canvas, create,
+                     [{callback, fun(_, _) -> wxGLCanvas:setCurrent(Canvas) end}]),
+    wxWindow:connect(Canvas, show,
+                     [{callback, fun(#wx{event=#wxShow{show=Show}}, _) ->
+                                         Show andalso (catch wxGLCanvas:setCurrent(Canvas)) end}]),
+    case os:type() of
+	{unix, darwin} ->
+	    wxWindow:connect(Canvas, paint, [{callback, fun redraw/2}]);
+	{unix, _} ->
+	    wxWindow:connect(Canvas, paint, [{skip, true}]),
+	    ok;
+	{win32, _} ->
+	    wxWindow:connect(Canvas, paint, [{callback, fun redraw/2}]),
+	    wxWindow:connect(Canvas, erase_background, [{callback, fun redraw/2}])
+    end,
+
+    wxWindow:connect(Canvas, size),
+    catch wxWindow:connect(Canvas, mouse_capture_lost), %% Not available in old wx's.
+
+    setup_std_events(Canvas),
+    wxWindow:setFocus(Canvas), %% Get keyboard focus
+    ok.
+
+redraw(#wx{obj=Canvas, event=#wxPaint{}}=Ev,_) ->
+    %% Must do a PaintDC and destroy it
+    DC = wxPaintDC:new(Canvas),
+    wxPaintDC:destroy(DC),
+    wings ! Ev,
+    ok;
+redraw(Ev, _) ->  %% For erase background events
+    wings ! Ev#wx{event=#wxPaint{type=paint}}.
+
+setup_std_events(Canvas) ->
+    wxWindow:connect(Canvas, motion),
+    wxWindow:connect(Canvas, left_up),
+    wxWindow:connect(Canvas, left_down),
+    wxWindow:connect(Canvas, middle_up),
+    wxWindow:connect(Canvas, middle_down),
+    wxWindow:connect(Canvas, left_dclick),
+    wxWindow:connect(Canvas, right_up),
+    wxWindow:connect(Canvas, right_down),
+    wxWindow:connect(Canvas, mousewheel),
+    %% wxWindow:connect(Canvas, char_hook, []),
+    wxWindow:connect(Canvas, key_down, [{callback, fun key_callback_win32/2}]),
+    wxWindow:connect(Canvas, key_up), %% Normally suppressed
+    wxWindow:connect(Canvas, char).
+
+key_callback_win32(Ev = #wx{event=Key=#wxKey{rawFlags=Raw}},Obj) ->
+    %% See WM_SYSKEYDOWN message in msdn
+    %% https://msdn.microsoft.com/en-us/library/windows/desktop/ms646286(v=vs.85).aspx
+    Repeat = (Raw band (1 bsl 30)) > 1,
+    %% AltGr  = (Raw band (1 bsl 24)) > 1,
+    %% Repeat orelse io:format("Ev ~p~n   ~.2B => Repeat ~p AltGr ~p~n",
+    %%  			    [Key, Raw, Repeat, AltGr]),
+    case forward_key(Key) of
+	true when Repeat -> ignore;
+	%% true when AltGr -> ignore;
+	true -> wings ! Ev;
+	false -> wxEvent:skip(Obj)
+    end.
+
+forward_key(#wxKey{controlDown=true}) -> true;
+forward_key(#wxKey{altDown=true}) -> true;
+forward_key(#wxKey{metaDown=true}) -> true;
+forward_key(#wxKey{shiftDown=true, keyCode=?WXK_SHIFT}) -> true;
+forward_key(_) -> false.
+
+
 %%%
 %%% OpenGL extensions.
 %%%
@@ -274,8 +371,8 @@ setup_fbo(Size, What) ->
     end.
 
 setup_fbo_1(Size, Types) ->
-    [FB] = gl:?genFramebuffers(1),
-    gl:?bindFramebuffer(?GL_FRAMEBUFFER_EXT, FB),
+    [FB] = gl:genFramebuffers(1),
+    gl:bindFramebuffer(?GL_FRAMEBUFFER_EXT, FB),
     {Bfs,_} = lists:foldl(fun(What, {Acc, ColCount}) ->
 				  case setup_fbo_2(What, Size, ColCount) of
 				      {color, _} = Res ->
@@ -315,42 +412,42 @@ setup_fbo_2({color, Options}, {W,H}, Count) ->
     GEN_MM = proplists:get_value(gen_mipmap, Options, ?GL_FALSE),
     gl:texParameteri(?GL_TEXTURE_2D, ?GL_GENERATE_MIPMAP, GEN_MM),
     if GEN_MM =:= ?GL_TRUE ->
-	    gl:?generateMipmap(?GL_TEXTURE_2D);
+	    gl:generateMipmap(?GL_TEXTURE_2D);
        true -> ok
     end,
 
-    gl:?framebufferTexture2D(?GL_FRAMEBUFFER_EXT,
+    gl:framebufferTexture2D(?GL_FRAMEBUFFER_EXT,
 			     ?GL_COLOR_ATTACHMENT0_EXT + Count,
 			     ?GL_TEXTURE_2D, Col, 0),
     {color, Col};
 setup_fbo_2({depth, Options}, {W,H}, _) ->
-    [Depth] = gl:?genRenderbuffers(1),
+    [Depth] = gl:genRenderbuffers(1),
     %% Init depth texture
-    gl:?bindRenderbuffer(?GL_RENDERBUFFER_EXT, Depth),
+    gl:bindRenderbuffer(?GL_RENDERBUFFER_EXT, Depth),
     Internal = proplists:get_value(internal, Options, ?GL_DEPTH_COMPONENT24),
-    gl:?renderbufferStorage(?GL_RENDERBUFFER_EXT,Internal, W, H),
-    gl:?framebufferRenderbuffer(?GL_FRAMEBUFFER_EXT,
+    gl:renderbufferStorage(?GL_RENDERBUFFER_EXT,Internal, W, H),
+    gl:framebufferRenderbuffer(?GL_FRAMEBUFFER_EXT,
 				?GL_DEPTH_ATTACHMENT_EXT,
 				?GL_RENDERBUFFER_EXT, Depth),
     {depth, Depth}.
 
 delete_fbo(List) ->
-    gl:?framebufferTexture2D(?GL_FRAMEBUFFER_EXT,
+    gl:framebufferTexture2D(?GL_FRAMEBUFFER_EXT,
 			     ?GL_COLOR_ATTACHMENT0_EXT,
 			     ?GL_TEXTURE_2D,0,0),
     Textures = [Col || {color, Col} <- List],
     deleteTextures(Textures),
-    gl:?framebufferRenderbuffer(?GL_FRAMEBUFFER_EXT,
+    gl:framebufferRenderbuffer(?GL_FRAMEBUFFER_EXT,
 				?GL_DEPTH_ATTACHMENT_EXT,
 				?GL_RENDERBUFFER_EXT, 0),
     Depth = [D || {depth, D} <- List],
     deleteRenderbuffers(Depth),
-    gl:?bindFramebuffer(?GL_FRAMEBUFFER_EXT, 0),
+    gl:bindFramebuffer(?GL_FRAMEBUFFER_EXT, 0),
     FB = [F || {framebuffer, F} <- List],
     deleteFramebuffers(FB).
 
 check_fbo_status(FB) ->
-    case gl:?checkFramebufferStatus(?GL_FRAMEBUFFER_EXT) of
+    case gl:checkFramebufferStatus(?GL_FRAMEBUFFER_EXT) of
 	?GL_FRAMEBUFFER_COMPLETE_EXT ->
 	    FB;
 	?GL_FRAMEBUFFER_UNSUPPORTED_EXT ->
@@ -379,20 +476,20 @@ check_fbo_status(FB) ->
 %%%%%%%%%%%%% Wrappers for functions that differs between wx and esdl
 
 bindFramebuffer(W, Fbo) ->
-    gl:?bindFramebuffer(W,Fbo).
-
-
--ifdef(USE_WX_OPENGL).
-callLists(List) ->  gl:callLists(List).
+    gl:bindFramebuffer(W,Fbo).
 
 project(X,Y,Z, Mod, Proj, View) ->
-    {_, RX,RY,RZ} = glu:project(X,Y,Z, list_to_tuple(Mod), list_to_tuple(Proj), View),
+    {_, RX,RY,RZ} = glu:project(X,Y,Z, mat(Mod), mat(Proj), View),
     {RX,RY,RZ}.
 
 unProject(X,Y,Z, Mod, Proj, View) ->
-    {_, RX,RY,RZ} = glu:unProject(X,Y,Z, list_to_tuple(Mod), list_to_tuple(Proj), View),
+    {_, RX,RY,RZ} = glu:unProject(X,Y,Z, mat(Mod), mat(Proj), View),
     {RX,RY,RZ}.
 
+mat(Mat) when tuple_size(Mat) =:= 16 ->  Mat;
+mat(Mat) when is_tuple(Mat) -> e3d_mat:expand(Mat);
+mat(List) when is_list(List) ->  mat(list_to_tuple(List)).
+
 triangulate(Normal, Pos0) ->
     {Tris0, BinPos} = glu:tesselate(Normal, Pos0),
     Tris = tris(Tris0),
@@ -423,31 +520,3 @@ drawElements(O,L,T = ?GL_UNSIGNED_INT,What) when is_list(What) ->
 drawElements(O,L,T,What) ->
     gl:drawElements(O,L,T,What).
 
--else.
-callLists(List) ->  gl:callLists(length(List), ?GL_UNSIGNED_INT, List).
-
-project(X,Y,Z, Mod, Proj, View) ->
-    glu:project(X,Y,Z, Mod, Proj, View).
-
-unProject(X,Y,Z, Mod, Proj, View) ->
-    glu:unProject(X,Y,Z, Mod, Proj, View).
-
-triangulate(Normal, Pos) ->
-    glu:triangulate(Normal,Pos).
-
-deleteTextures(List) ->
-    gl:deleteTextures(length(List), List).
-
-deleteRenderbuffers(List) ->
-    gl:deleteRenderbuffersEXT(length(List),List).
-
-deleteFramebuffers(List) ->
-    gl:deleteFramebuffersEXT(length(List),List).
-
-shaderSource(Handle, Src) ->
-    ok = gl:shaderSource(Handle, 1, Src, [-1]).
-
-drawElements(O,L,T,What) ->
-    gl:drawElements(O,L,T,What).
-
--endif.
