@@ -13,8 +13,9 @@
 
 -module(wings_hotkey).
 -export([event/1,event/2,matching/1,bind_unicode/3,bind_virtual/4,
-	 bind_from_event/2,unbind/1,hotkeys_by_commands/1,bindkey/2,
-	 set_default/0,listing/0,handle_error/2]).
+	 command/2, bind_from_event/2,unbind/1,hotkeys_by_commands/1,bindkey/2,
+	 set_default/0,listing/0,handle_error/2,
+	 format_hotkey/2]).
 
 -define(NEED_ESDL, 1).
 -include("wings.hrl").
@@ -25,6 +26,26 @@
 
 -define(KL, wings_state).
 
+%%%
+%%% Format hotkeys.
+%%%
+
+
+%% format_hotkey(Hotkey, wx|pretty) -> String.
+format_hotkey({bindkey,Hotkey}, Style) ->
+    format_hotkey(Hotkey, Style);
+format_hotkey({bindkey,_Mode,Hotkey}, Style) ->
+    format_hotkey(Hotkey, Style);
+format_hotkey(Hotkey, Style) ->
+    case Hotkey of
+	[] -> [];
+	[_|_] -> Hotkey;
+	{C,Mods} ->
+	    modname(Mods, Style) ++ vkeyname(C);
+	_ ->
+	    keyname(Hotkey, Style)
+    end.
+
 %%%
 %%% Hotkey lookup and translation.
 %%%
@@ -68,12 +89,150 @@ lookup(Ev, Cmd) ->
 %%% Binding and unbinding of keys.
 %%%
 
+-record(cs, {info, st, sc, op, action}).
+
+command(Cmd0, St) ->
+    Str = ?__(1, "Select an menu item to bind/unbind a hotkey"),
+    {Cmd, Sc} =
+    case Cmd0 of
+	{_,_} -> Cmd0;
+	_ -> {Cmd0, none}
+    end,
+    Cs = #cs{info=Str, st=St, op=Cmd, sc=Sc},
+    wings_wm:message(Str),
+    {push, fun(Ev) -> event_handler(Ev, Cs) end}.
+
+event_handler(redraw, CS=#cs{st=St}) ->
+    wings:redraw("", St),
+    {replace,fun(Ev) -> event_handler(Ev, CS) end};
+
+event_handler({action, Action}, #cs{op=unbind}) ->
+    do_unbind(Action);
+
+event_handler(Ev=#keyboard{which=menubar}, #cs{op=unbind}) ->
+    do_unbind(event(Ev));
+
+event_handler({action, Action}, CS=#cs{op=bind}) ->
+    wings_wm:message(hotkey_key_message(Action)),
+    wings_wm:dirty(),
+    {replace,fun(Ev) -> event_handler(Ev, CS#cs{action=Action}) end};
+
+event_handler(Ev0=#keyboard{which=menubar},
+	      CS=#cs{op=bind, action=undefined}) ->
+    Action = event(Ev0),
+    wings_wm:message(hotkey_key_message(Action)),
+    wings_wm:dirty(),
+    {replace,fun(Ev) -> event_handler(Ev, CS#cs{action=Action}) end};
+
+event_handler(Ev0= #mousebutton{}, #cs{info=Str, st=St, sc=Sc}) ->
+    case wings_menu:is_popup_event(Ev0) of
+        no ->
+	    wings_wm:message(Str),
+	    wings_wm:dirty(),
+	    keep;
+        {yes,Xglobal,Yglobal,Mod} ->
+            TweakBits = wings_msg:free_rmb_modifier(),
+            case Mod band TweakBits =/= 0 of
+		true ->  case Sc of
+			     none -> wings_tweak:menu(Xglobal, Yglobal);
+			     _ -> wpc_sculpt:sculpt_menu(Xglobal, Yglobal, Sc)
+			 end;
+		false -> wings:popup_menu(Xglobal, Yglobal, St)
+            end
+    end;
+
+event_handler(#mousemotion{}, _) -> keep;
+event_handler(#keyboard{sym=27}, _) ->
+    wings_wm:dirty(),
+    pop;
+
+event_handler(Ev = #keyboard{}, #cs{op=bind, action=Cmd})
+  when Cmd =/= undefined ->
+    case disallow_bind(Ev) of
+        true ->
+            keep;
+        false ->
+            case event(Ev, Cmd) of
+                next ->
+                    case hotkeys_by_commands([Cmd]) of
+                        [] -> do_bind(Ev, Cmd);
+                        Hotkeys ->
+                            HKs = ["[" ++ Hotkey ++"]" || {_, Hotkey, _, _} <- Hotkeys],
+                            Q = ?__(3,"This command is already bound to ") ++ string:join(HKs, ", ") ++
+                                ?__(4," hotkey. Do you want to re-define it?"),
+                            wings_u:yes_no(Q, fun() ->
+                                                      [wings_hotkey:unbind(Key) || {Key, _, _, _} <- Hotkeys],
+                                                      do_bind(Ev, Cmd)
+                                              end)
+                    end;
+                OtherCmd ->
+                    C = wings_util:stringify(OtherCmd),
+                    Q = ?__(1,"This key is already bound to the ") ++ C ++
+                        ?__(2," command. Do you want to re-define it?"),
+                    wings_u:yes_no(Q, fun() -> do_bind(Ev, Cmd) end)
+            end,
+            wings_wm:dirty(),
+            pop
+    end;
+
+event_handler(_Ev, _) ->
+    keep.
+
+disallow_bind(#keyboard{unicode=UC}) when UC =/= 0 -> false;
+disallow_bind(#keyboard{sym=Sym}) when ?SDLK_F1 =< Sym, Sym =< ?SDLK_F15 -> false;
+disallow_bind(#keyboard{sym=Sym}) when ?SDLK_KP0 =< Sym, Sym =< ?SDLK_KP_EQUALS -> false;
+disallow_bind(#keyboard{sym=Sym}) when ?SDLK_HOME =< Sym, Sym =< ?SDLK_PAGEDOWN -> false;
+disallow_bind(_) -> true.
+
+do_unbind(Action) ->
+    case hotkeys_by_commands([Action]) of
+	[] -> none;	%No hotkeys for this entry.
+	Hotkeys ->
+	    hotkey_delete_dialog(Hotkeys, Action)
+    end,
+    wings_wm:dirty(),
+    pop.
+
+do_bind(Ev, Cmd) ->
+    Keyname = bind_from_event(Ev, Cmd),
+    wings_menu:update_menu_hotkey(Cmd, Keyname),
+    ignore.
+
+hotkey_delete_dialog(Hotkeys, Action) ->
+    Fun = fun(Res) ->
+		  wings_menu:update_menu_hotkey(Action, ""),
+		  [wings_hotkey:unbind(K) || {K,true} <- Res],
+		  ignore
+	  end,
+    Dialog = mk_dialog(Hotkeys),
+    wings_dialog:dialog(?__(1,"Delete Hotkeys"), Dialog, Fun).
+
+mk_dialog([{Key,Keyname,Cmd,Src}|T]) ->
+    [mk_key_item(Key, Keyname, Cmd, Src)|mk_dialog(T)];
+mk_dialog([]) ->
+    [separator,{label,?__(1,"Check all hotkeys to be deleted.")}].
+
+mk_key_item(Key, Keyname, Cmd, _Src) when is_tuple(Key) ->
+    {Keyname ++ ": " ++ Cmd,false,[{key,Key}]}.
+
+hotkey_key_message(Cmd) ->
+    [?__(1,"Press the key to bind the \""),
+     wings_util:stringify(Cmd),
+     ?__(2,"\" command to.")].
+
+
 bind_from_event(Ev, Cmd) ->
     Bkey = bindkey(Ev, Cmd),
+    Key = case Bkey of
+	    {bindkey, Key1} -> Key1;
+	    {bindkey, _Mode, Key1} -> Key1
+    end,
     ets:insert(?KL, {Bkey,Cmd,user}),
-    keyname(Bkey).
+    format_hotkey(Key, wx).
 
 
+unbind({Key}) ->
+    unbind(Key);
 unbind(Key) ->
     ets:delete(?KL, Key).
 
@@ -86,8 +245,12 @@ hotkeys_by_commands_1([C|Cs], Acc) ->
 hotkeys_by_commands_1([], Acc) ->
     hotkeys_by_commands_2(sort(Acc)).
 
-hotkeys_by_commands_2([{Key,Cmd,Src}|T]) ->
-    Info = {Key,keyname(Key),wings_util:stringify(Cmd),Src},
+hotkeys_by_commands_2([{Key0,Cmd,Src}|T]) ->
+    Key = case Key0 of
+	      {bindkey,Key1} -> Key1;
+	      {bindkey,_Mode,Key1} -> Key1
+	  end,
+    Info = {Key0,format_hotkey(Key, wx),wings_util:stringify(Cmd),Src},
     [Info|hotkeys_by_commands_2(T)];
 hotkeys_by_commands_2([]) -> [].
     
@@ -109,8 +272,10 @@ bindkey(#keyboard{sym=Sym,mod=Mod,unicode=C}, Cmd) ->
 	    bkey(fix_bksp_and_del(Sym, C), Cmd);
 	[shift] when C =/= 0 ->
 	    bkey(C, Cmd);
+	Mods when Sym =/= 0 ->
+	    bkey({Sym,sort(Mods)}, Cmd);
 	Mods ->
-	    bkey({Sym,sort(Mods)}, Cmd)
+	    bkey({C,sort(Mods)}, Cmd)
     end.
 
 bkey(Key, {Mode,_}) ->
@@ -120,7 +285,7 @@ bkey(Key, Mode) ->
 	true -> {bindkey,Mode,Key};
 	false -> {bindkey,Key}
     end.
-    
+
 matching(Names) ->
     M0 = matching_global(Names) ++ matching_mode(Names),
     M = wings_util:rel2fam(M0),
@@ -131,7 +296,7 @@ matching_global(Names) ->
     Spec = [{{{bindkey,'$2'},Spec0,'$3'},
 	     [],
 	     [{{'$1',{{'$3','$2'}}}}]}],
-    [{Name,mkeyname(Key)} || {Name,Key} <- ets:select(?KL, Spec)].
+    [{Name,m_sortkey(Key)} || {Name,Key} <- ets:select(?KL, Spec)].
 
 matching_mode(Names) ->
     Mode = lists:last(Names),
@@ -142,12 +307,13 @@ matching_mode(Names) ->
 	    Spec = [{{{bindkey,Mode,'$2'},Spec0,'$3'},
 		     [],
 		     [{{'$1',{{'$3','$2'}}}}]}],
-	    [{Name,mkeyname(Key)} || {Name,Key} <- ets:select(?KL, Spec)]
+	    [{Name,m_sortkey(Key)} || {Name,Key} <- ets:select(?KL, Spec)]
     end.
 
-mkeyname({user,K}) -> {1,keyname(K)};
-mkeyname({default,K}) -> {2,keyname(K)};
-mkeyname({plugin,K}) -> {3,keyname(K)}.
+m_sortkey({user,K}) -> {1,K};
+m_sortkey({default,K}) -> {2,K};
+m_sortkey({plugin,K}) -> {3,K}.
+
 
 suitable_mode(light) -> true;
 suitable_mode(vertex) -> true;
@@ -175,15 +341,15 @@ suitable_mode(Menu) ->
 
 listing() ->
     MatchSpec = ets:fun2ms(fun({{bindkey,K},Cmd,Src}) ->
-				   {all,{{bindkey,K},Cmd,Src}};
+				   {all,{K,Cmd,Src}};
 			      ({{bindkey,Mode,K},Cmd,Src}) ->
-				   {Mode,{{bindkey,K},Cmd,Src}}
+				   {Mode,{K,Cmd,Src}}
 			   end),
     Keys = wings_util:rel2fam(ets:select(?KL, MatchSpec)),
     listing_1(Keys, []).
 
 listing_1([{Mode,Keys}|T], Acc0) ->
-    Acc = [list_keys(Keys),list_header(Mode)|Acc0],
+    Acc = [{table, 2, list_header(Mode), list_keys(Keys)}|Acc0],
     listing_1(T, Acc);
 listing_1([], Acc) -> reverse(Acc).
 
@@ -196,14 +362,13 @@ list_header(vertex) -> ?STR(list_header,6,"Hotkeys for vertices");
 list_header(A) -> atom_to_list(A).
 
 list_keys([{Key,Cmd,Src}|T]) ->
-    KeyStr = keyname(Key),
+    KeyStr = format_hotkey(Key, pretty),
     SrcStr = case Src of
 		 default -> "";
 		 user -> ?STR(list_keys,1," (user-defined)");
 		 plugin -> ?STR(list_keys,2," (plug-in-defined)")
 	     end,
-    KeyStr ++ ": " ++ wings_util:stringify(Cmd) ++ SrcStr ++ 
-	"\n" ++ list_keys(T);
+    [[KeyStr,wings_util:stringify(Cmd) ++ SrcStr]|list_keys(T)];
 list_keys([]) -> [].
 
 
@@ -212,31 +377,33 @@ list_keys([]) -> [].
 %%%
 handle_error(Ev, Cmd) ->
     Key = bindkey(Ev, Cmd),
-    KeyName = keyname(Key),
+    KeyName = format_hotkey(Key, pretty),
     CmdStr = wings_util:stringify(Cmd),
     Msg1 = "Executing the command \"" ++ CmdStr ++ "\"\nbound to the hotkey " ++
 	KeyName ++ " caused an error.",
     Msg2 = "Possible causes:",
-    Msg3 = [bullet,$\s|"The hotkey was defined in a previous version of Wings,\n"
-	"and the command that it refers to has been changed,\n"
-	"removed, or renamed in this version of Wings."],
-    Msg4 = [bullet,$\s|"The hotkey refers to a command in a "
-	    " plug-in that is currently disabled,\n"
-	    "or to a previous version of a plug-in."],
-    Msg5 = [bullet,$\s|"A bug in the command itself. Try executing the command\n"
-	    "from the menu directly (i.e. not through a hotkey) -\n"
-	    "if it crashes it IS a bug. (Please report it.)"],
-    Qs = {vframe,
-	  [{label,Msg1},{panel,[]},
+    Msg3 = "The hotkey was defined in a previous version of Wings,"
+	"and the command that it refers to has been changed,"
+	"removed, or renamed in this version of Wings.",
+    Msg4 = "The hotkey refers to a command in a "
+	" plug-in that is currently disabled,"
+	"or to a previous version of a plug-in.",
+    Msg5 = "A bug in the command itself. Try executing the command"
+	"from the menu directly (i.e. not through a hotkey) -"
+	"if it crashes it IS a bug. (Please report it.)",
+    Msg6 = "Delete Hotkey: " ++ KeyName ++ " or press cancel to avoid any changes",
+    Qs = {vframe_dialog,
+	  [{label,Msg1}, separator,
 	   {label,Msg2},
 	   {label,Msg3},
 	   {label,Msg4},
-	   {label,Msg5},{panel,[]},
-	   {hframe,[{button,"Ignore",fun(_) -> ignore end,
-		     [{info,"Do nothing"}]},
-		    {button,"Delete Hotkey",fun(_) -> unbind(Key) end,
-		     [{info,"Delete the hotkey "++KeyName}]}]}]},
-    wings_ask:dialog("", Qs, fun(_) -> ignore end).
+	   {label,Msg5}, separator,
+	   {label,Msg6}],
+	  [{buttons, [ok, cancel], {key, result}}]},
+    wings_dialog:dialog("Delete HotKey", Qs,
+			fun([{result, ok}]) -> unbind(Key);
+			   (_) -> ignore
+			end).
 
 %%%
 %%% Local functions.
@@ -263,41 +430,59 @@ modifiers(Mod, Acc) when Mod band ?KMOD_META =/= 0 ->
     modifiers(Mod bxor Pressed, [command|Acc]);
 modifiers(_, Acc) -> lists:sort(Acc).
 
-keyname({bindkey,Key}) ->
-    keyname(Key);
-keyname({bindkey,_Mode,Key}) ->
-    keyname(Key);
-keyname({C,Mods}) ->
-    modname(Mods) ++ vkeyname(C);
-keyname($\b) -> ?STR(keyname,1,"Bksp");
-keyname($\t) -> ?STR(keyname,2,"Tab");
-keyname($\s) -> ?STR(keyname,3,"Space");
-keyname(C) when $a =< C, C =< $z -> [C-32];
-keyname(C) when $A =< C, C =< $Z ->
-    case get(wings_os_type) of
-	{unix,darwin} -> [shift,C];
-	_ -> ?STR(keyname,4,"Shift+")++ [C]
-    end;
-keyname(C) when is_integer(C), C < 256 -> [C];
-keyname(C) when is_integer(C), 63236 =< C, C =< 63247 ->
-    [$F|integer_to_list(C-63235)];
-keyname(C) -> [$<|integer_to_list(C)++">"].
 
-modname(Mods) ->
-    case get(wings_os_type) of
-	{unix,darwin} -> mac_modname(Mods, []);
-	_ -> modname_1(Mods)
+%%%
+%%% Format hotkeys.
+%%%
+
+modname(Mods, Style) ->
+    case os:type() of
+	{unix,darwin} ->
+	    case Style of
+		wx -> mac_modname_wx(Mods);
+		pretty -> mac_modname(Mods)
+	    end;
+	_ ->
+	    modname_1(Mods)
     end.
 
 modname_1([command|T]) -> "Meta+" ++modname_1(T);
 modname_1([Mod|T]) -> wings_s:modkey(Mod) ++ "+" ++modname_1(T);
 modname_1([]) -> [].
 
-mac_modname([ctrl|T], Acc) -> [$^|mac_modname(T, Acc)];
-mac_modname([shift|T], Acc) -> mac_modname(T, [shift|Acc]);
-mac_modname([alt|T], Acc) -> mac_modname(T, [option|Acc]);
-mac_modname([command|T], Acc) -> mac_modname(T, Acc++[command]);
-mac_modname([], Acc) -> Acc.
+mac_modname_wx([ctrl|T]) -> "rawctrl+" ++ mac_modname_wx(T);
+mac_modname_wx([command|T]) -> "Ctrl+" ++ mac_modname_wx(T);
+mac_modname_wx([alt|T]) -> "Alt+" ++ mac_modname_wx(T);
+mac_modname_wx([shift|T]) -> "Shift+" ++ mac_modname_wx(T);
+mac_modname_wx([]) -> [].
+
+mac_modname(Mods0) ->
+    Mods1 = [{mac_mod_sortkey(M),M} || M <- Mods0],
+    Mods2 = sort(Mods1),
+    [wings_s:mac_mod(M) || {_,M} <- Mods2].
+
+mac_mod_sortkey(shift) -> 1;
+mac_mod_sortkey(alt) -> 2;
+mac_mod_sortkey(ctrl) -> 3;
+mac_mod_sortkey(command) -> 4.
+
+keyname($\b, _Style) -> ?STR(keyname,1,"Bksp");
+keyname($\t, _Style) -> ?STR(keyname,2,"Tab");
+keyname($\s, _Style) -> ?STR(keyname,3,"Space");
+keyname(C, _Style) when $a =< C, C =< $z -> [C-32];
+keyname(C, wx) when $A =< C, C =< $Z ->
+    ?STR(keyname,4,"Shift+") ++ [C];
+keyname(C, pretty) when $A =< C, C =< $Z ->
+    case os:type() of
+	{unix,darwin} ->
+	    wings_s:modkey(shift)++[C];
+	_ ->
+	    ?STR(keyname,4,"Shift+") ++ [C]
+    end;
+keyname(C, _Style) when is_integer(C), C < 256 -> [C];
+keyname(C, _Style) when is_integer(C), 63236 =< C, C =< 63247 ->
+    [$F|integer_to_list(C-63235)];
+keyname(C, _Style) -> [C].
 
 vkeyname(?SDLK_BACKSPACE) -> ?STR(vkeyname,1,"Bksp");
 vkeyname(?SDLK_TAB) -> ?STR(vkeyname,2,"Tab");
@@ -327,7 +512,7 @@ vkeyname(?SDLK_HOME) -> ?STR(vkeyname,20,"Home");
 vkeyname(?SDLK_END) -> ?STR(vkeyname,21,"End");
 vkeyname(?SDLK_PAGEUP) -> ?STR(vkeyname,22,"Page Up");
 vkeyname(?SDLK_PAGEDOWN) ->?STR(vkeyname,23,"Page Down");
-vkeyname(_) -> ?STR(vkeyname,24,"UKEY").
+vkeyname(C) -> [C].
 
 %%%
 %%% Default keybindings.
@@ -352,7 +537,7 @@ set_default() ->
       end, default_keybindings()).
 
 convert_modifiers(Mod) ->
-    case get(wings_os_type) of
+    case os:type() of
 	{unix,darwin} ->
 	    map(fun(ctrl) -> command;
 		   (Other) -> Other end, Mod);
@@ -381,6 +566,14 @@ default_keybindings() ->
      {{?SDLK_F5,[]},        {select,{by,{faces_with,5}}}},
      {{?SDLK_TAB,[]},       {view,workmode}},
      {{?SDLK_TAB,[shift]},  {view,quick_preview}},
+
+     {{?SDLK_INSERT,[ctrl]}, {hotkey, bind}},
+     {{$8, [ctrl, alt]},     {hotkey, bind}},   %% Swedish keyboards
+     {{$8, [ctrl]},          {hotkey, bind}},
+     {{?SDLK_DELETE,[ctrl]}, {hotkey, unbind}},
+     {{$9, [ctrl, alt]},     {hotkey, unbind}}, %% Swedish keyboards
+     {{$9, [ctrl]},          {hotkey, unbind}},
+
      {$\s,              {select,deselect}},
      {$a,               {view,highlight_aim}},
      {$A,               {view,frame}},
