@@ -320,69 +320,60 @@ edit(Id, #st{shapes=Shs}=St) ->
     case Type of
 	ambient ->
 	    {dialog,Qs,Fun} = edit_ambient_dialog(Name, Prop, We, Shs, St),
-	    wings_ask:dialog(?__(2,"Ambient Light Properties"), Qs, Fun);
+	    wings_dialog:dialog(?__(2,"Ambient Light Properties"), Qs, Fun);
 	_ ->
 	    {dialog,Qs,Fun} = edit_dialog(Name, Prop, We, Shs, St),
-	    wings_ask:dialog(?__(3,"Light Properties"), Qs, Fun)
+	    wings_dialog:dialog(?__(3,"Light Properties"), Qs, Fun)
     end.
 
 edit_ambient_dialog(Name, Prop0, 
 		    We0=#we{id=Id,light=#light{ambient=Amb0}=L0}, Shs, St) ->
-    Qs0 = [{hframe,
-	    [{vframe,[{label,?__(1,"Ambient")}]},
-	     {vframe,[{color,Amb0}]}],
-	    [{title,?__(2,"Color")}]}|qs_specific(L0)],
-    Qs1 = wings_plugin:dialog({light_editor_setup,Name,Prop0}, Qs0),
-    Qs = {hframe,[{vframe,Qs1},
-		  {vframe,[{button,?__(3,"OK"),done,
-			    [ok,{key,light_editor_ok}]},
-			   {button,wings_s:cancel(),cancel,[cancel]}]}]},
+    Qs0 = {vframe,
+	   [{hframe,
+	     [{label_column,
+	       [{?__(1,"Ambient"),{color,Amb0}}]}],
+	     [{title,?__(2,"Color")}]}|qs_specific(L0)]},
+    Qs1 = wings_plugin:dialog({light_editor_setup,Name,Prop0}, [{"Wings 3D", Qs0}]),
+    Qs = {vframe_dialog,
+	  [{oframe, Qs1, 1, [{style, buttons}]}],
+	  [{buttons, [ok, cancel]}, {key, result}]},
     Fun = fun([Amb|Res]) ->
-		  case plugin_results(Name, Prop0, Res) of
-		      {ok,Prop} ->
-			  L = L0#light{ambient=Amb,prop=Prop},
-			  We = We0#we{light=L},
-			  St#st{shapes=gb_trees:update(Id, We, Shs)};
-		      {again,Prop} -> edit_ambient_dialog(Name, Prop,
-							  We0, Shs, St)
-		  end
+		  {ok,Prop} = plugin_results(Name, Prop0, Res),
+		  L = L0#light{ambient=Amb,prop=Prop},
+		  We = We0#we{light=L},
+		  St#st{shapes=gb_trees:update(Id, We, Shs)}
 	  end,
     {dialog,Qs,Fun}.
 
 edit_dialog(Name, Prop0, #we{id=Id,light=L0}=We0, Shs, St) ->
     #light{diffuse=Diff0,ambient=Amb0,specular=Spec0} = L0,
-    Qs0 = [{hframe,
-	    [{vframe,
-	      [{label,?__(1,"Diffuse")},
-	       {label,?__(2,"Ambient")},
-	       {label,?__(3,"Specular")}]},
-	     {vframe,
-	      [{color,Diff0},
-	       {color,Amb0},
-	       {color,Spec0}]}],
-	    [{title,?__(4,"Colors")}]}|qs_specific(L0)],
-    Qs1 = wings_plugin:dialog({light_editor_setup,Name,Prop0}, Qs0),
-    Qs = {hframe,[{vframe,Qs1},
-		  {vframe,[{button,?__(5,"OK"),done,[ok,{key,light_editor_ok}]},
-			   {button,wings_s:cancel(),cancel,[cancel]}]}]},
+    Qs0 = {vframe,
+	   [{hframe,
+	     [{label_column,
+	       [{?__(1,"Diffuse"),{color,Diff0}},
+		{?__(2,"Ambient"),{color,Amb0}},
+		{?__(3,"Specular"),{color,Spec0}}]}],
+	     [{title,?__(4,"Colors")}]}|qs_specific(L0)]},
+    Qs1 = wings_plugin:dialog({light_editor_setup,Name,Prop0}, [{"Wings 3D", Qs0}]),
+    Qs = {vframe_dialog,
+ 	  [{oframe, Qs1, 1, [{style, buttons}]}],
+	  [{buttons, [ok, cancel]}, {key, result}]},
     Fun = fun([Diff,Amb,Spec|More0]) ->
 		  L1 = L0#light{diffuse=Diff,ambient=Amb,specular=Spec},
 		  {L,More} = edit_specific(More0, L1),
 		  case plugin_results(Name, Prop0, More) of
 		      {ok,Prop} ->
 			  We = We0#we{light=L#light{prop=Prop}},
-			  St#st{shapes=gb_trees:update(Id, We, Shs)};
-		      {again,Prop} -> edit_dialog(Name, Prop, We0, Shs, St)
+			  St#st{shapes=gb_trees:update(Id, We, Shs)}
+			  %%{again,Prop} -> edit_dialog(Name, Prop, We0, Shs, St)
 		  end
 	  end,
     {dialog,Qs,Fun}.
 
 plugin_results(Name, Prop0, Res0) ->
     case wings_plugin:dialog_result({light_editor_result,Name,Prop0}, Res0) of
-	{Prop,[{light_editor_ok,true}]} ->
+	{Prop,[{result, ok}]} ->
 	    {ok,keydelete(opengl, 1, Prop)};
-	{Prop,[{light_editor_ok,false}]} ->
-	    {again,Prop};
 	{_,Res} ->
 	  io:format(?__(1,
 			"Light editor plugin(s) left garbage:~n    ~P~n"), 
@@ -391,10 +382,11 @@ plugin_results(Name, Prop0, Res0) ->
     end.
 
 qs_specific(#light{type=spot,spot_angle=Angle,spot_exp=SpotExp}=L) ->
-    Spot = [{vframe,[{label,?__(1,"Angle")},
-		     {slider,{text,Angle,[{range,{0.0,89.9}}]}},
-		     {label,?__(2,"Falloff")},
-		     {slider,{text,SpotExp,[{range,{0.0,128.0}}]}}],
+    Spot = [{vframe,
+		[{label_column,
+		    [{?__(1, "Angle"), {slider, {text, Angle, [{range, {0.0, 89.9}}]}}},
+		     {?__(2, "Falloff"), {slider, {text, SpotExp, [{range, {0.0, 128.0}}]}}}]
+		}],
 	     [{title,?__(3,"Spot Parameters")}]}],
     qs_att(L, Spot);
 qs_specific(#light{type=point}=L) -> qs_att(L, []);
@@ -402,10 +394,11 @@ qs_specific(#light{type=area}=L) -> qs_att(L, []);
 qs_specific(_) -> [].
 
 qs_att(#light{lin_att=Lin,quad_att=Quad}, Tail) ->
-    [{vframe,[{label,?__(1,"Linear")},
-	      {slider,{text,Lin,[{range,{0.0,1.0}}]}},
-	      {label,?__(2,"Quadratic")},
-	      {slider,{text,Quad,[{range,{0.0,0.5}}]}}],
+    [{vframe,
+	[{label_column,
+	    [{?__(1,"Linear"),{slider,{text,Lin,[{range,{0.0,1.0}}]}}},
+	     {?__(2,"Quadratic"),{slider,{text,Quad,[{range,{0.0,0.5}}]}}}]
+	}],
       [{title,?__(3,"Attenuation")}]}|Tail].
     
 edit_specific([LinAtt,QuadAtt,Angle,SpotExp|More], #light{type=spot}=L) ->
@@ -471,63 +464,63 @@ update(#dlo{sel=none,src_we=#we{light=#light{}}=We}=D) ->
 update(D) -> D.
 
 update_1(#we{light=#light{type=Type}}=We, #dlo{src_sel=SrcSel}) ->
-    Selected = (SrcSel =/= none),
-    List = gl:genLists(1),
-    gl:newList(List, ?GL_COMPILE),
-    update_2(Type, Selected, We),
-    gl:endList(),
-    List.
+    SelColor = case SrcSel of
+		   none -> {0.0,0.0,1.0};
+		   _ -> wings_pref:get_value(selected_color)
+	       end,
+    update_fun(Type, SelColor, We).
 
-update_2(infinite, Selected, #we{light=#light{aim=Aim}}=We) ->
-    gl:lineWidth(1),
-    set_light_col(We),
-    gl:pushMatrix(),
-    {X,Y,Z} = Pos = light_pos(We),
-    gl:translatef(X, Y, Z),
-    Obj = glu:newQuadric(),
-    glu:quadricDrawStyle(Obj, ?GLU_FILL),
-    glu:quadricNormals(Obj, ?GLU_SMOOTH),
-    glu:sphere(Obj, 0.08, 25, 25),
-    glu:deleteQuadric(Obj),
-    Vec = e3d_vec:norm_sub(Aim, Pos),
-    set_sel_color(Selected),
-    gl:'begin'(?GL_LINES),
-    gl:vertex3fv(e3d_vec:mul(Vec, 0.2)),
-    gl:vertex3fv(e3d_vec:mul(Vec, 0.6)),
-    gl:'end'(),
-    gl:popMatrix();
-update_2(point, Selected, We) ->
-    gl:lineWidth(1),
-    set_light_col(We),
-    gl:pushMatrix(),
-    {X,Y,Z} = light_pos(We),
-    gl:translatef(X, Y, Z),
-    Obj = glu:newQuadric(),
-    glu:quadricDrawStyle(Obj, ?GLU_FILL),
-    glu:quadricNormals(Obj, ?GLU_FLAT),
-    glu:sphere(Obj, 0.08, 25, 25),
-    glu:deleteQuadric(Obj),
-    set_sel_color(Selected),
-    gl:'begin'(?GL_LINES),
-    lines({1.0,0.0,0.0}),
-    lines({0.0,1.0,0.0}),
-    lines({0.0,0.0,1.0}),
-    lines({0.71,0.71,0.0}),
-    lines({0.71,0.0,0.71}),
-    lines({0.0,0.71,0.71}),
-    gl:'end'(),
-    gl:popMatrix();
-update_2(spot, Selected, #we{light=#light{aim=Aim,spot_angle=Angle}}=We) ->
-    gl:lineWidth(1),
-    set_light_col(We),
-    gl:pushMatrix(),
-    Obj = glu:newQuadric(),
-    {Tx,Ty,Tz} = Top = light_pos(We),
-    gl:translatef(Tx, Ty, Tz),
-    glu:quadricDrawStyle(Obj, ?GLU_FILL),
-    glu:quadricNormals(Obj, ?GLU_SMOOTH),
-    glu:sphere(Obj, 0.08, 25, 25),
-    set_sel_color(Selected),
+update_fun(infinite, SelColor, #we{light=#light{aim=Aim}}=We) ->
+    LightPos = light_pos(We),
+    LightCol = get_light_col(We),
+    Vec = e3d_vec:norm_sub(Aim, LightPos),
+    Data = [e3d_vec:mul(Vec, 0.2),e3d_vec:mul(Vec, 0.6)],
+    D = fun() ->
+		gl:lineWidth(1.0),
+		gl:color4fv(LightCol),
+		gl:pushMatrix(),
+		{X,Y,Z} = LightPos,
+		gl:translatef(X, Y, Z),
+		Obj = glu:newQuadric(),
+		glu:quadricDrawStyle(Obj, ?GLU_FILL),
+		glu:quadricNormals(Obj, ?GLU_SMOOTH),
+		glu:sphere(Obj, 0.08, 25, 25),
+		glu:deleteQuadric(Obj),
+		gl:color3fv(SelColor),
+		gl:drawArrays(?GL_LINES, 0, 2),
+		gl:popMatrix()
+	end,
+    wings_vbo:new(D, Data);
+update_fun(point, SelColor, We) ->
+    LightPos = light_pos(We),
+    LightCol = get_light_col(We),
+    Data0 = [{1.0,0.0,0.0},
+	     {0.0,1.0,0.0},
+	     {0.0,0.0,1.0},
+	     {0.71,0.71,0.0},
+	     {0.71,0.0,0.71},
+	     {0.0,0.71,0.71}],
+    N = length(Data0) * 4,
+    Data = lines(Data0),
+    D = fun() ->
+		gl:lineWidth(1.0),
+		gl:color4fv(LightCol),
+		gl:pushMatrix(),
+		{X,Y,Z} = LightPos,
+		gl:translatef(X, Y, Z),
+		Obj = glu:newQuadric(),
+		glu:quadricDrawStyle(Obj, ?GLU_FILL),
+		glu:quadricNormals(Obj, ?GLU_FLAT),
+		glu:sphere(Obj, 0.08, 25, 25),
+		glu:deleteQuadric(Obj),
+		gl:color3fv(SelColor),
+		gl:drawArrays(?GL_LINES, 0, N),
+		gl:popMatrix()
+	end,
+    wings_vbo:new(D, Data);
+update_fun(spot, SelColor, #we{light=#light{aim=Aim,spot_angle=Angle}}=We) ->
+    Top = light_pos(We),
+    LightCol = get_light_col(We),
     SpotDir0 = e3d_vec:norm_sub(Aim, Top),
     SpotDir = case e3d_vec:is_zero(SpotDir0) of
 		  false -> SpotDir0;
@@ -536,28 +529,40 @@ update_2(spot, Selected, #we{light=#light{aim=Aim,spot_angle=Angle}}=We) ->
     Rad = Angle*math:pi()/180,
     R = math:sin(Rad),
     H = math:cos(Rad),
-    {Dx,Dy,Dz} = e3d_vec:mul(SpotDir, H),
-    gl:translatef(Dx, Dy, Dz),
+    Translate = e3d_vec:mul(SpotDir, H),
     Rot = e3d_mat:rotate_s_to_t({0.0,0.0,1.0}, e3d_vec:neg(SpotDir)),
-    gl:multMatrixd(Rot),
-    glu:quadricDrawStyle(Obj, ?GLU_LINE),
-    glu:cylinder(Obj, R, 0.08, H, 12, 1),
-    glu:deleteQuadric(Obj),
-    gl:popMatrix();
-update_2(ambient, _, _) -> ok.
+    fun() ->
+	    gl:lineWidth(1.0),
+	    gl:color4fv(LightCol),
+	    gl:pushMatrix(),
+	    Obj = glu:newQuadric(),
+	    {Tx,Ty,Tz} = Top,
+	    gl:translatef(Tx, Ty, Tz),
+	    glu:quadricDrawStyle(Obj, ?GLU_FILL),
+	    glu:quadricNormals(Obj, ?GLU_SMOOTH),
+	    glu:sphere(Obj, 0.08, 25, 25),
+	    gl:color3fv(SelColor),
+	    {Dx,Dy,Dz} = Translate,
+	    gl:translatef(Dx, Dy, Dz),
+	    gl:multMatrixd(Rot),
+	    glu:quadricDrawStyle(Obj, ?GLU_LINE),
+	    glu:cylinder(Obj, R, 0.08, H, 12, 1),
+	    glu:deleteQuadric(Obj),
+	    gl:popMatrix()
+    end;
+update_fun(ambient, _, _) ->
+    fun() -> ok end.
 
-lines(Vec) ->
-    gl:vertex3fv(e3d_vec:mul(Vec, 0.2)),
-    gl:vertex3fv(e3d_vec:mul(Vec, 0.6)),
-    gl:vertex3fv(e3d_vec:mul(Vec, -0.2)),
-    gl:vertex3fv(e3d_vec:mul(Vec, -0.6)).
+lines([Vec|Vecs]) ->
+    [e3d_vec:mul(Vec, 0.2),
+     e3d_vec:mul(Vec, 0.6),
+     e3d_vec:mul(Vec, -0.2),
+     e3d_vec:mul(Vec, -0.6)|lines(Vecs)];
+lines([]) -> [].
 
-set_light_col(#we{light=#light{diffuse=Diff}}) ->
-    gl:color4fv(Diff).
+get_light_col(#we{light=#light{diffuse=Diff}}) ->
+    Diff.
 
-set_sel_color(false) -> gl:color3f(0, 0, 1);
-set_sel_color(true) -> gl:color3fv(wings_pref:get_value(selected_color)).
-    
 render(#dlo{work=Light}) ->
     wings_dl:call(Light).
 
@@ -882,13 +887,13 @@ setup_light(Lnum, #light{type=ambient,ambient=Amb}, _We, _M) ->
     Lnum;
 setup_light(Lnum, #light{type=infinite,aim=Aim}=L, We, _M) ->
     {X,Y,Z} = e3d_vec:norm_sub(light_pos(We), Aim),
-    gl:lightfv(Lnum, ?GL_POSITION, {X,Y,Z,0}),
+    gl:lightfv(Lnum, ?GL_POSITION, {X,Y,Z,0.0}),
     setup_color(Lnum, L),
     gl:enable(Lnum),
     Lnum+1;
 setup_light(Lnum, #light{type=point}=L, We, _M) ->
     {X,Y,Z} = light_pos(We),
-    gl:lightfv(Lnum, ?GL_POSITION, {X,Y,Z,1}),
+    gl:lightfv(Lnum, ?GL_POSITION, {X,Y,Z,1.0}),
     gl:lightf(Lnum, ?GL_SPOT_CUTOFF, 180.0),
     setup_color(Lnum, L),
     setup_attenuation(Lnum, L),
@@ -898,7 +903,7 @@ setup_light(Lnum, #light{type=spot,aim=Aim,spot_angle=Angle,spot_exp=Exp}=L,
 	    We, _M) ->
     Pos = {X,Y,Z} = light_pos(We),
     Dir = e3d_vec:norm_sub(Aim, Pos),
-    gl:lightfv(Lnum, ?GL_POSITION, {X,Y,Z,1}),
+    gl:lightfv(Lnum, ?GL_POSITION, {X,Y,Z,1.0}),
     gl:lightf(Lnum, ?GL_SPOT_CUTOFF, Angle),
     gl:lightf(Lnum, ?GL_SPOT_EXPONENT, Exp),
     gl:lightfv(Lnum, ?GL_SPOT_DIRECTION, Dir),
