@@ -56,14 +56,13 @@ new(_) ->
     new_1(new).
 
 new_1(Act) ->
-    wings_ask:ask(?__(1,"New Material"),
-		  [{?__(2,"Material Name"),
-		    ?__(3,"New Material")}],
-		  fun([Name]) ->
-			  Action = {action,{material,{Act,Name}}},
-			  wings_wm:send_after_redraw(geom, Action),
-			  ignore
-		  end).
+    wings_dialog:ask(?__(1,"New Material"),
+		     [{?__(2,"Material Name"),?__(3,"New Material")}],
+		     fun([Name]) ->
+			     Action = {action,{material,{Act,Name}}},
+			     wings_wm:send_after_redraw(geom, Action),
+			     ignore
+		     end).
 
 command(new, _) ->
     new_1(assign_new);
@@ -83,6 +82,8 @@ command({duplicate,MatList}, St) ->
     duplicate_material(MatList, St);
 command({delete,MatList}, St) ->
     delete_material(MatList, St);
+command({rename, Old, New}, St) ->
+    rename_1([{list_to_atom(Old),New}], St, []);
 command({rename,MatList0}, St) ->
     case MatList0 -- ["default"] of
 	[] -> St;
@@ -128,10 +129,11 @@ delete_material([], St) ->
 
 rename(Mats, St) ->
     Qs = rename_qs(Mats),
-    wings_ask:dialog(?__(1,"Rename"), Qs,
-		     fun(NewNames) ->
-			     rename_1(NewNames, St, [])
-		     end).
+    wings_dialog:dialog(?__(1,"Rename"), Qs,
+			fun([{_,[]}]) -> ignore;
+			   (NewNames) ->
+				rename_1(NewNames, St, [])
+			end).
 
 rename_1([{Old,New}|Ms], #st{mat=Mat0}=St, Acc) ->
     MatPs = gb_trees:get(Old, Mat0),
@@ -194,35 +196,32 @@ select_material(Mat,SelAct,#st{selmode=SelMode,sel=Sel0,shapes=Shs}=St) ->
 %% sel_rem - remove #we's using Mat from any previous selection
 select_material_1(sel_rem,Sel1,Sel2,Acc) ->
     case Sel1 of
-    false -> Acc; % #we ins't in previous selection - nothing to do
-    _ ->
-        case subtract_sel(Sel1,Sel2) of
-        false -> Acc;
-        {_,GbNew}=Sel3 -> 
-        case gb_sets:is_empty(GbNew) of
-            true -> Acc;
-            _ -> [Sel3|Acc]
-            end
-        end
+	false -> Acc; % #we ins't in previous selection - nothing to do
+	_ ->
+	    {_,GbNew}=Sel3 = subtract_sel(Sel1,Sel2),
+	    case gb_sets:is_empty(GbNew) of
+		true -> Acc;
+		_ -> [Sel3|Acc]
+	    end
     end;
 %% select - select #we's using Mat - the original behavior
 %% sel_add - add #we's using Mat to any previous selection
 select_material_1(SelAct,Sel1,Sel2,Acc) ->
     case Sel2 of
-    false ->
-        case SelAct of
-        select -> Acc;
-        sel_add -> 
-            case concat_sel(Sel1,false) of
-            [] -> Acc;
-            Sel3 -> [Sel3|Acc]  % preserve previous selection of #we
-            end
-        end;
-    Sel2 ->
-        case SelAct of
-        select -> [Sel2|Acc]; % we are building a new selection 
-        sel_add -> [concat_sel(Sel1,Sel2)|Acc] % if so, we will update the previous selection
-        end
+	false ->
+	    case SelAct of
+		select -> Acc;
+		sel_add -> 
+		    case concat_sel(Sel1,false) of
+			[] -> Acc;
+			Sel3 -> [Sel3|Acc]  % preserve previous selection of #we
+		    end
+	    end;
+	Sel2 ->
+	    case SelAct of
+		select -> [Sel2|Acc]; % we are building a new selection 
+		sel_add -> [concat_sel(Sel1,Sel2)|Acc] % if so, we will update the previous selection
+	    end
     end.
 
 %% select the elements (face/edge/vertice) using the Mat
@@ -247,8 +246,6 @@ concat_sel(OldSel,false) -> OldSel;
 concat_sel({Id,GbSetOld},{Id,GbSetNew}) ->
     {Id,gb_sets:union(GbSetOld,GbSetNew)}.
 
-subtract_sel(false,false) -> false;
-subtract_sel(false,_) -> false;
 subtract_sel(OldSel,false) -> OldSel;
 subtract_sel({Id,GbSetOld},{Id,GbSetNew}) ->
     {Id,gb_sets:subtract(GbSetOld,GbSetNew)}.
@@ -320,9 +317,9 @@ add_defaults_1(P) ->
     VertexColor = valid_vertex_color(prop_get(vertex_colors, P, ignore)),
     [{diffuse,norm(prop_get(diffuse, P, Def))},
      {ambient,norm(prop_get(ambient, P, Def))},
-     {specular,norm(prop_get(specular, P, Def))},
-     {emission,norm(prop_get(emission, P, {0.0,0.0,0.0,0.0}))},
-     {shininess,prop_get(shininess, P, 1.0)},
+     {specular,norm(prop_get(specular, P, {0.0,0.0,0.0,1.0}))},
+     {emission,norm(prop_get(emission, P, {0.0,0.0,0.0,1.0}))},
+     {shininess,prop_get(shininess, P, 0.9)},
      {vertex_colors,VertexColor}].
 
 %% For future compatibility, ignore anything that we don't recognize.
@@ -388,7 +385,8 @@ load_map_1(File0, Dir) ->
 		      [File,file:format_error(Error)]),
 	    none
     end.
-    
+
+add(default, _, #st{}=St) -> St;
 add(Name, Mat0, #st{mat=MatTab}=St) ->
     Mat = sort([{K,sort(L)} || {K,L} <- Mat0]),
     case gb_trees:lookup(Name, MatTab) of
@@ -460,7 +458,7 @@ apply_material(Name, Mtab, ActiveVertexColors) when is_atom(Name) ->
 	end,
     gl:materialfv(?GL_FRONT_AND_BACK, ?GL_DIFFUSE, prop_get(diffuse, OpenGL)),
     gl:materialfv(?GL_FRONT_AND_BACK, ?GL_AMBIENT, prop_get(ambient, OpenGL)),
-    apply_texture(prop_get(diffuse, Maps, none)),    
+    apply_texture(prop_get(diffuse, Maps, false)),
     apply_normal_map(get_normal_map(Maps0)),  %% Combine with vertex colors
     DeApply.
 
@@ -478,7 +476,7 @@ shader_texture(What, Enable) ->
 	    ok
     end.
 
-apply_texture(none) -> no_texture();
+apply_texture(false) -> no_texture();
 apply_texture(Image) ->
     case wings_pref:get_value(show_textures) of
 	false -> no_texture();
@@ -583,12 +581,10 @@ is_transparent(Name, Mtab) ->
 
 is_mat_transparent(Mat) ->
     OpenGL = prop_get(opengl, Mat),
-    Trans = foldl(fun(_, true) -> true;
-		   ({emission,_}, _) -> false;
-		     ({_,{_,_,_,1.0}}, _) -> false;
-		     ({_,{_,_,_,_}}, _) -> true;
-		     (_, _) -> false
-		  end, false, OpenGL),
+    Trans = lists:any(fun({diffuse,{_,_,_,Alpha}}) when Alpha < 1.0 -> true;
+                         (_) -> false
+                      end, OpenGL),
+    %% Trans orelse proplists:is_defined(diffuse, prop_get(maps, Mat)).
     Trans.
 
 %% needed_attributes(We, St) -> [Attr]
@@ -641,8 +637,8 @@ needs_tangents(Mat) ->
 edit(Name, Assign, #st{mat=Mtab}=St) ->
     Mat = gb_trees:get(Name, Mtab),
     {dialog,Qs,Fun} = edit_dialog(Name, Assign, St, Mat),
-    wings_ask:dialog(?__(1,"Material Properties: ")++atom_to_list(Name),
-		     Qs, Fun).
+    wings_dialog:dialog(?__(1,"Material Properties: ")++atom_to_list(Name),
+			Qs, Fun).
 
 
 edit_dialog(Name, Assign, St=#st{mat=Mtab0}, Mat0) ->
@@ -653,55 +649,45 @@ edit_dialog(Name, Assign, St=#st{mat=Mtab0}, Mat0) ->
     {Spec0,_} = ask_prop_get(specular, OpenGL0),
     Shine0 = prop_get(shininess, OpenGL0),
     {Emiss0,_} = ask_prop_get(emission, OpenGL0),
-    Maps0 = show_maps(Mat0),
-    Preview = fun(A,S,D,F,G) ->
-		      mat_preview(A,S,D,F,G,prop_get(maps,Mat0))
+    Preview = fun(GLCanvas, Fields) ->
+		      mat_preview(GLCanvas,Fields,prop_get(maps,Mat0))
 	      end,
-    Hook = {hook,fun(is_disabled, {_Var,_I,Sto}) ->
-			 gb_trees:get(vertex_colors, Sto) =/= ignore;
-		    (_, _) -> void
-		 end},
+    Refresh = fun(_Key, _Value, Fields) ->
+		      GLCanvas = wings_dialog:get_widget(preview, Fields),
+		      wxWindow:refresh(GLCanvas)
+	      end,
+    RHook = {hook, Refresh},
     AnyTexture = has_texture(Mat0),
     VtxColMenu = vertex_color_menu(AnyTexture, VertexColors0),
-    Qs1 = [{vframe,
-	    [
-	     {hframe, 
-	      [{custom,?PREVIEW_SIZE,?PREVIEW_SIZE+5,Preview},
-	       {vframe,
-		[{label,?__(1,"Diffuse")},
-		 {label,?__(2,"Ambient")},
-		 {label,?__(3,"Specular")},
-		 {label,?__(4,"Emission")},
-		 {label,"Vertex Colors"}
-		]
-	       },
-	       {vframe,
-		[{slider,{color,Diff0,
-			  [{key,diffuse},Hook]}},
-		 {slider,{color,Amb0,[{key,ambient}]}},
-		 {slider,{color,Spec0,[{key,specular}]}},
-		 {slider,{color,Emiss0,[{key,emission}]}},
-		 VtxColMenu]}]},
-	     {hframe, [{vframe, [{label,?__(5,"Shininess")},
-				 {label,?__(6,"Opacity")}]},
-		       {vframe, [{slider,{text,Shine0,
-					  [{range,{0.0,1.0}},
-					   {key,shininess}]}},
-				 {slider,{text,Opacity0,
-					  [{range,{0.0,1.0}},
-					   {key,opacity}]}}]}]
-	     }|Maps0]
-	   }],
-    Qs2 = wings_plugin:dialog({material_editor_setup,Name,Mat0}, Qs1),
-    Qs = {hframe,[{vframe,Qs2},
-		  {vframe,[{button,?__(7,"OK"),done,[ok,{key,material_editor_ok}]},
-			   {button,wings_s:cancel(),cancel,[cancel]}]}]},
+    Qs1 = {vframe,
+	   [
+	    {hframe,
+	     [{custom_gl,?PREVIEW_SIZE,?PREVIEW_SIZE+5,Preview, [{key, preview}]},
+	      {label_column,
+	       [{?__(1,"Diffuse"), {slider,{color,Diff0, [{key,diffuse},RHook]}}},
+		{?__(2,"Ambient"), {slider,{color,Amb0,[{key,ambient}, RHook]}}},
+		{?__(3,"Specular"),{slider,{color,Spec0,[{key,specular}, RHook]}}},
+		{?__(4,"Emission"),{slider,{color,Emiss0,[{key,emission}, RHook]}}}
+	       ]}]},
+	    {label_column,
+	     [{"Vertex Colors", VtxColMenu},
+	      {?__(5,"Shininess"),
+	       {slider,{text,Shine0, [{range,{0.0,1.0}}, {key,shininess}, RHook]}}},
+	      {?__(6,"Opacity"),
+	       {slider,{text,Opacity0, [{range,{0.0,1.0}}, {key,opacity}, RHook]}}}
+	     ]}
+           ]
+	  },
+    Qs2 = wings_plugin:dialog({material_editor_setup,Name,Mat0}, [{"Wings 3D", Qs1}]),
+    Qs = {vframe_dialog,
+	  [{oframe, Qs2, 1, [{style, buttons}]}],
+	  [{buttons, [ok, cancel]}, {key, result}]},
     Ask = fun([{diffuse,Diff},
 	       {ambient,Amb},
 	       {specular,Spec},
 	       {emission,Emiss},
 	       {vertex_colors,VertexColors},
-	       {shininess,Shine},{opacity,Opacity}|More0]) ->
+	       {shininess,Shine},{opacity,Opacity}|More]) ->
 		  OpenGL = [ask_prop_put(diffuse, Diff, Opacity),
 			    ask_prop_put(ambient, Amb, Opacity),
 			    ask_prop_put(specular, Spec, Opacity),
@@ -709,13 +695,9 @@ edit_dialog(Name, Assign, St=#st{mat=Mtab0}, Mat0) ->
 			    {shininess,Shine},
 			    {vertex_colors,VertexColors}],
 		  Mat1 = keyreplace(opengl, 1, Mat0, {opengl,OpenGL}),
-		  {Mat2,More} = update_maps(Mat1, More0),
-		  case plugin_results(Name, Mat2, More) of
-		      {again,Mat} -> edit_dialog(Name, Assign, St, Mat);
-		      {ok,Mat}  ->
-			  Mtab = gb_trees:update(Name, Mat, Mtab0),
-			  maybe_assign(Assign, Name, St#st{mat=Mtab})
-		  end
+		  {ok,Mat} =  plugin_results(Name, Mat1, More),
+		  Mtab = gb_trees:update(Name, Mat, Mtab0),
+		  maybe_assign(Assign, Name, St#st{mat=Mtab})
 	  end,
     {dialog,Qs,Ask}.
 
@@ -741,48 +723,13 @@ maybe_assign(true, Name, St) -> set_material(Name, St).
 
 plugin_results(Name, Mat0, Res0) ->
     case wings_plugin:dialog_result({material_editor_result,Name,Mat0}, Res0) of
-	{Mat,[{material_editor_ok,true}]} ->
-	    {ok,Mat};
-	{Mat,[{material_editor_ok,false}]} ->
-	    {again,Mat};
+	{Mat,[{result,ok}]} -> {ok,Mat};
 	{_,Res} ->
-	    io:format(?__(1,"Material editor plugin(s) left garbage:~n    ~P~n"), 
+	    io:format(?__(1,"Material editor plugin(s) left garbage:~n    ~P~n"),
 		      [Res,20]),
-		      wings_u:error_msg(?__(2,"Plugin(s) left garbage"))
+            wings_u:error_msg(?__(2,"Plugin(s) left garbage"))
     end.
 
-update_maps(Mat0, More0) ->
-    Maps0 = sort(prop_get(maps, Mat0)),
-    {Maps,More} = update_maps_1(More0, Maps0, []),
-    Mat = [{maps,Maps}|keydelete(maps, 1, Mat0)],
-    {Mat,More}.
-
-update_maps_1([false|More], [M|Maps], Acc) ->
-    update_maps_1(More, Maps, [M|Acc]);
-update_maps_1([true|More], [_|Maps], Acc) ->
-    update_maps_1(More, Maps, Acc);
-update_maps_1(More, [], Acc) -> {Acc,More}.
-
-show_maps(Mat) ->
-    case prop_get(maps, Mat) of
-	[] -> [];
-	Maps ->
-	    MapDisp = [show_map(M) || M <- sort(Maps)],
-	    [{vframe,MapDisp,[{title,?__(1,"Textures")}]}]
-    end.
-
-show_map({Type,Image}) ->
-    Texture = 
-	case wings_image:info(Image) of
-	    none ->
-		[{label,flatten(io_lib:format(?__(1,"~p: <image deleted>"), [Type]))}];
-	    #e3d_image{name=Name,width=W,height=H,bytes_pp=PP} ->
-		Label = flatten(io_lib:format(?__(2,"~p: ~p [~px~px~p]"),
-					      [Type,Name,W,H,PP*8])),
-		[{label, Label},{button,?__(3,"Delete"),done}]
-	end,
-    {hframe, Texture}.
-
 ask_prop_get(Key, Props) ->
     {R,G,B,Alpha} = prop_get(Key, Props),
     {{R,G,B},Alpha}.
@@ -791,30 +738,30 @@ ask_prop_put(specular=Key, {R,G,B}, _) ->
     {Key,{R,G,B,1.0}};
 ask_prop_put(Key, {R,G,B}, Opacity) ->
     {Key,{R,G,B,Opacity}}.
-    
-mat_preview(X, Y, _W, _H, Common, Maps) ->
-    wings_io:border(X, Y, ?PREVIEW_SIZE, ?PREVIEW_SIZE, ?PANE_COLOR),
-    MM = gl:getDoublev(?GL_MODELVIEW_MATRIX),
-    PM = gl:getDoublev(?GL_PROJECTION_MATRIX),
-    ViewPort = wings_wm:viewport(),
-    {Ox,Oy,_} = wings_gl:project(X, Y+?PREVIEW_SIZE, 0, MM, PM, ViewPort),
+
+mat_preview(Canvas, Common, Maps) ->
     gl:pushAttrib(?GL_ALL_ATTRIB_BITS),
-    gl:viewport(trunc(Ox), trunc(Oy), ?PREVIEW_SIZE, ?PREVIEW_SIZE),
+    gl:viewport(0, 0, ?PREVIEW_SIZE, ?PREVIEW_SIZE),
+    {BR,BG,BB, _} = wxWindow:getBackgroundColour(wxWindow:getParent(Canvas)),
+    %% wxSystemSettings:getColour(?wxSYS_COLOUR_BACKGROUND),
+    BGC = fun(Col) -> (Col-15) / 255 end,
+    gl:clearColor(BGC(BR),BGC(BG),BGC(BB),1.0),
+    gl:clear(?GL_COLOR_BUFFER_BIT bor ?GL_DEPTH_BUFFER_BIT),
     gl:matrixMode(?GL_PROJECTION),
     gl:pushMatrix(),
     gl:loadIdentity(),
-    glu:perspective(60.0, 1, 0.01, 256.0),
+    glu:perspective(60.0, 1.0, 0.01, 256.0),
     gl:matrixMode(?GL_MODELVIEW),
     gl:pushMatrix(),
     gl:loadIdentity(),
     gl:translatef(0.0, 0.0, -2.0),
     wings_light:camera_lights(mat_preview),
     gl:shadeModel(?GL_SMOOTH),
-    Alpha = gb_trees:get(opacity, Common),
-    Amb = preview_mat(ambient, Common, Alpha),
-    Diff = preview_mat(diffuse, Common, Alpha),
-    Spec = preview_mat(specular, Common, Alpha),
-    Shine = gb_trees:get(shininess, Common),
+    Alpha = wings_dialog:get_value(opacity, Common),
+    Amb   = preview_mat(ambient, Common, Alpha),
+    Diff  = preview_mat(diffuse, Common, Alpha),
+    Spec  = preview_mat(specular, Common, Alpha),
+    Shine = wings_dialog:get_value(shininess, Common),
     gl:materialf(?GL_FRONT, ?GL_SHININESS, Shine*128.0),
     gl:materialfv(?GL_FRONT, ?GL_AMBIENT, Amb),
     gl:materialfv(?GL_FRONT, ?GL_DIFFUSE, Diff),
@@ -822,15 +769,16 @@ mat_preview(X, Y, _W, _H, Common, Maps) ->
     gl:blendFunc(?GL_SRC_ALPHA, ?GL_ONE_MINUS_SRC_ALPHA),
     gl:enable(?GL_LIGHTING),
     gl:enable(?GL_BLEND),
+    gl:enable(?GL_DEPTH_TEST),
     gl:enable(?GL_CULL_FACE),
-    gl:rotatef(-90,1,0,0),
+    gl:rotatef(-90.0,1.0,0.0,0.0),
     Obj = glu:newQuadric(),
     glu:quadricDrawStyle(Obj, ?GLU_FILL),
     glu:quadricNormals(Obj, ?GLU_SMOOTH),
-    case apply_texture(prop_get(diffuse, Maps, none)) of
-	true -> 
+    case apply_texture(prop_get(diffuse, Maps, false)) of
+	true ->
 	    glu:quadricTexture(Obj, ?GLU_TRUE);
-	false -> 
+	false ->
 	    ignore
     end,
     glu:sphere(Obj, 0.9, 50, 50),
@@ -846,9 +794,9 @@ mat_preview(X, Y, _W, _H, Common, Maps) ->
     gl:popAttrib().
 
 preview_mat(Key, Colors, Alpha) ->
-    {R,G,B} = gb_trees:get(Key, Colors),
+    {R,G,B} = wings_dialog:get_value(Key, Colors),
     {R,G,B,Alpha}.
-    
+
 %%% Return color in texture for the given UV coordinates.
 
 color(Face, UV, We, #st{mat=Mtab}) ->
