@@ -14,7 +14,7 @@
 -module(wings_pick).
 -export([event/2,event/3,hilite_event/3, hilite_event/4]).
 -export([do_pick/3,raw_pick/3]).
--export([marquee_pick/3,paint_pick/3]).
+-export([paint_pick/3]).
 
 -define(NEED_OPENGL, 1).
 -define(NEED_ESDL, 1).
@@ -32,8 +32,8 @@
 
 %% For marquee picking.
 -record(marquee,
-	{ox,oy,					%Original X,Y.
-	 cx,cy,					%Current X,Y.
+	{o, 					% start pos
+	 overlay,				% overlay win
 	 st
 	}).
 
@@ -56,7 +56,7 @@ event(#mousemotion{}=Mm, #st{selmode=Mode}=St, Redraw) ->
     end;
 event(#mousebutton{button=1,x=X,y=Y,mod=Mod,state=?SDL_PRESSED}, St, _)
         when Mod band (?SHIFT_BITS bor ?CTRL_BITS) =/= 0 ->
-    marquee_pick(X, Y, St);
+    setup_marquee(X, Y, St);
 event(#mousebutton{button=1,x=X,y=Y,state=?SDL_PRESSED}, St, _) ->
     paint_pick(X, Y, St);
 event(_, _, _) -> next.
@@ -77,14 +77,10 @@ hilite_event(_, _, _,_) -> next.
 hilite_enabled(body) -> wings_pref:get_value(body_hilite);
 hilite_enabled(_) -> true.
 
-marquee_pick(X, Y, St)  ->
-    Pick = #marquee{ox=X,oy=Y,st=St},
-    clear_hilite_marquee_mode(Pick).
 paint_pick(X, Y, St0) ->
     case do_pick(X, Y, St0) of
 	none ->
-	    Pick = #marquee{ox=X,oy=Y,st=St0},
-	    clear_hilite_marquee_mode(Pick);
+	    setup_marquee(X, Y, St0);
 	{PickOp,_,St} ->
 	    wings_wm:grab_focus(),
 	    wings_wm:dirty(),
@@ -93,6 +89,11 @@ paint_pick(X, Y, St0) ->
 	    {seq,push,get_pick_event(Pick)}
     end.
 
+setup_marquee(X,Y, St) ->
+    OL = wings_frame:get_overlay(),
+    Pick = #marquee{o=wings_wm:local2screen({X,Y}),st=St, overlay=OL},
+    clear_hilite_marquee_mode(Pick).
+
 %%
 %% Highlighting on mouse move.
 %%
@@ -235,28 +236,35 @@ accept_hl(Fun, Hit) when is_function(Fun) ->
 accept_hl(_,_) -> true.
 
 insert_hilite_dl(Hit, St) ->
-    wings_dl:map(fun(D, _) ->
-			 insert_hilite_dl_1(D, Hit, St)
-			end, []).
+    Extra = fun(_) -> [] end,
+    insert_hilite_dl(Hit, Extra, St).
 
-insert_hilite_dl_1(#dlo{src_we=We}=D, _, _) when ?IS_LIGHT(We) -> D;
-insert_hilite_dl_1(#dlo{open=Open,src_we=#we{id=Id}}=D, {Mode,_,{Id,Item}=Hit}, St) ->
-    List = gl:genLists(1),
-    gl:newList(List, ?GL_COMPILE),
-    hilite_color(Hit, St),
-    Pref = wings_pref:get_value(show_backfaces),
-    case wings_wm:lookup_prop(select_backface) of
-	Value when Value=:={value,true}; Pref andalso Open ->
-	    gl:disable(?GL_CULL_FACE),
-	    hilit_draw_sel(Mode, Item, D),
-	    gl:enable(?GL_CULL_FACE);
-	_ ->
-	    hilit_draw_sel(Mode, Item, D)
-    end,
-    gl:endList(),
-    D#dlo{hilite=List};
-insert_hilite_dl_1(#dlo{hilite=none}=D, _, _) -> D;
-insert_hilite_dl_1(D, _, _) -> D#dlo{hilite=none}.
+insert_hilite_dl(Hit, Extra, St) ->
+    wings_dl:map(fun(D, _) ->
+			 insert_hilite_dl_1(D, Hit, Extra, St)
+		 end, []).
+
+insert_hilite_dl_1(#dlo{src_we=We}=D, _, _, _) when ?IS_LIGHT(We) -> D;
+insert_hilite_dl_1(#dlo{open=Open,src_we=#we{id=Id}=We}=D,
+		   {Mode,_,{Id,Item}=Hit}, Extra, St) ->
+    DrawExtra = Extra(We),
+    HiliteColor = hilite_color(Hit, St),
+    DrawHilite = hilite_draw_sel_fun(Mode, Item, D),
+    Draw0 = [fun() -> gl:color3fv(HiliteColor) end,
+	     DrawHilite,DrawExtra],
+    ShowBack = wings_pref:get_value(show_backfaces),
+    SelBack = wings_wm:lookup_prop(select_backface) =:= {value,true},
+    Draw = if
+	       SelBack; ShowBack andalso Open ->
+		   [fun() -> gl:disable(?GL_CULL_FACE) end,
+		    Draw0,
+		    fun() -> gl:enable(?GL_CULL_FACE) end];
+	       true ->
+		   Draw0
+	   end,
+    D#dlo{hilite={Mode, Draw}};
+insert_hilite_dl_1(#dlo{hilite=none}=D, _, _, _) -> D;
+insert_hilite_dl_1(D, _, _, _) -> D#dlo{hilite=none}.
 
 tweak_vector() ->
     Draw = wings_wm:lookup_prop(tweak_draw) =:= {value,true},
@@ -278,36 +286,19 @@ tweak_vector() ->
       _other -> false
     end.
 
-insert_tweak_vector(Hit, Hit0, St) ->
-    wings_dl:map(fun(D, _) ->
-			  insert_tweak_vector_1(D, Hit, Hit0, St)
-			end, []).
-
-insert_tweak_vector_1(#dlo{src_we=We}=D, _, _, _) when ?IS_LIGHT(We) -> D;
-insert_tweak_vector_1(#dlo{src_we=#we{id=Id}=We}=D, {Mode,_,{Id,Item}=Hit},
-  {Mode0,_,{Id,Item0}}, St) ->
-    List = gl:genLists(1),
-    gl:newList(List, ?GL_COMPILE),
-    hilite_color(Hit, St),
-    {Normal,ENorm,Center} = wings_tweak:point_center(Mode0, Item0, We),
-    Norm = case wings_pref:get_value(tweak_axis) of
-      element_normal_edge -> ENorm;
-      _ -> Normal
-    end,
-	case wings_wm:lookup_prop(select_backface) of
-	{value,true} ->
-	    gl:disable(?GL_CULL_FACE),
-	    hilit_draw_sel(Mode, Item, D),
-	    draw_tweak_vector(Center, Normal),
-	    gl:enable(?GL_CULL_FACE);
-	_ ->
-	    hilit_draw_sel(Mode, Item, D),
-	    draw_tweak_vector(Center, Norm)
-    end,
-    gl:endList(),
-    D#dlo{hilite=List};
-insert_tweak_vector_1(#dlo{hilite=none}=D, _, _, _) -> D;
-insert_tweak_vector_1(D, _, _, _) -> D#dlo{hilite=none}.
+insert_tweak_vector(Hit, {Mode,_,{_,Item}}, St) ->
+    UseEdgeNormal = wings_pref:get_value(tweak_axis) =:=
+	element_normal_edge,
+    F = fun(We) ->
+		{Normal,EdgeNormal,Center} =
+		    wings_tweak:point_center(Mode, Item, We),
+		N = case UseEdgeNormal of
+			true -> EdgeNormal;
+			false -> Normal
+		    end,
+		draw_tweak_vector_fun(Center, N)
+	end,
+    insert_hilite_dl(Hit, F, St).
 
 hilite_color({Id,Item}, #st{sel=Sel}) ->
     Key = case keyfind(Id, 1, Sel) of
@@ -318,52 +309,60 @@ hilite_color({Id,Item}, #st{sel=Sel}) ->
 		      true -> selected_hlite
 		  end
 	  end,
-    gl:color3fv(wings_pref:get_value(Key)).
+    wings_pref:get_value(Key).
 
-draw_tweak_vector(Center, Normal) ->
+draw_tweak_vector_fun(Center, Normal) ->
     Length = wings_pref:get_value(tweak_vector_size),
     Width = wings_pref:get_value(tweak_vector_width),
     Color = wings_pref:get_value(tweak_vector_color),
     Point = e3d_vec:add_prod(Center, Normal, Length),
-    gl:color3fv(Color),
-    gl:lineWidth(Width),
-    gl:'begin'(?GL_LINES),
-    gl:vertex3fv(Center),
-    gl:vertex3fv(Point),
-    gl:'end'(),
-    gl:pointSize(Width+2),
-    gl:'begin'(?GL_POINTS),
-    gl:vertex3fv(Point),
-    gl:'end'().
+    Data = [Center,Point],
+    D = fun() ->
+		gl:lineWidth(Width),
+		gl:pointSize(Width+2),
+		gl:color3fv(Color),
+		gl:drawArrays(?GL_LINES, 0, 2),
+		gl:drawArrays(?GL_POINTS, 1, 1)
+	end,
+    wings_vbo:new(D, Data).
 
-hilit_draw_sel(vertex, V, #dlo{src_we=#we{vp=Vtab}}) ->
-    gl:pointSize(wings_pref:get_value(selected_vertex_size)),
-    gl:'begin'(?GL_POINTS),
-    gl:vertex3fv(array:get(V, Vtab)),
-    gl:'end'();
-hilit_draw_sel(edge, Edge, #dlo{src_we=#we{es=Etab,vp=Vtab}}) ->
+hilite_draw_sel_fun(vertex, V, #dlo{src_we=#we{vp=Vtab}}) ->
+    PointSize = wings_pref:get_value(selected_vertex_size),
+    Data = [array:get(V, Vtab)],
+    D = fun() ->
+		gl:pointSize(PointSize),
+		gl:drawArrays(?GL_POINTS, 0, 1)
+	end,
+    wings_vbo:new(D, Data);
+hilite_draw_sel_fun(edge, Edge, #dlo{src_we=#we{es=Etab,vp=Vtab}}) ->
     #edge{vs=Va,ve=Vb} = array:get(Edge, Etab),
-    gl:lineWidth(wings_pref:get_value(selected_edge_width)),
-    gl:'begin'(?GL_LINES),
-    gl:vertex3fv(array:get(Va, Vtab)),
-    gl:vertex3fv(array:get(Vb, Vtab)),
-    gl:'end'();
-hilit_draw_sel(face, Face, #dlo{vab=#vab{face_map=Map, face_vs=Vs}}) ->
-    gl:enable(?GL_POLYGON_STIPPLE),
-    gl:polygonMode(?GL_FRONT_AND_BACK, ?GL_FILL),
-    wings_draw_setup:enableVertexPointer(Vs),
+    LineWidth = wings_pref:get_value(selected_edge_width),
+    Data = [array:get(Va, Vtab),array:get(Vb, Vtab)],
+    D = fun() ->
+		gl:lineWidth(LineWidth),
+		gl:drawArrays(?GL_LINES, 0, 2)
+	end,
+    wings_vbo:new(D, Data);
+hilite_draw_sel_fun(face, Face, #dlo{vab=#vab{face_map=Map}=Vab}) ->
     {Start,NoElements} = array:get(Face, Map),
-    gl:drawArrays(?GL_TRIANGLES, Start, NoElements),
-    wings_draw_setup:disableVertexPointer(Vs),
-    gl:disable(?GL_POLYGON_STIPPLE);
-hilit_draw_sel(body, _, #dlo{vab=#vab{face_vs=Vs}}=D) ->
-    gl:enable(?GL_POLYGON_STIPPLE),
-    gl:polygonMode(?GL_FRONT_AND_BACK, ?GL_FILL),
-    wings_draw_setup:enableVertexPointer(Vs),
-    Count = wings_draw_setup:face_vertex_count(D),
-    gl:drawArrays(?GL_TRIANGLES, 0, Count),
-    wings_draw_setup:disableVertexPointer(Vs),
-    gl:disable(?GL_POLYGON_STIPPLE).
+    fun() ->
+	    gl:enable(?GL_POLYGON_STIPPLE),
+	    gl:polygonMode(?GL_FRONT_AND_BACK, ?GL_FILL),
+	    wings_draw_setup:enable_pointers(Vab, []),
+	    gl:drawArrays(?GL_TRIANGLES, Start, NoElements),
+	    wings_draw_setup:disable_pointers(Vab, []),
+	    gl:disable(?GL_POLYGON_STIPPLE)
+    end;
+hilite_draw_sel_fun(body, _, #dlo{vab=#vab{}=Vab}=D) ->
+    fun() ->
+	    gl:enable(?GL_POLYGON_STIPPLE),
+	    gl:polygonMode(?GL_FRONT_AND_BACK, ?GL_FILL),
+	    wings_draw_setup:enable_pointers(Vab, []),
+	    Count = wings_draw_setup:face_vertex_count(D),
+	    gl:drawArrays(?GL_TRIANGLES, 0, Count),
+	    wings_draw_setup:disable_pointers(Vab, []),
+	    gl:disable(?GL_POLYGON_STIPPLE)
+    end.
 
 enhanced_hl_info(Base,#hl{redraw=#st{sel=[],shapes=Shs},prev=Prev}) when is_tuple(Prev) ->
     case Prev of
@@ -427,10 +426,8 @@ clear_hilite_marquee_mode(#marquee{st=St}=Pick) ->
 	     wings_wm:later(now_enter_marquee_mode),
 	     keep;
 	(now_enter_marquee_mode) ->
+	     wings_io:grab(),
 	     wings_wm:grab_focus(wings_wm:this()),
-	     wings_io:ortho_setup(),
-	     gl:flush(),
-	     gl:drawBuffer(?GL_FRONT),
 	     get_marquee_event(Pick);
 	(Ev) ->
 	     wings_io:putback_event(Ev),
@@ -451,19 +448,17 @@ marquee_message() ->
 get_marquee_event(Pick) ->
     {replace,fun(Ev) -> marquee_event(Ev, Pick) end}.
 
-marquee_event(redraw, #marquee{cx=Cx,cy=Cy,st=St}=M) ->
-    gl:drawBuffer(?GL_BACK),
+marquee_event(redraw, #marquee{st=St}) ->
     wings:redraw(St),
-    gl:drawBuffer(?GL_FRONT),
-    wings_io:ortho_setup(),
-    draw_marquee(Cx, Cy, M),
     keep;
-marquee_event(init_opengl, #marquee{st=St}) ->
-    wings:init_opengl(St);
-marquee_event(#mousemotion{x=X,y=Y}, #marquee{cx=Cx,cy=Cy}=M) ->
-    draw_marquee(Cx, Cy, M),
-    draw_marquee(X, Y, M),
-    get_marquee_event(M#marquee{cx=X,cy=Y});
+marquee_event(#mousemotion{x=X0,y=Y0}, #marquee{o={OX,OY}, overlay=OL}) ->
+    {X,Y} = wings_wm:local2screen({X0,Y0}),
+    MinX = min(X,OX),
+    MinY = min(Y,OY),
+    W = max(X,OX) - MinX,
+    H = max(Y,OY) - MinY,
+    wings_frame:overlay_draw(OL, {MinX,MinY,W,H}, 170),
+    keep;
 marquee_event(#mousebutton{x=X0,y=Y0,mod=Mod,button=1,state=?SDL_RELEASED}, M) ->
     {Inside,Op} =
 	if
@@ -476,8 +471,8 @@ marquee_event(#mousebutton{x=X0,y=Y0,mod=Mod,button=1,state=?SDL_RELEASED}, M) -
 	    true ->
 		{false,add}
 	end,
-    #marquee{ox=Ox,oy=Oy,st=St0} = M,
-    gl:drawBuffer(?GL_BACK),
+    #marquee{o=Orig,st=St0,overlay=OL} = M,
+    {Ox,Oy} = wings_wm:screen2local(Orig),
     X = (Ox+X0)/2.0,
     Y = (Oy+Y0)/2.0,
     W = abs(Ox-X)*2.0,
@@ -488,6 +483,8 @@ marquee_event(#mousebutton{x=X0,y=Y0,mod=Mod,button=1,state=?SDL_RELEASED}, M) -
 	    St = marquee_update_sel(Op, Hits, St0),
 	    wings_wm:later({new_state,St})
     end,
+    wings_frame:overlay_hide(OL),
+    wings_io:ungrab(X0,Y0),
     wings_wm:release_focus(),
     wings_wm:later(revert_state),
     wings_wm:dirty_mode(back),
@@ -573,20 +570,6 @@ is_inside_rect({Px,Py,Pz}, {MM,PM,ViewPort,X1,Y1,X2,Y2}) ->
     X1 < Sx andalso Sx < X2 andalso
 	Y1 < Sy andalso Sy < Y2.
 
-draw_marquee(undefined, undefined, _) -> ok;
-draw_marquee(X, Y, #marquee{ox=Ox,oy=Oy}) ->
-    gl:color3f(1, 1, 1),
-    gl:enable(?GL_COLOR_LOGIC_OP),
-    gl:logicOp(?GL_XOR),
-    gl:'begin'(?GL_LINE_LOOP),
-    gl:vertex2i(X, Oy),
-    gl:vertex2i(X, Y),
-    gl:vertex2i(Ox, Y),
-    gl:vertex2i(Ox, Oy),
-    gl:'end'(),
-    gl:flush(),
-    gl:disable(?GL_COLOR_LOGIC_OP).
-
 marquee_update_sel(Op, Hits0, #st{selmode=body}=St) ->
     Hits1 = sofs:relation(Hits0, [{id,data}]),
     Hits2 = sofs:domain(Hits1),
@@ -672,8 +655,8 @@ raw_pick(X0, Y0, St) ->
     {W,H} = wings_wm:win_size(),
     X = float(X0),
     Y = H-float(Y0),
-    S = 5,
-    set_pick_matrix(X, Y, S, S, W, H),
+    S = 5.0,
+    Ms = set_pick_matrix(X, Y, S, S, W, H),
     case wings_wm:lookup_prop(select_backface) of
 	{value,true} ->
 	    %% Only in AutoUV windows.
@@ -681,23 +664,27 @@ raw_pick(X0, Y0, St) ->
 	_ ->
 	    wpc_pick:cull(true)
     end,
-    case dlo_pick(St, true) of
+    case dlo_pick(St, true, Ms) of
 	[] -> none;
 	[{Id,Face}] -> convert_hit(Id, Face, X, Y, St)
     end.
 
 set_pick_matrix(X, Y, Xs, Ys, W, H) ->
-    gl:matrixMode(?GL_PROJECTION),
-    gl:loadIdentity(),
-    wpc_pick:pick_matrix(X, Y, Xs, Ys, {0,0,W,H}),
-    wings_view:projection(),
-    wings_view:modelview(),
-    set_pick_matrix().
+    Pick = wpc_pick:pick_matrix(X, Y, Xs, Ys, {0,0,W,H}),
+    ProjMatrix = wings_view:projection(Pick),
+    {_, ModelMatrix} = wings_view:modelview(),
+    set_pick_matrix_1(e3d_transform:matrix(ProjMatrix),
+		      e3d_transform:matrix(ModelMatrix)).
 
-set_pick_matrix() ->
-    ModelMatrix = gl:getDoublev(?GL_MODELVIEW_MATRIX),
-    ProjMatrix = gl:getDoublev(?GL_PROJECTION_MATRIX),
-    wpc_pick:matrix(ModelMatrix, ProjMatrix).
+set_pick_matrix({PM, MM}) ->
+    set_pick_matrix_1(PM, MM).
+
+set_pick_matrix({PM, MM}, PostModel) ->
+    set_pick_matrix_1(PM, e3d_mat:mul(MM, PostModel)).
+
+set_pick_matrix_1(ProjMatrix, ModelMatrix) ->
+    wpc_pick:matrix(ModelMatrix, ProjMatrix),
+    {ProjMatrix, ModelMatrix}.
 
 %% update_selection({Mode,MM,{Id,Item}}, St0) ->
 %%                 {add|delete,{Id,Item,MM},St0}.
@@ -878,17 +865,17 @@ pick_all(_DrawFaces, _X, _Y, W, H, St) when W < 1.0; H < 1.0 ->
 pick_all(DrawFaces, X, Y0, W, H, St) ->
     {Ww,Wh} = wings_wm:win_size(),
     Y = Wh-Y0,
-    set_pick_matrix(X, Y, W, H, Ww, Wh),
+    Ms = set_pick_matrix(X, Y, W, H, Ww, Wh),
     case DrawFaces of
 	true ->
 	    wpc_pick:cull(true),
-	    {dlo_pick(St, false),St};
+	    {dlo_pick(St, false, Ms),St};
 	false ->
 	    wpc_pick:cull(false),
-	    {marquee_pick(St),St}
+	    {marquee_pick(St, Ms),St}
     end.
 
-marquee_pick(#st{selmode=edge}) ->
+marquee_pick(#st{selmode=edge}, Ms) ->
     PickFun = fun(#we{es=Etab,vp=Vtab,id=Id}=We, Acc) ->
 		      Vis = gb_sets:from_ordset(wings_we:visible(We)),
 		      EsPos = visible_edges(array:sparse_to_orddict(Etab),
@@ -899,8 +886,8 @@ marquee_pick(#st{selmode=edge}) ->
 			  Picked -> [{Id,E} || E <- Picked] ++ Acc
 		      end
 	      end,
-    setup_pick_context(PickFun);
-marquee_pick(#st{selmode=vertex}) ->
+    setup_pick_context(PickFun, Ms);
+marquee_pick(#st{selmode=vertex}, Ms) ->
     PickFun = fun(#we{vp=Vtab,id=Id}=We, Acc) ->
 		      VsPos = case wings_we:is_open(We) of
 				  false ->
@@ -916,8 +903,8 @@ marquee_pick(#st{selmode=vertex}) ->
 			      [{Id,V} || V <- Picked] ++ Acc
 		      end
 	      end,
-    setup_pick_context(PickFun);
-marquee_pick(St) -> dlo_pick(St, false).
+    setup_pick_context(PickFun, Ms);
+marquee_pick(St, Ms) -> dlo_pick(St, false, Ms).
 
 visible_edges([{Edge,#edge{vs=Va,ve=Vb,lf=Lf,rf=Rf}}|Es], Vtab, Vis, Acc) ->
     case gb_sets:is_member(Lf, Vis) orelse gb_sets:is_member(Rf, Vis) of
@@ -929,36 +916,33 @@ visible_edges([{Edge,#edge{vs=Va,ve=Vb,lf=Lf,rf=Rf}}|Es], Vtab, Vis, Acc) ->
     end;
 visible_edges([], _, _, Acc) -> Acc.
 
-setup_pick_context(PickFun) ->
+setup_pick_context(PickFun, Ms) ->
     Res = wings_dl:fold(fun(D, Acc) ->
-				setup_pick_context_fun(D, PickFun, Acc)
+				setup_pick_context_fun(D, PickFun, Ms, Acc)
 			end, []),
     sort(Res).
 
-setup_pick_context_fun(#dlo{src_we=#we{perm=Perm}}, _PickFun, Acc)
+setup_pick_context_fun(#dlo{src_we=#we{perm=Perm}}, _PickFun, _, Acc)
   when ?IS_NOT_SELECTABLE(Perm) ->
     Acc;
-setup_pick_context_fun(#dlo{mirror=none,src_we=We}, PickFun, Acc) ->
+setup_pick_context_fun(#dlo{mirror=none,src_we=We}, PickFun, _, Acc) ->
     PickFun(We, Acc);
-setup_pick_context_fun(#dlo{mirror=Matrix,src_we=We}, PickFun, Acc0) ->
+setup_pick_context_fun(#dlo{mirror=Mirror,src_we=We}, PickFun, Ms, Acc0) ->
     Acc1 = PickFun(We, Acc0),
-    gl:pushMatrix(),
-    gl:multMatrixf(Matrix),
-    set_pick_matrix(),
+    set_pick_matrix(Ms, Mirror),
     wpc_pick:front_face(cw),
     Acc = PickFun(We, Acc1),
     wpc_pick:front_face(ccw),
-    gl:popMatrix(),
-    set_pick_matrix(),
+    set_pick_matrix(Ms),
     Acc.
 
 %%
 %% Draw for the purpose of picking the items that the user clicked on.
 %%
 
-dlo_pick(St, OneHit) ->
+dlo_pick(St, OneHit, Matrices) ->
     Hits0 = wings_dl:map(fun(D, Acc) ->
-				 do_dlo_pick(D, St, OneHit, Acc)
+				 do_dlo_pick(D, St, OneHit, Matrices, Acc)
 			 end, []),
     case Hits0 of
 	{_,Hits} ->
@@ -969,26 +953,26 @@ dlo_pick(St, OneHit) ->
 	    usort(Hits0)
     end.
 
-do_dlo_pick(#dlo{src_we=#we{perm=Perm}}=D, _St, _OneHit, Acc)
+do_dlo_pick(#dlo{src_we=#we{perm=Perm}}=D, _St, _OneHit, _Ms, Acc)
   when ?IS_NOT_SELECTABLE(Perm) ->
     {D,Acc};
-do_dlo_pick(D=#dlo{vab=none}, St, OneHit, Acc) ->
-    do_dlo_pick(wings_draw_setup:work(D, St), St, OneHit, Acc);
-do_dlo_pick(D=#dlo{vab=#vab{face_vs=none}}, St, OneHit, Acc) ->
-    do_dlo_pick(wings_draw_setup:work(D, St), St, OneHit, Acc);
-do_dlo_pick(#dlo{mirror=none,src_we=#we{id=Id}=We}=D, _, OneHit, Acc)
+do_dlo_pick(D=#dlo{vab=none}, St, OneHit, Ms, Acc) ->
+    do_dlo_pick(wings_draw_setup:work(D, St), St, OneHit, Ms, Acc);
+do_dlo_pick(D=#dlo{vab=#vab{face_vs=none}}, St, OneHit, Ms, Acc) ->
+    do_dlo_pick(wings_draw_setup:work(D, St), St, OneHit, Ms, Acc);
+do_dlo_pick(#dlo{mirror=none,src_we=#we{id=Id}=We}=D, _, OneHit, _Ms, Acc)
   when ?IS_AREA_LIGHT(We) ->
     wpc_pick:cull(false),
     Res = do_dlo_pick_0(Id, D, OneHit, Acc),
     wpc_pick:cull(true),
     Res;
-do_dlo_pick(#dlo{mirror=none,open=Open,src_we=#we{id=Id}}=D, _, OneHit, Acc) ->
+do_dlo_pick(#dlo{mirror=none,open=Open,src_we=#we{id=Id}}=D, _, OneHit, _Ms, Acc) ->
     case wings_pref:get_value(show_backfaces) of
         true when Open -> wpc_pick:front_face(cw);
         _ -> wpc_pick:front_face(ccw)
     end,
     do_dlo_pick_0(Id, D, OneHit, Acc);
-do_dlo_pick(#dlo{mirror=Matrix,open=Open,src_we=#we{id=Id}}=D0, _, OneHit, Acc0) ->
+do_dlo_pick(#dlo{mirror=Matrix,open=Open,src_we=#we{id=Id}}=D0, _, OneHit, Ms, Acc0) ->
     case wings_pref:get_value(show_backfaces) of
         true when Open ->
             wpc_pick:front_face(cw),
@@ -997,16 +981,15 @@ do_dlo_pick(#dlo{mirror=Matrix,open=Open,src_we=#we{id=Id}}=D0, _, OneHit, Acc0)
             {D1,Acc1} = do_dlo_pick_0(Id, D0, OneHit, Acc0),
             wpc_pick:front_face(cw)
     end,
-    gl:pushMatrix(),
-    gl:multMatrixf(Matrix),
-    set_pick_matrix(),
+    set_pick_matrix(Ms, Matrix),
     {D,Acc} = do_dlo_pick_0(-Id, D1, OneHit, Acc1),
     wpc_pick:front_face(ccw),
-    gl:popMatrix(),
-    set_pick_matrix(),
+    set_pick_matrix(Ms),
     {D,Acc}.
 
-do_dlo_pick_0(Id, #dlo{vab=#vab{face_vs=Vs,face_map=Map0}}=D0, OneHit, Acc0) ->
+do_dlo_pick_0(Id, #dlo{vab=#vab{data=VsBin,face_vs={Stride,_},face_map=Map0}}=D0,
+	      OneHit, Acc0) ->
+    Vs = {Stride,VsBin},
     case wpc_pick:faces(Vs, OneHit) of
 	[] ->
 	    %% No hit.
