@@ -41,16 +41,16 @@
 
 init() ->
     ets:new(wings_seen_plugins, [named_table,public,ordered_set]),
-    put(wings_plugins, []),
+    ?SET(wings_plugins, []),
     put(wings_ui, def_ui_plugin()),
     case try_dir(wings_util:lib_dir(wings), "plugins") of
 	none -> ok;
 	PluginDir -> init_dir(PluginDir)
     end,
     wings_pref:set_default(disabled_plugins, []),
-    AllPlugins = get(wings_plugins),
+    AllPlugins = ?GET(wings_plugins),
     put(wings_all_plugins, AllPlugins),
-    put(wings_plugins, AllPlugins -- wings_pref:get_value(disabled_plugins)),
+    ?SET(wings_plugins, AllPlugins -- wings_pref:get_value(disabled_plugins)),
     ok.
 
 call_ui(What) ->
@@ -59,7 +59,7 @@ call_ui(What) ->
 
 menu(Name, Menu0) ->
     Menu = manager_menu(Name, Menu0),
-    menu_1(get(wings_plugins), Name, Menu).
+    menu_1(?GET(wings_plugins), Name, Menu).
 
 menu_1([M|Ps], Name, Menu0) ->
     case catch M:menu(Name, Menu0) of
@@ -73,7 +73,7 @@ menu_1([], _Name, Menu) -> Menu.
 
 
 dialog(Dialog, Ps) when is_list(Ps) ->
-    dialog_1(Dialog, Ps, get(wings_plugins)).
+    dialog_1(Dialog, Ps, ?GET(wings_plugins)).
 
 dialog_1(Dialog, Ps, [M|Tail]) ->
     case catch M:dialog(Dialog, Ps) of
@@ -83,7 +83,8 @@ dialog_1(Dialog, Ps, [M|Tail]) ->
 	    io:format("~w:dialog/2: crashed: ~P\n", [M,Reason,20]),
 	    wings_u:error_msg("~w:dialog/2: crashed", [M]);
 	NewPs when is_list(NewPs) ->
-	    dialog_1(Dialog, NewPs, Tail);
+	    Checked = check_dialog(NewPs, M),
+	    dialog_1(Dialog, Checked, Tail);
 	Other ->
 	    io:format("~w:dialog/2: bad return value: ~P\n", [M,Other,20]),
 	    wings_u:error_msg("~w:dialog/2: bad return value", [M])
@@ -91,8 +92,19 @@ dialog_1(Dialog, Ps, [M|Tail]) ->
 dialog_1(_Dialog, Ps, []) -> 
     Ps.
 
+% Check (and fix) plugin dialogs results (new format)
+check_dialog([Ok = {Name, Tuple}|Rest], Mod)
+  when is_list(Name), is_tuple(Tuple) ->
+    [Ok|check_dialog(Rest, Mod)];
+check_dialog([[PPs]|Rest], Mod) ->
+    [{atom_to_list(Mod), PPs}|check_dialog(Rest, Mod)];
+check_dialog([{Name, []}|Rest], Mod)
+  when is_list(Name) ->
+    [check_dialog(Rest, Mod)];
+check_dialog([], _) -> [].
+
 dialog_result(Dialog, Ps) when is_tuple(Dialog), is_list(Ps) ->
-    dialog_result1(Dialog, Ps, get(wings_plugins)).
+    dialog_result1(Dialog, Ps, ?GET(wings_plugins)).
 
 dialog_result1(Dialog, Ps, [M|Tail]) ->
     case catch M:dialog(Dialog, Ps) of
@@ -112,7 +124,7 @@ dialog_result1(Dialog, Ps, []) ->
     {element(tuple_size(Dialog), Dialog),Ps}.
 
 command(Cmd, St) ->
-    Ps = get(wings_plugins),
+    Ps = ?GET(wings_plugins),
     case manager_command(Cmd, St) of
 	next -> command(Ps, Cmd, St);
 	Other ->
@@ -163,7 +175,7 @@ init_plugin(user_interface, M) ->
 init_plugin(_, M) ->
     case catch M:init() of
 	true ->
-	    put(wings_plugins, [M|get(wings_plugins)]);
+	    ?SET(wings_plugins, [M|?GET(wings_plugins)]);
 	false ->
 	    ok;
 	Other ->
@@ -376,11 +388,13 @@ manager_command({edit,plugin_manager}, St) ->
     Cps = wings_util:rel2fam(Cps0),
     Fun = fun(Res) -> 
 		  Disabled = [M || {M,false} <- Res],
-		  put(wings_plugins, Ps -- Disabled),
+		  ?SET(wings_plugins, Ps -- Disabled),
+		  update_menus(Cps, Disabled),
+		  plugin_close_win(Disabled),
 		  update_disabled(Disabled, St)
 	  end,
     Dialog = mk_dialog(Cps, false),
-    wings_ask:dialog(?__(1,"Plug-In Manager"), Dialog, Fun);
+    wings_dialog:dialog(?__(1,"Plug-In Manager"), Dialog, Fun);
 manager_command(_, _) -> next.
 
 update_disabled(Disabled, St) ->
@@ -394,6 +408,46 @@ update_disabled(Disabled, St) ->
 	    St#st{repeatable=ignore}
     end.
 
+update_menus(Ps, Disabled) ->
+    DisabledS = sets:from_list(Disabled),
+    ToUpdate =
+	lists:foldr(fun({Category, Plugins}, Acc) ->
+	    PluginsS = sets:from_list(Plugins),
+	    case sets:to_list(sets:intersection(PluginsS,DisabledS)) of
+		[] -> Acc;
+		Acc0 -> Acc ++[{Category,Acc0}]
+	    end
+	end, [], Ps),
+    update_plugin_menus(ToUpdate).
+
+update_plugin_menus([]) -> ignore;
+update_plugin_menus([{Category, Ms}|Cps]) ->
+    [update_menu_category(Category, M) || M <- Ms],
+    update_plugin_menus(Cps).
+
+update_menu_category(_, []) -> ignore;
+update_menu_category(_, M) ->
+    Items = collect_menus([{file,render},{file,import},
+                           {file,export},{file,export_selected},
+			   {edit},{edit,plugin_preferences},
+                           {view},{select},{tools},{window},{help}], M),
+    [delete_menu_item(Item) || Item <- Items].
+
+
+delete_menu_item({{Menu},{_,SubMenu,_}}) ->
+    delete_menu_item(Menu, SubMenu, ignore);
+delete_menu_item({{Menu},{_,{SubMenu,_}}}) ->
+    delete_menu_item(Menu, SubMenu, ignore);
+delete_menu_item({{Menu,SubMenu},{_,Tag}}) ->
+    delete_menu_item(Menu, SubMenu, Tag);
+delete_menu_item({{Menu,SubMenu},{_,Tag,_}}) ->
+    delete_menu_item(Menu, SubMenu, {Tag,true}).
+
+delete_menu_item(Menu, SubMenu, ignore) ->
+    wings_menu:update_menu(Menu,SubMenu, delete);
+delete_menu_item(Menu, SubMenu, Tag) ->
+    wings_menu:update_menu(Menu, {SubMenu, Tag}, delete).
+
 mk_dialog(Cs, _Min) ->
     [{oframe,mk_dialog_1(Cs),1,[{style,buttons}]}].
 
@@ -402,12 +456,20 @@ mk_dialog_1([{C,Ms}|Cs]) ->
 mk_dialog_1([]) -> [].
 
 plugin_modules(C, Ms) ->
-    {hframe,[{vframe,[{atom_to_list(M),member(M, get(wings_plugins)),
-		       [{key,M},
-			{info,?__(1,"Enable or disable this plug-in ")++
-			 ?__(2,"(a disbled plug-in does not show up in menus)")}]} ||
-			 M <- Ms]},
-	     {vframe,[plugin_info(C, M) || M <- Ms]}]}.
+    Ps = [{info,?__(1,"Enable or disable this plug-in ")++
+	       ?__(2,"(a disbled plug-in does not show up in menus)")},
+	  {proportion, 1}],
+    {vframe,
+     [{hframe, [{atom_to_list(M), member(M, ?GET(wings_plugins)), [{key,M}|Ps]},
+		plugin_info(C,M)]}
+      || M <- Ms]}.
+
+plugin_info(C, M) ->
+    case plugin_info_1(C, M) of
+	panel -> panel;
+	{label, Str} -> {label, Str, [{proportion, 2}]}
+    end.
+
 
 cat_label(command) -> ?__(1,"Commands");
 cat_label(export_import) -> ?__(2,"Import/export");
@@ -462,22 +524,22 @@ try_menu([N|Ns], M, Category) ->
     end;
 try_menu([], _, _) -> next.
 
-plugin_info(export_import, M) -> export_import_info(M);
-plugin_info(render, M) -> export_import_info(M);
-plugin_info(command, M) ->
+plugin_info_1(export_import, M) -> export_import_info(M);
+plugin_info_1(render, M) -> export_import_info(M);
+plugin_info_1(command, M) ->
     Names = command_menus(),
     Menus = collect_menus(Names, M),
     plugin_menu_info(Menus);
-plugin_info(primitive, M) ->
+plugin_info_1(primitive, M) ->
     Menus = collect_menus([{shape},{shape}], M),
     plugin_menu_info(Menus);
-plugin_info(select, M) ->
+plugin_info_1(select, M) ->
     Menus = collect_menus([{select}], M),
     plugin_menu_info(Menus);
-plugin_info(tool, M) ->
+plugin_info_1(tool, M) ->
     Menus = collect_menus([{tools}], M),
     plugin_menu_info(Menus);
-plugin_info(_, _) -> panel.
+plugin_info_1(_, _) -> panel.
 
 export_import_info(M) ->
     case collect_menus([{file,import},{file,export},{file,render}], M) of
@@ -572,8 +634,9 @@ normalize_menu([{Name,Menu}|T]) ->
     end;
 normalize_menu([]) -> [].
 
-plugin_key({_,{Key,_},_}) when is_atom(Key) -> Key;
 plugin_key({_,{Key,_}}) when is_atom(Key) -> Key;
+plugin_key({_,{Key,_},_}) when is_atom(Key) -> Key;
+plugin_key({_,{Key,_},_, _}) when is_atom(Key) -> Key;
 plugin_key({_,Key,_}) when is_atom(Key) -> Key;
 plugin_key({_,Key,_,_}) when is_atom(Key) -> Key;
 plugin_key(Other) when is_tuple(Other) -> list_to_atom(element(1, Other));
@@ -640,7 +703,7 @@ check_plugin_against_flag(Flag, PData, Plugin, Acc) ->
 % It allows wings.erl module get/set the plugin window/s information (if there is/are any)
 % See wpc_sel_win.erl as an example.
 get_win_data(WinName) ->
-    Ps = get(wings_plugins),
+    Ps = ?GET(wings_plugins),
     get_win_data_1(Ps, WinName).
 
 %% win_data/1 function allows many plugin's windows to be saved.
@@ -653,7 +716,7 @@ get_win_data_1([M|Ps], WinName) ->
 get_win_data_1([], _) -> none.
 
 restore_window(M, WinName, Pos, Size, CtmData, St) ->
-    Ps = get(wings_plugins),
+    Ps = ?GET(wings_plugins),
     case module_found(M,Ps) of
 	true ->
 	    catch M:window(WinName, Pos, Size, CtmData, St),
@@ -664,3 +727,27 @@ restore_window(M, WinName, Pos, Size, CtmData, St) ->
 module_found(M,[M|_]) -> true;
 module_found(M,[_|T]) -> module_found(M,T);
 module_found(_,[]) -> false.
+
+
+%%%
+%%% Plugin Close Windows Utility
+%%%
+
+% It allows this module to close any plugin's window when a module is disabled.
+plugin_close_win([M|Ps]) ->
+    try M:win_name() of
+	WinName ->
+	    plugin_close_win_0(WinName),
+	    plugin_close_win(Ps)
+    catch
+    	_Exception:_Reason ->
+	    plugin_close_win(Ps)
+    end;
+plugin_close_win([]) -> none.
+
+plugin_close_win_0([]) -> ignore;
+plugin_close_win_0([Win|Wins]) ->
+    plugin_close_win_0(Win),
+    plugin_close_win_0(Wins);
+plugin_close_win_0(Win) ->
+    wings_frame:close(Win).
