@@ -46,7 +46,7 @@ load() ->
             set_value(legacy_colors_checked, true),
             ok;
         PrefFile ->
-            io:format("wings-~s\nReading preferences from: ~s\n",
+            io:format("wings-~s\nReading preferences from: ~ts\n",
                 [?WINGS_VERSION, PrefFile]),
             case local_consult(PrefFile) of
                 {ok,List0} ->
@@ -113,7 +113,7 @@ no_more_basic_menus() ->
 	    %% Either this is the first launch and the user used
 	    %% the basic menus, or (s)he has still not turned off
 	    %% the informational dialog.
-	    set_value(no_basic_menu_info, false),
+	    set_value(no_basic_menu_info, true),
 	    self() ! {external,no_more_basic_menus},
 	    ok
     end,
@@ -139,8 +139,8 @@ finish() ->
     end.
 
 finish_save_prefs(PrefFile) ->
-    List0 = ets:tab2list(wings_state),
-    List = prune_defaults(List0),
+    List0 = prune_temp(ets:tab2list(wings_state)),
+    List  = prune_defaults(List0),
     Format = "~p. \n",
     PostProcess = case os:type() of
 		      {win32,_} -> fun insert_crs/1;
@@ -182,6 +182,12 @@ insert_crs(C) when is_integer(C) -> C.
 prune_defaults(List) ->
     List -- defaults().
 
+prune_temp([{{temp,_}, _}|List]) ->
+    prune_temp(List);
+prune_temp([First|Rest]) ->
+    [First | prune_temp(Rest)];
+prune_temp([]) -> [].
+
 win32_save_maximized() ->
     case os:type() of
 	{win32,_} ->
@@ -371,7 +377,7 @@ delete_scene_value(Key) ->
 
 
 defaults() ->
-    wings_theme:olive_theme()++
+    wings_theme:native_theme()++
     [
      %% Put any non-constant preferences here.
      {jumpy_camera,os:type() =:= {unix,darwin}},
@@ -398,7 +404,6 @@ defaults() ->
      {force_ortho_along_axis,false},
      {body_hilite,true},
      {auto_rotate_angle,1.0},
-     {auto_rotate_delay,60},
      {autosave,true},
      {autosave_time,10},
      {active_vector_size,1.0},
@@ -414,7 +419,7 @@ defaults() ->
      {dummy_axis_letter,false},
      {polygon_offset_f,1.0},
      {polygon_offset_r,1.0},
-     {multisample,undefined},
+     {multisample, true},
      {ungrab_bug, false},
 
      %% Advanced features.
@@ -473,6 +478,7 @@ defaults() ->
      %% User interface preferences.
      {interface_icons,bluecube},
      {aa_edges,false},
+     {show_toolbar, true},
      {extended_toolbar,true},
      {bitmap_icons, false},
      {menu_toolbar, true},
@@ -562,6 +568,7 @@ not_bad(workmode, _) -> false;
 not_bad(orthogonal_view, _) -> false;
 not_bad(show_memory_used, _) -> false;
 not_bad(show_axes, _) -> false;
+not_bad(show_cam_imageplane, _) -> false;
 not_bad(show_groundplane, _) -> false;
 not_bad(current_view, _) -> false;
 not_bad(camera_fov, _) -> false;
@@ -647,100 +654,91 @@ colors(ColorPrefs, List) ->
 %% Command is initiated in wings_file.erl
 pref({load,Request,St}) ->
     case Request of
-      custom_theme ->
-        pref(load);
-      olive_theme ->
-        LegacyColors = wings_theme:legacy_colors(),
-        Defaults = defaults(),
-        Colors = colors(LegacyColors, Defaults),
-        load_pref_category([{graphical,true}],[{graphical,Colors}],St),
-        init_opengl(),
-        keep;
-      Theme when is_atom(Theme) ->
-        Colors = wings_theme:Theme(),
-        load_pref_category([{graphical,true}],[{graphical,Colors}],St),
-        init_opengl(),
-        keep;
-      Key when is_integer(Key) ->
-        Recent0 = get_value(recent_prefs),
-        PrefDir = lists:nth(Key, Recent0),
-        case filelib:is_file(PrefDir) of
-          true ->
-              wings_pref:set_value(pref_directory, PrefDir),
-              pref(load);
-          false ->
-              Recent = delete_nth(Recent0, Key),
-              wings_pref:set_value(recent_prefs, Recent),
-              wings_u:error_msg(?__(11,"This file has been moved or deleted."))
-        end
+	custom_theme ->
+	    pref(load);
+	Theme when is_atom(Theme) ->
+	    Colors = wings_theme:Theme(),
+	    load_pref_category([{graphical,true}],[{graphical,Colors}],St),
+	    init_opengl(),
+	    keep;
+	Key when is_integer(Key) ->
+	    Recent0 = get_value(recent_prefs),
+	    PrefDir = lists:nth(Key, Recent0),
+	    case filelib:is_file(PrefDir) of
+		true ->
+		    wings_pref:set_value(pref_directory, PrefDir),
+		    pref(load);
+		false ->
+		    Recent = delete_nth(Recent0, Key),
+		    wings_pref:set_value(recent_prefs, Recent),
+		    wings_u:error_msg(?__(11,"This file has been moved or deleted."))
+	    end
     end;
 
 pref(Action) -> %% load|save dialog
-    Disable = fun (is_disabled, {_Var,_I, Store}) ->
-              not gb_trees:get(hotkeys,Store);
-              (_, _) -> void
+    FileName = "Preference Subset.pref",
+    Directory = case get_value(pref_directory) of
+		    undefined -> get_pref_directory(FileName);
+		    Dir -> Dir
+		end,
+
+    Keys = case file:consult(Directory) of
+	       {ok,List} when Action =:= load -> orddict:fetch_keys(List);
+	       _ -> []
+	   end,
+    Disable = fun (_Var, Enable, Store) ->
+		      Action =:= load andalso
+			  wings_dialog:enable(hotkey_radio, Enable, Store)
               end,
     case Action of
         load ->
-          Title = ?__(1,"Load Preference Subset"),
-          Button = [{button,
-                     ?__(14,"Select Valid Preference Fields for Current File"),
-                     done,[{key,update}]},separator],
-          Dialog = open_dialog,
-          Options =
-           [separator,
-            {hframe,[{vradio,[{?__(9,"Merge hotkeys"),merge},
-                              {?__(10,"Remove existing hotkeys first"),remove}],
-                               merge}],[{hook,Disable}]},panel];
+	    Title = ?__(1,"Load Preference Subset"),
+	    Dialog = open_dialog,
+	    Options =
+		[separator,
+		 {vradio,[{?__(9,"Merge hotkeys"),merge},
+			  {?__(10,"Remove existing hotkeys first"),remove}],
+		  merge, [{key, hotkey_radio}]},
+		 panel];
         save ->
-          Title = ?__(2,"Save Preference Subset"),
-          Button = [],
-          Dialog = save_dialog,
-          Options = [panel]
-    end,
-    FileName = "Preference Subset.pref",
-    Directory = case get_value(pref_directory) of
-      undefined -> get_pref_directory(FileName);
-      Dir -> Dir
-    end,
-
-    Keys = case file:consult(Directory) of
-      {ok,List} when Action =:= load -> orddict:fetch_keys(List);
-      _ -> []
+	    Title = ?__(2,"Save Preference Subset"),
+	    Dialog = save_dialog,
+	    Options = [panel]
     end,
     PrefFeilds =
         [{hframe,
           [{vframe,
             [{?__(3,"Graphical Settings"),member(graphical,Keys),[{key,graphical}]},
              {?__(4,"Camera Settings"),member(camera,Keys),[{key,camera}]},
-             {?__(5,"Hotkeys"),member(hotkeys,Keys),[{key,hotkeys}]}]},
+             {?__(5,"Hotkeys"),member(hotkeys,Keys),[{key,hotkeys}, {hook, Disable}]}]},
            {vframe,
             [{?__(6,"Window and View Settings"),member(windows,Keys),[{key,windows}]},
              {?__(7,"Constraints"),member(constraints,Keys),[{key,constraints}]},
              {?__(8,"General Settings"),member(settings,Keys),[{key,settings}]}]}
-         ]}],
+	  ]}],
     FileBrowser =
-        [{button, {text, Directory, [{key, pref_directory},
-            {props, [{dialog_type, Dialog},
-            {extensions, [{".pref", "Preference Subset"}]}]}]}}],
-    Qs = Button ++ PrefFeilds ++ Options ++ FileBrowser,
-    wings_ask:dialog(true, Title, Qs,
-        fun(Res) ->
-            case lists:keyfind(update, 1, Res) of
-              {_,true} ->
-                  {_,PrefDir} = lists:keyfind(pref_directory, 1, Res),
-                  case filelib:is_file(PrefDir) of
-                    true -> wings_pref:set_value(pref_directory, PrefDir);
-                    false -> ok
-                  end,
-                  {file,{load_pref,custom_theme}};
-              _ -> {file,{pref,{Action,Res}}}
-            end
-        end).
+        [{button, {text, Directory,
+		   [{key, pref_directory},
+		    {props, [{dialog_type, Dialog},
+			     {extensions, [{".pref", "Preference Subset"}]}]}]}}],
+    Qs = PrefFeilds ++ Options ++ FileBrowser,
+    Do = fun(Res) ->
+		 case lists:keyfind(update, 1, Res) of
+		     {_,true} ->
+			 {_,PrefDir} = lists:keyfind(pref_directory, 1, Res),
+			 case filelib:is_file(PrefDir) of
+			     true -> wings_pref:set_value(pref_directory, PrefDir);
+			     false -> ok
+			 end,
+			 {file,{load_pref,custom_theme}};
+		     _ -> {file,{pref,{Action,Res}}}
+		 end
+	 end,
+    wings_dialog:dialog(Title, Qs, Do).
 
 pref({save, Res}, St) -> %save a .pref
     DelayedPrefs = ets:tab2list(wings_delayed_update),
-    Prefs0 = ets:tab2list(wings_state),
+    Prefs0 = prune_temp(ets:tab2list(wings_state)),
     List = foldl(fun({Key, _}=Pref, Acc) ->
             lists:keystore(Key, 1, Acc, Pref)
         end, Prefs0, DelayedPrefs),
@@ -804,65 +802,65 @@ write_pref(Dir, ColorPref) ->
     set_value(pref_directory,Dir).
 
 treat_hotkeys(Res) ->
-    case lists:member(merge,Res) of
-      true ->
-        lists:delete(merge,Res);
-      false ->
-        case lists:member(remove,Res) of
-          true ->
-            List = ets:tab2list(wings_state),
-            lists:foreach(fun({Hotkey,_,_}) ->
-                      ets:delete(wings_state,Hotkey);
-                            (_) -> ok
-                    end, List),
-            lists:delete(remove,Res);
-          false -> Res
-        end
+    case proplists:get_value(hotkeys, Res, false) of
+	false -> Res;
+	true ->
+	    case proplists:get_value(hotkey_radio, Res) of
+		merge -> Res;
+		remove ->
+		    List = prune_temp(ets:tab2list(wings_state)),
+		    lists:foreach(fun({Hotkey,_,_})
+					when element(1, Hotkey) =:= bindkey ->
+					  ets:delete(wings_state,Hotkey);
+				     (_) -> ok
+				  end, List),
+		    Res
+	    end
     end.
 
 init_opengl() ->
-    case wings_io:is_maximized() of
-      false ->
-        {W, H} = TopSize = get_value(window_size),
-        put(wm_top_size, TopSize),
-        wings_wm:reinit_opengl(),
-        wings_wm:resize_windows(W, H),
-        wings_wm:dirty();
-      true ->
-        wings_wm:reinit_opengl(),
-        wings_wm:dirty()
-    end.
+    wings_wm:dirty().
 
 update_recent_prefs(Dir) ->
     Recent0 = get_value(recent_prefs,[]),
+    PrevLen = length(Recent0),
     Recent1 = [Dir|lists:delete(Dir,Recent0)],
     Recent = lists:sublist(Recent1, 5),
+    Update = fun(File, N) when N =< PrevLen ->
+                     {_,Pref} = split_dir(lists:reverse(File),[]),
+                     wings_menu:update_menu(file, {load_pref, N}, Pref, recent_pref_help()),
+                     N+1;
+                (_, N) ->
+                     N
+             end,
+    Next = lists:foldl(Update, 1, Recent),
+    case length(Recent) > PrevLen of
+        false -> ok;
+        true -> %% Add the last one
+            {_,Pref} = split_dir(lists:reverse(lists:last(Recent)),[]),
+            wings_menu:update_menu(file, {load_pref, Next},
+                                   {append, -1, Pref},
+                                   recent_pref_help())
+    end,
     set_value(recent_prefs, Recent).
 
 recent_prefs() ->
     Recent = get_value(recent_prefs,[]),
     recent_prefs(Recent,1).
-recent_prefs([Dir|Recent],1) ->
-    case lists:suffix(".pref",Dir) of
-      true ->
-        {_,Pref} = split_dir(lists:reverse(Dir),[]),
-        [separator,{Pref,1,recent_pref_help()}|recent_prefs(Recent,2)];
-      false ->
-        P0 = get_value(recent_prefs),
-        P = P0 -- Dir,
-        set_value(recent_prefs,P),
-        recent_prefs(Recent,1)
-    end;
 recent_prefs([Dir|Recent],N) ->
     case lists:suffix(".pref",Dir) of
-      true ->
-        {_,Pref} = split_dir(lists:reverse(Dir),[]),
-        [{Pref,N,recent_pref_help()}|recent_prefs(Recent,N+1)];
-      false ->
-        P0 = get_value(recent_prefs),
-        P = P0 -- Dir,
-        set_value(recent_prefs,P),
-        recent_prefs(Recent,N)
+        true ->
+            {_,Pref} = split_dir(lists:reverse(Dir),[]),
+            PrefEntries = [{Pref,N,recent_pref_help()}|recent_prefs(Recent,N+1)],
+            case N of
+                1 -> [separator|PrefEntries];
+                _ -> PrefEntries
+            end;
+        false ->
+            P0 = get_value(recent_prefs),
+            P = P0 -- Dir,
+            set_value(recent_prefs,P),
+            recent_prefs(Recent,N)
     end;
 recent_prefs([],_) -> [].
 
@@ -992,11 +990,12 @@ save_pref_category([{windows,Bool}|Options], List, Defaults, St, Acc0) ->
     C2 = lists:keyfind(console_save_lines,1,List),
     C3 = lists:keyfind(console_width,1,List),
     WinSize = lists:keyfind(window_size,1,List),
-    WP = wings:save_windows_1(wings_wm:windows()),
-    Windows = [WinSize,C1,C2,C3|WP],
-    Acc = if Bool -> [{windows,Windows}|Acc0];
-             true -> Acc0 end,
-    NewList = (List--Windows)--[{saved_windows,WP}],
+    WCWF = wings_frame:export_layout(),
+    Windows = [WinSize,C1,C2,C3],
+    Acc = if Bool -> [{saved_windows2,WCWF}|Acc0];
+             true -> Acc0
+          end,
+    NewList = (List--Windows)--[{saved_windows2,WCWF}],
     save_pref_category(Options, NewList,  Defaults--Windows, St, Acc);
 save_pref_category([], _, _, _, Acc) -> Acc.
 
@@ -1008,62 +1007,28 @@ load_pref_category([{pref_directory,Dir}|Options], List, St) ->
 load_pref_category([{_,false}|Options], List, St) ->
     load_pref_category(Options, List, St);
 load_pref_category([{graphical,true}|Options], List, St) ->
-%% Load color preferences
+    %% Load color preferences
     Colors = case lists:keyfind(graphical, 1, List) of
-        {_,C} -> C;
-        false -> []
-    end,
+		 {_,C} -> C;
+		 false -> []
+	     end,
     catch wings_pref_dlg:set_values(Colors, St),
     load_pref_category(Options, List, St);
 load_pref_category([{windows,true}|Options], List, St) ->
-%% Load preference windows and remove any old windows
-    OldWindows = wings:save_windows_1(wings_wm:windows()),
-    case lists:keyfind(windows, 1, List) of
-        {_,Windows0} ->
-            foreach(fun
-              ({_,Prop}=P) when is_integer(Prop) ->
-                ets:insert(wings_state,P);
-              ({window_size,_}=P) ->
-                ets:insert(wings_state,P),
-                init_opengl();
-              (Window) ->
-                Name = element(1,Window),
-                case wings_wm:is_window(Name) of
-                  true when Name =:= geom ->
-                    Props = element(4,Window),
-                    case proplists:get_bool(toolbar_hidden, Props) of
-                      true -> wings_wm:hide({toolbar,geom});
-                      false -> ok
-                    end,
-                    wings:set_geom_props(Props,Name),
-                    Pos = element(2,Window),
-                    Size = element(3,Window),
-                    wings_wm:move(Name,Pos,Size);
-                  true ->
-                    wings_wm:delete(Name),
-                    wings:restore_windows_1([Window], St);
-                  false ->
-                    wings:restore_windows_1([Window], St)
-                end
-            end,Windows0),
-            %% Delete old windows
-            foreach(fun(OldWindow) ->
-                OldName = element(1,OldWindow),
-                case lists:keymember(OldName,1,Windows0) of
-                  false -> wings_wm:delete(OldName);
-                  _ -> ok
-                end
-              end, OldWindows);
-        false -> ok
+    %% Load preference windows and remove any old windows
+    case lists:keyfind(saved_windows2, 1, List) of
+        {saved_windows2, {_WC,_WF} = WCWF} ->
+            wings_frame:import_layout(WCWF, St);
+        _ -> ok
     end,
     load_pref_category(Options, List, St);
-% Load Hotkeys
+%% Load Hotkeys
 load_pref_category([{hotkeys,true}|Options], List, St) ->
     case lists:keyfind(hotkeys, 1, List) of
         {_,Prefs} ->
-          foreach(fun(Hkey) ->
-            ets:insert(wings_state, Hkey)
-          end, clean(Prefs));
+	    foreach(fun(Hkey) ->
+			    ets:insert(wings_state, Hkey)
+		    end, clean(Prefs));
         false -> wings_hotkey:set_default()
     end,
     load_pref_category(Options, List, St);
@@ -1071,12 +1036,14 @@ load_pref_category([{hotkeys,true}|Options], List, St) ->
 load_pref_category([{Other,true}|Options], List, St) ->
     case lists:keyfind(Other, 1, List) of
         {_,Prefs} ->
-          foreach(fun(P) ->
-            ets:insert(wings_state, P)
-          end, clean(Prefs));
+	    foreach(fun(P) ->
+			    ets:insert(wings_state, P)
+		    end, clean(Prefs));
         false -> ok
     end,
     load_pref_category(Options, List, St);
+load_pref_category([{hotkey_radio, _}|Options], List, St) ->
+    load_pref_category(Options, List, St);
 load_pref_category([], _, _) -> ok.
 
 delete_nth([_|T], 1) -> T;
