@@ -22,7 +22,7 @@
 
 init() ->
     init_shaders(),
-    init_multisample(),
+    wings_pref:set_default(multisample, true),
     init_polygon_stipple().
 
 %% render(St)
@@ -41,23 +41,18 @@ render(#st{selmode=Mode}=St) ->
     gl:enable(?GL_CULL_FACE),
     case wings_pref:get_value(multisample) of
 	true -> gl:enable(?GL_MULTISAMPLE);
-	false -> gl:disable(?GL_MULTISAMPLE);
-	undefined -> ok
+	_ -> gl:disable(?GL_MULTISAMPLE)
     end,
-    SceneLights = wings_view:load_matrices(true),
-    Yon = ground_and_axes(),
-    mini_axis_icon(),
+    {PM,MM,SceneLights} = wings_view:load_matrices(true),
+    Yon = ground_and_axes(PM,MM),
+    mini_axis_icon(MM),
     show_saved_bb(St),
     show_bb_center(St),
     user_clipping_planes(on),
-    render_objects(Mode, SceneLights),
+    render_objects(Mode, PM, SceneLights),
     user_clipping_planes(off),
-    axis_letters(Yon),
-    gl:lineWidth(1),
-    wings_io:ortho_setup(),
-    gl:polygonMode(?GL_FRONT_AND_BACK, ?GL_LINE),
-    {W,H} = wings_wm:win_size(),
-    gl:rectf(W-0.5, 0.5, 0.5, H-0.5),
+    axis_letters(PM,MM,Yon),
+    show_camera_image_plane(),
     gl:popAttrib(),
     wings_develop:gl_error_check("Rendering scene").
 
@@ -68,38 +63,27 @@ polygonOffset(M) ->
 	    R = wings_pref:get_value(polygon_offset_r,1.0),
 	    put(polygon_offset, {F,R}),
 	    gl:polygonOffset(M*F, M*R);
-	{F,R} ->
+        {F,R} ->
 	    gl:polygonOffset(M*F, M*R)
     end.
 
+%% According Game Programming Gems chap 4.1 (By Eric Lengyel)
+%% http://www.terathon.com/books/code/Listing9.1.txt
+%% http://terathon.com/gdc07_lengyel.pdf
+%% Minimum Eps for 24b depth buffer (skip calc and use min value)
+non_polygon_offset(Offset, TPM) ->
+    ProjMat = e3d_transform:matrix(TPM),
+    Eps = 1.0 - Offset*5.0e-7,
+    E33 = element(11, ProjMat),  %% Modify entry {3,3}
+    gl:loadMatrixd(setelement(11, ProjMat, E33*Eps)).
+
 %%%
 %%% Internal functions follow.
 %%%
 
 init_shaders() ->
-    case wings_gl:support_shaders() of
-	true ->
-	    try
-		wings_shaders:init()
-	    catch _:_Err ->
-		ok
-	    end;
-	false ->
-	    ok
-    end.
-
-init_multisample() ->
-    case wings_gl:is_ext('GL_ARB_multisample') of
-	false ->
-	    %% Not supported.
-	    %%
-	    %% It would have been better to use something
-	    %% clearer like 'not_supported' here, but that
-	    %% would cause older versions of Wings from
-	    %% 0.99.54 through 1.1.2 to crash.
-	    wings_pref:set_value(multisample, undefined);
-	true ->
-	    wings_pref:set_default(multisample, true)
+    try wings_shaders:init()
+    catch _:_Err -> ok
     end.
 
 init_polygon_stipple() ->
@@ -121,69 +105,69 @@ init_polygon_stipple() ->
 	 16#DD,16#DD,16#DD,16#DD,16#77,16#77,16#77,16#77>>,
     gl:polygonStipple(P).
 
-render_objects(Mode, SceneLights) ->
+render_objects(Mode, PM, SceneLights) ->
     Dls = wings_dl:display_lists(),
     case wings_wm:get_prop(workmode) of
 	false ->
-	    render_smooth_objects(Dls, Mode, false, SceneLights),
-	    render_smooth_objects(Dls, Mode, true, SceneLights);
+	    render_smooth_objects(Dls, Mode, false, PM, SceneLights),
+	    render_smooth_objects(Dls, Mode, true, PM, SceneLights);
 	true ->
-	    render_work_objects(Dls, Mode, SceneLights)
+	    render_work_objects(Dls, Mode, PM, SceneLights)
     end.
 
-render_smooth_objects([D|Dls], Mode, RenderTrans, SceneLights) ->
-    render_object(D, Mode, false, RenderTrans, SceneLights),
-    render_smooth_objects(Dls, Mode, RenderTrans, SceneLights);
-render_smooth_objects([], _, _, _) -> ok.
+render_smooth_objects([D|Dls], Mode, RenderTrans, PM, SceneLights) ->
+    render_object(D, Mode, false, RenderTrans, PM, SceneLights),
+    render_smooth_objects(Dls, Mode, RenderTrans, PM, SceneLights);
+render_smooth_objects([], _, _, _, _) -> ok.
 
-render_work_objects([D|Dls], Mode, SceneLights) ->
-    render_object(D, Mode, true, false, SceneLights),
-    render_work_objects(Dls, Mode, SceneLights);
-render_work_objects([], _, _) -> ok.
+render_work_objects([D|Dls], Mode, PM, SceneLights) ->
+    render_object(D, Mode, true, false, PM, SceneLights),
+    render_work_objects(Dls, Mode, PM, SceneLights);
+render_work_objects([], _, _, _) -> ok.
 
-render_object(#dlo{drag={matrix,_,_,Matrix}}=D, Mode, Work, RT, SceneLights) ->
+render_object(#dlo{drag={matrix,_,_,Matrix}}=D, Mode, Work, RT, PM, SceneLights) ->
     gl:pushMatrix(),
     gl:multMatrixf(Matrix),
-    render_object_1(D, Mode, Work, RT, SceneLights),
+    render_object_1(D, Mode, Work, RT, PM, SceneLights),
     gl:popMatrix();
-render_object(D, Mode, Work, RT, SceneLights) ->
-    render_object_1(D, Mode, Work, RT, SceneLights).
+render_object(D, Mode, Work, RT, PM, SceneLights) ->
+    render_object_1(D, Mode, Work, RT, PM, SceneLights).
 
-render_object_1(#dlo{mirror=none}=D, Mode, Work, RenderTrans, SceneLights) ->
-    render_object_2(D, Mode, Work, RenderTrans, SceneLights);
-render_object_1(#dlo{mirror=Matrix}=D, Mode, Work, RenderTrans, SceneLights) ->
-    render_object_2(D, Mode, Work, RenderTrans, SceneLights),
+render_object_1(#dlo{mirror=none}=D, Mode, Work, RenderTrans, PM, SceneLights) ->
+    render_object_2(D, Mode, Work, RenderTrans, PM, SceneLights);
+render_object_1(#dlo{mirror=Matrix}=D, Mode, Work, RenderTrans, PM, SceneLights) ->
+    render_object_2(D, Mode, Work, RenderTrans, PM, SceneLights),
     gl:frontFace(?GL_CW),
     gl:pushMatrix(),
     gl:multMatrixf(Matrix),
-    render_object_2(D, Mode, Work, RenderTrans, SceneLights),
+    render_object_2(D, Mode, Work, RenderTrans, PM, SceneLights),
     gl:popMatrix(),
     gl:frontFace(?GL_CCW).
 
-render_object_2(#dlo{src_we=We}=D, _, _, false, _) when ?IS_LIGHT(We) ->
+render_object_2(#dlo{src_we=We}=D, _, _, false, _, _) when ?IS_LIGHT(We) ->
     wings_light:render(D);
-render_object_2(#dlo{src_we=We}, _, _, true, _) when ?IS_LIGHT(We) ->
+render_object_2(#dlo{src_we=We}, _, _, true, _, _) when ?IS_LIGHT(We) ->
     ok;
-render_object_2(D, Mode, true, _, SceneLights) ->
-    render_plain(D, Mode, SceneLights);
-render_object_2(#dlo{transparent=true}=D, _, false, false, SceneLights) ->
+render_object_2(D, Mode, true, _, PM, SceneLights) ->
+    render_plain(D, Mode, PM, SceneLights);
+render_object_2(#dlo{transparent=true}=D, _, false, false, PM, SceneLights) ->
     gl:disable(?GL_CULL_FACE),
-    render_smooth(D, false, SceneLights),
+    render_smooth(D, false, PM, SceneLights),
     gl:enable(?GL_CULL_FACE);
-render_object_2(#dlo{transparent=true}=D, _, false, true, SceneLights) ->
-    render_smooth(D, true, SceneLights);
-render_object_2(#dlo{transparent=false}=D, _, false, RenderTrans, SceneLights) ->
-    render_smooth(D, RenderTrans, SceneLights).
+render_object_2(#dlo{transparent=true}=D, _, false, true, PM, SceneLights) ->
+    render_smooth(D, true, PM, SceneLights);
+render_object_2(#dlo{transparent=false}=D, _, false, RenderTrans, PM, SceneLights) ->
+    render_smooth(D, RenderTrans, PM, SceneLights).
 
 render_plain(#dlo{work=Faces,edges=Edges,open=Open,
-		  src_we=We,proxy=false}=D, SelMode, SceneLights) ->
+		  src_we=We,proxy=false}=D, SelMode, PM, SceneLights) ->
     %% Draw faces for winged-edge-objects.
     Wire = wire(We),
     case Wire of
 	false ->
 	    gl:polygonMode(?GL_FRONT_AND_BACK, ?GL_FILL),
 	    gl:enable(?GL_POLYGON_OFFSET_FILL),
-	    polygonOffset(2),
+	    polygonOffset(2.0),
 	    gl:shadeModel(?GL_SMOOTH),
 	    enable_lighting(SceneLights),
 	    case wings_pref:get_value(show_backfaces) of
@@ -222,8 +206,7 @@ render_plain(#dlo{work=Faces,edges=Edges,open=Open,
 	    end,
 	    gl:lineWidth(edge_width(SelMode)),
 	    gl:polygonMode(?GL_FRONT_AND_BACK, ?GL_LINE),
-	    gl:enable(?GL_POLYGON_OFFSET_LINE),
-	    polygonOffset(1),
+            gl:depthFunc(?GL_LEQUAL),
 	    case Wire andalso wings_wm:get_prop(show_wire_backfaces) of
 		true ->
 		    gl:disable(?GL_CULL_FACE),
@@ -231,42 +214,53 @@ render_plain(#dlo{work=Faces,edges=Edges,open=Open,
 		    gl:enable(?GL_CULL_FACE);
 		false ->
 		    wings_dl:call(Edges)
-	    end
+	    end,
+            gl:depthFunc(?GL_LESS),
+            ok
     end,
-    render_plain_rest(D, Wire, SelMode);
-render_plain(#dlo{src_we=We}=D, SelMode, SceneLights) ->
+    render_plain_rest(D, Wire, SelMode, PM);
+render_plain(#dlo{src_we=We}=D, SelMode, PM, SceneLights) ->
     Wire = wire(We),
     wings_proxy:draw(D, Wire, SceneLights),
-    render_plain_rest(D, Wire, SelMode).
+    render_plain_rest(D, Wire, SelMode, PM).
 
-render_plain_rest(#dlo{}=D, Wire, SelMode) ->
+render_plain_rest(#dlo{}=D, Wire, SelMode, PM) ->
     gl:disable(?GL_POLYGON_OFFSET_LINE),
     gl:disable(?GL_POLYGON_OFFSET_FILL),
 
-    draw_hilite(D),
+    NeedPush = sel_need_matrix_push(D),
+    gl:matrixMode(?GL_PROJECTION),
+    gl:pushMatrix(),
+    non_polygon_offset(1.0, PM),
+    gl:depthFunc(?GL_LEQUAL),
+    draw_hard_edges(D, SelMode),
+    draw_vertices(D, SelMode),
+    draw_normals(D),
+    NeedPush orelse gl:popMatrix(),
     case Wire of
 	true ->
 	    gl:disable(?GL_CULL_FACE),
-	    draw_sel(D),
 	    draw_orig_sel(D),
+	    draw_sel(D),
 	    gl:enable(?GL_CULL_FACE);
 	false ->
-	    draw_sel(D),
-	    draw_orig_sel(D)
+            draw_orig_sel(D),
+	    draw_sel(D)
     end,
-    draw_vertices(D, SelMode),
-    draw_hard_edges(D, SelMode),
-    draw_normals(D),
+    draw_hilite(D),
+    gl:depthFunc(?GL_LESS),
+    NeedPush andalso gl:popMatrix(),
+    gl:matrixMode(?GL_MODELVIEW),
     draw_plugins(plain,D,SelMode). %% arbitrary placement in the grand scheme of things
 
 render_smooth(#dlo{work=Work,edges=Edges,smooth=Smooth0,transparent=Trans0,
 		   src_we=We,proxy=Proxy,proxy_data=PD,open=Open}=D,
-	      RenderTrans, SceneLights) ->
+	      RenderTrans, PM, SceneLights) ->
     gl:shadeModel(?GL_SMOOTH),
     gl:polygonMode(?GL_FRONT_AND_BACK, ?GL_FILL),
     enable_lighting(SceneLights),
     gl:enable(?GL_POLYGON_OFFSET_FILL),
-    gl:polygonOffset(2, 2),
+    polygonOffset(2.0),
 
     case Proxy of
 	true ->
@@ -308,27 +302,42 @@ render_smooth(#dlo{work=Work,edges=Edges,smooth=Smooth0,transparent=Trans0,
     gl:depthMask(?GL_TRUE),
     disable_lighting(),
     gl:shadeModel(?GL_FLAT),
+    gl:depthFunc(?GL_LEQUAL),
     case wire(We) of
 	true when Proxy =:= false ->
 	    gl:color3fv(wings_pref:get_value(edge_color)),
-	    gl:lineWidth(1),
+	    gl:lineWidth(1.0),
 	    gl:polygonMode(?GL_FRONT_AND_BACK, ?GL_LINE),
-	    gl:enable(?GL_POLYGON_OFFSET_LINE),
-	    gl:polygonOffset(1, 1),
 	    wings_dl:call(Edges);
 	true ->
 	    wings_proxy:draw_smooth_edges(D);
 	false -> ok
     end,
-    draw_hilite(D),
-    draw_sel(D),
+    NeedPush = sel_need_matrix_push(D),
+    if NeedPush ->
+            gl:matrixMode(?GL_PROJECTION),
+            gl:pushMatrix(),
+            non_polygon_offset(1.0, PM);
+       true -> false
+    end,
     draw_orig_sel(D),
+    draw_sel(D),
+    draw_hilite(D),
+    gl:depthFunc(?GL_LESS),
+    NeedPush andalso gl:popMatrix(),
+    gl:matrixMode(?GL_MODELVIEW),
     draw_plugins(smooth,D,none).
 
 wire(#we{id=Id}) ->
     W = wings_wm:get_prop(wireframed_objects),
     gb_sets:is_member(Id, W).
 
+sel_need_matrix_push(#dlo{src_sel={edge,_}})   -> true;
+sel_need_matrix_push(#dlo{src_sel={vertex,_}}) -> true;
+sel_need_matrix_push(#dlo{hilite={edge,_}})    -> true;
+sel_need_matrix_push(#dlo{hilite={vertex,_}})  -> true;
+sel_need_matrix_push(_) -> false.
+
 draw_sel(#dlo{sel=none}) -> ok;
 draw_sel(#dlo{sel=SelDlist,src_sel={edge,_}}) ->
     gl:lineWidth(wings_pref:get_value(selected_edge_width)),
@@ -342,11 +351,12 @@ draw_sel(#dlo{open=Open,sel=SelDlist}) ->
     sel_color(),
     gl:polygonMode(?GL_FRONT_AND_BACK, ?GL_FILL),
     gl:enable(?GL_POLYGON_OFFSET_FILL),
-    gl:polygonOffset(1, 1),
+    polygonOffset(1.0),
     %Stippled selection style.
     gl:enable(?GL_POLYGON_STIPPLE),
     draw_face_sel(Open, SelDlist),
-    gl:disable(?GL_POLYGON_STIPPLE).
+    gl:disable(?GL_POLYGON_STIPPLE),
+    gl:disable(?GL_POLYGON_OFFSET_FILL).
 
 draw_face_sel(true, SelDlist) ->
     case wings_pref:get_value(show_backfaces) of
@@ -371,7 +381,8 @@ draw_vertices(#dlo{src_we=#we{perm=P},vs=VsDlist}, vertex) when ?IS_SELECTABLE(P
     wings_dl:call(VsDlist);
 draw_vertices(_, _) -> ok.
 
-draw_hilite(#dlo{hilite=DL}) -> 
+draw_hilite(#dlo{hilite=none}) -> ok;
+draw_hilite(#dlo{hilite={_Mode,DL}}) ->
     wings_dl:call(DL).
 
 draw_orig_sel(#dlo{orig_sel=none}) -> ok;
@@ -399,9 +410,11 @@ draw_orig_sel_1(_, DlistSel) ->
     gl:blendFunc(?GL_SRC_ALPHA, ?GL_ONE_MINUS_SRC_ALPHA),
     {R0,G0,B0} = wings_pref:get_value(selected_color),
     gl:color4f(R0, G0, B0, 0.5),
-    gl:polygonOffset(1, 1),
+    gl:enable(?GL_POLYGON_OFFSET_FILL),
     gl:polygonMode(?GL_FRONT_AND_BACK, ?GL_FILL),
-    wings_dl:call(DlistSel).
+    polygonOffset(1.0),
+    wings_dl:call(DlistSel),
+    gl:disable(?GL_POLYGON_OFFSET_FILL).
 
 draw_hard_edges(#dlo{hard=none}, _) -> ok;
 draw_hard_edges(#dlo{hard=Hard}, SelMode) ->
@@ -411,7 +424,7 @@ draw_hard_edges(#dlo{hard=Hard}, SelMode) ->
 	
 draw_normals(#dlo{normals=none}) -> ok;
 draw_normals(#dlo{normals=Ns}) ->
-    gl:color3f(0, 0, 1),
+    gl:color3f(0.0, 0.0, 1.0),
     gl:lineWidth(wings_pref:get_value(normal_vector_width)),
     wings_dl:call(Ns).
 
@@ -424,73 +437,50 @@ hard_edge_width(_) -> max(wings_pref:get_value(hard_edge_width) - 1, 1).
 draw_plugins(Flag,D,Selmode) ->
     wings_plugin:draw(Flag, D, Selmode).
 
-ground_and_axes() ->
+ground_and_axes(PM,MM) ->
     Axes = wings_wm:get_prop(show_axes),
-    GridSize = groundplane(Axes),
+    GridSize = groundplane(Axes, PM, MM),
     Yon = case wings_pref:get_value(constrain_axes) of
 	      true  -> GridSize;
 	      false -> (wings_view:current())#view.yon
 	  end,
-    case Axes of
-	true ->
-	    axis(1, Yon, get_pref(x_color), get_pref(neg_x_color)),
-	    axis(2, Yon, get_pref(y_color), get_pref(neg_y_color)),
-	    axis(3, Yon, get_pref(z_color), get_pref(neg_z_color));
-	false -> ok
-    end,
+    Key = case Axes of
+	      true ->
+		  axis_data([{1,x_color,neg_x_color},
+			     {2,y_color,neg_y_color},
+			     {3,z_color,neg_z_color}],
+			    Yon);
+	      false ->
+		  none
+	  end,
+    Update = fun(Data) ->
+		     D = fun() ->
+				 gl:drawArrays(?GL_LINES, 0, 3*4)
+			 end,
+		     wings_vbo:new(D, Data, [color,vertex])
+	     end,
+    wings_dl:draw(axes, Key, Update),
     Yon.
 
 get_pref(Key) ->
     wings_pref:get_value(Key).
 
-axis(I, Yon, Pos, Neg) ->
+axis_data([{I,PosKey,NegKey}|T], Yon) ->
+    Pos = get_pref(PosKey),
+    Neg = get_pref(NegKey),
     A0 = {0.0,0.0,0.0},
     A = setelement(I, A0, Yon),
     B = setelement(I, A0, -Yon),
-    gl:'begin'(?GL_LINES),
-    gl:color3fv(Pos),
-    gl:vertex3fv(A0),
-    gl:vertex3fv(A),
-    gl:color3fv(Neg),
-    gl:vertex3fv(A0),
-    gl:vertex3fv(B),
-    gl:'end'().
+    [Pos,A0,Pos,A,Neg,A0,Neg,B|axis_data(T, Yon)];
+axis_data([], _) -> [].
 
-dummy_axis_letter() ->
-    %% Attempt to work around a crash occurring with Matrox cards.
-    MM = list_to_tuple(gl:getDoublev(?GL_MODELVIEW_MATRIX)),
-    PM = list_to_tuple(gl:getDoublev(?GL_PROJECTION_MATRIX)),
-    %% Since this is a workaround, we will do a real fetching
-    %% of the viewport (rather than wings_wm:viewport/0).
-    [X,Y,W,H|_] = gl:getIntegerv(?GL_VIEWPORT),
-    Viewport = {X,Y,W,H},
-    dummy_axis_letter(MM, PM, Viewport).
-
-dummy_axis_letter(_, _, {_,_,W,H}) ->
-    gl:matrixMode(?GL_PROJECTION),
-    gl:pushMatrix(),
-    gl:loadIdentity(),
-    glu:ortho2D(0, W, 0, H),
-    gl:matrixMode(?GL_MODELVIEW),
-    gl:pushMatrix(),
-    gl:loadIdentity(),
-    wings_io:set_color(wings_pref:get_value(background_color)),
-    axis_text(10, 90, axisx),
-    gl:popMatrix(),
-    gl:matrixMode(?GL_PROJECTION),
-    gl:popMatrix(),
-    gl:matrixMode(?GL_MODELVIEW).
-
-axis_letters(Yon0) ->
+axis_letters(TPM, TMM, Yon0) ->
     case wings_wm:get_prop(show_axes) of
 	false ->
-	    case wings_pref:get_value(dummy_axis_letter) of
-		false -> ok;
-		true -> dummy_axis_letter()
-	    end;
+	    ok;
 	true ->
-	    MM = list_to_tuple(gl:getDoublev(?GL_MODELVIEW_MATRIX)),
-	    PM = list_to_tuple(gl:getDoublev(?GL_PROJECTION_MATRIX)),
+	    PM = e3d_transform:matrix(TPM),
+	    MM = e3d_transform:matrix(TMM),
 	    ViewPort = wings_wm:viewport(),
 	    Start = {0.0,0.0,0.0},
 	    Origin = proj(Start, MM, PM),
@@ -499,10 +489,11 @@ axis_letters(Yon0) ->
 	    gl:matrixMode(?GL_PROJECTION),
 	    gl:loadIdentity(),
 	    {_,_,W,H} = ViewPort,
-	    glu:ortho2D(0, W, 0, H),
+	    glu:ortho2D(0.0, W, H, 0.0),
 	    gl:matrixMode(?GL_MODELVIEW),
 	    gl:loadIdentity(),
 	    Yon = Yon0 + ?GROUND_GRID_SIZE,
+	    gl:polygonMode(?GL_FRONT_AND_BACK, ?GL_FILL),
 	    axis_letter_1(1, Yon, axisx, x_color, Info),
 	    axis_letter_1(2, Yon, axisy, y_color, Info),
 	    axis_letter_1(3, Yon, axisz, z_color, Info)
@@ -521,6 +512,54 @@ axis_letter_1(I, Yon, Char, Color0, {Start,{Ox,Oy,_,Ow},MM,PM,Viewport}) ->
 	    clip(Ox, Oy, Ow, Px, Py, Pw, Char, Viewport)
     end.
 
+show_camera_image_plane() ->
+    case wings_wm:get_prop(show_cam_imageplane) of
+	false ->
+	    ok;
+	true ->
+	    {WinW,WinH} = wings_wm:win_size(),
+	    CamW = wings_pref:get_value(negative_width),
+	    CamH = wings_pref:get_value(negative_height),
+	    AspRatio = CamW/CamH,
+	    %% it's used only the height as reference because view horizon changes only for height variation
+	    {W,H} = {round(WinH*AspRatio)/2, WinH/2},
+
+	    X1 = (WinW/2.0)-W+5.0,
+	    Y1 = (WinH/2.0)-H+5.0,
+	    X2 = (WinW/2.0)+W-5.0,
+	    Y2 = (WinH/2.0)+H-5.0,
+
+	    Quads = [{X1,Y1,0.0},{X2,Y1,0.0},  			% top
+		     {WinW*1.0,0.0,0.0},{0.0,0.0,0.0},
+		     {X1,Y1,0.0},{0.0,0.0,0.0},			% left
+		     {0.0,WinH*1.0,0.0},{X1,Y2,0.0},
+		     {X1,Y2,0.0},{0.0,WinH*1.0,0.0},		% bottom
+		     {WinW*1.0,WinH*1.0,0.0},{X2*1.0,Y2,0.0},
+		     {X2*1.0,Y2,0.0},{WinW*1.0,WinH*1.0,0.0},	% right
+		     {WinW*1.0,0.0,0.0},{X2,Y1,0.0}],
+	    Poly = << <<X:?F32,Y:?F32,Z:?F32>> || {X,Y,Z} <- Quads >>,
+
+	    Update = fun draw_camera_image_plane/1,
+	    wings_dl:draw(draw_cam_imageplane, {Poly,{X1,Y1,X2,Y2}}, Update)
+    end.
+
+draw_camera_image_plane({Poly,{X1,Y1,X2,Y2}}) ->
+    NoVs = byte_size(Poly) div 12,
+    Draw =
+    fun() ->
+	wings_io:ortho_setup(),
+	gl:enable(?GL_BLEND),
+	gl:blendFunc(?GL_SRC_ALPHA, ?GL_ONE_MINUS_SRC_ALPHA),
+	gl:color4f(0.0, 0.4, 0.8, 0.5),
+	gl:drawArrays(?GL_QUADS, 0, NoVs),
+	gl:color3f(0.0, 0.4, 0.8),
+	gl:lineWidth(2.0),
+	gl:polygonMode(?GL_FRONT_AND_BACK, ?GL_LINE),
+	gl:rectf(X2, Y1, X1, Y2),
+	gl:flush()
+    end,
+    wings_vbo:new(Draw, Poly).
+
 clip(Ox, Oy, Ow, Px, Py, Pw, Char, Viewport) ->
     AxisRay = line(Ox, Oy, Px, Py),
     NegOw = -Ow,
@@ -551,12 +590,11 @@ clip_1(_, [], _W) -> none.
 
 show_letter(X0, Y0, W, Char, {_,_,Vw,Vh}) ->
     X = trunc((0.5*X0/W+0.5)*(Vw-20) + 10),
-    Y = trunc((0.5*Y0/W+0.5)*(Vh-16) + 7),
+    Y = Vh - trunc((0.5*Y0/W+0.5)*(Vh-16) + 7),
     axis_text(X, Y, Char).
 
 axis_text(X, Y, C) ->
-    gl:rasterPos2i(X, Y),
-    wings_text:char(C).
+    wings_io:unclipped_text(X, Y, [C]).
 
 proj({X0,Y0,Z0}, MM, PM) ->
     e3d_mat:mul(PM, e3d_mat:mul(MM, {X0,Y0,Z0,1.0})).
@@ -572,103 +610,133 @@ sub({X1,Y1}, {X2,Y2}) ->
 add_prod({X1,Y1}, {X2,Y2}, S) when is_float(S) ->
     {S*X2+X1,S*Y2+Y1}.
 
-calc_grid_size() ->
-    {MM,PM,{_,_,W0,H0}=Viewport} = wings_u:get_matrices(0, original),
-    W1=max(W0,H0)/2.0,
-    {S,T,U} = wings_gl:unProject(W1, 0.0, 0.0, MM, PM, Viewport),
+calc_grid_size(PM,MM) ->
+    Viewport = {_,_,W,H} =  wings_wm:viewport(),
+    W1=max(W,H)/2.0,
+    {S,T,U} = wings_gl:unProject(W1, 0.0, 0.0,
+				 e3d_transform:matrix(MM),
+				 e3d_transform:matrix(PM),
+				 Viewport),
     ?GROUND_GRID_SIZE*max(round(max(max(abs(S),abs(T)),abs(U))),10.0).
 
-groundplane(Axes) ->
-    GridSize = calc_grid_size(),
-    case (wings_wm:get_prop(show_groundplane) orelse
+groundplane(Axes, PM, MM) ->
+    GridSize = calc_grid_size(PM, MM),
+    Show = wings_wm:get_prop(show_groundplane) orelse
 	  (wings_pref:get_value(force_show_along_grid) andalso
-	   (wings_view:current())#view.along_axis =/= none)) of
-	true -> groundplane_1(Axes, GridSize);
-	false -> ok
-    end,
+	   (wings_view:current())#view.along_axis =/= none),
+    Key = case Show of
+	      true ->
+		  #view{along_axis=Along} = wings_view:current(),
+		  Color = wings_pref:get_value(grid_color),
+		  {Along,GridSize,Axes,Color};
+	      false ->
+		  none
+	  end,
+    wings_dl:draw(groundplane, Key, fun update_groundplane/1),
     float(GridSize).
 
-groundplane_1(Axes, Sz) ->
-    #view{along_axis=Along} = wings_view:current(),
-    gl:color3fv(wings_pref:get_value(grid_color)),
-    gl:lineWidth(1),
-    gl:matrixMode(?GL_MODELVIEW),
-    gl:pushMatrix(),
-    case Along of
-	x -> gl:rotatef(90, 0, 1, 0);
-	z -> ok;
-	_ -> gl:rotatef(90, 1, 0, 0)
-    end,
-    gl:'begin'(?GL_LINES),
-    groundplane_2(-Sz, Sz, Sz, Axes),
-    gl:'end'(),
-    gl:popMatrix(),
-    ?CHECK_ERROR().
+update_groundplane({Along,Sz,Axes,Color}) ->
+    Data = groundplane_2(-Sz, Sz, Sz, Axes),
+    N = length(Data),
+    Draw = fun() ->
+		   gl:color3fv(Color),
+		   gl:lineWidth(1.0),
+		   gl:matrixMode(?GL_MODELVIEW),
+		   gl:pushMatrix(),
+		   case Along of
+		       x -> gl:rotatef(90.0, 0.0, 1.0, 0.0);
+		       z -> ok;
+		       _ -> gl:rotatef(90.0, 1.0, 0.0, 0.0)
+		   end,
+		   gl:drawArrays(?GL_LINES, 0, N),
+		   gl:popMatrix(),
+		   ?CHECK_ERROR()
+	   end,
+    wings_vbo:new(Draw, Data).
 
-groundplane_2(X, Last, _Sz, _Axes) when X > Last -> ok;
-groundplane_2(X, Last, Sz, true) when X == 0 ->
+groundplane_2(X, Last, _Sz, _Axes) when X > Last ->
+    [];
+groundplane_2(X, Last, Sz, true) when X == 0->
+    %% Skip ground plane where the axes go.
     groundplane_2(?GROUND_GRID_SIZE, Last, Sz, true);
 groundplane_2(X, Last, Sz, Axes) ->
-    gl:vertex2f(X, -Sz),
-    gl:vertex2f(X, Sz),
-    gl:vertex2f(-Sz, X),
-    gl:vertex2f(Sz, X),
-    groundplane_2(X+?GROUND_GRID_SIZE, Last, Sz, Axes).
+    NegSz = -Sz,
+    [{X,NegSz,0},{X,Sz,0},{NegSz,X,0},{Sz,X,0}|
+     groundplane_2(X+?GROUND_GRID_SIZE, Last, Sz, Axes)].
 
-show_saved_bb(#st{bb=[{X1,Y1,Z1},{X2,Y2,Z2}]}) ->
-    case wings_pref:get_value(show_bb) of
-	false -> ok;
-	true ->
-	    gl:enable(?GL_LINE_STIPPLE),
-	    gl:lineStipple(4, 2#1110111011101110),
-	    gl:color3fv(wings_pref:get_value(active_vector_color)),
-	    gl:'begin'(?GL_LINE_STRIP),
-	    gl:vertex3f(X1, Y1, Z1),
-	    gl:vertex3f(X2, Y1, Z1),
-	    gl:vertex3f(X2, Y2, Z1),
-	    gl:vertex3f(X1, Y2, Z1),
-	    gl:vertex3f(X1, Y1, Z1),
-	    gl:vertex3f(X1, Y1, Z2),
-	    gl:vertex3f(X2, Y1, Z2),
-	    gl:vertex3f(X2, Y2, Z2),
-	    gl:vertex3f(X1, Y2, Z2),
-	    gl:vertex3f(X1, Y1, Z2),
-	    gl:'end'(),
-	    gl:'begin'(?GL_LINES),
-	    gl:vertex3f(X1, Y2, Z1),
-	    gl:vertex3f(X1, Y2, Z2),
-	    gl:vertex3f(X2, Y2, Z1),
-	    gl:vertex3f(X2, Y2, Z2),
-	    gl:vertex3f(X2, Y1, Z1),
-	    gl:vertex3f(X2, Y1, Z2),
-	    gl:'end'(),
-	    gl:disable(?GL_LINE_STIPPLE)
-    end;
-show_saved_bb(_) -> ok.
+show_saved_bb(St) ->
+    Key = get_saved_bb_key(St),
+    Update = fun update_saved_bb/1,
+    wings_dl:draw(saved_bb, Key, Update).
 
-show_bb_center(#st{bb=[_,_]=BB}) ->
-    case wings_pref:get_value(show_bb_center) of
-      false -> ok;
-      true ->
-        {Cx,Cy,Cz} = Center = e3d_vec:average(BB),
-        Colour = gl:color3fv(wings_pref:get_value(active_vector_color)),
-        Colour,
-        gl:pointSize(8.0),
-        gl:'begin'(?GL_POINTS),
-        gl:vertex3fv(Center),
-        gl:'end'(),
-        gl:'begin'(?GL_LINES),
-        Colour,
-        gl:vertex3fv({Cx,Cy+0.2,Cz}),
-        gl:vertex3fv({Cx,Cy-0.2,Cz}),
-        gl:vertex3fv({Cx+0.2,Cy,Cz}),
-        gl:vertex3fv({Cx-0.2,Cy,Cz}),
-        gl:vertex3fv({Cx,Cy,Cz+0.2}),
-        gl:vertex3fv({Cx,Cy,Cz-0.2}),
-        gl:'end'()
+get_saved_bb_key(#st{bb=BB}) ->
+    case {wings_pref:get_value(show_bb),BB} of
+	{true,[A,B]} ->
+	    Color = wings_pref:get_value(active_vector_color),
+	    {A,B,Color};
+	{_,_} ->
+	    none
+    end.
 
-    end;
-show_bb_center(_) -> ok.
+update_saved_bb({{X1,Y1,Z1},{X2,Y2,Z2},Color}) ->
+    %% 10 vertices in a line strip.
+    Data = [{X1,Y1,Z1},
+	    {X2,Y1,Z1},
+	    {X2,Y2,Z1},
+	    {X1,Y2,Z1},
+	    {X1,Y1,Z1},
+	    {X1,Y1,Z2},
+	    {X2,Y1,Z2},
+	    {X2,Y2,Z2},
+	    {X1,Y2,Z2},
+	    {X1,Y1,Z2},
+	    %% 6 vertices / 3 lines
+	    {X1,Y2,Z1},
+	    {X1,Y2,Z2},
+	    {X2,Y2,Z1},
+	    {X2,Y2,Z2},
+	    {X2,Y1,Z1},
+	    {X2,Y1,Z2}],
+    D = fun() ->
+		gl:enable(?GL_LINE_STIPPLE),
+		gl:lineStipple(4, 2#1110111011101110),
+		gl:color3fv(Color),
+		gl:drawArrays(?GL_LINE_STRIP, 0, 10),
+		gl:drawArrays(?GL_LINES, 10, 6),
+		gl:disable(?GL_LINE_STIPPLE)
+	end,
+    wings_vbo:new(D, Data).
+
+show_bb_center(St) ->
+    Key = get_bb_center_key(St),
+    Update = fun update_bb_center/1,
+    wings_dl:draw(saved_bb_center, Key, Update).
+
+get_bb_center_key(#st{bb=BB}) ->
+    case {wings_pref:get_value(show_bb_center),BB} of
+	{true,[_,_]} ->
+	    Center = e3d_vec:average(BB),
+	    Color = wings_pref:get_value(active_vector_color),
+	    {Center,Color};
+	{_,_} ->
+	    none
+    end.
+
+update_bb_center({{Cx,Cy,Cz}=Center,Color}) ->
+    Data = [Center,
+	    {Cx,Cy+0.2,Cz},
+	    {Cx,Cy-0.2,Cz},
+	    {Cx+0.2,Cy,Cz},
+	    {Cx-0.2,Cy,Cz},
+	    {Cx,Cy,Cz+0.2},
+	    {Cx,Cy,Cz-0.2}],
+    D = fun() ->
+		gl:color3fv(Color),
+		gl:pointSize(8.0),
+		gl:drawArrays(?GL_POINTS, 0, 1),
+		gl:drawArrays(?GL_LINES, 1, 6)
+	end,
+    wings_vbo:new(D, Data).
 
 enable_lighting(SceneLights) ->
     Progs = get(light_shaders),
@@ -696,112 +764,120 @@ disable_lighting() ->
 	false -> ok
     end.
 
-mini_axis_icon() ->
-    case wings_pref:get_value(mini_axis) of 
-    false -> ok;
-    true ->
-      gl:pushAttrib(?GL_ALL_ATTRIB_BITS),
-      {W,H} = wings_wm:win_size(),
-      Matrix1 = list_to_tuple(gl:getDoublev(?GL_MODELVIEW_MATRIX)),
-      Matrix2 = setelement(13, Matrix1, -W/H+0.11),
-      Matrix3 = setelement(14, Matrix2, -1.0+0.11),
-      Matrix4 = setelement(15, Matrix3, 0.0),
-      gl:matrixMode(?GL_PROJECTION),
-      gl:pushMatrix(),
-      gl:loadIdentity(),
-      gl:ortho(-W/H, W/H, -1, 1, 0.00001, 10000000.0),
-      gl:matrixMode(?GL_MODELVIEW),
-      gl:pushMatrix(),
-      gl:loadIdentity(),
-      gl:loadMatrixd(Matrix4),
-      draw_mini_axis(),
-      gl:popMatrix(),
-      gl:matrixMode(?GL_PROJECTION),
-      gl:popMatrix(),
-      gl:matrixMode(?GL_MODELVIEW),
-      gl:popAttrib()
-	end.
+mini_axis_icon(MM) ->
+    case mini_axis_icon_key() of
+	none -> ok;
+	Key -> draw_mini_axis_icon(Key, MM)
+    end.
 
-draw_mini_axis() ->
-    {PA,PB} = {0.08,0.01},
-    X = wings_pref:get_value(x_color),
-    Y = wings_pref:get_value(y_color),
-    Z = wings_pref:get_value(z_color),
-    gl:'begin'(?GL_LINES),
-    %% X Axis
-    gl:color3fv(X),
-    gl:vertex3f(0,0,0),
-    gl:vertex3f(0.1,0.0,0.0),
-    %% Y Axis
-    gl:color3fv(Y),
-    gl:vertex3f(0,0,0),
-    gl:vertex3f(0.0,0.1,0.0),
-    %% Z Axis
-    gl:color3fv(Z),
-    gl:vertex3f(0,0,0),
-    gl:vertex3f(0.0,0.0,0.1),
-    View = wings_view:current(),
-    case View#view.along_axis of
+draw_mini_axis_icon(Key, MM) ->
+    {W,H} = wings_wm:win_size(),
+    Ratio = W/H,
+    Matrix0 = e3d_transform:matrix(MM),
+    Matrix1 = setelement(15, Matrix0, 0.0),
+    Matrix2 = setelement(14, Matrix1, -1.0+0.11),
+    Matrix  = setelement(13, Matrix2, 0.11-Ratio),
+    gl:pushAttrib(?GL_ALL_ATTRIB_BITS),
+    gl:matrixMode(?GL_PROJECTION),
+    gl:pushMatrix(),
+    gl:loadIdentity(),
+    gl:ortho(-Ratio, Ratio, -1.0, 1.0, 0.00001, 10000000.0),
+    gl:matrixMode(?GL_MODELVIEW),
+    gl:pushMatrix(),
+    gl:loadMatrixd(Matrix),
+
+    Update = fun update_mini_axis_icon/1,
+    wings_dl:draw(mini_axis_icon, Key, Update),
+
+    gl:popMatrix(),
+    gl:matrixMode(?GL_PROJECTION),
+    gl:popMatrix(),
+    gl:matrixMode(?GL_MODELVIEW),
+    gl:popAttrib().
+
+mini_axis_icon_key() ->
+    case wings_pref:get_value(mini_axis) of
+	false ->
+	    none;
+	true ->
+	    #view{along_axis=Along} = wings_view:current(),
+	    X = wings_pref:get_value(x_color),
+	    Y = wings_pref:get_value(y_color),
+	    Z = wings_pref:get_value(z_color),
+	    {Along,{X,Y,Z}}
+    end.
+
+update_mini_axis_icon({Along,{X,Y,Z}}) ->
+    Arrows = mini_axis_arrows(Along, X, Y, Z),
+    Data = [X,{0.0,0.0,0.0},			%X Axis
+	    X,{0.1,0.0,0.0},
+	    Y,{0.0,0.0,0.0},			%Y Axis
+	    Y,{0.0,0.1,0.0},
+	    Z,{0.0,0.0,0.0},			%Z Axis
+	    Z,{0.0,0.0,0.1}|Arrows],
+    N = case Along of
+	    none -> 3*2 + 3*4;
+	    _ -> 3*2 + 2*4
+	end,
+    D = fun() ->
+		gl:drawArrays(?GL_LINES, 0, N)
+	end,
+    wings_vbo:new(D, Data, [color,vertex]).
+
+mini_axis_arrows(Along, X, Y, Z) ->
+    PA = 0.08,
+    PB = 0.01,
+    case Along of
 	none ->
 	    %% X Arrows
-	    gl:color3fv(X),
-	    gl:vertex3f(PA,0.0,-PB),
-	    gl:vertex3f(0.1,0.0,0.0),
-	    gl:vertex3f(PA,0.0,PB),
-	    gl:vertex3f(0.1,0.0,0.0),
-	    %% Y Arrows
-	    gl:color3fv(Y),
-	    gl:vertex3f(-PB,PA,0.0),
-	    gl:vertex3f(0.0,0.1,0.0),
-	    gl:vertex3f(PB,PA,0.0),
-	    gl:vertex3f(0.0,0.1,0.0),
-	    %% Z Arrows
-	    gl:color3fv(Z),
-	    gl:vertex3f(-PB,0.0,PA),
-	    gl:vertex3f(0.0,0.0,0.1),
-	    gl:vertex3f(PB,0.0,PA),
-	    gl:vertex3f(0.0,0.0,0.1);
+	    [X,{PA,0.0,-PB},
+	     X,{0.1,0.0,0.0},
+	     X,{PA,0.0,PB},
+	     X,{0.1,0.0,0.0},
+	     %% Y Arrows
+	     Y,{-PB,PA,0.0},
+	     Y,{0.0,0.1,0.0},
+	     Y,{PB,PA,0.0},
+	     Y,{0.0,0.1,0.0},
+	     %% Z Arrows
+	     Z,{-PB,0.0,PA},
+	     Z,{0.0,0.0,0.1},
+	     Z,{PB,0.0,PA},
+	     Z,{0.0,0.0,0.1}];
 	x ->
 	    %% Y Arrows
-	    gl:color3fv(Y),
-	    gl:vertex3f(0.0,PA,-PB),
-	    gl:vertex3f(0.0,0.1,0.0),
-	    gl:vertex3f(0.0,PA,PB),
-	    gl:vertex3f(0.0,0.1,0.0),
-	    %% Z Arrows
-	    gl:color3fv(Z),
-	    gl:vertex3f(0.0,-PB,PA),
-	    gl:vertex3f(0.0,0.0,0.1),
-	    gl:vertex3f(0.0,PB,PA),
-	    gl:vertex3f(0.0,0.0,0.1);
+	    [Y,{0.0,PA,-PB},
+	     Y,{0.0,0.1,0.0},
+	     Y,{0.0,PA,PB},
+	     Y,{0.0,0.1,0.0},
+	     %% Z Arrows
+	     Z,{0.0,-PB,PA},
+	     Z,{0.0,0.0,0.1},
+	     Z,{0.0,PB,PA},
+	     Z,{0.0,0.0,0.1}];
 	y ->
 	    %% X Arrows
-	    gl:color3fv(X),
-	    gl:vertex3f(PA,0.0,-PB),
-	    gl:vertex3f(0.1,0.0,0.0),
-	    gl:vertex3f(PA,0.0,PB),
-	    gl:vertex3f(0.1,0.0,0.0),
-	    %% Z Arrows
-	    gl:color3fv(Z),
-	    gl:vertex3f(-PB,0.0,PA),
-	    gl:vertex3f(0.0,0.0,0.1),
-	    gl:vertex3f(PB,0.0,PA),
-	    gl:vertex3f(0.0,0.0,0.1);
+	    [X,{PA,0.0,-PB},
+	     X,{0.1,0.0,0.0},
+	     X,{PA,0.0,PB},
+	     X,{0.1,0.0,0.0},
+	     %% Z Arrows
+	     Z,{-PB,0.0,PA},
+	     Z,{0.0,0.0,0.1},
+	     Z,{PB,0.0,PA},
+	     Z,{0.0,0.0,0.1}];
 	z ->
 	    %% X Arrows
-	    gl:color3fv(X),
-	    gl:vertex3f(PA,-PB,0.0),
-	    gl:vertex3f(0.1,0.0,0.0),
-	    gl:vertex3f(PA,PB,0.0),
-	    gl:vertex3f(0.1,0.0,0.0),
-	    %% Y Arrows
-	    gl:color3fv(Y),
-	    gl:vertex3f(-PB,PA,0.0),
-	    gl:vertex3f(0.0,0.1,0.0),
-	    gl:vertex3f(PB,PA,0.0),
-	    gl:vertex3f(0.0,0.1,0.0)
-    end,
-    gl:'end'().
+	    [X,{PA,-PB,0.0},
+	     X,{0.1,0.0,0.0},
+	     X,{PA,PB,0.0},
+	     X,{0.1,0.0,0.0},
+	     %% Y Arrows
+	     Y,{-PB,PA,0.0},
+	     Y,{0.0,0.1,0.0},
+	     Y,{PB,PA,0.0},
+	     Y,{0.0,0.1,0.0}]
+    end.
 
 user_clipping_planes(on) ->
     case wings_wm:get_prop(clip_plane) of
@@ -836,4 +912,3 @@ draw_clip_disk(Direction, Expand) ->
     glu:disk(Obj, 0.0, wings_pref:get_value(clip_plane_size), 35, 1),
     gl:popMatrix(),
     glu:deleteQuadric(Obj).
-
