@@ -12,10 +12,10 @@
 %%
 
 -module(wings_u).
--export([error_msg/1,error_msg/2,message/1,get_matrices/2,
-	 geom_windows/0,menu_restriction/2,
+-export([error_msg/1,error_msg/2,message/1,debug/1,
+	 get_matrices/2, geom_windows/0,
 	 yes_no/2,yes_no/3,yes_no_cancel/3,
-	 export_we/2,win_crash/1,crash_log/2,crash_log/3,
+	 export_we/2,win_crash/1,win_crash/2,crash_log/2,crash_log/3,
 	 pretty_filename/1,relative_path_name/2,caption/1,win32_special_folder/2]).
 
 -define(NEED_OPENGL, 1).
@@ -34,53 +34,41 @@ error_msg(Format, Arg) ->
     error_msg(lists:flatten(io_lib:format(Format, Arg))).
 
 message(Message) ->
-    Qs = {vframe,
-	  [{label,Message},
-	   {hframe,[{button,ok,[ok]}]}]},
-    wings_ask:dialog("", Qs, fun(_) -> ignore end).
-
-menu_restriction(Win, Allowed) ->
-    case wings_wm:get_menubar(Win) of
-	none -> wings_wm:menubar(Win, []);
-	Mb0 ->
-	    Mb = [Item || {_,Name,_}=Item <- Mb0, member(Name, Allowed)],
-	    wings_wm:menubar(Win, Mb)
-    end.
+    Qs = {vframe_dialog,
+	  [{label,Message}],
+	  [{buttons,[ok]}]},
+    wings_dialog:dialog("", Qs, fun(_) -> ignore end).
 
 geom_windows() ->
     geom_windows_1(wings_wm:windows()).
 
 get_matrices(Id, MM) ->
-    wings_view:load_matrices(false),
-    case MM of
-	mirror ->
-	    Matrix = wings_dl:mirror_matrix(Id),
-	    gl:multMatrixf(Matrix);
-	original -> ok
-    end,
+    {TPM, TMV0, _} = wings_view:load_matrices(false),
+    TMV = case MM of
+	      mirror ->
+		  Matrix = wings_dl:mirror_matrix(Id),
+		  e3d_mat:mul(e3d_transform:matrix(TMV0),Matrix);
+	      original -> e3d_transform:matrix(TMV0)
+	  end,
     {_,_,W,H} =  wings_wm:viewport(),
-    ModelMatrix = gl:getDoublev(?GL_MODELVIEW_MATRIX),
-    ProjMatrix = gl:getDoublev(?GL_PROJECTION_MATRIX),
-    {ModelMatrix,ProjMatrix,{0,0,W,H}}.
+    {TMV,e3d_transform:matrix(TPM),{0,0,W,H}}.
 
 yes_no(Question, Yes) ->
     yes_no(Question, Yes, ignore).
 
 yes_no(Question, Yes, No) ->
-    Qs = {vframe,
-	  [{label,Question,[{break,45*?CHAR_WIDTH}]},
-	   {hframe,[{button,wings_s:yes(),yes_no_fun(Yes)},
-		    {button,wings_s:no(),yes_no_fun(No),[cancel]}]}]},
-    wings_ask:dialog("", Qs, fun(_) -> ignore end).
+    Qs = {vframe_dialog,
+	  [{label,Question,[{break,75*?CHAR_WIDTH}]}],
+	  [{buttons, [yes, no]}, {key, result}]
+	 },
+    wings_dialog:dialog("", Qs, fun([{result,Res}]) -> yes_no_cancel(Res, Yes, No, ignore) end).
 
 yes_no_cancel(Question, Yes, No) ->
-    Qs = {vframe,
-	  [{label,Question,[{break,45*?CHAR_WIDTH}]},
-	   {hframe,[{button,wings_s:yes(),yes_no_fun(Yes)},
-		    {button,wings_s:no(),yes_no_fun(No)},
-		    {button,wings_s:cancel(),
-		     yes_no_fun(ignore),[cancel]}]}]},
-    wings_ask:dialog("", Qs, fun(_) -> ignore end).
+    Qs = {vframe_dialog,
+	  [{label,Question,[{break,75*?CHAR_WIDTH}]}],
+	  [{buttons, [yes, no, cancel]}, {key, result}]
+	 },
+    wings_dialog:dialog("", Qs, fun([{result,Res}]) -> yes_no_cancel(Res, Yes, No, ignore) end).
 
 %% export_we(Filename, State)
 %%  Dump the winged-edge structure in a textual format.
@@ -94,7 +82,10 @@ export_we(Name, #st{shapes=Shs}) ->
     end.
 
 win_crash(Reason) ->
-    LogName = crash_log(wings_wm:this(), Reason),
+    win_crash(wings_wm:this(), Reason).
+
+win_crash(Window, Reason) ->
+    LogName = crash_log(Window, Reason),
     wings_wm:send(geom, {crash_in_other_window,LogName}).
 
 crash_log(WinName, Reason) ->
@@ -102,13 +93,14 @@ crash_log(WinName, Reason) ->
     crash_log(WinName, Reason, StackTrace).
 
 crash_log(WinName, Reason, StackTrace) ->
-    wings_pb:cancel(),
+    catch wings_pb:cancel(),
     LogFileDir = log_file_dir(),
     LogName = filename:absname("wings_crash.dump", LogFileDir),
     F = open_log_file(LogName),
-    io:format(F, "Version: ~s\n", [?WINGS_VERSION]),
-    io:format(F, "Window: ~p\n", [WinName]),
-    io:format(F, "Reason: ~p\n\n", [Reason]),
+    io:format("Internal Error~n",[]),
+    [io:format(Fd, "Version: ~s\n", [?WINGS_VERSION]) || Fd <- [F, group_leader()]],
+    [io:format(Fd, "Window: ~p\n", [WinName])  || Fd <- [F, group_leader()]],
+    [io:format(Fd, "Reason: ~p\n\n", [Reason]) || Fd <- [F, group_leader()]],
     report_stacktrace(F, StackTrace),
     analyse(F, StackTrace),
     file:close(F),
@@ -119,6 +111,7 @@ report_stacktrace(F, [_|_]=StackTrace) ->
 				is_list(A) -> length(A);
 				true -> A
 			    end} || {M,N,A} <- StackTrace],
+    io:format("Stack trace:\n~P\n\n", [StackTrace, 20]),
     case ShortStackTrace =:= StackTrace of
 	false ->
 	    io:format(F, "Short stack trace:\n~p\n\n", [ShortStackTrace]),
@@ -151,22 +144,10 @@ caption(#st{file=Name}=St) ->
     St.
 
 %% pretty_filename(Name) -> PrettyName
-%%  If the Name *seems* to be encoded in UTF8, convert to a
-%%  list of Unicode characters. Leave it alone otherwise.
-%%
+%% This function tried to handle utf8 conversions
+%% but now filename may be full Unicode lists.
 pretty_filename(Name0) ->
-    case all(fun(C) -> C < 128 end, Name0) of
-	true ->
-	    %% Only 7-bit ASCII characters. Already fine.
-	    Name0;
-	false ->
-	    %% Try to convert it from UTF8 to a list of Unicode characters.
-	    Name1 = list_to_binary(Name0),
-	    case unicode:characters_to_list(Name1, utf8) of
-		Name when is_list(Name) -> unicode_combine(Name);
-		_ -> Name0
-	    end
-    end.
+    Name0.
 
 %% it returns a new file name using relative path
 %% Dst: it's the relative folder (used to be the project directory)
@@ -176,30 +157,49 @@ relative_path_name(Dst,Src) ->
     Fn=filename:basename(Src),
     Dir0=filename:split(get_dir(Dst)),
     Dir1=filename:split(get_dir(Src)),
-	case get_rel_path(Dir0,Dir1,[]) of
-		[] -> Fn;
-		Dir3 -> filename:join(Dir3++[Fn])
-	end.
+    case get_rel_path(Dir0,Dir1) of
+        [] -> Fn;
+        Dir3 -> filename:join(Dir3++[Fn])
+    end.
 
 get_dir(Dir) ->
-	case filelib:is_dir(Dir) of
-		true -> Dir;
-		_ -> filename:dirname(Dir)
-	end.
+    case filelib:is_dir(Dir) of
+        true -> Dir;
+        _ -> filename:dirname(Dir)
+    end.
 
-get_rel_path([]=_Dst, []=_Src, Acc) -> Acc;  % export and image dir are the same
-get_rel_path([_|T], [], Acc) -> get_rel_path(T,[],["../"]++Acc);  % image is in some up level directory
-get_rel_path([], [H|T], Acc) -> get_rel_path([],T,[H]++Acc);  % image is in some down level directory
-get_rel_path([H|T0], [H|T1], Acc) -> get_rel_path(T0,T1, Acc);  % continue checking in next dir level
-get_rel_path(_, Dst, _) -> Dst.  % image is in an other disk
+get_rel_path([D|_]=Dst,[D|_]=Src) ->    % paths are in the same drive - check relative path
+    get_rel_path(Dst,Src,[]);
+get_rel_path(_,Src) -> Src.   % paths are in a different Drive - ignore routine
+
+get_rel_path([]=_Dst, []=_Src, Acc) -> Acc;     % export and file dir are the same
+get_rel_path([_|T], [], Acc) ->             % file is in some dir level in the Dst
+    get_rel_path(T,[],["../"]++Acc);
+get_rel_path([], [H|T], Acc)->              % all previous dir match - file is in uppper dir level
+    get_rel_path([],T,[H]++Acc);
+get_rel_path([H|T0], [H|T1], Acc) ->        % continue checking in next dir level
+    get_rel_path(T0,T1,Acc);
+get_rel_path(Dst, Src, _Acc) ->             % no more compatible subdir and file is in some down dir level
+    get_rel_path_0(Dst,Src).
+
+get_rel_path_0([], Acc) -> Acc;
+get_rel_path_0([_|T], Acc) ->
+    get_rel_path_0(T, ["../"]++Acc).
 
 
 wings() ->
-    case ?wings_branch of
+    case wings_branch() of
 	"" -> debug("Wings3D");
 	_ -> "Wings3D " ++ ?wings_version ++ " (" ++ ?wings_branch ++ ")"
     end.
 
+wings_branch() ->
+    %% Fool dialyzer
+    case is_process_alive(self()) of
+        true -> ?wings_branch;
+        false -> ""
+    end.
+
 -ifdef(DEBUG).
 debug(Caption) -> Caption ++ " [debug]".
 -else.
@@ -221,15 +221,22 @@ win32_special_folder(R, FolderType) ->
 %%% Local functions.
 %%%
 
-yes_no_fun(ignore) -> fun(_) -> ignore end;
-yes_no_fun(Fun) ->
-    This = wings_wm:this(),
-    fun(_) ->
-	    case Fun() of
-		ignore -> ignore;
-		Action -> wings_wm:send(This, {action,Action})
-	    end
-    end.
+yes_no_cancel(ok,  Yes, _, _) -> invoke(Yes);
+yes_no_cancel(yes, Yes, _, _) -> invoke(Yes);
+yes_no_cancel(no, _, No, _)   -> invoke(No);
+yes_no_cancel(cancel, _, _, Cancel)   -> invoke(Cancel).
+
+invoke(Fun) when is_function(Fun) -> Fun();
+invoke(ignore) -> ignore.
+
+%% yes_no_fun(Fun) ->
+%%     This = wings_wm:this(),
+%%     fun(_) ->
+%% 	    case Fun() of
+%% 		ignore -> ignore;
+%% 		Action -> wings_wm:send(This, {action,Action})
+%% 	    end
+%%     end.
 
 geom_windows_1([geom|T]) ->
     [geom|geom_windows_1(T)];
@@ -333,45 +340,3 @@ show_edge(F, Edge, #edge{vs=Vs,ve=Ve,lf=Lf,rf=Rf,ltpr=Lpred,ltsu=Lsucc,
 
 show_face(F, Face, Edge) ->
     io:format(F, "~p: edge=~p\n", [Face,Edge]).
-
-
-%% unicode_combine(String0) -> String
-%%  Combine diacritical characters with the previous character.
-%%  (We need to do this because Mac OS X encodes filenames in
-%%  this way. Sigh!)
-%%
-%%  For instance, the characters 'LATIN CAPITAL LETTER A' (U+0041) and
-%%  'COMBINING DIAERESIS' (U+0308) combines to form the letter
-%%  'LATIN CAPITAL LETTER A WITH DIAERESIS' (U+00C4).
-%%
-%%  Currently, we only handle a few of the possible character combinations,
-%%  namely those that occur in Swedish/Finnish, German, and Russian.
-%%
-unicode_combine([$A,16#0308|T]) ->
-    [16#C4|unicode_combine(T)];			%"Ä"
-unicode_combine([$a,16#0308|T]) ->
-    [16#E4|unicode_combine(T)];			%"ä"
-unicode_combine([$A,16#030A|T]) ->
-    [16#C5|unicode_combine(T)];			%"Å"
-unicode_combine([$a,16#030A|T]) ->
-    [16#E5|unicode_combine(T)];			%"å"
-unicode_combine([$O,16#0308|T]) ->
-    [16#D6|unicode_combine(T)];			%"Ö"
-unicode_combine([$o,16#0308|T]) ->
-    [16#F6|unicode_combine(T)];			%"ö"
-unicode_combine([$U,16#0308|T]) ->
-    [16#DC|unicode_combine(T)];			%"Ü"
-unicode_combine([$u,16#0308|T]) ->
-    [16#FC|unicode_combine(T)];			%"ü"
-%% Russian.
-unicode_combine([16#0415,16#0308|T]) ->
-    [16#0401|unicode_combine(T)];		%"Ё"
-unicode_combine([16#0435,16#0308|T]) ->
-    [16#0451|unicode_combine(T)];		%"ё"
-unicode_combine([16#0418,16#0306|T]) ->
-    [16#0419|unicode_combine(T)];		%"Й"
-unicode_combine([16#0438,16#0306|T]) ->
-    [16#0439|unicode_combine(T)];		%"й"
-unicode_combine([H|T]) ->
-    [H|unicode_combine(T)];
-unicode_combine([]) -> [].
