@@ -23,11 +23,13 @@
 	 gb_trees_smallest_key/1,gb_trees_largest_key/1,
 	 array_keys/1,array_smallest_key/1,array_greatest_key/1,
 	 array_is_empty/1,array_entries/1,
+	 mapsfind/3,
+	 wxequal/2, wxset_pid/2, min_wx/1,
 	 nice_float/1,
 	 unique_name/2,
 	 is_name_masked/2,
 	 lib_dir/1,
-	 tc/3,
+	 tc/3, profile_start/1, profile_stop/1,
 	 limit/2]).
 
 -define(NEED_OPENGL, 1).
@@ -78,6 +80,7 @@ make_vector(Axis) when Axis == last_axis; Axis == default_axis ->
     {_,Vec} = wings_pref:get_value(Axis),
     Vec.
 
+
 key_format(Key, Msg) ->
     [Key,160,Msg].
 
@@ -194,6 +197,39 @@ simplify_float_1("0."++_=F) -> F;
 simplify_float_1("0"++F) -> simplify_float_1(F);
 simplify_float_1(F) -> F.
 
+%%
+%% Finds the first map containing Key:=Value
+%%
+-spec mapsfind(term(), term(), list()) -> map()|false.
+mapsfind(Value, Key, [H|T]) ->
+    case H of
+	#{Key:=Value} -> H;
+	_ -> mapsfind(Value, Key, T)
+    end;
+mapsfind(_, _, []) -> false.
+
+
+%% Missing wx funcs
+-record(wx_ref, {ref, type, state}).
+wxequal(#wx_ref{ref=Ref1}, #wx_ref{ref=Ref2}) -> Ref1 =:= Ref2.
+wxset_pid(#wx_ref{}=R, Pid) when is_pid(Pid) ->
+    R#wx_ref{state=Pid}.
+
+min_wx({_,_}=Ver) ->
+    case get(wx_version) of
+        undefined ->
+            case filename:basename(code:lib_dir(wx)) of
+                [$w,$x,$-,Major,_,Minor|_] ->
+                    Version = {Major-$0, Minor-$0},
+                    put(wx_version, Version),
+                    Version >= Ver;
+                _Vsn -> %% erlang src build? assume 19.2 or later
+                    true
+            end;
+        Version ->
+            Version >= Ver
+    end.
+
 %%
 %% Create a unique name by appending digits.
 %%
@@ -229,6 +265,33 @@ tc(Fun,Mod,Line) ->
     io:format("~p:~p: Time: ~p\n", [Mod, Line, timer:now_diff(After,Before)]),
     R.
 
+profile_start(fprof) ->
+    fprof:trace(start),
+    ok;
+profile_start(eprof) ->
+    eprof:start(),
+    profiling = eprof:start_profiling([self()]),
+    ok.
+
+profile_stop(fprof) ->
+    fprof:trace(stop),
+    spawn_link(fun() ->
+                       File = "fprof.analysis",
+                       fprof:profile(),
+                       fprof:analyse([{dest, File}, {cols, 120}]),
+                       io:format("Analysis in: ~p~n", [filename:absname(File)]),
+                       eprof:stop()
+               end),
+    ok;
+profile_stop(eprof) ->
+    eprof:stop_profiling(),
+    spawn_link(fun() ->
+                       File = "eprof.analysis",
+                       eprof:log(File),
+                       eprof:analyze(),
+                       io:format("Analysis in: ~p~n", [filename:absname(File)])
+               end).
+
 limit(Val, {'-infinity',infinity}) -> Val;
 limit(Val, {Min,infinity}) when Val < Min -> Min;
 limit(Val, {'-infinity',Max}) when Val > Max -> Max;
@@ -690,9 +753,4 @@ translation_string(materials_to_colors)  -> ?__(288,"Materials to Colors");
 
 %%%% Others as yet to be added are proccessed here
 translation_string(Atom) when is_atom(Atom) ->
-    wings_util:cap(atom_to_list(Atom));
-
-translation_string(Int) when is_integer(Int) ->
-    integer_to_list(Int);
-
-translation_string(_Other) -> [].
+    wings_util:cap(atom_to_list(Atom)).
