@@ -12,11 +12,11 @@
 %%
 
 -module(wings_view).
--export([menu/1,command/2,
+-export([menu/0,command/2,
 	 virtual_mirror/2,
 	 init/0,initial_properties/0,delete_all/1,reset/0,
 	 current/0,set_current/1,
-	 load_matrices/1,projection/0,
+	 load_matrices/1,projection/0,projection/1,
 	 modelview/0,align_view_to_normal/1,
 	 eye_point/0,export_views/1,import_views/2,camera_info/2,
 	 freeze_mirror/1]).
@@ -27,12 +27,13 @@
 
 -import(lists, [foldl/3,zip/2]).
 
-menu(#st{views={CurrentView,Views}}=St) ->
+menu() ->
     L = wings_pref:get_value(number_of_lights),
     [{?__(68,"Show"),{show,
      [{?__(1,"Ground Plane"),show_groundplane,?__(2,"Show the ground plane"),
        crossmark(show_groundplane)},
       {?__(3,"Axes"),show_axes,?__(4,"Show the coordinate axes"),crossmark(show_axes)},
+      {?__(78,"Image Plane"),show_cam_imageplane,?__(79,"Show the camera image plane"),crossmark(show_cam_imageplane)},
       {?__(48,"Show Info Text"),show_info_text,
        ?__(49,"Show an informational text at the top of this Geometry window"),
        crossmark(show_info_text)},
@@ -65,7 +66,7 @@ menu(#st{views={CurrentView,Views}}=St) ->
      {?__(7,"Wireframe"),wireframe,?__(8,"Display selected objects as a wireframe (same for all objects if nothing is selected)")},
      {?__(9,"Shade"),shade,?__(10,"Display selected objects as shaded (same for all objects if nothing is selected)")},
      {?__(11,"Toggle Wireframe"),toggle_wireframe,
-      ?__(12,"Toggle display mode for selected objects (same for all objects if nothing is selected)"),wireframe_crossmark(St)},
+      ?__(12,"Toggle display mode for selected objects (same for all objects if nothing is selected)")},
      {?__(19,"Show Edges"),show_edges,?__(20,"Show edges in workmode"),crossmark(show_edges)},
      {?__(72,"Show Backfaces"),show_backfaces,
       ?__(73,"Show backfaces when there is a hole or hiddwn faces in an object"),crossmark(show_backfaces)},
@@ -83,9 +84,8 @@ menu(#st{views={CurrentView,Views}}=St) ->
      [{?__(36,"Scene Lights"),scene_lights,
        ?__(37,"Use the lights defined in the scene"),
        crossmark(scene_lights)},
-      {one_of(L == 1, ?__(38,"Two Lights"),?__(39,"One Light")),toggle_lights,
-       one_of(L == 1, ?__(40,"Use two work lights"),
-       ?__(41,"Use one work light"))},
+      {one_of(L == 1, ?__(38,"Use Camera Light"),?__(39,"Use a camera work Light")),toggle_lights,
+       one_of(L == 1, ?__(40,"Use two work lights"),?__(41,"Use a simple sky light simulation"))},
       separator,
       {?__(31,"Orthographic View"),orthogonal_view,
        ?__(32,"Toggle between orthographic and perspective views"),
@@ -95,12 +95,13 @@ menu(#st{views={CurrentView,Views}}=St) ->
       {?__(66,"Highlight Aim"),highlight_aim,
        ?__(67,"Aim camera at mouseover highlight. (Requires 'Use Highlight as Temporary Selection' enabled, and an assigned hotkey)")},
       {?__(29,"Frame"),frame,?__(30,"Dolly to show all selected elements (or all objects if nothing is selected)")},
-      {?__(65,"Frame Disregards Mirror"),frame_mode,crossmark(frame_disregards_mirror)},
+      {?__(65,"Frame Disregards Mirror"),frame_disregards_mirror,crossmark(frame_disregards_mirror)},
       {?__(57,"Align to Selection"),align_to_selection,
        ?__(58,"Align the view to the normal of the selection")},
       separator,
-      {?__(33,"Saved Views: ")++integer_to_list(tuple_size(Views)),
-       {views,views_submenu(CurrentView, Views)}},
+      {?__(33,"Saved Views "),
+       {views, 	views_submenu()}},
+
       {?__(50,"View Along"),{along,[{?__(51,"+X"),x},
 				    {?__(52,"+Y"),y},
 				    {?__(53,"+Z"),z},
@@ -115,72 +116,16 @@ menu(#st{views={CurrentView,Views}}=St) ->
       {?__(59,"Auto Rotate"),auto_rotate,?__(60,"Spin the view")}]].
 
 crossmark(Key) ->
-    wings_menu_util:crossmark(Key).
+    %% Start with everything false, update when values loaded
+    [{crossmark, wings_pref:get_value(Key, false)}].
 
-wireframe_crossmark(#st{sel=[],shapes=Shs}) ->
-    {menubar,Client} = wings_wm:this(),
-    Wire = wings_wm:get_prop(Client, wireframed_objects),
-    case {gb_sets:size(Wire),gb_trees:size(Shs)} of
-	{0,_} -> [];
-	{Same,Same} -> [crossmark];
-	{_,_} -> [grey_crossmark]
-    end;
-wireframe_crossmark(#st{sel=Sel0}) ->
-    {menubar,Client} = wings_wm:this(),
-    Wire0 = wings_wm:get_prop(Client, wireframed_objects),
-    Sel = gb_sets:from_list([Id || {Id,_} <- Sel0]),
-    Wire = gb_sets:intersection(Sel, Wire0),
-    case {gb_sets:size(Wire),gb_sets:size(Sel)} of
-	{0,_} -> [];
-	{Same,Same} -> [crossmark];
-	{_,_} -> [grey_crossmark]
-    end.
+views_submenu() ->
+    [{?__(1,"Next"),next, ?__(11, "Move camera to next view")},
+     {?__(2,"Current"),current, ?__(12, "Move camera to current view")},
+     {?__(3,"Prev"),prev, ?__(13, "Move camera to current view")},
+     {?__(4,"Save..."),save, ?__(14,"Save this view"), [option]},
+     {?__(8,"Delete"),delete, ?__(18, "Delete the current view")}].
 
-views_submenu(CurrentView, Views) ->
-    {_,_,_,H} = wings_wm:viewport(desktop),
-    Lines = max((H div ?LINE_HEIGHT) - 3, 4),
-    S = tuple_size(Views),
-    C = if S > 0 -> view_index(CurrentView, S); true -> 0 end,
-    [{?__(1,"Next"),next,views_submenu_help(CurrentView, Views, next)},
-     {?__(2,"Current"),current,views_submenu_help(CurrentView, Views, current)},
-     {?__(3,"Prev"),prev,views_submenu_help(CurrentView, Views, prev)},
-     views_jumpmenu(CurrentView, Views, Lines),
-     {?__(4,"Save"),save,
-      ?__(5,"Save this view at ") ++"["++integer_to_list(C+1)++"]",[option]},
-     views_movemenu(CurrentView, Views, Lines),
-     {?__(7,"Rename..."),rename,views_submenu_help(CurrentView, Views, rename)},
-     {?__(8,"Delete"),delete,views_submenu_help(CurrentView, Views, delete)},
-     {?__(9,"Delete All..."),delete_all,
-      views_submenu_help(CurrentView, Views, delete_all)}].
-
-views_submenu_help(_CurrentView, {}, _Action) ->
-    ?__(1,"No saved views!");
-views_submenu_help(CurrentView, Views, Action) ->
-    S = tuple_size(Views),
-    case Action of
-	next ->
-	    N = view_index(CurrentView+1, S),
-	    {_,Legend} = element(N, Views),
-	    ?__(2,"Jump to \"")++Legend++"\"["++integer_to_list(N)++"]";
-	current ->
-	    C = view_index(CurrentView, S),
-	    {_,Legend} = element(C, Views),
-	    ?__(2,"Jump to \"")++Legend++"\"["++integer_to_list(C)++"]";
-	prev ->
-	    P = view_index(CurrentView-1, S),
-	    {_,Legend} = element(P, Views),
-	    ?__(2,"Jump to \"")++Legend++"\"["++integer_to_list(P)++"]";
-	rename ->
-	    C = view_index(CurrentView, S),
-	    {_,Legend} = element(C, Views),
-	    ?__(5,"Rename \"")++Legend++"\"["++integer_to_list(C)++"]";
-	delete ->
-	    C = view_index(CurrentView, S),
-	    {_,Legend} = element(view_index(CurrentView, S), Views),
-	    ?__(8,"Delete \"")++Legend++"\"["++integer_to_list(C)++"]";
-	delete_all ->
-	    ?__(11,"Delete all saved views")
-    end.
 
 view_index(I, N) when is_integer(I), is_integer(N), N > 0 ->
     J = (I-1) rem N,
@@ -188,84 +133,6 @@ view_index(I, N) when is_integer(I), is_integer(N), N > 0 ->
        true -> J + 1
     end.
 
-views_jumpmenu(_CurrentView, {}, _Lines) ->
-    {?__(1,"Jump"),current,?__(2,"No saved views!")};
-views_jumpmenu(CurrentView, Views, Lines) ->
-    S = tuple_size(Views),
-    P = view_index(CurrentView-1, S),
-    C = view_index(CurrentView, S),
-    N = view_index(CurrentView+1, S),
-    F = fun (I) ->
-		{_,Legend} = element(I, Views),
-		Help =
-		    case I of
-			P -> ?__(3,"Jump to prev[")++integer_to_list(I)++"]";
-			C -> ?__(5,"Jump to current[")++integer_to_list(I)++"]";
-			N -> ?__(7,"Jump to next[")++integer_to_list(I)++"]";
-			_ -> ?__(9,"Jump to [")++integer_to_list(I)++"]"
-		    end,
-		{Legend,I,Help}
-	end,
-    {?__(11,"Jump"),{jump,viewmenu(F, S, C, Lines)}}.
-
-views_movemenu(_CurrentView, {}, _Lines) ->
-    {?__(1,"Move Current"),current,?__(2,"No saved views!")};
-views_movemenu(CurrentView, Views, Lines) ->
-    S = tuple_size(Views),
-    P = view_index(CurrentView-1, S),
-    C = view_index(CurrentView, S),
-    N = view_index(CurrentView+1, S),
-    Lc = integer_to_list(C),
-    {_,CL} = element(C, Views),
-    F = fun (I) ->
-		{_,Legend} = element(I, Views),
-		Li = integer_to_list(I),
-		Help =
-		    case I of
-			P -> ?__( 3,"Move \"")++CL++"\"["++Lc++?__(5,"] to prev[")++Li++"]";
-			C -> ?__( 7,"Move \"")++CL++"\"["++Lc++?__(9,"] nowhere");
-			N -> ?__(10,"Move \"")++CL++"\"["++Lc++?__(12,"] to next[")++Li++"]";
-			_ -> ?__(14,"Move \"")++CL++"\"["++Lc++?__(16,"] to [")++Li++"]"
-		    end,
-		{Legend,I,Help}
-	end,
-    {?__(18,"Move Current"),{move,viewmenu(F, S, C, Lines)}}.
-
-%% Build a list of F(I) values for all view_index(I, S) starting from C
-%% half of them after C and half before, no more than Lines If the
-%% list has to be limited by Lines - insert an element 'separator'
-%% between the after and before elements, otherwise all view
-%% indexes from 1 upto S will be represented in the result with
-%% C last in the list.
-%%
-%% Like this when Lines == 4, S > 4 ->
-%%   [F(view_index(C+1, S)), F(view_index(C+2, S)), separator,
-%%    F(view_index(C-1, S)), F(view_index(C, S))].
-%%
-viewmenu(F, S, C, Lines) ->
-    N = view_index(C+1, S),
-    viewmenu_2(F, S, N, C, Lines, [], []).
-
-viewmenu_1(F, _S, N, N, _Lines, Next, Prev) ->
-    lists:reverse(Next, [F(N)|Prev]);
-viewmenu_1(F, S, N, P, Lines, Next, Prev) ->
-    Cnt = view_index(N-P, S),
-    if Cnt >= Lines ->
-	    lists:reverse(Next, [F(N),separator|Prev]);
-       true ->
-	    viewmenu_2(F, S, view_index(N+1, S), P, Lines, [F(N)|Next], Prev)
-    end.
-
-viewmenu_2(F, _S, P, P, _Lines, Next, Prev) ->
-    lists:reverse(Next, [F(P)|Prev]);
-viewmenu_2(F, S, N, P, Lines, Next, Prev) ->
-    Cnt = view_index(N-P, S),
-    if Cnt >= Lines ->
-	    lists:reverse(Next, [separator,F(P)|Prev]);
-       true ->
-	    viewmenu_1(F, S, N, view_index(P-1, S), Lines, Next, [F(P)|Prev])
-    end.
-
 command(reset, St) ->
     reset(),
     St;
@@ -299,12 +166,12 @@ command(quick_preview, St) ->
 command(orthogonal_view, St) ->
     toggle_option(orthogonal_view),
     St;
-command({show,What}, St) ->
-    Prev = toggle_option(What),
-    if 
+command(Option={show,What}, St) ->
+    Prev = toggle_option(Option),
+    if
 	What =:= show_normals ->
 	    Prev andalso wings_dl:map(fun(D, _) -> D#dlo{normals=none} end, []);
-	What =:= show_materials; What =:= filter_texture ->
+	What =:= filter_texture ->
 	    wings_dl:map(fun(#dlo{proxy_data=PD}=D, _) ->
 				 %% We only need to invalidate display lists.
 				 D#dlo{work=none,smooth=none,
@@ -320,8 +187,7 @@ command({show,What}, St) ->
     end,
     St;
 command(show_edges, St) ->
-    Bool = wings_pref:get_value(show_edges),
-    wings_pref:set_value(show_edges, not Bool),
+    Bool = toggle_option(show_edges),
     case Bool of
 	false -> St;
 	true ->
@@ -329,8 +195,7 @@ command(show_edges, St) ->
 	    St
     end;
 command(show_backfaces, St) ->
-    Bool = wings_pref:get_value(show_backfaces),
-    wings_pref:set_value(show_backfaces, not Bool),
+    toggle_option(show_backfaces),
     St;
 command({highlight_aim,{Type,{Selmode,Sel,MM}}}, St) ->
     highlight_aim(Type, Selmode, Sel, MM, St),
@@ -344,10 +209,6 @@ command(aim, St) ->
 command(frame, St) ->
     frame(St),
     St;
-command(frame_mode, St) ->
-    Bool = wings_pref:get_value(frame_disregards_mirror),
-    wings_pref:set_value(frame_disregards_mirror, not Bool),
-    St;
 command({views,Views}, St) ->
     views(Views, St);
 command({along,Axis}, St) ->
@@ -357,7 +218,7 @@ command(auto_rotate, St) ->
     auto_rotate(St);
 command(rotate_left, St) ->
     #view{azimuth=Az0} = View = current(),
-    Az = Az0 + wings_pref:get_value(auto_rotate_angle),
+    Az = Az0 + ?GET(auto_rotate),
     set_current(View#view{azimuth=Az}),
     St;
 command(align_to_selection, St) ->
@@ -379,8 +240,7 @@ command({shader_set,N}, St) ->
     shader_set(N),
     St;
 command(camera_settings, St) ->
-    camera(),
-    St;
+    camera(St);
 command(Key, St) ->
     toggle_option(Key),
     St.
@@ -470,13 +330,14 @@ sel_mirror_objects(St) ->
 
 
 
--define(RANGE_FOV, {1.0,180.0}).
+-define(RANGE_FOV, {1.0,179.9}).
 -define(RANGE_NEAR_CLIP, {0.01,1000.0}).
 -define(RANGE_FAR_CLIP, {100.0,infinity}).
 -define(RANGE_ZOOM_SLIDER, {-1.0,4.0}).
 -define(RANGE_NEGATIVE_SIZE, {1,infinity}).
+-define(RANGE_LENS_LENGTH_ZOOM, {0.01,infinity}).
 
-camera() ->
+camera(St) ->
     Active = wings_wm:this(),
     View0 = wings_wm:get_prop(Active, current_view),
     NegH = wings_pref:get_value(negative_height),
@@ -485,6 +346,8 @@ camera() ->
     NegativeFormat =
 	case {NegH,NegW} of
 	    {24,36} -> {24,36};
+	    {34,60} -> {34,60};
+	    {45,60} -> {45,60};
 	    {60,60} -> {60,60};
 	    _ -> custom
 	end,
@@ -496,49 +359,53 @@ camera() ->
     LensLength = pget(lens_length, Props),
     Zoom = pget(zoom, Props),
     ZoomSlider = pget(zoom_slider, Props),
-    FovHook =
-	fun (update, {Var,_I,Val,Sto}) ->
-		{store,camera_update_1(Var, Val, Sto)};
-	    (_, _) -> void
-	end,
+    FovHook = fun(Var,Val,Sto) ->
+		    camera_update_1(Var, Val, Sto)
+	      end,
+    FovHook2 = fun(Var,Val,Sto) -> camera_update_2(Var, Val, Sto) end,
+    NegHook = fun(_Var, Val, Sto) ->
+		    case Val of
+			default ->
+			    #view{fov=Fov1,hither=Hither1,yon=Yon1} = DefView = default_view(),
+			    Props =
+				camera_propconv(from_fov,
+						[{negative_height,NegH},{negative_width,NegW},{fov,Fov1}]),
+			    gbupdate([{fov, Fov0},{height, Hither1},{yon, Yon1},
+				      {lens_type, pget(lens_type, Props)},
+				      {lens_length, pget(lens_length, Props)},
+				      {zoom, pget(zoom, Props)},
+				      {zoom_slider, pget(zoom_slider, Props)},
+				      {negative_format,{NegH,NegW}}], Sto),
+			    Keys = [negative_height, negative_width],
+			    wings_dialog:enable(Keys, false, Sto),
+			    wings_dialog:enable(lens_type, true, Sto),
+			    wings_wm:set_prop(Active, current_view, DefView);
+			_ ->
+			    Keys = [negative_height, negative_width],
+			    wings_dialog:enable(Keys, Val =:= custom, Sto),
+			    wings_dialog:enable(lens_type, Val =/= custom, Sto),
+			    FovHook2(lens_type, wings_dialog:get_value(lens_type, Sto), Sto)
+		    end
+	      end,
     LensFrame =
 	{vframe,
 	 [{hframe,
 	   [{label,?__(1,"Negative Format")},
 	    {menu,
-	     [{"24x36",{24,36}},{"60x60",{60,60}},{?__(4,"Custom"),custom}],
+	     [{"24x36 [3:2]" ,{24,36}},
+	      {"34x60 [16:9]",{34,60}},
+	      {"45x60 [4:3]" ,{45,60}},
+	      {"60x60 [1:1]" ,{60,60}},
+	      {?__(4,"Custom"),custom},
+	      {?__(19,"Reset to default"),default}],
 	     NegativeFormat,
-	     [{key,negative_format},layout,
-	     {hook,
-	      fun (update, {Var,_I,Val,Sto}) ->
-		      {store,camera_update_1(Var, Val, Sto)};
-		  (_, _)  -> void
-	      end}]},
+	     [{key,negative_format}, {hook, NegHook}]},
 	    {hframe,
-	     [{text,
-	       NegH,
-	       [{key,negative_height},{range,?RANGE_NEGATIVE_SIZE},
-		{hook,
-		 fun (update, {Var,_I,Val,Sto}) ->
-			 {store,camera_update_1(Var, Val, Sto)};
-		     (_, _)  -> void
-		 end}]},
+	     [{text, NegH, [{key,negative_height},{range,?RANGE_NEGATIVE_SIZE},
+			    {hook,FovHook}]},
 	      {label,?__(5,"x")},
-	      {text,
-	       NegW,
-	       [{key,negative_width},{range,?RANGE_NEGATIVE_SIZE},
-		{hook,
-		 fun (update, {Var,_I,Val,Sto}) ->
-			 {store,camera_update_1(Var, Val, Sto)};
-		     (_, _)  -> void
-		 end}]}],
-	     [{hook,
-	       fun (is_disabled, {_Var,_I,Sto}) ->
-		       gbget(negative_format, Sto) =/= custom;
-		   (update, {Var,_I,Val,Sto}) ->
-		       {store,camera_update_2(Var, Val, Sto)};
-		   (_, _) -> void
-	       end}]}]},
+	      {text, NegW, [{key,negative_width},{range,?RANGE_NEGATIVE_SIZE},
+			    {hook,FovHook}]}]}]},
 	  {hframe,
 	   [{menu,
 	     [{?__(6,"Wide-Angle Lens"),wide_angle},
@@ -548,41 +415,15 @@ camera() ->
 	      {?__(10,"Telephoto Lens"),tele},
 	      {?__(11,"Custom Lens"),custom}],
 	     LensType,
-	     [{key,lens_type},
-	      {hook,
-	       fun (is_minimized, {_Var,_I,Sto}) ->
-		       gbget(negative_format, Sto) =:= custom;
-		   (update, {Var,_I,Val,Sto}) ->
-		       {store,camera_update_2(Var, Val, Sto)};
-		   (_, _) -> void
-	       end}]},
+	     [{key,lens_type}, {hook,FovHook2}]},
 	    panel,
 	    {label,?__(12,"Length")},
-	    {text,
-	     LensLength,
-	     [{key,lens_length},
-	      {hook,
-	       fun (update, {Var,_I,Val,Sto}) ->
-		       {store,camera_update_2(Var, Val, Sto)};
-		   (_, _) -> void
-	       end}]}]},
+	    {text,LensLength,[{key,lens_length}, {range,?RANGE_LENS_LENGTH_ZOOM}, {hook,FovHook2}]}]},
 	  {hframe,
 	   [{slider,
 	     [{key,zoom_slider},{range,?RANGE_ZOOM_SLIDER},
-	      {value,ZoomSlider},
-	      {hook,
-	       fun (update, {Var,_I,Val,Sto}) ->
-		       {store,camera_update_2(Var, Val, Sto)};
-		   (_, _) -> void
-	       end}]},
-	    {text,
-	     Zoom,
-	     [{key,zoom},
-	      {hook,
-	       fun (update, {Var,_I,Val,Sto}) ->
-		       {store,camera_update_2(Var, Val, Sto)};
-		   (_, _) -> void
-	       end}]},
+	      {value,ZoomSlider}, {hook,FovHook2}]},
+	    {text, Zoom,[{key,zoom}, {range,?RANGE_LENS_LENGTH_ZOOM}, {hook,FovHook2}]},
 	    {label,?__(13,"x Zoom")}]}],
 	 [{title,?__(14,"Lens")},{minimized,true}]},
     Qs =
@@ -592,25 +433,41 @@ camera() ->
 		    {label,?__(16,"Near Clipping Plane")},
 		    {label,?__(17,"Far Clipping Plane")}]},
 	   {vframe,[{text,Fov0,[{range,?RANGE_FOV},{key,fov},{hook,FovHook}]},
-		    {text,Hither0,[{range,?RANGE_NEAR_CLIP}]},
-		    {text,Yon0,[{range,?RANGE_FAR_CLIP}]}]},
+		    {text,Hither0,[{key,height},{range,?RANGE_NEAR_CLIP}]},
+		    {text,Yon0,[{key,yon},{range,?RANGE_FAR_CLIP}]}]},
 	   {vframe,[help_button(camera_settings_fov),
 		    panel,
 		    panel]}]}],
-    wings_ask:dialog(?__(18,"Camera Settings"), Qs,
-		     fun([_,
-			  {negative_format,_},
-			  {negative_height,_},{negative_width,_},
-			  {lens_type,_},{lens_length,_},
-			  {zoom_slider,_},{zoom,_},
-			  {fov,Fov},Hither,Yon]=Ps) ->
-			     {NH,NW} = camera_propconv_negative_format(Ps),
-			     View = View0#view{fov=Fov,hither=Hither,yon=Yon},
-			     wings_wm:set_prop(Active, current_view, View),
-			     wings_pref:set_value(negative_height, NH),
-			     wings_pref:set_value(negative_width, NW),
-			     ignore
-		     end).
+    ShowImgPlane = wings_wm:get_prop(show_cam_imageplane),
+    wings_wm:set_prop(show_cam_imageplane, true),
+    wings_dialog:dialog(?__(18,"Camera Settings"), {preview,Qs},
+			fun
+			    ({dialog_preview,Res}) ->
+				update_camera(Active, View0, Res),
+				{preview,St,St};
+			    (cancel) ->
+				wings_wm:set_prop(Active, show_cam_imageplane, ShowImgPlane),
+				wings_wm:set_prop(Active, current_view, View0),
+				St;
+			    (Res) ->
+				update_camera(Active, View0, Res),
+				wings_wm:set_prop(Active, show_cam_imageplane, ShowImgPlane),
+				{commit,St,St}
+			end).
+
+update_camera(Win, View0, Ps) ->
+    [{negative_format,_},
+     {negative_height,_},{negative_width,_},
+     {lens_type,_},{lens_length,_},
+     {zoom_slider,_},{zoom,_},
+     {fov,Fov},{height,Hither},{yon,Yon}] = Ps,
+
+    {NH,NW} = camera_propconv_negative_format(Ps),
+    View = View0#view{fov=Fov,hither=Hither,yon=Yon},
+    wings_wm:set_prop(Win, current_view, View),
+    wings_pref:set_value(negative_height, NH),
+    wings_pref:set_value(negative_width, NW),
+    ignore.
 
 camera_update_1(Var, Val, Sto) ->
     Props = gbget(lists:delete(Var, [fov,negative_format,
@@ -700,6 +557,24 @@ camera_lens_type({24,36}, LensLength) ->
 	135 -> tele;
 	_ -> custom
     end;
+camera_lens_type({34,60}, LensLength) ->
+    case round(LensLength) of
+	35 -> wide_angle;
+	55 -> moderate_wide_angle;
+	80 -> standard;
+	135 -> short_tele;
+	215 -> tele;
+	_ -> custom
+    end;
+camera_lens_type({45,60}, LensLength) ->
+    case round(LensLength) of
+	40 -> wide_angle;
+	60 -> moderate_wide_angle;
+	85 -> standard;
+	150 -> short_tele;
+	235 -> tele;
+	_ -> custom
+    end;
 camera_lens_type({60,60}, LensLength) ->
     case round(LensLength) of
 	40 -> wide_angle;
@@ -719,6 +594,22 @@ camera_lens_length({24,36}, LensType) ->
 	      short_tele	  -> 85;
 	      tele		  -> 135
 	  end);
+camera_lens_length({34,60}, LensType) ->
+    float(case LensType of
+	      wide_angle	  -> 35;
+	      moderate_wide_angle -> 55;
+	      standard		  -> 80;
+	      short_tele	  -> 135;
+	      tele		  -> 215
+	  end);
+camera_lens_length({45,60}, LensType) ->
+    float(case LensType of
+	      wide_angle	  -> 40;
+	      moderate_wide_angle -> 60;
+	      standard		  -> 85;
+	      short_tele	  -> 150;
+	      tele		  -> 250
+	  end);
 camera_lens_length({60,60}, LensType) ->
     float(case LensType of
 	      wide_angle	  -> 40;
@@ -729,13 +620,10 @@ camera_lens_length({60,60}, LensType) ->
 	  end);
 camera_lens_length(_, _) -> undefined.
 
-
-
 gbget([], _Sto) -> [];
-gbget([Key|Keys], Sto) -> [{Key,gb_trees:get(Key, Sto)}|gbget(Keys, Sto)];
-gbget(Key, Sto) -> gb_trees:get(Key, Sto).
+gbget([Key|Keys], Sto) -> [{Key,wings_dialog:get_value(Key, Sto)}|gbget(Keys, Sto)].
 
-gbupdate(Key, Val, Sto) -> gb_trees:update(Key, Val, Sto).
+gbupdate(Key, Val, Sto) -> wings_dialog:set_value(Key, Val, Sto).
 
 gbupdate([], Sto) -> Sto;
 gbupdate([{Key,Val}|KVs], Sto) ->
@@ -758,12 +646,17 @@ pget(Key, Props) -> proplists:get_value(Key, Props).
 
 auto_rotate(St) ->
     auto_rotate_help(),
-    Delay = wings_pref:get_value(auto_rotate_delay),
+    Delay = 16,
+    ?SET(auto_rotate, min(0.1333, 0.0333+wings_pref:get_value(auto_rotate_angle)/5)),
     Tim = #tim{delay=Delay,st=St},
     Active = wings_wm:this(),
-    wings_wm:callback(fun() -> wings_u:menu_restriction(Active, []) end),
+    {W,H} = wings_wm:win_size(Active),
+    X = W div 2, Y = H div 2,
+    wings_io:warp(X,Y),
     {seq,push,set_auto_rotate_timer(Tim)}.
 
+auto_rotate_event({action, Cmd={view, rotate_left}}, Tim) ->
+    auto_rotate_event_1(Cmd, Tim);
 auto_rotate_event(Event, #tim{timer=Timer,st=St}=Tim) ->
     case wings_camera:event(Event, St) of
 	next -> auto_rotate_event_1(Event, Tim);
@@ -780,13 +673,19 @@ auto_rotate_event_1(redraw, Tim) ->
     auto_rotate_redraw(Tim),
     keep;
 auto_rotate_event_1(#mousemotion{}, _) -> keep;
+auto_rotate_event_1(got_focus, _) -> keep;
 auto_rotate_event_1(#mousebutton{state=?SDL_PRESSED}, _) -> keep;
-auto_rotate_event_1(#keyboard{}=Kb, #tim{delay=Delay}=Tim) ->
+auto_rotate_event_1(#keyboard{}=Kb, Tim) ->
+    Deg = ?GET(auto_rotate),
+    Incr = max(0.1, abs(Deg)*0.1),
     case wings_hotkey:event(Kb) of
 	{select,more} ->
-	    get_event(Tim#tim{delay=Delay-10});
+            ?SET(auto_rotate, min(15,  Deg+Incr)),
+            get_event(Tim);
 	{select,less} ->
-	    get_event(Tim#tim{delay=Delay+10});
+            Deg = ?GET(auto_rotate),
+            ?SET(auto_rotate, max(-15, Deg-Incr)),
+	    get_event(Tim);
 	_ ->
 	    keep
     end;
@@ -815,10 +714,10 @@ auto_rotate_help() ->
     Message = wings_msg:join([Msg1,Msg2,P,M]),
     wings_wm:message(Message).
 
-set_auto_rotate_timer(#tim{delay=Delay}=Tim) when Delay < 0 ->
-    set_auto_rotate_timer(Tim#tim{delay=0});
+set_auto_rotate_timer(#tim{delay=Delay}=Tim) when Delay < 16 ->
+    set_auto_rotate_timer(Tim#tim{delay=16});
 set_auto_rotate_timer(#tim{delay=Delay}=Tim0) ->
-    Timer = wings_io:set_timer(Delay, {view,rotate_left}),
+    Timer = wings_io:set_timer(Delay, {action, {view,rotate_left}}),
     Tim = Tim0#tim{timer=Timer},
     get_event(Tim).
 
@@ -829,17 +728,20 @@ get_event(Tim) ->
 %%% Other stuff.
 %%%
 
-toggle_option({show,Key}) ->
-%% process Show menu items
-    toggle_option(Key);
-toggle_option(Key) ->
+toggle_option(Key0) ->
+    Key = case Key0 of
+	      {show, K} -> K;
+	      _ -> Key0
+	  end,
     case wings_wm:lookup_prop(Key) of
 	none ->
 	    Prev = wings_pref:get_value(Key, false),
 	    wings_pref:set_value(Key, not Prev),
+	    wings_wm:send(top_frame, {menu,{view, Key0, not Prev}}),
 	    Prev;
 	{value,Bool} ->
 	    wings_wm:set_prop(Key, not Bool),
+	    wings_wm:send(top_frame, {menu,{view, Key0, not Bool}}),
 	    Bool
     end.
 
@@ -872,6 +774,7 @@ initial_properties() ->
      {clip_plane,false},
      {show_axes,true},
      {show_groundplane,true},
+     {show_cam_imageplane,false},
      {wireframed_objects,gb_sets:empty()},
      {current_view,default_view()},
      {allow_rotation,true},
@@ -903,11 +806,15 @@ default_view() ->
 
 load_matrices(IncludeLights) ->
     gl:matrixMode(?GL_PROJECTION),
-    gl:loadIdentity(),
-    projection(),
-    modelview(IncludeLights).
+    TPM = projection(e3d_transform:identity()),
+    {UseSceneLights, TMM} = modelview(IncludeLights),
+    {TPM,TMM, UseSceneLights}.
 
 projection() ->
+    OP0 = gl:getDoublev(?GL_PROJECTION_MATRIX),
+    projection(e3d_transform:init(list_to_tuple(OP0))).
+
+projection(In) ->
     {W,H} = wings_wm:win_size(),
     Aspect = W/H,
     #view{distance=D,fov=Fov,hither=Hither,yon=Yon,along_axis=AA} =
@@ -915,13 +822,16 @@ projection() ->
     Ortho = wings_wm:get_prop(orthogonal_view)
 	orelse ((AA =/= none) andalso
 		wings_pref:get_value(force_ortho_along_axis)),
-    case Ortho of
-	false ->
-	    glu:perspective(Fov, Aspect, Hither, Yon);
-	true ->
-	    Sz = D*math:tan(Fov*math:pi()/180/2),
-	    gl:ortho(-Sz*Aspect, Sz*Aspect, -Sz, Sz, Hither, Yon)
-    end.
+    TP = case Ortho of
+	     false ->
+		 e3d_transform:perspective(Fov, Aspect, Hither, Yon);
+	     true ->
+		 Sz = D*math:tan(Fov*math:pi()/180/2),
+		 e3d_transform:ortho(-Sz*Aspect, Sz*Aspect, -Sz, Sz, Hither, Yon)
+	 end,
+    TPM = e3d_transform:mul(In, TP),
+    gl:loadMatrixd(e3d_transform:matrix(TPM)),
+    TPM.
 
 modelview() ->
     modelview(false).
@@ -944,17 +854,18 @@ modelview(IncludeLights) ->
 	    UseSceneLights = false
     end,
 
-    gl:translatef(PanX, PanY, -Dist),
-    gl:rotatef(El, 1, 0, 0),
-    gl:rotatef(Az, 0, 1, 0),
-    {OX,OY,OZ} = Origin,
-    gl:translatef(OX, OY, OZ),
+    TM0 = e3d_transform:translate(e3d_transform:identity(), {PanX, PanY, -Dist}),
+    TM1 = e3d_transform:rotate(TM0, El, {1.0,0.0,0.0}),
+    TM2 = e3d_transform:rotate(TM1, Az, {0.0,1.0,0.0}),
+    TMM = e3d_transform:translate(TM2, Origin),
+
+    gl:loadMatrixd(e3d_transform:matrix(TMM)),
 
     case UseSceneLights of
 	false -> ok;
 	true -> wings_light:global_lights()
     end,
-    UseSceneLights.
+    {UseSceneLights, TMM}.
 
 %% Calculate the location of the viewer in 3D space.
 %% (The (0,0,0) point multiplied by the inverse model transformation matrix.)
@@ -1123,13 +1034,13 @@ views({move,J}, #st{views={CurrentView,Views}}=St) ->
 views(rename, #st{views={CurrentView,Views}}=St) ->
     J = view_index(CurrentView, tuple_size(Views)),
     {View,Legend} = element(J, Views),
-    wings_ask:dialog(
-      ?__(3,"Rename view"),
-      views_rename_qs([Legend]),
-      fun([NewLegend]) ->
-	      St#st{views={CurrentView,
-			   setelement(J, Views, {View,NewLegend})}}
-      end);
+    wings_dialog:dialog(?__(3,"Rename view"),
+			views_rename_qs([Legend]),
+			fun([[]]) -> ignore;
+			   ([NewLegend]) ->
+				St#st{views={CurrentView,
+					     setelement(J, Views, {View,NewLegend})}}
+			end);
 views(delete, #st{views={CurrentView,Views}}=St) ->
     View = current(),
     J = view_index(CurrentView, tuple_size(Views)),
@@ -1151,9 +1062,9 @@ views(delete_all, St) ->
       end).
 
 views_save_dialog(Ask, Options) ->
-    wings_ask:dialog(Ask, ?__(1,"Save view as"),
-		     views_rename_qs(Options),
-		     fun(Opts) -> {view,{views,{save,Opts}}} end).
+    wings_dialog:dialog(Ask, ?__(1,"Save view as"),
+			views_rename_qs(Options),
+			fun(Opts) -> {view,{views,{save,Opts}}} end).
 
 views_rename_qs([Legend]) ->
     [{hframe,[{label,?__(1,"Name")},{text,Legend}]}].
@@ -1197,14 +1108,24 @@ views_move(J, St, CurrentView, Views) ->
 toggle_lights() ->
     %% Invalidate displaylists so that shader data get set correctly
     %% for materials
+    case wings_pref:get_value(scene_lights, true) of
+        false -> ok;
+        true -> toggle_option(scene_lights)
+    end,
+
     wings_dl:map(fun(#dlo{proxy_data=PD}=D, _) ->
 			 D#dlo{work=none,smooth=none,
 			       proxy_data=wings_proxy:invalidate(PD, dl)}
 		 end, []),
-    Lights = case wings_pref:get_value(number_of_lights) of
-		 1 -> 2;
-		 2 -> 1
-	     end,
+    Lights0 = wings_pref:get_value(number_of_lights),
+    wings_menu:update_menu(view, toggle_lights,
+			   one_of(Lights0 == 1,
+				  ?__(2,"Use Camera Light"),
+				  ?__(1,"Use Hemisphere Light")),
+			   one_of(Lights0 == 1,
+				  ?__(4,"Use a camera work light"),
+				  ?__(3,"Use a simple sky light simulation"))),
+    Lights = 1 + (2 + Lights0) rem 2,
     wings_pref:set_value(number_of_lights, Lights).
 
 shader_set(N) ->
@@ -1466,8 +1387,6 @@ import_view([], #view{azimuth=Az,elevation=El}=View, Name) ->
 %%% import/export utilities
 %%%
 
-get_view_index(#view{}=View, Views) when is_tuple(Views) ->
-    get_view_index(View, tuple_to_list(Views));
 get_view_index(#view{}=View, Views) ->
     get_view_index_1(View, Views, 0).
 get_view_index_1(#view{}, [], _) -> undefined;
