@@ -19,6 +19,8 @@
 #include <shlobj.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <shellapi.h>
+#include <direct.h>
 
 static void install(void);
 static void print_path(FILE* fp, char* path);
@@ -63,14 +65,23 @@ WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrev, LPSTR szCmdLine, int sw)
 
   pref_dir[0] = L'\0';
   SHGetFolderPathW(NULL,	CSIDL_APPDATA|CSIDL_FLAG_CREATE, NULL, 0, pref_dir);
-  _snwprintf(cmd_line, 3*MAX_PATH,
-	   L"\"%s\\bin\\werl.exe\" -smp enable -detached -run wings_start start_halt",
-	   install_dir);
-  if (argc > 1) {
-      _snwprintf(cmd_line+wcslen(cmd_line), 3*MAX_PATH, L" \"%s\"", argv[1]);
+
+  _snwprintf(cmd_line, 3*MAX_PATH, L"\"%s\\bin\\werl.exe\" -smp enable", install_dir);
+  i=0;
+  if ((argc > 1 && wcscmp(argv[1], L"--debug") == 0)) {
+      i++;
+      _snwprintf(cmd_line+wcslen(cmd_line), 3*MAX_PATH,  L" -run wings_start start");
+  } else {
+      _snwprintf(cmd_line+wcslen(cmd_line), 3*MAX_PATH,  L" -detached");
+      _snwprintf(cmd_line+wcslen(cmd_line), 3*MAX_PATH,  L" -run wings_start start_halt");
+  }
+
+  if (argc > (1+i)) {
+      _snwprintf(cmd_line+wcslen(cmd_line), 3*MAX_PATH, L" \"%s\"", argv[1+i]);
   }
   _snwprintf(cmd_line+wcslen(cmd_line), 3*MAX_PATH, L" -extra \"%s\"", pref_dir);
 
+  // fprintf(stderr, "Cmd %S\r\n", cmd_line);
   siStartInfo.cb = sizeof(STARTUPINFO); 
   siStartInfo.wShowWindow = SW_MINIMIZE;
   siStartInfo.dwFlags = STARTF_USESHOWWINDOW;
@@ -98,7 +109,7 @@ install(void)
   FILE* fp = fopen("bin/erl.ini", "w");
   char dir[MAX_PATH];
 
-  getcwd(dir, MAX_PATH);
+  _getcwd(dir, MAX_PATH);
   if (fp == NULL) {
     MessageBox(NULL, "Failed to install Erlang/OTP components", NULL, MB_OK);
     exit(1);
